/* authuser.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
1/3/95: turned to autoconf, removed zmailer stuff, uwe@tirka.gun.de
21/7/92 Fixed SIGPIPE bug in auth_tcpuser3(). <pen@lysator.liu.se>
2/9/92: authuser 4.0. Public domain.
2/9/92: added bunches of zeroing just in case.
2/9/92: added auth_tcpuser3. uses bsd 4.3 select interface.
2/9/92: added auth_tcpsock, auth_sockuser.
2/9/92: added auth_fd2, auth_tcpuser2, simplified some of the code.
12/27/91: fixed up usercmp to deal with restricted tolower XXX
5/6/91 DJB baseline authuser 3.1. Public domain.
*/

#include "config.h"

#ifdef HAVE_SOCKET

#include <stdio.h>
#include <sys/types.h>
#ifdef TIME_WITH_SYS_TIME
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#include <time.h>
#else
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif
#endif

#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#else
#ifdef HAVE_SYS_FCNTL_H
#include <sys/fcntl.h>
#endif
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
extern int errno;
#include "authuser.h"

#ifndef FNDELAY
#define FNDELAY O_NDELAY
#endif

unsigned short auth_tcpport = 113;

#define SIZ 500 /* various buffers */

#if defined(SA_NOCLDSTOP)||defined(SA_ONSTACK)||defined(SA_RESTART)
    /* ================ POSIX.1 STUFF ================ */
#define SIGNAL_HANDLE(X,Y) \
	{                  \
		struct sigaction act;       \
		act.sa_handler = Y;     \
		act.sa_flags   = 0;     \
		sigemptyset(&act.sa_mask);  \
		sigaction (X, &act, NULL);  \
	}
#define SIGNAL_HANDLESAVE(X,Y,OLD) \
	{   				\
		struct sigaction act, oact; \
		act.sa_handler = Y;     \
		act.sa_flags   = 0;     \
		sigemptyset(&act.sa_mask);  \
		sigemptyset(&oact.sa_mask); \
		sigaction (X, &act, &oact); \
		OLD = oact.sa_handler;      \
	}
#define SIGNAL_HOLD(SIG) \
	{   			\
		sigset_t    sigmask;    \
		sigemptyset(&sigmask);      \
		sigaddset(&sigmask,SIG);    \
		sigprocmask(SIG_BLOCK,&sigmask,NULL);   \
	}
#define SIGNAL_RELEASE(SIG) \
	{   			\
		sigset_t    sigmask;    \
		sigemptyset(&sigmask);      \
		sigaddset(&sigmask,SIG);    \
		sigprocmask(SIG_UNBLOCK,&sigmask,NULL); \
	}
#define SIGNAL_IGNORE(SIG) \
	SIGNAL_HANDLE(SIG,SIG_IGN)

#else   /* No SIGACTION (POSIX) stuff */
#ifdef  SV_INTERRUPT    /* BSDism */
/* ================ BSD 4.3+ (?) STUFF ================ */
#define SIGNAL_HANDLE(X,Y)  \
	{   struct sigvec sv;           \
	sigvec(X, (struct sigvec *)NULL, &sv);  \
	sv.sv_handler = Y;          \
	sv.sv_flags |= SV_INTERRUPT;        \
	sigvec(X, &sv, (struct sigvec *)NULL);  \
	}
#define SIGNAL_HANDLESAVE(X,Y,OLD)  \
    {   struct sigvec sv;           \
    sigvec(X, (struct sigvec *)NULL, &sv);  \
    OLD = sv.sv_handler;            \
    sv.sv_handler = Y;          \
    sv.sv_flags |= SV_INTERRUPT;        \
    sigvec(X, &sv, (struct sigvec *)NULL);  \
    }
#define SIGNAL_HOLD(SIG) \
	sigblock(sigmask(SIG))
#define SIGNAL_RELEASE(SIG) \
	sigblock((~(sigmask(SIG))) & sigblock(0))
#define SIGNAL_IGNORE(SIG) \
	SIGNAL_HANDLE(SIG,SIG_IGN)

#else   /* No SIGACTION (POSIX), nor SIGVEC stuff */
/* ================ Everything  else.. ================ */
/*         ...  Well, I think it is  SVR3 ...       */
#define SIGNAL_HANDLE(X,Y)  \
	signal(X,Y)
#define SIGNAL_HANDLESAVE(X,Y,OLD)  \
	OLD=signal(X,Y)
#define SIGNAL_HOLD(SIG) \
	sighold(SIG)
#define SIGNAL_RELEASE(SIG) \
	sigrelse(SIG)
#define SIGNAL_IGNORE(SIG) \
	sigignore(SIG)

#endif
#endif


static void clearsa(struct sockaddr_in *sa)
{
  register char *x;
  for (x = (char *) sa;x < sizeof(*sa) + (char *) sa;++x)
    *x = 0;
}

static int usercmp(char *u, char *v)
{
  register char uc;
  register char vc=0;
  register char ucvc;
  /* is it correct to consider Foo and fOo the same user? yes */
  /* but the function of this routine may change later */
  while ((uc = *u) && (vc = *v))
    {
      ucvc = (isupper(uc) ? tolower(uc) : uc) - (isupper(vc) ? tolower(vc) : vc);
      if (ucvc)
	return ucvc;
      else
	++u,++v;
    }
  return uc || vc;
}

static char authline[SIZ];

/*
 * user: the supposed name of the user, NULL if unknown 
 * fd: the file descriptor of the connection 
 */
char *auth_infoline(char *user,int fd, unsigned long *in)
{
  unsigned short local;
  unsigned short remote;
  register char *ruser;

  if (auth_fd(fd,in,&local,&remote) == -1)
    return 0;
  ruser = auth_tcpuser(*in,local,remote);
  if (!ruser)
    return 0;
  if (!user)
    user = ruser;		/* forces X-Auth-User */
  sprintf(authline,
	  (usercmp(ruser,user) ? "forgery %s" : "authuser %s"),
	  ruser);
  return authline;
}

int auth_fd(int fd, unsigned long *in,unsigned short *local,
	unsigned short *remote)
{
  unsigned long inlocal;
  return auth_fd2(fd,&inlocal,in,local,remote);
}

int auth_fd2(int fd, unsigned long *inlocal, unsigned long *inremote,
	unsigned short * local, unsigned short *remote)
{
  struct sockaddr_in sa;
  size_t dummy;

  dummy = sizeof(sa);
  if (getsockname(fd,(struct sockaddr *)&sa,&dummy) == -1)
    return -1;
  if (sa.sin_family != AF_INET)
    {
      errno = EAFNOSUPPORT;
      return -1;
    }
  *local = ntohs(sa.sin_port);
  *inlocal = sa.sin_addr.s_addr;
  dummy = sizeof(sa);
  if (getpeername(fd,(struct sockaddr *)&sa,&dummy) == -1)
    return -1;
  *remote = ntohs(sa.sin_port);
  *inremote = sa.sin_addr.s_addr;
  return 0;
}

static char ruser[SIZ];
static char realbuf[SIZ];
static char *buf;

char *auth_tcpuser(
	register unsigned long in,
	register unsigned short local,
	register unsigned short remote)
{
  return auth_tcpuser2(0,in,local,remote);
}

#define CLORETS(e) { saveerrno = errno; close(s); errno = saveerrno; return e; }

int auth_tcpsock(unsigned long inlocal,unsigned long inremote)
{
  struct sockaddr_in sa;
  register int s;
  register int fl;
  register int saveerrno;

  if ((s = socket(AF_INET,SOCK_STREAM,0)) == -1)
    return -1;
  if (inlocal)
    {
      clearsa(&sa);
      sa.sin_family = AF_INET;
      sa.sin_port = 0;
      sa.sin_addr.s_addr = inlocal;
      if (bind(s,(struct sockaddr *)&sa,sizeof(sa)) == -1)
	CLORETS(-1);
    }
  if ((fl = fcntl(s,F_GETFL,0)) == -1)
    CLORETS(-1);
  if (fcntl(s,F_SETFL,FNDELAY | fl) == -1)
    CLORETS(-1);
  clearsa(&sa);
  sa.sin_family = AF_INET;
  sa.sin_port = htons(auth_tcpport);
  sa.sin_addr.s_addr = inremote;
  if (connect(s,(struct sockaddr *)&sa,sizeof(sa)) == -1)
    if (errno != EINPROGRESS)
      CLORETS(-1);
  return s;
}

char *auth_tcpuser2(unsigned long inlocal,unsigned long inremote,
	unsigned short local,unsigned short remote)
{
  register int s;

  s = auth_tcpsock(inlocal,inremote);
  if (s == -1)
    return 0;
  return auth_sockuser(s,local,remote);
}

char *auth_tcpuser3(unsigned long inlocal,unsigned long inremote,unsigned short local,unsigned short remote,int timeout)
{
  register int s;
  struct timeval tv;
  fd_set wfds;
  register int r;
  register int saveerrno;
  RETSIGTYPE *old_sig;
  char *retval;
 

  SIGNAL_HANDLESAVE(SIGPIPE, SIG_IGN, old_sig);
 
  s = auth_tcpsock(inlocal,inremote);
  if (s == -1)
    {
      SIGNAL_HANDLE(SIGPIPE, old_sig);
      return 0;
    }
  tv.tv_sec = timeout;
  tv.tv_usec = 0;
  FD_ZERO(&wfds);
  FD_SET(s,&wfds);
  r = select(s + 1,(fd_set *) 0,&wfds,(fd_set *) 0,&tv);
  /* XXX: how to handle EINTR? */
  if (r == -1)
    {
      SIGNAL_HANDLE(SIGPIPE, old_sig);
      CLORETS(0);
    }
  if (!FD_ISSET(s,&wfds))
    {
      close(s);
      errno = ETIMEDOUT;
      SIGNAL_HANDLE(SIGPIPE, old_sig);
      return 0;
    }
  retval = auth_sockuser(s,local,remote);
  SIGNAL_HANDLE(SIGPIPE, old_sig);

  return retval;
}

char *auth_sockuser(int s,unsigned short local,unsigned short remote)
{
  register int buflen;
  register int w;
  register int saveerrno;
  char ch;
  unsigned short rlocal;
  unsigned short rremote;
  register int fl;
  fd_set wfds;
  RETSIGTYPE *old_sig;
 
  SIGNAL_HANDLESAVE(SIGPIPE, SIG_IGN, old_sig);
 
  FD_ZERO(&wfds);
  FD_SET(s,&wfds);
  select(s + 1,(fd_set *) 0,&wfds,(fd_set *) 0,(struct timeval *) 0);
  /* now s is writable */
  if ((fl = fcntl(s,F_GETFL,0)) == -1)
    {
      SIGNAL_HANDLE(SIGPIPE, old_sig);
      CLORETS(0);
    }
  if (fcntl(s,F_SETFL,~FNDELAY & fl) == -1)
    {
      SIGNAL_HANDLE(SIGPIPE, old_sig);
      CLORETS(0);
    }
  buf = realbuf;
  sprintf(buf,"%u , %u\r\n",(unsigned int) remote,(unsigned int) local);
  /* note the reversed order---the example in RFC 931 is misleading */
  buflen = strlen(buf);
  while ((w = write(s,buf,buflen)) < buflen)
    if (w == -1)		/* should we worry about 0 as well? */
      {
	SIGNAL_HANDLE(SIGPIPE, old_sig);
	CLORETS(0);
      }
    else
      {
	buf += w;
	buflen -= w;
      }
  buf = realbuf;
  while ((w = read(s,&ch,1)) == 1)
    {
      *buf = ch;
      if ((ch != ' ') && (ch != '\t') && (ch != '\r'))
	++buf;
      if ((buf - realbuf == sizeof(realbuf) - 1) || (ch == '\n'))
	break;
    }
  SIGNAL_HANDLE(SIGPIPE, old_sig);
  if (w == -1)
    CLORETS(0);
  *buf = 0;

  if (sscanf(realbuf,"%hd,%hd: USERID :%*[^:]:%s",&rremote,&rlocal,ruser) < 3)
    {
      close(s);
      errno = EIO;
      /* makes sense, right? well, not when USERID failed to match ERROR */
      /* but there's no good error to return in that case */
      return 0;
    }
  if ((remote != rremote) || (local != rlocal))
    {
      close(s);
      errno = EIO;
      return 0;
    }
  /* we're not going to do any backslash processing */
  close(s);
  return ruser;
}
#endif /* HAVE_SOCKET */
