/* bb_read.c: read a brett.
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/
#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <syslog.h>
#include <assert.h>
#include <errno.h>

#include "config.h"
#include "libquark.h"

#include "brett_db.h"
#include "brett_int.h"
#include "filelock.h"



/*
BB_FUNCTION(Funktion,BOOLEAN,bb_read_brett,
[[(BRETTNR @var{nr},BOOLEAN @var{for_write})]],[[
    Liest die Daten des Brettes mit der Nummer @var{nr} von der Platte
	in den Cache. Falls @var{for_write} TRUE ist wird das Brett gelockt,
	der Lock kann durch @code{bb_write_brett} wieder freigegeben
	werden.

	Die Funktion sollte nur benutzt werden wenn es auf Aktualitt
	ankommt. Fr normale Anwendungen reicht der Cache.

	fr Brettnr 0 (Postfach/Programmfach) wird ein Pseudoeintrag erzeugt,
    in dem alle Strings leer und alle Zahlen 0 sind.

    Returniert TRUE bei Erfolg, FALSE falls das Brett nicht existiert
    oder gelockt ist.
]])
*/
BOOLEAN bb_read_brett(BRETTNR nr,BOOLEAN for_write)
{
	char *old_sysop;
	char *old_alias;
	char *old_user;
	char *old_lokal;
	BRETT *b;
	ULONG proc_flags;
	size_t got;

	/* Brettnummer zu gro oder Brett schon gelockt? */
	if (nr > max_brettnr
	|| (for_write && br_ind[nr].brett->flags & B_LOCKED))
		return FALSE;

	/* Existiert das Brett? */
	if (nr && br_ind[nr].brett == NULL)
		return FALSE;
	
	if (!nr)
	{
		/* Postfach */
		return TRUE;
	}


	if (!bb_lock_brett(nr,for_write ? (BB_WRITELOCK) : (BB_READLOCK),FALSE))
		return FALSE;

	/* das Brett in den Cache holen */

	b=br_ind[nr].brett;

	/* Prozessinterne Flags merken */
	proc_flags= (b->flags) & ~(B_MASK_DATABASE);

	/* Diese Texte werden nie neu gelesen, das wre zu teuer */
	/* Sie sind aber verhltnismig unwichtig */
	old_sysop=b->sysopinfo;
	old_alias=b->aliasname;
	old_user=b->userinfo;
	old_lokal=b->lokalinfo;

	if (-1==lseek(bb_fh,POS_NR(nr),SEEK_SET))
	{
		syslog(LOG_ERR,"cannot seek pos of group %ld: %m", (ULONG) nr);
		return FALSE;
	}

	got=xread(bb_fh,(void *)b,sizeof(BRETT));
	if (got == -1) {
		syslog(LOG_ERR,"Fehler beim Lesen von brett.dat: %m");
		CLOSE_IT;
		return FALSE;
	}
	if (got != sizeof(BRETT)) {
		syslog(LOG_ERR,"Fehler beim Lesen von brett.dat: Datei zu kurz");
		CLOSE_IT;
		return FALSE;
	}
	/* Flags zurcksetzen, insbesondere B_LOCKED */
	/* Flags zurcksetzen, insbesondere B_LOCKED */
	b->flags |= proc_flags;
	b->tmpflags=0;

	if (!for_write) /* Readlock wieder aufheben. */
		bb_unlock_brett(nr);

	/* Textinfos und Flags restaurieren */
	b->sysopinfo=old_sysop;
	b->aliasname=old_alias;
	b->userinfo=old_user;
	b->lokalinfo=old_lokal;

	return TRUE;

} /* bb_read_brett */


