/* bcl_sort.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact1: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
 * Brett Datenbank, Routinen zur Bereitstellung einer sortierten Liste
 */

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "userbase.h"
#include "user.h"
#include "modem.h"

extern USER user;

extern BRETTINFO *br_ind;				/* Feld mit Brettnummern, -namen, -flags */
extern long max_brettnr;			/* hchstmgliche Brettnummer */


static int compare_brettnr (const void *a, const void *b);

static BRETTNR *sort_anker;

static int
compare_brettnr (const void *a, const void *b)
{
	const BRETTNR *za = a;
	const BRETTNR *zb = b;
	const char *ca;
	const char *cb;
	int r;
	ca=br_ind[*za].canon;
	cb=br_ind[*zb].canon;

	if (!ca && !cb)
		return 0;
	if (!ca)
		return 1;
	if (!cb)
		return -1;

	r = *ca - *cb;
	if (r != 0)	
		return r;
	return strcmp (ca, cb);
}


BRETTNR *
b_sortierung(void)
{
	BRETTNR i;
	BOOLEAN found=FALSE;
	BRETTNR j;

	/* Wenn schon mal sortiert wurde, dann haben wir es einfach.
	 * Das setzt aber voraus, da b_free_sortierung beim Logout des
	 * Users, bei Anlage neuer Bretter, beim Lschen vorhandener
	 * Bretter und beim Umbenennen aufgerufen wird.
	 * Und da b_sortierung immer dann aufgerufen wird, wenn auf
	 * das Array zugegriffen wird.
	 */
	if (sort_anker)
		return sort_anker;

	sort_anker = malloc (sizeof (BRETTNR) * max_brettnr);

	if (!sort_anker)
	{
		mbfehler ("Abbruch wegen Speichermangel");
		return NULL;
	}


	for (i = 1, j=0; i < max_brettnr; i++)
	{
		if (br_ind[i].brett && br_ind[i].brett->flags & B_READ)
		{
			sort_anker[j++] = i;
			found = TRUE;
		}
	}
	/* Damit ist sichergestellt, da immer ein Feldelement mit Nummer NULL
	 * als Endesignal vorhanden ist. Das Postfach soll ja nicht in der
	 * Liste auftauchen, daher entsteht hier kein Problem.
	 */
	sort_anker[j] = 0;

	if (!found)
	{
		mboutput ("Keine Bretter vorhanden!\r\n");
		sort_anker[0]=0;
		return sort_anker;
	}

	qsort (sort_anker, j, sizeof (BRETTNR), compare_brettnr);

	return sort_anker;
}

void
b_free_sortierung(void)
{

	if (!sort_anker)
		return;

	free(sort_anker);
	sort_anker=NULL;
}
