/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/

/*
 * befehle/brett.c:
 * - bretterliste anzeigen
 * - brett wechseln
 */

#include "tomsysdep.h"

#include <portab.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include "fnmatch.h"

#include "prototyp.h"
#include "box_conf.h"
#include "brett.h"
#include "befehle.h"
#include "user.h"
#include "modem.h"
#include "terminal.h"
#include "msg.h"
#include "bin_msg.h"
#ifdef OFFLINE2
#include "offline/inc/mdb_global.h"
#else
#include "offline/mdb_global.h"
#endif

extern USER user;
extern BRETT akt_brett;
extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;
extern long max_brettnr;

extern BRETTART postfach_art;



static BOOLEAN brett_info(BRETTNR nr);
static BOOLEAN brett_info_bin(BRETT *b);

static int srt_name(const void *a,const void *b);


typedef struct
{
	int grp_nr;
	int wert;

} GRP_SORT;

typedef struct
{
	enum {
		SORT_NAME, SORT_NETZ, SORT_SPRACHE
	} art;
	char *str;
	UWORD nr; /* Sprach/Netzcode */
} GRP_SORT_CRIT;




/*
	Sortierung der Brettnamen anhand Eintrge in "tom/system/order.cnf"

	Ablauf:
	- array(Wertung,Gruppennummer) ber alle mglichen Gruppennummern anlegen
	- alle mglichen Nummern durchlaufen:
	  - ex. eine Gruppe nicht, wird 0 eingetragen
	  - ex. eine Gruppe, wird eine eindeutige Wertungsnummer aus
	    a) Index des Eintrags in "sort"
	    b) Gruppenname, lexikographisch, case insensitive
	    erzeugt.
	- array nach Wertung sortieren
	  dann ist ein array vorhanden, in dem die Gruppe mit der hchsten Prioritt 
	  ganz vorne steht und zu jedem Arrayelement ber die Gruppennummer der Name 
	  etc. erreicht werden kann.
*/
static int srt_name(const void *a,const void *b)
{
	int vgl;

	vgl = (*(const GRP_SORT *)a).wert - (*(const GRP_SORT *)b).wert;

	if (vgl > 0)
		return -1;
	if (vgl < 0)
		return 1;

	/* nur real ex. Gruppennamen drfen/knnen verglichen werden */
	if ((*(const GRP_SORT *)a).wert == 0)
		return 0;

	return stricmp(br_ind[(*(const GRP_SORT *)a).grp_nr].brett->name,
	               br_ind[(*(const GRP_SORT *)b).grp_nr].brett->name);

} /* srt_name */

static void
bef_brett_sortierung(int *sortierung, int *anz_sort, GRP_SORT **sort_ix)
{
	FILE *ord_f;

	ord_f = fopen(tprintf("%sorder.cnf",box_conf.config_dir),"r");
	if (ord_f == NULL)
	{
		*sortierung = -1;
	}
	else
	{
		char zeile[256];
		char *p;

		*anz_sort = 0;
		*sort_ix = (GRP_SORT *) calloc(max_brettnr,sizeof(GRP_SORT));
		if (*sort_ix == NULL)
			*sortierung = -1;
		else
		{
#define SORT_BLOCK 100
			ULONG sort_zahl=SORT_BLOCK;
			GRP_SORT_CRIT *sort;	/* Sortierkriterien selbst, absteigend */
			sort=XMALLOC(sort_zahl*sizeof(GRP_SORT_CRIT));
			while (fgets(zeile,sizeof(zeile),ord_f))
			{
				BOOLEAN is_sprache=FALSE;
				BOOLEAN is_netz=FALSE;
				if (*anz_sort+1>=sort_zahl)
				{
					sort_zahl+=SORT_BLOCK;
					sort=XREALLOC(sort,sort_zahl*sizeof(GRP_SORT_CRIT));
				}
				if ((p = strpbrk(zeile,"\r\n")) != NULL)
					*p = 0;

				for (p=zeile;isspace(*p);p++)
					;

				if (*p == '#' || *p == ';' || !*p)
					continue;
				is_netz=(0==strnicmp(p,"netz",4) && isspace(p[4]));
				is_sprache=(0==strnicmp(p,"sprache",7) && isspace(p[7]));
				if (is_sprache || is_netz)
				{
					int i;
					const char *str;
					if (is_netz)
						p+=5;
					else
						p+=8;
					while(isspace(*p))
						p++;
					for (i=0;;i++)
					{
						if (is_netz)
							str=bb_ursprungsnetz(i);
						else
							str=bb_sprache(i);
						if (!str)
							break; /* Ende der Liste */

						if (!stricmp(str,p))
						{
							if (is_netz)
								sort[*anz_sort].art = SORT_NETZ;
							else
								sort[*anz_sort].art = SORT_SPRACHE;
							sort[*anz_sort].str = 0;
							sort[*anz_sort].nr = i;
							(*anz_sort)++;
							break;
						}
					}
				}
				else
				{
					sort[*anz_sort].str = entumlauten1(p);
					sort[*anz_sort].art = SORT_NAME;
					sort[*anz_sort].nr  = 0;
					XNULLCHECK(sort[*anz_sort].str);
					strlwr(sort[(*anz_sort)++].str);
				}
			}
			fclose(ord_f);

			/* Kostet nicht viel und sorgt dafr, da wirklich jede
			 * Gruppe erfat wird. Platz dafr haben wir oben freigelassen.
			 */
			sort[*anz_sort].art = SORT_NAME;
			sort[*anz_sort].nr  = 0;
			sort[(*anz_sort)++].str = XSTRDUP("*");

			if (!*anz_sort)
				*sortierung = -1;
			else
			{
				int n;
				int i;
				char buf[sizeof(akt_brett.name)*3];

				/* allen Brettnummern einen Wert gem. "order.cnf" zuweisen */
				(*sort_ix)[0].wert = 0;
				(*sort_ix)[0].grp_nr = 0;
				for (i = 1; i < max_brettnr; i++)
				{

					if (!br_ind[i].brett)
						continue;

					(*sort_ix)[i].wert = 0;
					(*sort_ix)[i].grp_nr = i;

					entumlauten(br_ind[i].brett->name,buf);
#ifndef FNM_CASEFOLD
					strlwr(buf);
#endif

					for (n = 0; n < *anz_sort; n++)
					{
						if(sort[n].art==SORT_NAME)
						{
#ifdef FNM_CASEFOLD
							if (0==fnmatch(sort[n].str,buf,FNM_CASEFOLD))
#else
							if (0==fnmatch(sort[n].str,buf,0))
#endif
							{
								(*sort_ix)[i].wert = *anz_sort - n;
								break;
							}
						}
						else if (sort[n].art == SORT_NETZ)
						{
							if (br_ind[i].brett->ursprungsnetz==sort[n].nr)
							{
								(*sort_ix)[i].wert = *anz_sort - n;
								break;
							}
						}
						else /* SORT_SPRACHE */
						{
							if (br_ind[i].brett->sprache==sort[n].nr)
							{
								(*sort_ix)[i].wert = *anz_sort - n;
								break;
							}
						}
					}
				}
				qsort((*sort_ix)+1,max_brettnr-1,sizeof(GRP_SORT),srt_name);
				for (n = 0; n < *anz_sort; n++)
				{
					if (sort[n].art==SORT_NAME)
						free(sort[n].str);
				}
				free(sort);
				*sortierung = 1;
			}
		}
	}
}

BOOLEAN bef_brett_sortierung_maybe_wrong=TRUE;

/*
	27.11.92 Frank:
	Ausgabe in zwei Schritten: erst Untersuchung aller Bretter auf neue bzw. 
	passende Nachrichten und Markierung der zutreffenden Bretter durch ein 
	Flag, dann erst die eigentliche Ausgabe.
	22.04.93 Frank, Um LS-Level erweitern.
	25.10.93 Frank, um Anzeige nur lokaler Bretter erweitert
	27.12.93 Uwe, BRETT liefert nur Bretter, die man auch lesen kann.

	Achtung: diese Funktion ruft sich ein Mal rekursiv auf. (BRETT +/-)
*/
BOOLEAN bef_brett(const char *para)
{
	static UBYTE *brett_flag = NULL;
	static int sortierung = 0;

	BOOLEAN brett_anzeigeB = FALSE;
	BOOLEAN brett_infoB = FALSE;
	BOOLEAN paraB = FALSE;
	BOOLEAN lokal = FALSE;
	BOOLEAN binaer = FALSE;
	WORD decideW = 0;
	BRETTNR iW;
	WORD kW = 0;
	int ix;										/* Brettindex, abh. von Sortierung */
	size_t parlen;
	static int anz_sort;		/* Anzahl Sortierinformationen fr Brettnamenanzeige */
	static GRP_SORT *sort_ix = NULL;

	if (!brettanzahl)
	{
		mbfehler("Es sind noch keine Bretter in der Box installiert");
		return FALSE;
	}

	if (!brett_flag)
	{
		brett_flag = (UBYTE *) calloc(max_brettnr,sizeof(UBYTE));

		if (!brett_flag)
		{
			speicherproblem(FALSE,"bef_brett");
			return FALSE;
		}
	}
	else
	{
		/* Flags immer Lschen - es knnte mittlerweile temporr der
		 * Level verndert worden sein, oder ein Brett angelegt
		 * worden sein, oder der Befehl mit anderen Parametern
		 * aufgerufen worden sein.
		 * Ein Cache fr die Lese/Schreibrechtsflags lohnt sich hier
		 * berhaupt nicht, er spart hchstens 1 % Zeit.
		 * old_brett_flag wird bentigt um festzustellen ob es 
		 * notwendig ist, die Sortierung zu erneuern.
		 */
		memset(brett_flag,0,max_brettnr * sizeof(UBYTE));
	}

	if (bef_brett_sortierung_maybe_wrong)
	{
		anz_sort=0;
		sortierung=0;
		if (sort_ix)
			free(sort_ix);
		bef_brett_sortierung_maybe_wrong=FALSE;
		sort_ix=NULL;
	}

	if (sortierung==0)
		bef_brett_sortierung(&sortierung, &anz_sort, &sort_ix);

	/* nur lokale Bretter? */
	if (*para && !strnicmp(para,"-L",2))
	{
		para += 2;
		while (isspace(*para))
			para++;
		lokal = TRUE;
	}
	/* nur Binaer Bretter? */
	if (*para && !strnicmp(para,"-B",2))
	{
		para += 2;
		while (isspace(*para))
			para++;
		binaer = TRUE;
	}

	parlen = strlen(para);

	if (!*para)
	{
		if (b_check_new())
		{
			set_status_line(tprintf("%s %sretter mit neuen Eintrgen am %s um %s",
								(lokal) ? "Lokale" : "",
								(binaer) ? "Binrb" : "B",
								dd_mm_yy(),hh_mm_ss()));
			for (iW = 1; iW < max_brettnr; iW++)
			{
				if (br_ind[iW].brett
				&& (br_ind[iW].brett->flags & B_READ)
				&& (br_ind[iW].brett->flags & B_NEU)
				&& (!binaer 
					|| (binaer && (br_ind[iW].brett->art == BINAER_BRETT)))
				&& (!lokal || (lokal && (br_ind[iW].brett->flags & B_LOKAL))))
				{
					brett_flag[iW] = TRUE;
					brett_anzeigeB = TRUE;
				}
			}
		}
		else
		{
			mbprintf(" Keine neuen Eintrge in den %s%srettern vorhanden\r\n\n",
			          (lokal) ? "lokalen " : "",
			          (binaer) ? "Binrb" : "B");
		}
		paraB = TRUE;
	}
	else if (!strcmp(para,"*"))
	{
		for (iW = 1; iW < max_brettnr; iW++)
		{
			if (br_ind[iW].brett && br_ind[iW].brett->flags & B_READ)
			{
				if (lokal)
				{
					if (br_ind[iW].brett->flags & B_LOKAL)
					{
						brett_flag[iW] = TRUE;
						brett_anzeigeB = TRUE;
					}
				}
				else if (binaer)
				{
					if (br_ind[iW].brett->art == BINAER_BRETT)
					{
						brett_flag[iW] = TRUE;
						brett_anzeigeB = TRUE;
					}
				}
				else
				{
					brett_flag[iW] = TRUE;
					brett_anzeigeB = TRUE;
				}
			}
		}
		if (brett_anzeigeB)
		{
			char tmp[100];

			strcpy(tmp,"Verzeichnis aller ");
			if (lokal)
				strcat(tmp,"lokalen ");
			if (binaer)
				strcat(tmp,"Binrbretter am ");
			else
				strcat(tmp,"Bretter am ");
			strcat(tmp,dd_mm_yy());
			strcat(tmp," um ");
			strcat(tmp,hh_mm_ss());
			if (!level())
				strcat(tmp," fr Gste");
			set_status_line(tmp);
		}
		paraB = TRUE;
	}
	else if (!strcmp(para,"+") || !strcmp(para,"-"))
	{
		WORD richtung = 1;
		if (!strcmp(para,"-"))
			richtung = -1;
		if (brettanzahl>0)
		{
			int start;
			set_status_line("Neue Eintrge in:");

			if (sortierung == 1)
			{
				if (!brettnrW)
					start = 1;
				else for (start = 1; start < brettanzahl; start++)
					if (brettnrW == sort_ix[start].grp_nr)
						break;
			}
			else
				start = brettnrW;
			for (iW = start+richtung; iW < max_brettnr && iW>0; iW+=richtung)
			{
				if (sortierung == 1)
					ix = sort_ix[iW].grp_nr;
				else
					ix = iW;

				if (!br_ind[ix].brett)
					continue;

				/* ggf. Bretter mit Lese-Mitgliedschaft berspringen */
				if ((br_ind[ix].brett->flags & B_LESMITGLIED) 
					&& level() < 9 && !b_tst_access(ix,user.nummer))
					continue;

				if ((br_ind[ix].brett->flags & B_READ)
				&&  (br_ind[ix].brett->datum > user.letzter_login))
				{
					if (level())
					{
						mbprintf(" <%s> A,I,L,N,? : ",br_ind[ix].brett->name);
						do
						{
							decideW = mbdecide("AILN?","Abbruch\r\n|Inhalt\r\n\n|Lesen\r\n\n|Nchstes\r\n|\? = Hilfe\r\n");
							if (decideW==5)
								mbprintf("\r\n <%s> Abbruch,Inhalt,Lesen,Nchstes: ",br_ind[ix].brett->name);
						}
						while (decideW == 5);
					}
					else /* Gste */
					{
						mbprintf(" <%s> Abbruch,Inhalt,Lesen,Nchstes: ",br_ind[ix].brett->name);
						decideW = mbdecide("AILN","Abbruch\r\n|Inhalt\r\n\n|Lesen\r\n\n|Nchstes\r\n");
					}
				}

				switch (decideW)
				{
					case 1 :
						/* Abbruch */
						redraw_status();
						brettnrW = ix;
						akt_brett=*br_ind[ix].brett;
						break;

					case 2 :
						/* Inhalt */
						bef_brett(br_ind[ix].brett->name);
						if (brettnrW==ix)
						{
							redraw_status();
							bef_inhalt("");
						}
						kW = 1;
						break;

					case 3 :
						/* Lesen */
						bef_brett(br_ind[ix].brett->name);
						if (brettnrW==ix)
						{
							redraw_status();
							bef_lesen("");
						}
						kW = 1;
						break;

					case 5 :
						/* vorheriges */
						/* ... */
						break;
				}
				if (kW==1 || decideW==1)
					break;
			}
			if (!kW && decideW != 1)
			{
				mbprintf("\r\n Keine %s Bretter mit neuen Eintrgen mehr\r\n",
				         (richtung == 1) ? "weiteren" : "vorangehenden");
				brettnrW = 0;
				redraw_status();
			}
		}
		paraB = TRUE;
	}
	else if (parlen > 1 && strpbrk(para,"*?[]"))
	{
		BOOLEAN ist_nummer;
		size_t vgllen=0;
		int nullen=0;
		char vgl1[10];
		char vgl2[10];
		BOOLEAN fnm=FALSE;
		char *p;
		long nr;
		char para_klein[BDAT_GRP_NAME_LEN];

		entumlauten(para,para_klein);
		strlwr(para_klein);

		nr=strtol(para,&p,10);
		if (nr==0 && p!=para+parlen-1)
		{
			ist_nummer=FALSE;
			fnm=TRUE;
		}
		else
		{
			const char *q;
			ist_nummer=TRUE;
			q=para;
			while (*q=='0')
			{
				q++;
				nullen++;
			}
			sprintf(vgl1,"%lu",(ULONG) nr);
			if (isdigit(*q))
				vgllen=strlen(vgl1);
		}
		
		for (iW = 1; iW < max_brettnr; iW++)
		{
			if (!br_ind[iW].brett || !(br_ind[iW].brett->flags & B_READ))
				continue;
			if (lokal && !(br_ind[iW].brett->flags & B_LOKAL))
				continue;
			if (binaer && br_ind[iW].brett->art != BINAER_BRETT)
				continue;
			if (fnm)
			{
				char vergleich[BDAT_GRP_NAME_LEN];
				entumlauten(br_ind[iW].brett->name,vergleich);
#ifdef FNM_CASEFOLD
				if (fnmatch(para_klein,vergleich,FNM_CASEFOLD))
#else
				strlwr(vergleich);
				if (fnmatch(para_klein,vergleich,0))
#endif
					continue;
			}
			else /* ist_nummer */
			{
				/* Die Problematik 0* wird simpel abgehandelt: 0*
				 * heit "alle Bretter unter hundert", 00* alle unter 10
				 */
				if (nullen)
				{
					if (nullen==1 && iW>99)
						continue;
					if (nullen==2 && iW>9)
						continue;
				}
				sprintf(vgl2,"%lu",(ULONG) iW);
				if (vgllen && strnicmp(vgl1,vgl2,vgllen))
					continue;
			}
			brett_flag[iW] = TRUE;
			brett_anzeigeB = TRUE;
			paraB = TRUE;
		}
		if (brett_anzeigeB)
		{
			set_status_line(tprintf("Brettinhaltsverzeichnis am %s um %s  fr Userlevel %d",
											dd_mm_yy(),hh_mm_ss(),level()));
		}
	}
	else if (*para && *(para+parlen-1) == '#')
	{
		if (!strcmp(para,"#") && (brettnrW || postfach_art==BINAER_BRETT))
		{
			iW = brettnrW;
			brett_infoB = TRUE;
		}
		else if (!stricmp(para,"PP#"))
		{
			iW = 0;
			brett_infoB = TRUE;
		}
		else
		{
			for (iW = 1; iW < max_brettnr; iW++)
			{
				if (!br_ind[iW].brett)
					continue;

				if (((br_ind[iW].brett->flags & B_READ) && atoi(para) == iW)
				|| !strnicmp(para,br_ind[iW].brett->name,parlen-1))
				{
					brett_infoB = TRUE;
					break;
				}
			}
		}
		if (brett_infoB)
			return brett_info(iW);
		else
			mbfehler("Brett nicht vorhanden");
		paraB = TRUE;
	}
	else
	{
		iW = find_brett(para);
		if (iW && iW != brettnrW && level() <= 6)
			set_status_line("Brettwechsel");
		if (iW > 0 && (br_ind[iW].brett->flags & B_READ))
		{
			if ((br_ind[iW].brett->flags & B_ZAHLER) && !user.bezahlt)
			{
				mboutput(" Zugriff nur fr zahlende User\r\n\n");
				iW = brettnrW;
				goto abbruch;
			}

			if ((br_ind[iW].brett->flags & B_LESMITGLIED) && level() < 9 
				&& !b_tst_access(iW,user.nummer))
			{
				mboutput(" Lesezugriff nur fr Mitglieder\r\n");
				mbprintf(" Bitte an 'Chef %s' schreiben.\r\n\n",
						 br_ind[iW].brett->name);
				iW = brettnrW;
				goto abbruch;
			}

			if (br_ind[iW].brett->flags & B_ACL)
			{
				char fname[PATH_MAX];
				const char *err=NULL;
				sprintf(fname,"%s%s",brettpfad(iW),"acl.lst");
				if (!acl_check(fname,user.name,level(), 'R', &err))
				{
					if (!err)
						err="Sie sind nicht die in Zugriffskontrolliste eingetragen.\r\n";
					mbprintf(" Lesezugriff verweigert.\r\n"
					    " %s.\r\n"
						" Bitte an 'Chef %s' schreiben.\r\n\n",
						err, br_ind[iW].brett->name);
					iW = brettnrW;
					goto abbruch;
				}
			}

			if (*br_ind[iW].brett->passwort 
				&& (br_ind[iW].brett->flags & B_PASSWORT))
			/* && !(br_ind[iW].brett->brett->flags & (B_GEHEIM | B_ANONYM | B_ZAHLER | B_PASSWORT))) weshalb? */
			{
				if (level()>=8)
				{
					mboutput(" Pawortabfrage bersprungen\r\n\n");
				}
				else
				{
					char pass[12]="";

					if (!mbfrage(10,MBF_TOUPPER|MBF_STAR|MBF_TRIM," Pawort > ",pass))
						goto abbruch;
					if (strcmp(pass,br_ind[iW].brett->passwort))
					{
						mboutput(" <== falsches Pawort\r\n\n");
						goto abbruch;
					}
					else
						mboutput("\r\n\n");
				}
			}

			brettnrW = iW;
			akt_brett = *br_ind[iW].brett;
			redraw_status();

			if (level()<6)
			{
				/* einzeilige Userinfo */
				mboutput("\r\n\n\n");
				if (akt_brett.userinfo)
					mbprintf("%s\r\n",akt_brett.userinfo);
				/* mehrzeilige lokale Info */
				if (akt_brett.lokalinfo)
					mboutput(akt_brett.lokalinfo);
			}
abbruch:
			paraB = TRUE;
		}
	}

	if (brett_anzeigeB)
	{
		int old_mode;
		mbprintf("%sArt%s   ",vt_S(ATTR_FETT),vt_S(ATTR_OFF));
		if (sortierung != 1)
			mbprintf("%sNr%s   ",vt_S(ATTR_FETT),vt_S(ATTR_OFF));
		mbprintf("%sName%s%-52.52s",vt_S(ATTR_FETT),vt_S(ATTR_OFF),"");
		mbprintf("%sDatum%s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF));
		mbprintf("----------------------------------------------------------------------------\r\n");
		old_mode=set_scroll_area(2);

		for (iW = 1; iW < max_brettnr; iW++)
		{
			if (sortierung == 1)
			{
				if (!sort_ix[iW].wert)
					continue;
				ix = sort_ix[iW].grp_nr;
				if (!brett_flag[ix])
					continue;
			}
			else
			{
				if (!brett_flag[iW])
					continue;
				ix = iW;
			}

			if (br_ind[ix].brett->art == BINAER_BRETT)	/* Lokal, Bin */
				mboutput("Binr ");
			else
			if (br_ind[ix].brett->flags & B_LOKAL)		/* Lokal, Text */
				mboutput("Lokal ");
			else
				mboutput("Netz  ");					/* Netz */
#if 1
			/* einspaltig */
			if (sortierung == 1)
			{
				if (1>mbprintf("%-54.54s %-8.8s\r\n",
					br_ind[ix].brett->name, 
					zeige_datum_S(br_ind[ix].brett->datum)))
					break;
			}
			else if (1>mbprintf("%4d-%-54.54s %-8.8s\r\n",
					ix, br_ind[ix].brett->name, 
					zeige_datum_S(br_ind[ix].brett->datum)))
				break;
#else
			/* zweispaltig */
			mbprintf("%4d-%-16.16s %-8.8s",ix,br_ind[ix].brett->name,zeige_datum_S(br_ind[ix].ref));
			kW++;
			if ((kW % 2==0) || kW==brettanzahl)
			{
				if (mboutput("\r\n") < 0)
					break;
			}
			else
				mboutput("  ");
#endif
			if (userbreak())  
				break;
		}
		paraB = TRUE;
		mboutput("\r\n");
		set_scroll_area(old_mode);
	}

	if (!paraB)  
	{
		parameter_falsch("BRETT",para);
		return FALSE;
	}
	return TRUE;

} /* bef_brett */



static BOOLEAN brett_info(BRETTNR nr)
{
	BRETT *b;
	b=br_ind[nr].brett;

	set_status_line(tprintf("Informationen zum Brett <%s>",b->name));

	mbprintf(" %sBrettname     :%s %s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),b->name);
	mbprintf(" %sBrettnummer   :%s %ld\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),(long)b->nr);
	mbprintf(" %sBearbeiter    :%s %s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),b_verwalter_S(nr));
	mbprintf(" %sLeselevel     :%s %d\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),b->lesen_level);
	mbprintf(" %sSchreiblevel  :%s %d\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),b->schreib_level);
	mbprintf(" %sStatus        :%s ",vt_S(ATTR_FETT),vt_S(ATTR_OFF));
	if (b->flags & B_ANONYM)
		mboutput("anonym ");
	if (b->flags & B_GEHEIM)
		mboutput("geheim ");
	if (b->flags & B_MITGLIED)
		mboutput("Mitgliedschaft ");
	if (b->flags & B_ZAHLER)
		mboutput("Zahler ");
	if (b->flags & B_PASSWORT)
		mboutput("Pawort ");
	if (b->flags & B_ZWANG)
		mboutput("Zwang ");
	if (b->flags & B_DEFAULT)
		mboutput("Default ");
	if (b->flags & B_MODERIERT)
		mboutput("Moderiert ");
	if (b->flags & B_ACL)
		mboutput("ACL ");
	if (b->flags & B_LESMITGLIED)
		mboutput("Lesen ");
	if (b->flags & B_PAYWRITE)
		mboutput("Paywrite ");
	mboutput("\r\n");
	mbprintf(" %sHerkunft      :%s %s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),
		bb_ursprungsnetz(br_ind[nr].brett->ursprungsnetz));
	mbprintf(" %sSprache       :%s %s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),
		bb_sprache(br_ind[nr].brett->sprache));

	mbprintf(" %sL. Eintrag    :%s %s\r\n",vt_S(ATTR_FETT),vt_S(ATTR_OFF),
		zeige_datum_S(br_ind[nr].brett->datum));
	if (b->userinfo)
		mbprintf(" %s",b->userinfo);

	if (b->art == TEXT_BRETT) {
		return gruppeninfo(b->name,(nr == 0));
	} else
		return brett_info_bin(b);

} /* brett_info */



static BOOLEAN brett_info_bin(BRETT *b)
{
	long files = 0L;
	long bytes = 0L;
	int geloescht = 0;
	BIN_MSG bin_info;
	FILE *bin_f;
	long gesamt = 0;

	if (level() < 6 && b->grp_chef != user.nummer)
	{
		mbfehler("Zugriff nicht erlaubt");
		return FALSE;
	}

	if ((bin_f = binaer_dat_oeffnen(b->nr,FALSE)) != NULL)
	{
		while (fread(&bin_info,sizeof(BIN_MSG),1,bin_f))
		{
			if (!bin_info.nr)
				geloescht++;
			else
				bytes += bin_info.bytes;
			gesamt++;
		}
		binaer_dat_schliessen(bin_f);
	}
#ifdef HAVE_DOSDRIVES
	mbprintf(" %sFrei auf Part %c:%s %ld Bytes\r\n",
			vt_S(ATTR_FETT),'A'+b->partition,vt_S(ATTR_OFF),
			bytes_free('A'+b->partition-64));
#endif
	mbprintf(" %sBrettgre    :%s %ld kB\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),bytes / 1000);
	mbprintf(" %sGesamtanzahl  :%s %ld\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),gesamt);
	mbprintf(" %sdavon gelscht:%s %d\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),geloescht);
	files = anzahl_dateien(brettpfad(b->nr),"*.txt");
	mbprintf(" %sAnzahl Infos  :%s %ld\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),files);
	files = anzahl_dateien(brettpfad(b->nr),"*.bin");
	if (files != gesamt)
	mbprintf(" %sAnzahl Uploads:%s %ld\r\n",
			vt_S(ATTR_FETT),vt_S(ATTR_OFF),files);

	return TRUE;

} /* brett_info_bin */
