/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "prototyp.h"
#include "befehle.h"
#include "brett.h"
#include "user.h"
#include "modem.h"
#include "terminal.h"
#include "bin_msg.h"
#include "bin_out.h"
#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/message.h"
#include "offline/inc/mdb_global.h"
#else
#include "offline/version.h"
#include "offline/message.h"
#include "offline/mdb_global.h"
#endif
#include "routing.h"

extern USER user;
extern BRETT akt_brett;
extern BRETTNR brettnrW;
extern BRETTINFO *br_ind;

extern BYTE
pm_schutzB;

extern WORD
count_crW,
vtmodeW;

extern BOOLEAN mehr_disabled;



static BOOLEAN lesen_text(const char *para);
BOOLEAN lesen_binaer(const char *para);
/* damit offline den Messagestatus setzen kann */
void bef_lesen_status_callback(const char *id, char status);



/*
	Nachrichten lesen

	14.12.92 Frank
*/
BOOLEAN bef_lesen(const char *para)
{
	if (!(br_ind[brettnrW].brett->flags & B_READ))
	{
		mbfehler("Lesen hier nicht erlaubt");
		return FALSE;
	}
	if (user.typ!=UTYP_USER && brettnrW==0)
	{
		mbfehler("Das Lesen von Nachrichten ist Benutzern erlaubt");
		return FALSE;
	}

	if (brettart(brettnrW) == BINAER_BRETT)
		return lesen_binaer(para);
	else
		return lesen_text(para);

} /* bef_lesen */



static BOOLEAN lesen_text(const char *para)
{
	const char *name;			/* Gruppen- bzw. Username */
	long max;					/* Anzahl Msgs. in der Gruppe */
	BOOLEAN pm;
	BOOLEAN old_mehr;
	long start;
	int gh;

	set_status_line("Nachrichten lesen");

	pm = (brettnrW == 0);
	name = (pm) ? user.name : akt_brett.name;

	start = atol(para);
	if (start <= 0 && !strcmp(para,"*"))
		start=1; /* quark 1 Kompatibilitt */

	if (*para && start <= 0)
	{
		mbfehler("Ungltiger Parameter");
		return FALSE;
	}
	else if (!*para)
	{
#ifdef OFFLINE_VERSION
		if (!pm && !(br_ind[brettnrW].brett->flags & B_NEU))
#else
		if ((!pm && !(br_ind[brettnrW].brett->flags & B_NEU))
		|| !group_info_msgs(name,pm,&max,NULL) || !max)
#endif
		{
			mboutput(" Keine ");
			if (!*para)
				mboutput("neuen ");
			mboutput("Eintrge ");
			if (pm)
				mboutput("im Postfach");
			else
				mboutput("in diesem Brett");
			mboutput(" vorhanden\r\n");
			mboutput("\r\n");

			return TRUE;
		}
	}

	if (pm)
		pm_schutzB = TRUE;

#ifdef OFFLINE_VERSION
	if ((gh = group_open(name,pm)) < 0)
#else
	if ((gh = group_open(getpid(),name,pm,pm)) < 0)
#endif
	{
		mbfehler("Keine Eintrge vorhanden");
		return FALSE;
	}
	group_close(gh,getpid());

	if (vtmodeW)
	{
		mbprintf("%s%s",vt_pos_S(2,1),vt_S(DEL_SCR_REST));
		count_crW = 0;
	}

	/* das Paging bernimmt der Browser selbst */
	old_mehr = user.mehr;
	user.mehr = FALSE;
	if (!old_mehr)
		mehr_disabled=TRUE;
	msg_browser(name,pm,FALSE, (start) ? start-1 : -2,user.letzter_login);
	mehr_disabled=FALSE;
	user.mehr = old_mehr;

	akt_anruf.hat_neue_pms=FALSE;

	pm_schutzB = FALSE;
	redraw_status();

	return TRUE;

} /* lesen_text */



BOOLEAN lesen_binaer(const char *para)
{
	return bef_download(para);
}

void 
bef_lesen_status_callback(const char *id, char status)
{
	char buf[2];
	buf[0]=status;
	buf[1]=0;
	(void) r_st_set_status(id, user.name,NULL,buf,time(NULL));
}
