#! /usr/local/bin/tclsh 

#
# Das ist die einzige Variable, die anzupassen ist. Hinweis: In dem 
# Verzeichnis mu ein Symlink namens "db" auf offline's log-Verzeichnis
# sein.
#

set logdir [tom_getvar "%pl"]

#############################################################################
#
# Installation:
# 1. Das Script nach tom/bin/tcl kopieren.
# 2. Das Script in die befehle.cnf eintragen:
#    S|VIEWlog     /    |TCL /tom/bin/tcl/viewlog.tcl |--------89-|-|?|B|-|
#    Hinweis: ab Level 9 wird "less" aufgerufen und von dem aus kommt man
#    in die Shell!
# 3. will man auch gepackten Protokolle anzeigen knnen, ist die 
#    Environmentvariablen LESSOPEN zu setzen. Siehe Manpage zum less, oder
#    in Kurzform: export LESSOPEN="|lessopen.sh %s",
#    und ein Script lessopen.sh irgendwo im $PATH anlegen, mit z.B. diesem
#    Inhalt:
#		#!/bin/sh
#
#		echo $1
#		case "$1" in
#		*.Z|*.z|*.gz)
#			zcat -c $1  2>/dev/null
#			;;
#		*)
#			cat $1
#			;;
#		esac
# 4. Das Programm kann auch von auerhalb der Quark aufgerufen werden.
# 
#############################################################################

set freestanding [catch tom_time]

set lf_count 0

proc less { file } {
	global freestanding
	set use_less 0
	if {$freestanding} {
		set use_less 1
	} elseif {[tom_getvar %ul] == 9} {
		set use_less 1
	}
	if {$use_less} {
		exec sh -c "less $file </dev/tty 2>/dev/tty >/dev/tty"
		return
	}

	if [string match "*.gz" $file] {
		set ok [catch {set f [open "|zcat $file" r]}]
	} else {
		set ok [catch {set f [open $file r]}]
	}
	if {$ok != 0} {
		error "can't read file $file"
	}	
	while {[gets $f line] >= 0} {
		output $line
	}
	close $f
}

proc output { what } {
	global freestanding
	if {$freestanding} {
		puts stdout "$what"
	} else {
		tom_puts "$what"
	}
}

proc normdate { offset } {
	global now
	global freestanding
	if {$freestanding} {	
		set cmd "|date -d \"$offset days ago\" +%y_%m_%d"
		if {0 == [catch {set f [open $cmd r]} errt ]} {
			gets $f this
			close $f
			return $this
		} else {
			error "can't get date: $errt"
		}
	} else {
		set dat [expr $now - ( $offset * 86400)]
		set mt [tom_time_t_to_mtzeit $dat]
		# mt ist z.B. 199410151800
		#             0   4 6 8 0
		return [string range $mt 2 3]_[string range $mt 4 5]_[string range $mt 6 7]
	}
}
proc offdate { offset } {
	global now
	global freestanding
	set dat {expr $now - $offset}
	if {$freestanding} {	
		set cmd "|date -d \"$offset days ago\" +%y%m%d"
		if {0 == [catch {set f [open $cmd r]} errt ]} {
			gets $f this
			close $f
			return $this
		} else {
			error "can't get date: $errt"
		}
	} else {
		set mt [tom_time_t_to_mtzeit $dat]
		# mt ist z.B. 199410151800
		#             0   4 6 8 0
		return [string range $mt 2 3][string range $mt 4 5][string range $mt 6 7]
	}
}

proc init { } {
	global now
	global freestanding
	if {$freestanding} {
		if {0 == [catch {set f [open "|date -d 00:00:00 +%s" r]}]} {
			gets $f now
			close $f
		} else {
			error "can't get date"
		}
	} else {
		set now [tom_time]
		set daystart [tom_strftime "%Y%m%d000000" $now]
		tom_puts $daystart
		set now [tom_mtzeit_to_time_t $daystart]
		tom_puts $now
	}
}

proc find_files { offset } {
	global lf_count
	global lf_name
	global lf_fname
	global lf_key
	global lf_size
	global logdir
	set d [normdate $offset]
	set lf_count 0

	# L  Loopscript     .loop
	# M  Messageprot.   .msg
	# P  Tagesprotokoll .log
	# Q  Quarkoutfile   .qo
	# U  Userprot.      .inf
	# A  PAniklogfile   .panic
	# I  Importlog      db/tr*
	# D  DB-Putzlog     db/pt*

	if {[catch {set f [glob "$logdir/$d*"]}] == 0} {
		foreach i $f {
			incr lf_count
			set lf_fname($lf_count) "$i"
			set lf_size($lf_count) [file size $i]
			set ind [string last "/" $lf_fname($lf_count)]
			set s [string range $lf_fname($lf_count) [expr $ind + 1] end]
			if [string match "$d.log*" $s] {
				set lf_name($lf_count) "Tagesprotokoll"
				set lf_key($lf_count) "P"
			} elseif [string match "$d.inf*" $s] {
				set lf_name($lf_count) "Userprotokoll"
				set lf_key($lf_count) "U"
			} elseif [string match "$d.msg*" $s] {
				set lf_name($lf_count) "Messageprotokoll"
				set lf_key($lf_count) "M"
			} elseif [string match "$d.panic*" $s] {
				set lf_name($lf_count) "Paniklogfile"
				set lf_key($lf_count) "A"
			} elseif [string match "$d.qo*" $s] {
				if {$lf_size($lf_count) == 0} {
					incr lf_count -1
				} else {
						set lf_name($lf_count) "Quarkoutfile"
						set lf_key($lf_count) "Q"
				}
			} elseif [string match "$d.loop*" $s] {
				set lf_name($lf_count) "Loopscript"
				set lf_key($lf_count) "L"
			} else {
				set lf_name($lf_count) "???"
				set lf_key($lf_count) "$lf_count"
			}
		}
	}
	set d [offdate $offset]
	if {[catch {set f [glob "$logdir/db/*$d*"]}] == 0} {
		foreach i $f {
			incr lf_count
			set lf_fname($lf_count) "$i"
			set lf_size($lf_count) [file size $i]
			set ind [string last "/" $lf_fname($lf_count)]
			set s [string range $lf_fname($lf_count) [expr $ind + 1] end]
			if [string match "tr$d.log*" $s] {
				set lf_name($lf_count) "Datenbankimportlog"
				set lf_key($lf_count) "I"
			} elseif [string match "pt$d.inf*" $s] {
				set lf_name($lf_count) "Datenbankputzprotokoll"
				set lf_key($lf_count) "D"
			} else {
				set lf_name($lf_count) "???"
				set lf_key($lf_count) "$lf_count"
			}
		}
	}
}


proc menu_day { offset } {
	global lf_count
	global lf_name
	global lf_fname
	global lf_key
	global lf_size
	global freestanding
	find_files $offset
	if {$lf_count == 0} {
		output "kein Logfile vom [normdate $offset] gefunden"
		return
	}
	while {1} {
		set d [normdate $offset] 
		output ""
		output "Protokolle vom [format "%s.%s.%s" \
			[string range $d 6 7] \
			[string range $d 3 4] \
			[string range $d 0 1]]"
		output ""
		for {set i 1 } { $i <= $lf_count } { incr i } {
			if {$lf_size($i) < 4096} {
				set size [format "(%d Bytes)" $lf_size($i)]
			} else {
				set size [format "(%d KB)" [expr $lf_size($i) / 1024]]
			}
			set ind [string last "/" $lf_fname($i)]
			set s [string range $lf_fname($i) [expr $ind + 1] end]
			output [format "%3s  %-20.20s  %s %s" $lf_key($i) $s $lf_name($i) $size]
		}
		output [format "%3s  %-20.20s  %s" "#"   "" "Datum wechseln"]
		if {$offset != 0} {
			output [format "%3s  %-20.20s  %s" "0"   "" "heutige Logs"]
		}
		if {$offset != -1} {
			output [format "%3s  %-20.20s  %s" "-"   "" "gestrige Logs"]
		}
		output [format "%3s  %-20.20s  %s" "."   "" "Zurck"]
		if {$freestanding} {
			puts stdout "Ihre Wahl:"
			set wahl [string toupper [gets stdin]]
		} else {
			tom_puts -nonewline "Ihre Wahl: "
			set wahl [string toupper [tom_getchar]]
		}
		if {[string compare $wahl "#"] == 0} {
			output "Datum setzen"
			set s [get_date ""]
			return $s
		} elseif {[string match "#*" $wahl]} {
			# neat little hack for freestanding mode.
			output "Datum setzen auf [string range $wahl 1 end]"
			set day [get_date [string range $wahl 1 end]]
			if {$day != -99} {
				return $day
			}
		} elseif {[string compare $wahl "-"] == 0} {
			output "Datum setzen auf gestern"
			return 1
		} elseif {[string compare $wahl "0"] == 0} {
			output "Datum setzen heute"
			return 0
		} elseif {[string compare $wahl "."] == 0} {
			return -99
		} else {
			output ""
			for {set i 1 } { $i <= $lf_count } { incr i } {
				if {[string compare $wahl $lf_key($i)]==0} {
					less $lf_fname($i)
				}
			}
		}
	}
}

proc get_date { vorschlag } {
	global now
	global freestanding
	while {1} {
		if {$vorschlag == ""} {
output "Kurzhilfe: negative Zahl: vor x Tagen"
output "           Postive Zahl (kein + davor!): Tag in diesem Monat"
output "           alles andere: Datum im von GNU date verstandenen Format"
output "             Beispiele: Sep 11, Sep 11 0 1994"
			output "Welches Datum:"

			if {$freestanding} {
				set wahl [string toupper [gets stdin]]
			} else {
				set wahl [string toupper [tom_gets]]
			}
		} else {
			set wahl $vorschlag
		}
		if [string match "-\[0-9\]*" $wahl] {
			return [expr -$wahl]
		} elseif [string match "\[0-9\]*" $wahl] {
			set cmd "|date +%b"
			if {0 == [catch {set f [open $cmd r]} errt ]} {
				gets $f this
				close $f
			} else {
				error "can't get date: $errt"
			}
			set cmd "|date -d \"$this $wahl 00:00:00\" +%s"
			if {0 == [catch {set f [open $cmd r]} errt ]} {
				gets $f this
				close $f
			} else {
				error "can't get date: $errt"
			}
			set date [expr ($this - $now) / 86400]
			if {$date <= 0} {
				return [expr -$date]
			} 
			puts "illegales Datum (in der Zukunft)"
		} else {
			if {0 == [catch {set f [open "|date -d {$wahl} +%s" r]} errt]} {
				gets $f this
				close $f
				set date [expr ($this - $now) / 86400]
				if {$date <= 0} {
					return [expr -$date]
				} 
				puts "illegales Datum (in der Zukunft)"
			} else {
				puts "illegales Datum ($errt)"
			}
		} else {
			puts "illegales Datum"
			if {$vorschlag == ""} {
				return -99
			}
		}
	}
}

init
if [info exist argc] {
	if {$argc > 1} {
		error "too many arguments"
	} elseif {$argc==1} {
		set offset [get_date [lindex $argv 0]]
	} else {
		set offset 0
	}
} else {
	set offset 0
}
while {1} {
	set offset [menu_day $offset]
	if {$offset == -99} break
}

