/*
 * quarkoutfile - cripple down quark outfiles to maus infiles
 * Copyright (C) 1993,1994,1995,1996  Uwe Ohse
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * For questions contact:
 * e-mail: (prefered)
 * uwe@tirka.gun.de
 * Uwe_Ohse@mausdu3.gun.de
 * Uwe Ohse @ DU3 (inside MausNet)
 */

#include "tomsysdep.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <ctype.h>

#include "portab.h"
#include "quarkoutfile.h"
#include "libquark.h"
#include "dynhash.h"
#include "crc32.h"


typedef struct bouncedata_t {
	const char *mausid;
	const char *realid;
	const char *gruppe;
	const char *empfaenger;
	const char *absender;
} bouncedata_t;

static FILE *bfile;

void 
create_bouncefile (const char *bouncefile)
{
	bfile = fopen (bouncefile, "wb");
	if (!bfile) {
		fprintf (stderr, "%s: cannot create %s: %s\n",
				 program_name, bouncefile, strerror (errno));
		exit (1);
	}
}

void 
add_bounceinfo (const char *mausid, const char *realid,
		const char *gruppe, const char *empfaenger, const char *absender)
{
	if (!bfile)
		return;
	fprintf (bfile, "#%s\nI%s\n", mausid, realid);
	if (gruppe && *gruppe)
		fprintf (bfile, "G%s\n", gruppe);
	if (empfaenger && *empfaenger)
		fprintf (bfile, "A%s\n", empfaenger);
	if (absender && *absender)
		fprintf (bfile, "V%s\n", absender);
}

void 
close_bouncefile (void)
{
	if (!bfile)
		return;
	if (0 != fclose (bfile)) {
		fprintf (stderr, "%s: cannot write to bouncefile: %s", program_name, strerror (errno));
		exit (1);
	}
	bfile = NULL;
}

static void *hashhandle = NULL;

void 
load_bouncefiles (char *bouncefile)
{
	char *next;
	hashhandle = hhcreate (1000);
	if (!hashhandle) {
		fprintf (stderr, "%s: out of memory\n", program_name);
		exit (1);
	}
	for (; bouncefile && *bouncefile; bouncefile = next) {
		FILE *f;
		char *p;
		char buf[1024];
		bouncedata_t *b;
		DYN_ENTRY ele;
		p = strchr (bouncefile, ':');
		if (p) {
			*p++ = 0;
			next = p;
		} else
			next = NULL;
		f = fopen (bouncefile, "rb");
		if (!f) {
			fprintf (stderr, "%s: cannot open %s: %s",
					 program_name, bouncefile, strerror (errno));
			continue;
		}
		b = XCALLOC (1, sizeof (bouncedata_t));
		while (fgets (buf, sizeof (buf), f)) {
			p = buf + strlen (buf) - 1;
			if (*p == '\n')
				*p = 0;
			switch (*buf) {
			case '#':
				if (b->mausid && b->realid) {
					ele.crc = crc32ansi (b->mausid, strlen (b->mausid));
					ele.u.ptr = b;
					hhsearch (hashhandle, &ele, DYN_ENTER);
				}
				b = XCALLOC (1, sizeof (bouncedata_t));
				b->mausid = XSTRDUP (buf + 1);
				break;
			case 'I':
				b->realid = XSTRDUP (buf + 1);
				break;
			case 'A':
				b->empfaenger = XSTRDUP (buf + 1);
				break;
			case 'V':
				b->absender = XSTRDUP (buf + 1);
				break;
			case 'G':
				b->gruppe = XSTRDUP (buf + 1);
				break;
			}					/* switch */
		}						/* while fgets */
		if (b->mausid && b->realid && b->absender
			&& (b->gruppe || b->empfaenger)) {
			ele.crc = crc32ansi (b->mausid, strlen (b->mausid));
			ele.u.ptr = b;
			hhsearch (hashhandle, &ele, DYN_ENTER);
		}
	}							/* for durch alle Files */
}

static FILE *any_tmpf;
static FILE *sysop_tmpf;
static BOOLEAN sysop_tmpf_used = FALSE;
static long statusprobleme = 0;
static long newsprobleme = 0;
static long unbekannteprobleme = 0;
static long mailprobleme = 0;
static int lfd_nr = 0;

void 
merke_bounce (const char *mausid, const char *fehler)
{
	DYN_ENTRY such;
	DYN_ENTRY *e;
	bouncedata_t *b;
	time_t now;
	char now_mt[128];
	now = time (NULL);
	time_t_to_mtzeit (&now, now_mt);
	such.crc = crc32ansi (mausid, strlen (mausid));
	such.u.ptr = NULL;
	if (!any_tmpf) {
		any_tmpf = tmpfile ();
		sysop_tmpf = tmpfile ();
	}
	if (!any_tmpf || !sysop_tmpf) {
		fprintf (stderr, "%s: cannot create temporary file: %s\n",
				 program_name, strerror (errno));
		exit (1);
	}
	e = hhsearch (hashhandle, &such, DYN_FIND);
	if (!e) {
		/* Hh? */
		fprintf (sysop_tmpf,
				 ":Zu bouncende Nachricht nicht auffindbar:\n"
				 ":  ID:     %s\n"
				 ":  Fehler: %s\n"
				 ":\n",
				 mausid, fehler);
		sysop_tmpf_used = TRUE;
		unbekannteprobleme++;
		return;
	}
	b = e->u.ptr;
	if (!b->absender && !b->gruppe && !b->empfaenger) {
		fprintf (sysop_tmpf,
				 ":Status nicht zustellbar:\n"
				 ":  ID:     %s\n"
				 ":  Fehler: %s\n"
				 ":\n",
				 mausid, fehler);
		sysop_tmpf_used = TRUE;
		statusprobleme++;
		return;
	}
	if (b->gruppe) {
		/* also ein ffentliche Nachricht. Damit belasten wir nur den 
		 * Sysop. 
		 */
		fprintf (sysop_tmpf,
				 ":ffentliche Nachricht nicht angenommen:\n"
				 ":  ID:     %s\n"
				 ":  Gruppe: %s\n"
				 ":  Fehler: %s\n"
				 ":\n",
				 mausid, b->gruppe, fehler);
		sysop_tmpf_used = TRUE;
		newsprobleme++;
		return;
	}
	/* Eine PM */
	fprintf (any_tmpf, "#quarkoutfile.%s.%d@%s\n",
			 now_mt, ++lfd_nr, mysite);
	fprintf (any_tmpf, "A%s\n", b->absender);
	fprintf (any_tmpf, "V%s\n", mailer_daemon);
	fprintf (any_tmpf, "WFehler: %s\n", fehler);
	fprintf (any_tmpf, "E%s\n", now_mt);
	fprintf (any_tmpf, "BN\n");
	fprintf (any_tmpf,
			 ":Ihre Nachricht / your message:\n"
			 ":  %s\n"
		   ":konnte nicht zugestellt werden an / was not delivered to:\n"
			 ":  %s\n"
			 ":Grund / Reason:\n"
			 ":  %s\n",
			 b->realid, b->empfaenger, fehler);
	fprintf (sysop_tmpf,
			 ":Mail nicht zugestellt:\n"
			 ":  ID:     %s\n"
			 ":  MausID: %s\n"
			 ":  Abs.:   %s\n"
			 ":  Empf.:  %s\n"
			 ":  Fehler: %s\n"
			 ":\n",
			 b->realid, b->mausid, b->absender, b->empfaenger, fehler);
	mailprobleme++;
	sysop_tmpf_used = TRUE;
	return;
}

void 
generate_bounces (FILE * out, char *flameadressen)
{
	char buf[1024];
	time_t now;
	char now_mt[128];
	now = time (NULL);
	time_t_to_mtzeit (&now, now_mt);
	if (sysop_tmpf_used) {
		/* Wenn Flamen gewnscht ist und diese Kiste selbst einigermaen
		 * richtig konfiguriert ist ... 
		 */
		if (flameadressen && mailprobleme
			&& 0 != strcasecmp (mysite, "quark")
			&& strchr (mysite, '.')) {
			char *p;
			fprintf (out, "#quarkoutfile.%s.%d@%s\n",
					 now_mt, ++lfd_nr, mysite);
			fprintf (out, "V%s\n", mailer_daemon);
			fprintf (out, "WBounces\n");
			fprintf (out, "E%s\n", now_mt);
			fprintf (out, "BN\n");
			while (flameadressen && *flameadressen) {
				p = strchr (flameadressen, ':');
				if (p)
					*p++ = 0;
				fprintf (out, "A%s\n", flameadressen);
				flameadressen = p;
			}
			fprintf (out,
			  ":Eben hat die Quark wieder %ld mal den Job der MAUS getan "
					 "und Nachrichten gebounced.\n", mailprobleme);
		}
		fprintf (out, "#quarkoutfile.%s.%d@%s\n",
				 now_mt, ++lfd_nr, mysite);
		fprintf (out, "A%s\n", sysop_adresse);
		fprintf (out, "V%s\n", mailer_daemon);
		fprintf (out, "WQuarkoutfilemeldungen\n");
		fprintf (out, "E%s\n", now_mt);
		fprintf (out, "BN\n");
		fprintf (out,
				 ":\n"
				 ":Zusammenfassung:\n"
				 ":   Mails abgelehnt:  %ld\n"
				 ":   News abgelehnt:   %ld\n"
				 ":   Status abgelehnt: %ld\n"
				 ":   nicht zuzuordnen: %ld\n"
				 ":\n",
		 mailprobleme, newsprobleme, statusprobleme, unbekannteprobleme);
		fseek (sysop_tmpf, 0, SEEK_SET);
		while (fgets (buf, sizeof (buf), sysop_tmpf))
			fputs (buf, out);
		fclose (sysop_tmpf);
	}
	fseek (any_tmpf, 0, SEEK_SET);
	while (fgets (buf, sizeof (buf), any_tmpf))
		fputs (buf, out);
	fclose (any_tmpf);
}
