/* boxio.c : allgemeine, high-level Box-IO */
/*
    Copyright (C) 1994 1995 1996 1997 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <syslog.h>
#include <setjmp.h>

#include "tom_termcap.h"

#include "io.h"
#include "modem.h"
#include "prototyp.h"
#include "terminal.h"
#include "user.h"					/* nur fr die User-Umlaute */
#include "brett.h"
#include "box_conf.h"
#include "cslib.h"
#include "aktdata.h"
#include "libquark.h"
#include "log.h"
#include "tompaths.h"

#ifdef HAVE_TERMIOS_H
#  include <termios.h>
#elif defined(HAVE_TERMIO_H)
#  include <termio.h>
#elif defined(HAVE_SYS_TERMIOS_H)
#  include <sys/termios.h>
#elif defined(HAVE_SYS_TERMIO_H)
#  include <sys/termio.h>
#endif

extern USER user;

extern int col_ix[];
extern volatile int got_sigusr2;
extern volatile int got_sigterm;

extern int user_in_fd;
extern int user_out_fd;
extern int sysop_in_fd;
extern int sysop_out_fd;

int keine_ausgaben;

extern BYTE
batchB,
loginB,
pm_schutzB,
tausch_auto_loginB;

extern WORD
count_crW,
temp_levelW,
anz_lineW,
vtmodeW;

extern int logout_modus;

extern BOOLEAN mehr_disabled;
extern BOOLEAN mitten_im_chat;

static const char *urgent_text=
"\r\n"
"Die Box mu leider sofort beendigt werden. Vielleicht mu der Sysop\r\n"
"dringende Arbeiten vornehmen oder es mu ein Autoevent ausgefhrt\r\n"
"werden.\r\n";
static int urgent_printed=0;

const char *mb_output_prefix;


void online_key(int pressed_key);
static void online_status(const char *text);
char handle_escape(char pressed_key);

static BOOLEAN online_protokollB = FALSE;
static FILE *onlp_f = NULL;
static BOOLEAN befehls_protokoll = FALSE;
static FILE *befp_f = NULL;


/*
	Zeichenkette ausgeben
	(mit Umlautwandlung und Tastaturabfrage)

	<-			Anzahl Zeichen bzw. EOF/Timeout/Abbruch (-1)

	22.11.92 Frank, optimiert
	07.12.92 Frank, testweise direkte Ausgabe auf Standardkanle
	10.02.93 Frank, schweren logischen Fehler behoben (fhrte bei Consolen-
				       login auch zu Modemausgabe)
	02.12.93 Frank, Umlenkung von Ausgaben in Datei
	7.1.94, Uwe, Umlautwandlung eingebaut.

	Bugs:
	kein timeout
	kein Userbreak
	Bei Ausgabe von mehr als einer Zeile zu sptes "mehr".

	Nach Mglichkeit sollte jeweils nicht mehr als eine Zeile ausgegeben 
	werden damit die <MEHR> Funktion arbeiten kann.
	Achtung: Niemals mehr als 1K in einem Aufruf ausgeben, sonst kann es
 	passieren, da ungewandelt ausgegeben wird!
*/
int mboutput(const char *daten)
{
	size_t len = strlen(daten);
	char *pos;
	static long lines_since_drain=0;
	static BOOLEAN last_line_terminated=TRUE;

	if (got_sigusr2||got_sigterm)
	{
		logout_modus=1;
		return -1;
	}

	if (keine_ausgaben)
		return (int) len;

	/* Befehlsumlenkung immer ohne Umlautwandlung, da die Umlautwandlung
	 * des Tauschs noch darber luft. Auerdem keine MEHR-Zhlung.
	 */
	if (befehls_protokoll)
	{
		fputs(daten,befp_f);
		return (int) len;
	}

	if (mb_output_prefix && last_line_terminated)
		xwrite_tty(user_out_fd,mb_output_prefix, strlen(mb_output_prefix));

	/* Wenn nicht auf der Console und wenn Zeichensatzwandlung erwnscht */
	if (!akt_connection.is_console)
	{
		/* Dann durch Umlautwandlung */
		char obuf[4096];
		size_t newlen;
		newlen=string_to_foreign((unsigned char *)obuf, (unsigned const char *)daten,sizeof(obuf));
		if (newlen!=0)
			xwrite_tty(user_out_fd,obuf,newlen);
		else
			xwrite_tty(user_out_fd,daten,len);
	}
	else
		xwrite_tty(user_out_fd,daten, len);

	if (!pm_schutzB)
	{
		if (online_protokollB)
			fputs(daten,onlp_f);

		if (box_conf.view_user && sysop_out_fd>=0) {
			write(sysop_out_fd,daten,(unsigned) len);
			if (mb_output_prefix && last_line_terminated)
				xwrite_tty(user_out_fd,mb_output_prefix, strlen(mb_output_prefix));
		}
	}

	last_line_terminated=FALSE;
	/* LFs zhlen fr <mehr> */
	while ((pos = strchr(daten,'\n')) != NULL)
	{
		count_crW++;
		lines_since_drain++;
		daten = pos+1;
		if (!*daten)
			last_line_terminated=TRUE;
	}
	/* drinlassen, wegen verlorener Zeichen ber serielle */
	if (box_conf.drain_lines && lines_since_drain >= box_conf.drain_lines)
	{
		lines_since_drain=0;
		io_drain(user_out_fd);
	}

	if (count_crW > SCROLLZEILEN)
		if (!wait_mehr())
			return -1;

	return (int)len;

} /* mboutput */




/*
	Befehls-Fehlermeldung ausgeben

	Sinn dieser Funktion ist ein einheitliches Erscheinungsbild (inkl. Farbe) 
	und ggf. Debugging von Befehls-Fehlermeldungen
*/
void mbfehler(const char *s)
{
	int old=keine_ausgaben;
	keine_ausgaben=0;
	mbprintf(" %sFehler:%s %s\r\n",vt_S(col_ix[C_ROT]),vt_S(ATTR_OFF),s);
	keine_ausgaben=old;

} /* mbfehler */


/*
	Unterbrechung durch User oder Console via Ctrl-X/C

	<-			TRUE (Abbruch) oder FALSE (weiter)

	07.12.92 Frank
	10.02.93 Frank, Bug entfernt
	13.07.94 Frank, Abbruch bei remote Logins nicht mehr ber Konsole mglich
*/
BOOLEAN userbreak(void)
{
	char pressed_key = 0;

	if (tausch_auto_loginB)
		return FALSE;

	if (!io_charavail(user_in_fd))
		return FALSE;

	pressed_key = mbchar();

	switch (pressed_key)
	{
		case 19:            /* Ctrl-S */
			(void) mbchar();
			break;
		case 3:				/* Ctrl-C */
		case 14:				/* Ctrl-N */
		case 27:				/* Escape */
		case 24:				/* Ctrl-X */
			mboutput("\r\n");
			if (batchB)
				mbprintf("%s >> Batch-Abbruch <<%s\r\n",vt_S(ATTR_INVERS),vt_S(ATTR_OFF));
			else
				mbprintf("%s >> Abbruch <<%s\r\n",vt_S(ATTR_INVERS),vt_S(ATTR_OFF));
			return TRUE;
	}

	return FALSE;

} /* userbreak */


/*
	zeilenweise Eingabe
	verwendet in Editor und Utility (Chat)
*/
void mbinput(char *text,char *rest)
{
	int zeichen;
	size_t zaehler;

	if (*rest)
		*text = 0;
	else
		*rest = 0;

	if (*rest)
	{
		strcpy(text,rest);
		*rest = 0;
	}
	if (*text)  mboutput(text);

	count_crW = 0;

	do
	{
		zeichen = mbchar();
		if (!zeichen)
			break;
		if (zeichen>31 || zeichen=='\b' || zeichen=='\r' || zeichen=='\n')
		{
			if (zeichen=='\b' || zeichen==127)
			{
				zeichen = 0;
				if (*text)
				{
					text[strlen(text)-1] = 0;
					mboutput("\b ");
					zeichen = '\b';
				}
			}
			else
			{
				zaehler = strlen(text);
				if (zeichen != '\r' && zeichen != '\n')
				{
					text[zaehler++] = zeichen;
					text[zaehler] = 0;
				}
				if (zaehler >= 74)
				{
					/* Umbruch */
					if (strchr(text,' '))
					{
						while (--zaehler)
						{
							if (text[zaehler] == ' ')
								break;
							mboutput("\b \b");
						}
						zaehler++;
					}
					strcpy(rest,text+zaehler);
					text[zaehler-1] = 0;
					zeichen = '\r';
				}
			}
			if (zeichen == '\r' || zeichen == '\n')
				mboutput("\r\n");
			else
			{
				char smallbuf[2];
				smallbuf[0]=zeichen;
				smallbuf[1]=0;
				mboutput(smallbuf);
			}
		}
	}
	while (zeichen != '\r' && zeichen != '\n');

} /* mbinput */

void
tprint_fd(int fd, const char *cap)
{
#ifdef HAVE_TERMCAP
	char buf[2048];
	char *p=buf;
	char *q;
	q=tgetstr(cap,&p);
	if (q)
		write(fd,q,strlen(q));
#endif
}

/*
	Statusnderung (z.B. "Konsole besetzt") in oberster Zeile ausgeben
	(whrend ein User online ist)
*/
static void online_status(const char *text)
{
	if (sysop_out_fd>=0)
	{
		tprint_fd(sysop_out_fd,"sc");
		tprint_fd(sysop_out_fd,"ho");
		tprint_fd(sysop_out_fd,"so");
		write(sysop_out_fd,text,strlen(text));
		tprint_fd(sysop_out_fd,"se");
		tprint_fd(sysop_out_fd,"rc");
	}
} /* online_status */


char 
handle_escape(char pressed_key)
{
	static BOOLEAN in_flag = FALSE;		/* Flag, ob in online-key */
	BYTE screenB;
	clock_t wait_timeL;
	WORD voidW;
	char voidS[10];

	/* keine Verschachtelungen zulassen */
	if (in_flag)
		return 0;

	switch(pressed_key)
	{
		case 's':
			box_conf.console_besetzt = !box_conf.console_besetzt;
			if (box_conf.console_besetzt)
				online_status("Console besetzt");
			else
				online_status("Console nicht besetzt");
			break;

		case 'b':
			box_conf.view_user = !box_conf.view_user;
			if (box_conf.view_user)
				online_status("Bildschirm an");
			else
				online_status("Bildschirm aus");
			break;

		case 'c':
		case 'k':
			if (mitten_im_chat)
				break;
			in_flag = TRUE;
			screenB = box_conf.view_user;
			set_status_line("Chat erzwungen");
			mbprintf("%s%s",vt_pos_S(4,1),vt_S(DEL_SCR_REST));
			box_conf.view_user = TRUE;
			mboutput("\r\n\n");
			mboutput("\r\n\n");
			mboutput(" ** Chat mit Sysop beginnt:  (Wechsel mit Leerzeile)\r\n");
			chat_modus();
			mboutput("\r\n");
			mboutput("\r\n\n");
			mboutput(" ** Chat mit Sysop beendet.");
			box_conf.view_user = screenB;
			in_flag = FALSE;
			break;
/*			return '\r';*/
		case 'h':
			{
				char help[]=
"\r\nb   Bildschirm an (Spannermodus)?\r\n"
    "s   Console besetzt?\r\n"
    "c/k Chat erzwingen\r\n"
    "i   Userinfo ausgeben\r\n"
    "l   Userlevel fr diesen Anruf ndern\r\n"
    "L   Userlevel permanent ndern\r\n" 
    "o   Restonlinezeit ndern\r\n"
    "Q   User aus der Box werfen\r\n"
    "p   Protokoll ein/aus\r\n";
				write(sysop_out_fd,help,strlen(help));
    		}
    		break;

		case 'i':
			online_status(tprintf("Userinfo ==> Name: %s /// Level: %d",user.name,level()));
			break;

		case 'l':
			wait_timeL = time(NULL);
			online_status("neuen Userlevel eingeben ==> ");
			voidW = level()+'0';
			while ((time(NULL)-wait_timeL < 5) && !io_charavail(sysop_in_fd))
				msleep(50);
			if (io_charavail(sysop_in_fd))
			{
				char zeichen;
				read(sysop_in_fd,&zeichen,1);
				voidW = zeichen;
				if (voidW > '0' && voidW <= '9')
				{
					USER tmp_user = user;

					temp_levelW = voidW-'0';
					tmp_user.level = temp_levelW;
					b_set_flags(&tmp_user);
				}
				else
					voidW = level()+'0';
			}
			online_status(tprintf("neuer Userlevel ==> %c",voidW));
			break;

		case 'L':
			wait_timeL = time(NULL);
			online_status("neuen dauerhaften Userlevel eingeben ==> ");
			voidW = level()+'0';
			while ((time(NULL)-wait_timeL < 5) && !io_charavail(sysop_in_fd))
				msleep(50);
			if (io_charavail(sysop_in_fd))
			{
				char zeichen;
				read(sysop_in_fd,&zeichen,1);
				voidW = zeichen;
				if (isdigit(voidW))
				{
					user.level = voidW - '0';
					temp_levelW = -1;

					b_set_flags(&user);
				}
				else
					voidW = level()+'0';
			}
			online_status(tprintf("neuer dauerhafter Userlevel ==> %d",user.level));
			break;

		case 'o':
			tprint_fd(sysop_out_fd,"cl");
			online_status(tprintf(
				"Rest-Onlinezeit fr heute ndern von %ld auf:     \b\b\b\b",
				restzeit()/60));
			fgets(voidS,3,stdin);
			if (atol(voidS))
			{
				/*
					Restzeit des User fr heute
					= Vergangene Zeit plus angegebene Zeit.
					(vergangene Zeit wg. Restzeitberechnung nach Logoff)
				*/
				user.restzeit = useronline() + atol(voidS)*60;
				/* Die Onlinezeit fr diesen Anruf */
				akt_anruf.soft_limit = time(NULL) + atol(voidS)*60;
				if (akt_anruf.soft_limit > akt_anruf.hard_limit)
				{
					akt_anruf.soft_limit = akt_anruf.hard_limit;
				}
			}
			online_status(tprintf("neue Rest-Onlinezeit ==> %ld Min.",restzeit()/60));
			break;

		case 'Q':
			online_status("Rauswurf!");
			set_status_line("Logoff");
			logout_modus = 1;
			mboutput("\r\n\n Rauswurf: Der Sysop mu leider dringend an die Box...\r\n");
			mboutput("Bitte um Entschuldigung.\r\n");
			sleep(1);
			hangup_modem(user_in_fd);
			takeoff_modem(user_in_fd);
			break;

		case 'p':
			if (!online_protokollB)
			{
				char proto_path[PATH_MAX];

				in_flag = TRUE;
				online_status(" Online-Protokoll ->");
				akt_connection.is_console=TRUE;
				sprintf(proto_path,"%s%s.cons",box_conf.log_dir, dd_mm_yy());
				if ((onlp_f = fopen(proto_path,BIN_R_APPEND)) == NULL)
					break;
				online_status(" Online-Protokoll -> EIN");
				online_protokollB = TRUE;
				fprintf(onlp_f,"\r\n");
				fprintf(onlp_f," ==============================================================================\r\n");
				fprintf(onlp_f," //    Consolen-Protokoll vom: %s %s     User: %s //\r\n",dd_mm_yy(),hh_mm_ss(),user.name);
					fprintf(onlp_f," ==============================================================================\r\n");
				in_flag = FALSE;
			}
			else
			{
				online_status(" Online-Protokoll -> AUS");
				fclose(onlp_f);
				online_protokollB = FALSE;
			}
			break;
	}
	return 0;
} /* online_key */



void online_protokoll_close(void)
{
	if (!online_protokollB)
		return;
	if (onlp_f)
		fclose(onlp_f);
	onlp_f = NULL;
	online_protokollB = FALSE;

} /* online_protokoll_close */



/*
	Umlenkung von Befehlsausgaben in eine Datei

	->			datei	:	Dateiname (einschalten) bzw. NULL (ausschalten)

	<-			TRUE	:	ok
				FALSE	:	Datei ex. nicht

	02.12.93 Frank
*/
BOOLEAN befehls_umlenkung(const char *datei)
{
	if (!datei)
	{
		befehls_protokoll = FALSE;

		if (befp_f)
		{
			fclose(befp_f);
			befp_f = NULL;
		}

		return TRUE;
	}

	if ((befp_f = fopen(datei,BINAPPEND)) == NULL)
		return FALSE;

	befehls_protokoll = TRUE;

	return TRUE;

} /* befehls_umlenkung */



/*
	29.08.92 Frank, kein MEHR bei Auto-Tausch
	05.12.92 Frank

	<-			TRUE = weiter , FALSE = Abbruch

	Abbruch wird durch ESCAPE, 'q', Ctrl-C und Ctrl-X bewirkt
*/
BOOLEAN wait_mehr(void)
{
	int zeichen = 0;

	count_crW = 0;
	if (!logout_modus && user.mehr 
		&& !mehr_disabled && !tausch_auto_loginB && !batchB)
	{
		mbprintf("\r%s%s <<< MEHR >>> %s",
					vt_S(col_ix[C_ROT]),
					vt_S(ATTR_INVERS),
					vt_S(ATTR_OFF));
		if (user.glocke)
			userglocke();
		zeichen = mbchar();
		mboutput("\r               \r");
		mboutput(vt_S(ATTR_OFF));

		if (strchr("\033q\003\030",zeichen))
			return FALSE;
	}
	return TRUE;

} /* wait_mehr */

void userglocke(void)
{
	const char *gl="\a";
	write(user_out_fd,gl,1);
}

static void
alarm_handler(int sig TOM_ATTRIB_UNUSED)
{
	/* dummy */
}

/*
 * eine Zeile beliebiger Lnge lesen. Verwendet im smtp-Code und im
 * internen ASCII-Protokoll.
 * Keine Umlautwandlung.
 */
char *
mbgetline(ULONG timeout_sec, BOOLEAN *timed_out, BOOLEAN mit_echo)
{
	static char readbuf[1024];
	static size_t zurueckgestellt=0;

	size_t place=0;
	int zeichen;
	size_t timeout_counter=0;
	char *start=NULL;
	char *ptr=NULL;

	if (logout_modus)
		return NULL;

	signal(SIGALRM,alarm_handler);
	alarm(timeout_sec);

	while (TRUE)
	{
		char *p;
		if (got_sigusr2 || logout_modus||got_sigterm)
		{
			if (!urgent_printed++) mboutput(urgent_text);
			logout_modus=1;
			if (start)
				free(start);
			alarm(0);
			return 0;
		}

		if (zurueckgestellt)
		{
			zeichen=zurueckgestellt;
			zurueckgestellt=0;
		}
		else
		{
			zeichen=read(user_in_fd,readbuf,sizeof(readbuf));
			if (zeichen==-1 && errno==EINTR)
			{
				if (start)
					free(start);
				if (timed_out)
					*timed_out=TRUE;
				return NULL;
			}
		}
		/* hmmm .. keine Ahnung, ob das so gut ist! */
		if (!zeichen)
			break;
		if (zeichen<0)
		{
			kill(getpid(),SIGHUP);
			logout_modus=1;
			alarm(0);
			return NULL;
		}
		timeout_counter+=zeichen;
		if (timeout_counter>512)
		{
			timeout_counter=0;
			alarm(60);
		}
		if (!start || (ptr+zeichen+1>=start+place))
		{
			size_t off;
			if (start)
				off=ptr-start;
			else
				off=0;
			if (zeichen>1024)
				place+=zeichen*4;
			else
				place+=1024;
			start=realloc(start,place);
			if (!start)
			{
				errno=ENOMEM;
				return NULL;
			}
			ptr=start+off;
		}
		memcpy(ptr,readbuf,zeichen);
		ptr[zeichen]=0;

		p=ptr;
		ptr+=zeichen;
		while (zeichen && *p!='\r' && *p!='\n')
		{
			p++;
			zeichen--;
		}
		if (*p=='\r' || *p=='\n')
		{
			alarm(0);
			if (*p=='\r' && zeichen)
			{
				p++;
				zeichen--;
			}
			if (*p=='\n' && zeichen)
			{
				p++;
				zeichen--;
			}
			if (zeichen)
			{
				memcpy(readbuf,p,zeichen);
				zurueckgestellt=zeichen;
			}
			*p=0;
			if (mit_echo && zeichen) {
				xwrite(user_out_fd,start,strlen(start));
				xwrite(user_out_fd,"\r\n",2);
			}
			return start;
		}
	}
	return start;
} /* mbreadline */

/*
 * Taste einlesen, inkl. Cursortasten-Auswertung
 * 
 * Problem:
 * Erkennung ist auf vt100/vt52/ANSI hardcoded, das eigentliche Problem sind
 * aber die mglw. unterschiedlichen Terminaltypen remote/lokal und die
 * wnschenswerte Mglichkeit, von beiden Seiten aus Cursortasten eingeben
 * zu knnen.
 * Die jetzige Lsung ist zumindest praxisnah, evtl. werden PGUP etc. noch
 * ergnzt.
 */
int 
mbkey (void)
{
	BOOLEAN esc = FALSE;		/* zur Erkennung von Cursortasten */
	char cursor[10];
	int cur_anz = 0;
	int zeichen;

	do
	{
		zeichen = mbchar ();

	   /* Cursortastenerkennung */
		if (zeichen == '\033')
		{
			*cursor = 0;
			cur_anz = 0;
			esc = TRUE;
			continue;
		}

		if (!esc)
			return zeichen;

		cursor[cur_anz++] = zeichen;
		cursor[cur_anz] = 0;

		if (cur_anz < 5 && (zeichen == '[' || zeichen == 'O'))
			continue;
		if (cur_anz > 5) 
			/* das ist kein Cursor */
			return '\033';

		if (!strcmp (cursor, "[A") || !strcmp (cursor, "OA"))
			return C_UP;
		if (!strcmp (cursor, "[B") || !strcmp (cursor, "OB"))
			return C_DOWN;
		if (!strcmp (cursor, "[C") || !strcmp (cursor, "OC"))
			return C_RIGHT;
		if (!strcmp (cursor, "[D") || !strcmp (cursor, "OD"))
			return C_LEFT;
	}
	while (1);

}								/* mbkey */
