/* brettacc.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Zugriffslisten-Verwaltung

	Um deadlocks zu vermeiden, mu dies auf einer anderen Ebene
	geschehen als in der Brettverwaltung/Brettserver.
	Es wird nmlich sowohl auf Brett- als auch auf Userdaten zugegriffen.

	18.01.94 Frank
*/


#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <sys/stat.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "brett.h"
#include "filelock.h"
#include "modem.h"
/*
TOMDOC_FILE([[lib/brettacc.c]],Zugriffslisten)
TOMDOC_TEXT([[Idee:
Hier knnte man irgendwann einmal Caching einbauen. Das jetzige
Verfahren macht mehr Zugriffe als unbedingt ntig. Es werden
einmal alle Zugriffslisten durchwhlt, um festzustellen, ob der
User zugreifen darf (das ist durchaus effizient). Auerdem wird
fr jedes Tausch-Gruppen-Infofile nocheinmal die Mitgliederliste
herausgesucht, um festzustellen, ob der User Mitglied ist.
Etliche Tauschkommandos greifen auch auf die Mitgliedschaft zu.

Wahrscheinlich reicht es, nur die Flags fr den aktuellen User
zu cachen. Das knnte man in br_ind[] machen, als zustzliches
boxlokales Flag.
]])
*/


/*
	User in Brettzugriffsliste eintragen

	29.04.93 Frank, neu
*/
BOOLEAN b_add_access(BRETTNR nr,USERNR usernr)
{
	FILE *acc_f;
	char zug_lst[PATH_MAX];
	BRETT *b;
	USERNR tmp_nr;

	CHECK_BRETTNR(nr,return FALSE);

	b=br_ind[nr].brett;

	/* Brett nicht durch Userliste geschtzt ? */
	if (!B_MITLISTE(b->flags))
		return TRUE;

	sprintf(zug_lst,"%s%s",brettpfad(nr),"zugriff.dat");
	if (!fileexist(zug_lst))
	{
		char *x=strdup(zug_lst);
		if (x) {
			char *p=strrchr(x,'/');
			if (p)
				*p=0;
			mkdir(x,S_IRUSR|S_IWUSR|S_IXUSR|S_IRGRP|S_IXGRP);
			free(x);
		}
		if ((acc_f = fopen(zug_lst,BIN_R_APPEND)) == NULL)
			return FALSE;
		fd_writew_lock(fileno(acc_f),0,SEEK_SET,1);
		fwrite(&usernr,sizeof(USERNR),1,acc_f);
		fclose(acc_f);
		return TRUE;
	}

	if ((acc_f = fopen(zug_lst,BIN_R_APPEND)) == NULL)
		return FALSE;

	fd_writew_lock(fileno(acc_f),0,SEEK_SET,1);
	while (fread(&tmp_nr,sizeof(USERNR),1,acc_f))
	{
		if (tmp_nr == usernr)
		{
			fclose(acc_f);
			return TRUE;
		}
	}
	fseek(acc_f,- sizeof(USERNR),SEEK_CUR);
	fwrite(&usernr,sizeof(USERNR),1,acc_f);
	fclose(acc_f);

	return TRUE;

} /* b_add_access */



/*
	User aus Brettzugriffsliste austragen

	29.04.93 Frank, neu

	da die Zugriffsdatei bei nderungen ex- und wieder importiert wird, 
	reicht es aus, den Eintrag auf 0 zu setzen.
*/
BOOLEAN b_del_access(BRETTNR nr,USERNR usernr)
{
	FILE *acc_f;
	char zug_lst[PATH_MAX];
	BRETT *b;
	USERNR tmp_nr;
	CHECK_BRETTNR(nr,return FALSE);

	b=br_ind[nr].brett;

	/* Brett durch Userliste geschtzt ? */
	if (!B_MITLISTE(b->flags))
		return TRUE;

	sprintf(zug_lst,"%s%s",brettpfad(nr),"zugriff.dat");

	if ((acc_f = fopen(zug_lst,BIN_RW_OPEN)) == NULL)
		return TRUE;
	fd_writew_lock(fileno(acc_f),0,SEEK_SET,1);

	while (fread(&tmp_nr,sizeof(USERNR),1,acc_f))
	{
		if (tmp_nr == usernr)
		{
			fseek(acc_f,- sizeof(USERNR),SEEK_CUR);
			tmp_nr = 0;
			fwrite(&tmp_nr,sizeof(USERNR),1,acc_f);
			fflush(acc_f);
		}
	}
	fclose(acc_f);

	return TRUE;

} /* b_del_access */



/*
	Brettmitgliedschaft berprfen

	<-			TRUE	:	User ist in Zugriffsliste eingetragen
				FALSE	:	User nicht eingetragen oder keine Liste vorhanden

	22.11.93 Frank, neu
*/
BOOLEAN b_tst_access(BRETTNR nr,USERNR usernr)
{
	FILE *acc_f;
	char zug_lst[PATH_MAX];
	USERNR tmp_nr;
	BRETT *b;

	CHECK_BRETTNR(nr,return FALSE);

	b=br_ind[nr].brett;

	/* Brett nicht durch Userliste geschtzt ? */
	if (!B_MITLISTE(b->flags))
		return FALSE;

	sprintf(zug_lst,"%s%s",brettpfad(nr),"zugriff.dat");
	if ((acc_f = fopen(zug_lst,BINREAD)) == NULL)
		return FALSE;
	fd_readw_lock(fileno(acc_f),0,SEEK_SET,1);

	while (fread(&tmp_nr,sizeof(USERNR),1,acc_f))
	{
		if (tmp_nr == usernr)
		{
			fclose(acc_f);
			return TRUE;
		}
	}
	fclose(acc_f);

	return FALSE;

} /* b_tst_access */



/*
	binre Zugriffsliste in Textdatei umsetzen

	14.10.1993 Frank

	->			nr		:	interne Brettnummer

	Die Umsetzung findet direkt im Brettpfad statt.
	Falls keine binre Zugriffsdatei existiert, wird eine leere Zugriffs-
	liste erzeugt.
*/
BOOLEAN b_export_access(BRETTNR nr)
{
	char zug_lst[PATH_MAX],
	     zug_dat[PATH_MAX];
	FILE *dat_f,*lst_f;
	USERNR usernr;
	USER user;

	sprintf(zug_dat,"%s%s",brettpfad(nr),"zugriff.dat");
	sprintf(zug_lst,"%s%s",brettpfad(nr),"zugriff.lst");

	if ((lst_f = fopen(zug_lst,BINWRITE)) == NULL)
		return FALSE;

	if ((dat_f = fopen(zug_dat,BINREAD)) == NULL)
	{
		fclose(lst_f);
		return TRUE;
	}
	fd_readw_lock(fileno(dat_f),0,SEEK_SET,1);

	while (fread(&usernr,sizeof(USERNR),1,dat_f))
	{
		if (!ub_read_user(usernr,&user,FALSE))
			continue;
		fprintf(lst_f,"%s\r\n",user.name);
	}

	fclose(dat_f);
	fclose(lst_f);

	return TRUE;

} /* b_export_access */



/*
	ASCII-Zugriffsliste in Binrdatei umsetzen

	14.10.1993 Frank

	->			nr		:	interne Brettnummer

	Die Umsetzung findet direkt im Brettpfad statt.
	Falls keine Zugriffsliste existiert, wird eine leere binre Zugriffs-
	datei erzeugt.
*/
BOOLEAN b_import_access(BRETTNR nr)
{
	char zug_lst[PATH_MAX],
	     zug_dat[PATH_MAX];
	FILE *dat_f,*lst_f;
	BOOLEAN ok=TRUE;
	char name[200];
	USERNR usernr;

	sprintf(zug_lst,"%s%s",brettpfad(nr),"zugriff.lst");
	sprintf(zug_dat,"%s%s",brettpfad(nr),"zugriff.dat");

	if ((dat_f = fopen(zug_dat,BINWRITE)) == NULL)
		return FALSE;
	fd_writew_lock(fileno(dat_f),0,SEEK_SET,1);

	if ((lst_f = fopen(zug_lst,BINREAD)) == NULL)
	{
		fclose(dat_f);
		return TRUE;
	}

	while (fgets(name,190,lst_f))
	{
		char *p;

		if ((p = strpbrk(name,"\r\n")) != NULL)
			*p = 0;
		if (!*name)
			continue;

		if (!ub_find_username(&usernr,name))
		{
			mbprintf("unbekannter Username in Zugriffsliste: %s\r\n", name);
			continue;
		}

		if (!fwrite(&usernr,sizeof(USERNR),1,dat_f))
		{
			ok=FALSE;
			break;
		}
	}

	fclose(lst_f);
	fclose(dat_f);

	return ok;

} /* b_import_access */
