/* brettnr.c: return the BRETTNR.
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/
#include <portab.h>


#include "config.h"
#include "libquark.h"

#include "brett_db.h"

#ifdef TOM_CACHE_STATISTICS
#  ifndef TOM_ATTRIB_CONSTRUCTOR
#    undef TOM_CACHE_STATISTICS
#  endif
#endif
#ifdef TOM_CACHE_STATISTICS
#include <syslog.h>
#endif

/*
BB_FUNCTION(Funktion,BRETTNR,brettnr,[[(const char *@var{name})]],[[
    Wandelt den angegebenen Brettnamen mittels bb_canonical in die
    interne Form und sucht ihn in der Liste aller Bretter (also nicht
    nur in einer der dem aktuellen User bekannten).

    Liefert 0 bei Nichtfinden, sonst die Nummer des Brettes.
]])
*/
#define CACHESIZE 25
#ifdef TOM_CACHE_STATISTICS
static unsigned long brettnr_cache_hit;
static unsigned long brettnr_cache_miss;
static unsigned long brettnr_cache_reorder;

static void brettnr_cache_print(void) TOM_ATTRIB_DESTRUCTOR;
static void 
brettnr_cache_print(void)
{
	if (brettnr_cache_hit || brettnr_cache_miss)
		syslog(LOG_DEBUG,"brettnr: %lu,%lu,%lu",
			brettnr_cache_hit, brettnr_cache_miss, brettnr_cache_reorder);
}
#endif

BRETTNR brettnr(const char *name)
{
	BRETTINFO *bi = br_ind;
	long i;
	char bufname[512];
	static BRETTNR cache[CACHESIZE]={0};

	bb_canonical(name,bufname);
	for (i=0;i<CACHESIZE;i++)
	{
		if (!cache[i])
			break;
		if (!br_ind[cache[i]].canon)
		{
			cache[0]=0; /* Hmmm? */
			break;
		}
		if (0==strcmp(br_ind[cache[i]].canon,bufname))
		{
			BRETTNR bnr;
#ifdef TOM_CACHE_STATISTICS
			brettnr_cache_hit++;
#endif
			if (i==0)
				return cache[i];
			bnr=cache[i];
			memcpy(&cache[1],&cache[0],i*sizeof(BRETTNR));
			cache[0]=bnr;
#ifdef TOM_CACHE_STATISTICS
			brettnr_cache_reorder++;
#endif
			return bnr;
		}
	}
#ifdef TOM_CACHE_STATISTICS
	brettnr_cache_miss++;
#endif

	i=0;
	/* kein strncmp! Wegen MAUS & MAUS.GRP */
	while (i++ < max_brettnr)
	{
		if (bi->canon)
		{
			if (bi->canon[0]==bufname[0] && 0==strcmp(bi->canon,bufname))
			{
				memcpy(&cache[1],&cache[0],(CACHESIZE-1)*sizeof(BRETTNR));
				cache[0]=i-1;
				return i-1;
			}
		}
		bi++;
	}
	return 0;

} /* brettnr */
