/* call_editor.c: still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact1: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include <portab.h>
#include <stdio.h>
#include <limits.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
#include <pwd.h>

#include "tomsysdep.h"
#include "tompaths.h"
#include "prototyp.h"
#include "modem.h"
#include "terminal.h"
#include "box_conf.h"
#include "aktdata.h"
#include "user.h"
#include "cslib.h"

extern USER user;

extern int logout_modus; /* TCL-Schnittstelle */
extern WORD vtmodeW;

static BOOLEAN recall_backup(const char *backup, const char *datei);

/*
	Editor aufrufen

	->			datei	:	zu edierende Datei

	je nach Terminaleinstellung sowie Login (remote oder lokal) wird ein 
	anderer Editor aufgerufen (s. quark.cnf)

	3.4.1994, Uwe, total umgestellt, auf Nutzung von Securestart und
	          defaultmiges Benutzen von ascii_edit, wenn der 
	          eigentlich zu verwendende Editor nicht konfiguriert ist.
*/
int 
call_editor (const char *datei, BOOLEAN do_backup)
{
	const char *which = NULL;
	int cswan = 0;
	const char *charset = "usascii";
	BOOLEAN not = FALSE;
	int ret;
	char editor_exe[PATH_MAX];
	char workfile[PATH_MAX];
	BOOLEAN need_userchange=FALSE;
	uostr_t cspath=UOSTR_INIT;
	uostr_xdup_cstrmulti(&cspath,box_conf.bin_dir,"cstrans",NULL);
	uostr_x0(&cspath);

	if (akt_connection.is_console)
		which = box_conf.cons_edit;
	else if (vtmodeW == T_ANSI)
		which = box_conf.ansi_edit;
	else if (vtmodeW == T_VT52)
		which = box_conf.vt52_edit;
	else if (vtmodeW == T_VT100)
		which = box_conf.vt100_edit;
	if (!which || !*which)
		which = box_conf.ascii_edit;
	if (*which == '!')
	{
		not = TRUE;
		which++;
	}
	if (!akt_connection.is_console && fileexist (cspath.data))
	{
		cstl_t *cs;

		cswan = 1;
		cs = get_charset_from_nr (user.zeichensatz);
		if (!cs)
			charset = "usascii";
		else
			charset = cs->name;
	}
	uostr_freedata(&cspath);

	if (!which || !*which)
		return -1;

	if (akt_connection.euid!=akt_connection.ruid)
	{
		const char *home=getenv("HOME");
		if (!home)
		{
			struct passwd *pwd;
			pwd=getpwuid(akt_connection.ruid);
			if (pwd)
				home=pwd->pw_dir;
		}
		if (!home)
		{
			mboutput("Kein $HOME!\r\n");
			sprintf (workfile, "%se%lu.tmp", tom_tmp_dir, (ULONG) getpid ());
		}
		else
		{
			if (access(home,W_OK)!=0)
			{
				mbprintf("%s: %s\r\n",workfile,strerror(errno));
				sprintf (workfile, "%se%lu.tmp", tom_tmp_dir, (ULONG) getpid ());
				unlink(workfile);
			} else {
				sprintf (workfile, "%s/tom.%lu.tmp", home, (ULONG) getpid ());
				need_userchange=TRUE;
			}
		}
	}
	else
		sprintf (workfile, "%se%lu.tmp", tom_tmp_dir, (ULONG) getpid ());

	if (fileexist (datei))
	{
		/* 
		 * Eine Arbeitskopie der Datei erzeugen, auf die der User
		 * Schreibzugriff hat.
		 */
		FILE *fin;
		FILE *fout;
		char buf[1024];
		size_t gelesen;
		BOOLEAN err=FALSE;

		fin=fopen(datei,"rb");

		/* Es kann sein, da ein Shelluser mit Sysoprechten 
		 * EDIT TEXT aufruft, um irgendeine Datei in seinem Homeverzeichnis
		 * zu editieren.
		 */
		if (!fin && need_userchange)
		{
			int errno1=errno;
			set_user_ids();
			fin=fopen(datei,"rb");
			if (!fin)
			{
				mboutput("Kann Datei nicht lesen!\r\n");
				mbprintf("  %s: %s\r\n",datei,strerror(errno1));
				return -1;
				set_box_ids();
			}
		}
		else if (need_userchange)
			set_user_ids();

		fout=fopen(workfile,"w");
		if (!fout)
		{
			int errno1=errno;
			fclose(fin);
			if (need_userchange)
				set_box_ids();
			mboutput("Kann Temporrdatei nicht schreiben!\r\n");
			mbprintf("  %s: %s\r\n",datei,strerror(errno1));
			return -1;
		}

		while ((gelesen=fread(buf,1,sizeof(buf),fin))>0)
		{
			if (fwrite(buf,1,gelesen,fout)!=gelesen)
			{
				err=TRUE;
				mbprintf("kann Daten nicht schreiben: %s\r\n",	
					strerror(errno));
			}
		}
		if (fclose(fout)!=0)
		{
			err=TRUE;
			mbprintf("kann Daten nicht schreiben: %s\r\n",	
				strerror(errno));
		}
		if (ferror(fin))
		{
			err=TRUE;
			mbprintf("kann Daten nicht lesen: %s\r\n",	
				strerror(errno));
		}
		fclose(fin);
		if (err)
		{
			unlink(workfile);
			return -1;
		}

		if (cswan)
		{
			/* cstrans kann In-Place arbeiten */
			uostr_t cmd=UOSTR_INIT;
			uostr_xdup_cstrmulti(&cmd,"-t ",charset," ",workfile,NULL);
			uostr_x0(&cmd);
			if (call_system_prg ("cstrans", cmd.data)) {
				uostr_freedata(&cmd);
				return -1;
			}
			uostr_freedata(&cmd);
		}

		/* es mag eine PM sein, oder etwas geheimes ... */
		chmod(workfile,0600);
		
		if (need_userchange)
			set_box_ids();
	} else {
		if (need_userchange)
			set_user_ids();
		unlink(workfile);
			set_box_ids();
	}

	/* der Editoraufruf - so wenig Arbeit ist das ... */
	sprintf (editor_exe, "%s%s %s", box_conf.bin_dir, which,workfile);
	ret=call_extern(
		editor_exe,
		NULL,                 /* spezielles Arbeitsverzeichnis */
		akt_anruf.soft_limit, /* killzeitpunkt */
		TRUE,                 /* ignore_exitcode */
		TRUE);                /* unter UID des Users */

	if (not)
		ret = !ret;
	if (need_userchange) {
		set_user_ids();
		if (!fileexist(workfile))
			ret=-1;
		set_box_ids();
	} else {
		if (!fileexist(workfile))
			ret=-1;
	}


	if (!ret)
	{
		/* 
		 * Hier wird es etwas schwierig. Die Datei gehrt dem User,
		 * und ihre Zugriffsrechte sind (mssen sein) 0600 - unter
		 * unserer eigenen Kennung kommen wir da nicht ran. 
		 * Wechseln wir aber die uid, mssen wir irgendwie dafr
		 * sorgen, da die Daten nicht einen Moment lang fr
		 * unbefugte lesbar sind.
		 */
		FILE *fin,*fout;
		char buf[1024];
		char backup[PATH_MAX];
		size_t gelesen;
		BOOLEAN err=FALSE;
		BOOLEAN did_backup=FALSE;

		/* Wenn die Datei vorher schon existiert hat, legen wir
		 * jetzt ein Backup an.
		 */
		if (fileexist(datei))
		{
			char *p,*q;
			sprintf(backup,datei);
			p=strrchr(backup,'/');
			if (p)
				p++;
			else
				p=backup;
			q=strrchr(p,'.');
			if (q)
				q++;
			else
				q=p+strlen(p);
			strcpy(q,"bak");
			unlink(backup);
			if (!file_copy(datei,backup))
			{
				mbprintf("kann Backup nicht anlegen: %s\r\n",	
					strerror(errno));
				unlink(workfile);
				return -1;
			}
			did_backup=TRUE;
		}

		/* Ausgabedatei ffnen */
		fout=fopen(datei,"w");
		if (!fout)
		{
			int errno1=errno;
			mbprintf("kann Datei %s nicht schreiben: %s\r\n",	
				datei,strerror(errno1));
			syslog(LOG_ERR,"fopen(%s,\"w\"): %s",
				datei,strerror(errno1));
			unlink(workfile);
			unlink(datei);
			if (did_backup)
				recall_backup(backup,datei);
			unlink(backup);
			return -1;
		}

		/* zur User-ID wechseln, da die Temporrdatei sonst nicht gelesen
		 * werden kann. 
		 */
		if (need_userchange)
			set_user_ids();
		fin=fopen(workfile,"r");
		if (!fin)
		{
			int errno1=errno;
			mbprintf("kann Datei %s nicht lesen: %s\r\n",	
				workfile,strerror(errno1));
			syslog(LOG_ERR,"fopen(%s,\"r\"): %s",
				workfile,strerror(errno1));
			unlink(workfile);
			unlink(datei);
			if (did_backup)
				recall_backup(backup,datei);
			unlink(backup);
			return -1;
		}
		/* und zurck */
		if (need_userchange)
			set_box_ids();

		/* Datei kopieren */
		while ((gelesen=fread(buf,1,sizeof(buf),fin))>0)
		{
			if (fwrite(buf,1,gelesen,fout)!=gelesen)
			{
				int errno1=errno;
				mbprintf("kann Daten nicht schreiben: %s\r\n",	
					strerror(errno));
				syslog(LOG_ERR,"fwrite in %s: %s",
					datei,strerror(errno1));
				err=TRUE;
			}
		}
		if (fclose(fout)!=0 && !err)
		{
			int errno1=errno;
			err=TRUE;
			mbprintf("kann Daten nicht schreiben: %s\r\n",	
				strerror(errno));
			syslog(LOG_ERR,"fclose(%s): %s",
				datei,strerror(errno1));
		}
		if (ferror(fin))
		{
			int errno1=errno;
			err=TRUE;
			mbprintf("kann Daten nicht lesen: %s\r\n",	
				strerror(errno));
			syslog(LOG_ERR,"fread aus %s: %s",
				workfile,strerror(errno1));
		}
		fclose(fin);
		if (err)
		{
			/* Backup einspielen */
			unlink(workfile);
			unlink(datei);
			if (did_backup)
				recall_backup(backup,datei);
			unlink(backup);

			return -1;
		}

		unlink(workfile);

		ret=0;
		if (cswan)
		{
			uostr_t cmd=UOSTR_INIT;
			uostr_xdup_cstrmulti(&cmd,"-f ",charset," ",workfile,NULL);
			uostr_x0(&cmd);
			if (need_userchange)
				set_user_ids();
			if (call_system_prg ("cstrans", cmd.data)) {
				uostr_freedata(&cmd);
				if (need_userchange)
					set_box_ids();
				return -1;
			}
			if (need_userchange)
				set_box_ids();
			uostr_freedata(&cmd);
		}
		if (ret==-1)
		{
			/* es ist anzunehmen, da "datei" nun Mll enthlt - jedenfalls
			 * nichts, was wir gebrauchen knnen ... ergo brauchen wir 
			 * das Backup.
			 */
			int errno1=errno;
			mboutput("Umlautwandlung der Datei fehlgeschlagen!\r\n");
			syslog(LOG_ERR,"cstrans von %s: %s",
				datei,strerror(errno1));
			unlink(datei); /* Plattenplatz schaffen */
			if (did_backup)
				recall_backup(backup,datei);
			return -1;
		}
		/* alles OK */
		if (!do_backup && did_backup)
		{
			/* das Backup lschen */
			unlink(backup);
		}
	}
	else
		unlink(workfile);
	return ret;
}								/* call_editor */

static BOOLEAN
recall_backup(const char *backup, const char *datei)
{
	mboutput("Das Backup wird verwendet!\r\n");
	if (!file_move(backup,datei))
	{
		/* das ist die einzige fatale Stelle hier */
		mboutput("Das Zurckholen des Backups ist gescheitert!\r\n");
		mboutput("Bitte benachrichtigen Sie den Sysop:\r\n");
		mbprintf("  Von:    %s\r\n",backup);
		mbprintf("  Zu:     %s\r\n",datei);
		mbprintf("  Fehler: %s\r\n",strerror(errno));
		syslog(LOG_CRIT, /* naja, mglicherweise ist es das! */
			"need backup, but cannot copy it: %m");
		syslog(LOG_CRIT, "backup: %s",backup);
		syslog(LOG_CRIT, "datei: %s",datei);
		/* *der* Grund, hier nicht mit "exit" aufzuhren ist, da der
		 * User eventuell das Problem beheben knnte ...
		 */
		return FALSE;
	}
	else
		syslog(LOG_ERR, "backup %s verwendet",backup);
	return TRUE;
}
