#include "tomsysdep.h"
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <syslog.h>
#include <string.h>
#include "box_conf.h"
#include "user.h"
#include "uostr.h"

int error_txtbsy =
#ifdef ETXTBSY
ETXTBSY;
#else
-4;
#endif

USER user;
char *program_name;
extern char **environ;

static void 
doit (const char *login)
{
	USERNR unr;
	if (!ub_find_username (&unr,login)) {
		syslog(LOG_NOTICE,"user %s not found",login);
		_exit (1);
	}
	if (!ub_read_user (unr, &user, 0)) {
		syslog(LOG_ERR,"usernummer %d not found",unr);
		_exit (1);
	}
}

static char *
str1e2(const char *name, const char *value)
{
	uostr_t nv={0};
	char *s;
	uostr_xadd_cstr(&nv,name);
	uostr_xadd_char(&nv,'=');
	uostr_xadd_cstr(&nv,value);
	uostr_x0(&nv);
	s=malloc(nv.len);
	if (!s) {
		syslog(LOG_ERR,"out of memory");
		_exit(1);
	}
	memcpy(s,nv.data,nv.len);
	return s;
}

char up[513];
int uplen;

void 
main (int argc TOM_ATTRIB_UNUSED, char **argv)
{
	char *login;
	char *password;
	int r;
	int i;
	char **newenv;
	int numenv;
	uostr_t dirname = {0};
	program_name=argv[0];

	if (getuid()==0) {
		syslog(LOG_ERR,"refused to run as root");
		_exit(4);
	}
	if (getgid()==0) {
		syslog(LOG_ERR,"refused to run in group 0");
		_exit(4);
	}

	if (!argv[1]) {
		syslog(LOG_ERR,"incorrect number of arguments");
		_exit (2);
	}
	if (!read_quark_configuration (argv[1])) {
		syslog(LOG_ERR,"can't read configuration file %s",argv[1]);
		_exit (3);
	}
	if (!ub_init(box_conf.var_dir)) {
		syslog(LOG_ERR,"can't open user database in %s",box_conf.var_dir);
		_exit(5);
	}
	argv++;
	if (!argv[1]) {
		syslog(LOG_ERR,"incorrect number of arguments");
		_exit (2);
	}

	uplen = 0;
	for (;;) {
		do
			r = read (3, up + uplen, sizeof (up) - uplen);
		while ((r == -1) && (errno == EINTR));
		if (r == -1) {
			syslog(LOG_ERR,"lost connection reading from fd 3");
			_exit (111);
		}
		if (r == 0)
			break;
		uplen += r;
		if (uplen >= (int) sizeof (up)) {
			syslog(LOG_ERR,"got too much from fd 3");
			_exit (1);
		}
	}

	close (3);
	i = 0;
	login = up + i;
	while (up[i++])
		if (i == uplen) {
			syslog(LOG_ERR,"user name too long");
			_exit (2);
		}
	password = up + i;
	if (i == uplen) {
		syslog(LOG_ERR,"user name too long");
		_exit (2);
	}
	while (up[i++]) {
		if (i == uplen) {
			syslog(LOG_ERR,"user name and password too long");
			_exit (2);
		}
	}

	doit (login);
	if (!passwort_vergleich (&user, password)) {
		syslog(LOG_NOTICE,"wrong password for %s",login);
		_exit (1);
	}

	{
		char utmp[UDAT_NAME_LEN*2];
		char *p;
		ub_canonical (user.name, utmp);
		for (p = utmp; *p; p++) {
			if (*p == ' ')
				*p = '_';
		}
		uostr_xdup_cstrmulti (&dirname, box_conf.user_dir, utmp, NULL);
		uostr_x0 (&dirname);
	}

	if (chdir (dirname.data) == -1) {
		syslog(LOG_ERR,"can't chdir to %s: %s",dirname.data,strerror(errno));
		_exit (111);
	}

	numenv=0;
	while (environ[numenv])
		++numenv;
	newenv = (char **) malloc ((numenv + 4) * sizeof (char *));
	if (!newenv) {
		syslog(LOG_ERR,"out of memory");
		_exit (111);
	}
	for (i = 0; i < numenv; ++i)
		newenv[i] = environ[i];
	newenv[numenv++] = str1e2 ("USER", user.name);
	newenv[numenv++] = str1e2 ("HOME", dirname.data);
	newenv[numenv++] = str1e2 ("SHELL", "tom");
	newenv[numenv] = 0;
	environ = newenv;

	syslog(LOG_INFO,"authenticated %s for %s",login,argv[1]);
	execvp (argv[1], argv + 1);
	syslog(LOG_ERR,"can't exec %s: %s",argv[1],strerror(errno));
	_exit (111);
}
