/* cmd.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <syslog.h>
#include <errno.h>
#include <unistd.h> /* write */

#include "prototyp.h"
#include "befehle.h"
#include "terminal.h"
#include "brett.h"
#include "modem.h"
#include "user.h"
#include "box_conf.h"
#include "tompaths.h"
#include "cmd.h"
#include "tomtcl.h"
#include "aktdata.h"
#include "log.h"

#define HISTORY_LINES		16

extern USER user;
extern BRETT akt_brett;

static box_cmd_t *befehls_anker;
static box_cmd_t *last;
static BOOLEAN init_done;

extern BRETTINFO *br_ind;
extern long max_brettnr;

extern BYTE
batchB,
loginB;

extern WORD count_crW;
extern int logout_modus;

BYTE befehl_anzeigeB;

struct cmd_typ
{
	const char *befehl;
	BOOLEAN (*fn)(const char *);
}

sbef[] =
{
	{ "\?",				bef_fragezeichen },
	{ "ACL",			bef_acl },
/*	{ "ANTWORT",		bef_reply },*/
	{ "AUSLASTUNG",	bef_auslastung },
	{ "BRETT",			bef_brett },
	{ "BATCH",			batch },
	{ "BAUD",			bef_baud },
	{ "BETREFF",		bef_betreff },
	{ "BOXSTATISTIK",	bef_boxstatistik },
	{ "BRIEF",			bef_senden },
	{ "CHAT",			bef_chat },
	{ "CLSCREEN",		bef_clearscreen },
	{ "CRUNCH",			bef_crunch },
	{ "DATUM",			bef_datum },
	{ "DOWNLOAD",		bef_download },
	{ "DUMP",	     	bef_dump },
	{ "EDIT",			bef_editieren },
	{ "EVENT",        bef_event },
	{ "EXTERN",			bef_extern_prg },
	{ "HILFE",			bef_hilfe },
	{ "HITLISTE",		bef_hitliste },
	{ "INFO",			bef_info },
	{ "INHALT",			bef_inhalt },
	{ "INSTALL",		install_vt },
	{ "KOPIEREN",		bef_kopieren },
	{ "KILL",			bef_kill },
	{ "LEAVE",			bef_leave },
	{ "LESEN",			bef_lesen },
	{ "LISTE",			bef_liste },
	{ "LOGOUT",			bef_logoff },
	{ "MEHR",			bef_mehr },
	{ "MIDNIGHT",		bef_midnight },
	{ "OFFLINE",		bef_offline },
	{ "PROTOKOLL",		bef_protokoll },
	{ "PASSWORT",		bef_passwort },
	{ "PAUSE",			bef_pause },
	{ "POSTFACH",		bef_postfach },
	{ "PP",				bef_programmfach },
	{ "PTINFO",			bef_proginfo },
	{ "PUTZ",			bef_putz },
	{ "RELOGIN",		bef_relogin },
/*	{ "REPLY",			bef_reply },*/
	{ "RESET",			bef_reset },
	{ "ROUTE",			bef_route },
	{ "SENDEN",			bef_senden },
	{ "SHOW",			bef_show },
	{ "STATUS",			bef_status },
	{ "SUCHEN",			bef_suchen },
	{ "SYSTEMSTATUS",	bef_sys_status },
	{ "TAUSCH",			bef_tausch },
#ifdef HAVE_LIBTCL
	{ "TCL",			bef_tcl },
#endif
	{ "TEILNEHMER",	bef_teilnehmer },
	{ "TEST",			bef_test },
	{ "TYPE",			bef_type },
	{ "USERANTRAG",	bef_userantrag },
	{ "UMKOPIEREN",	bef_umkopieren },
	{ "UMLAUT",			bef_umlaut },
	{ "UNKILL",			bef_unkill },
	{ "UPLOAD",			bef_senden },
	{ "VERBOSE",		bef_verbose },
	{ "VERSION",		bef_version },
	{ "VT",				bef_vt },
	{ "ZEIT",			bef_zeit },
	{ "ZUGRIFF",		bef_zugriff },
};

/*
 * Aus irgendeinem unerfindlichen Grund hat gcc gemeint, absolut miesen
 * code erzeugen zu drfen. Die Inlines verhindern das.
 */
static TOM_ATTRIB_INLINE void befehl_anhaengen (box_cmd_t * new);
static TOM_ATTRIB_INLINE BOOLEAN 
	parse_bool (char s, char c) TOM_ATTRIB_CONST;
static void befehl_do_init(void);

static TOM_ATTRIB_INLINE void
befehl_anhaengen (box_cmd_t * new)
{
	if (!new)
		return;
	if (!befehls_anker)
		befehls_anker = new;
	else
		last->next = new;
	last = new;
	new->next = NULL;
}

box_cmd_t *
befehl_get_anker (void)
{
	if (!init_done)
		befehl_do_init ();
	return befehls_anker;
}

static TOM_ATTRIB_INLINE BOOLEAN 
parse_bool (char s, char c)
{
	char d = tolower (s);

	if (d == 'j' || d == 't' || d == 'y' || isdigit (s) || (c && d == c))
		return TRUE;
	return FALSE;
}

static void 
befehl_do_init(void)
{
	char datei[PATH_MAX];
	FILE *f;
	char buf[1024];	
	char *p;
	box_cmd_t *new=NULL;
	int line=0;
	BOOLEAN fehler_gemeldet=FALSE;
	box_cmd_t leer_cmd=
	{
		/* name.ext,para.ext, name.int,para.int, optionen */
		NULL,NULL,NULL,NULL,NULL,
		/* 10 Level */
		{ FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE,FALSE}, 
		/* systembefehl, auch_chef, nur_chef, nur_zahler, fragezeichen, */
		FALSE,FALSE,FALSE,FALSE,FALSE,
		/* fragezeichen,hilfe,ist_relogin, acl,acltype,para_hint, next */
		   FALSE,       FALSE,FALSE,       NULL,NULL,  0,         NULL
	};

	if (init_done)
		return;
	init_done=TRUE;

	/* Portspezifische befehle.cnf laden */
	f=NULL;
	if (tom_port_config_dir)
	{
		sprintf(datei,"%s%s",tom_port_config_dir,"befehle.cnf");
		f=fopen(datei,"r");
	}
	if (f==NULL)
	{
		sprintf(datei,"%s%s",box_conf.config_dir,"befehle.cnf");
		f=fopen(datei,"r");
	}
	if (!f)
	{
		syslog(LOG_NOTICE,"Fehler: keine befehle.cnf: %s",strerror(errno));
		/* wenn kein User online ist, oder nur ein Autotauscher, knnen
		 * wir prchtig damit leben.
		 */
		return;
	}
#define MELDE_FEHLER(meldung) do { \
	fprintf(stderr,"befehle.cnf, Zeile %d, %s\n", line, meldung); \
	if (!fehler_gemeldet) syslog(LOG_NOTICE,"befehle.cnf, Z. %d: %s", \
		line, meldung); \
	fehler_gemeldet=TRUE; \
	continue; \
} while(0)
	while (fgets(buf,sizeof(buf),f))
	{
		line++;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		if (*buf=='#')
			continue;
		p=strchr(buf,'|');
		if (p)
		{
			if (new)
				befehl_anhaengen(new);
			new=XMALLOC(sizeof(box_cmd_t));
			*new=leer_cmd;

			p=strtok(buf,"|");
			if (tolower(*p)=='s')
				new->systembefehl=TRUE;
			else
				new->systembefehl=FALSE;

			new->name_ext=strtok(NULL,"/");
			if (!new->name_ext)
			{
				free(new);
				new=NULL;
				MELDE_FEHLER("externer Name nicht vorhanden");
			}
			new->name_ext=XSTRDUP(strtrim(new->name_ext));

			p=strtok(NULL,"|");
			if (p)
				strtrim(p);
			if (p && *p)
				new->para_ext=XSTRDUP(strtrim(p));
			else if (!p)
				continue;

			p=strtok(NULL,"/");
			if (p)
				strtrim(p);
			if (p && *p)
				new->name_int=XSTRDUP(strtrim(p));
			else if (!p)
				continue;

			if (!stricmp(new->name_int,"RELOGIN"))
				new->ist_relogin_cmd=TRUE;

			p=strtok(NULL,"|");
			if (p)
				strtrim(p);
			if (p && *p)
				new->para_int=XSTRDUP(strtrim(p));
			else if (!p)
				continue;

			p=strtok(NULL,"|");
			if (p)
				strtrim(p);
			else
				continue;
			while (p && *p)
			{
				if (isdigit(*p))
					new->level[*p-'0']=TRUE;
				else if (*p=='B')
					new->auch_chef=TRUE;
				else if (*p=='b')
					new->nur_chef=TRUE;
				p++;
			}

			p=strtok(NULL,"|");
			if (!p)
				continue;
			strtrim(p);
			if (*p=='$')
				new->nur_zahler=TRUE;

			p=strtok(NULL,"|");
			if (!p)
				continue;
			strtrim(p);
			if (*p=='?')
				new->fragezeichen=TRUE;

			p=strtok(NULL,"|");
			if (!p)
				continue;
			strtrim(p);
			if (tolower(*p)=='b')
				new->batch_freigabe=TRUE;

			p=strtok(NULL,"|");
			if (!p)
				continue;
			strtrim(p);
			if (tolower(*p)=='b')
				new->hilfe=TRUE;

			p=strtok(NULL,"|");
			if (!p)
				continue;
			strtrim(p);
			if (strpbrk(p,"XxBb"))
				new->parameter_hint|=CMDPARA_BRETT;
			else if (strpbrk(p,"uU"))
				new->parameter_hint|=CMDPARA_USER;

			p=strtok(NULL,"|");
			if (!p)
				continue;
			strtrim(p);
			if (*p)
				new->optionen=XSTRDUP(p);
		}
		else 
		{
			char *name;
			char *inhalt;
			inhalt=strchr(buf,'=');
			if (!inhalt)
				MELDE_FEHLER("Syntaxfehler");
			*inhalt++=0;
			strtrim(inhalt);
			name=strtrim(buf);
			if (!stricmp(name,"befehl"))
			{
				char *para_p;
				befehl_anhaengen(new);
				new=XMALLOC(sizeof(box_cmd_t));
				para_p=strpbrk(inhalt,"\t ");
				if (!para_p)
					new->name_ext=XSTRDUP(inhalt);
				else
				{
					*para_p++='\0';
					strtrim(para_p);
					strtrim(inhalt);
					new->name_ext=XSTRDUP(inhalt);
					new->para_ext=XSTRDUP(para_p);
				}
			}
			else if (!new)
			{
				MELDE_FEHLER("Strukturfehler");
			}
			else if (!stricmp(name,"intern"))
			{
				char *para_p;
				para_p=strpbrk(inhalt,"\t ");
				if (!para_p)
					new->name_int=XSTRDUP(inhalt);
				else
				{
					*para_p++='\0';
					strtrim(para_p);
					strtrim(inhalt);
					new->name_int=XSTRDUP(inhalt);
					new->para_int=XSTRDUP(para_p);

					if (!stricmp(new->name_int,"RELOGIN"))
						new->ist_relogin_cmd=TRUE;
				}
			}
			else if (!stricmp(name,"optionen"))
				new->optionen=XSTRDUP(inhalt);
			else if (!stricmp(name,"level"))
			{
				char *para_p=inhalt;
				while(*para_p)
				{
					if (isdigit(*para_p))
						new->level[*para_p-'0']=TRUE;
					else if (*para_p=='B')
						new->auch_chef=TRUE;
					else if (*para_p=='b')
						new->nur_chef=TRUE;
					para_p++;
				}
			}
			else if (!stricmp(name,"system") || !stricmp(name,"systembefehl"))
				new->systembefehl=parse_bool(*inhalt,'\0');
			else if (!stricmp(name,"chef"))
			{
				if (0==stricmp(inhalt,"nur"))
					new->nur_chef=TRUE;
				else if (0==stricmp(inhalt,"auch"))
					new->auch_chef=TRUE;
				else if (*inhalt=='B')
					new->auch_chef=TRUE;
				else if (*inhalt=='b')
					new->nur_chef=TRUE;
			}
			else if (!stricmp(name,"zahler"))
				new->nur_zahler=parse_bool(*inhalt,'$');
			else if (!stricmp(name,"batch"))
				new->batch_freigabe=parse_bool(*inhalt,'B');
			else if (!stricmp(name,"hilfe"))
				new->hilfe=parse_bool(*inhalt,'H');
			else if (!stricmp(name,"?") || !stricmp(name,"fragezeichen"))
				new->fragezeichen=parse_bool(*inhalt,'?');
			else if (!stricmp(name,"brettpara") || !stricmp(name,"brettparameter"))
			{
				if (parse_bool(*inhalt,'B'))
					new->parameter_hint|=CMDPARA_BRETT;
			}
			else if (!stricmp(name,"userpara") || !stricmp(name,"userparameter"))
			{
				if (parse_bool(*inhalt,'U'))
					new->parameter_hint|=CMDPARA_USER;
			}
			else if (!stricmp(name,"acl"))
				new->acl=XSTRDUP(inhalt);
			else if (!stricmp(name,"acltype"))
				new->acltype=XSTRDUP(inhalt);
			else
				MELDE_FEHLER("unbekannte Variable");
		}
	}
	befehl_anhaengen(new);
	fclose(f);

} /* read_befehle */



/* ========================  Befehls - Routinen  ======================== */

#define ALLOWED_ALT(bef,lv) \
	(bef->level[lv] \
	|| (bef->auch_chef && user.nummer == akt_brett.grp_chef) \
	|| (akt_anruf.ist_relogin && box_conf.relogin_for_relogins \
		&& !stricmp(akt->name_int,"RELOGIN")))


/*
	17.11.92 Frank, zwar weitgehend auf real C umgestellt
						 aber besser neu schreiben, dann evtl. mit Cursor/History
	06.03.93 Frank, Command completion funktionsfhig, dito interne Kommando-
				       vervollstndigung
	14.02.94 Frank, optimiert
	14.02.94 Frank, mit Cursortastenerkennung begonnen
*/
char *befehl_eingabe(const char *prompt)
{
	char *returnstring=NULL;
	static char *history = NULL;
	static int hist_akt_line = 0;
	static int hist_anz_lines = 0;
	static USERNR hist_last_user = 0;
	int zeichen;
	size_t anz = 0;
	WORD lvl;				/* aktueller Userlevel */
	BOOLEAN brettexpansion=FALSE;
	char befehl[80]="";

	/* Speicher fr history anfordern */
	if (!history)
	{
		history = (char *) malloc(80 * HISTORY_LINES);
		if (history)
		{
			hist_anz_lines = HISTORY_LINES;
			memset(history,0,80 * HISTORY_LINES);
		}
		hist_akt_line = 0;
	}

	if (!init_done)
		befehl_do_init();

	/* bei Userwechsel history leeren */
	/* ggf. kann spter eine userspezifische history geladen werden */
	if (history)
	{
		if (hist_last_user != user.nummer)
		{
			hist_akt_line = 0;
			hist_anz_lines = 0;
			hist_last_user = user.nummer;
		}
	}
	strcpy(history + 80*hist_akt_line, "");

	count_crW = 0;

	lvl = level();

#ifdef WANT_READLINE
	if (user.flags & UB_FLAG_READLINE)
	{
		returnstring=do_readline(prompt,'C');
		if (!returnstring)
			return NULL;
		count_crW = 0;

		if (loginB && user.protokoll)
			write_info_protokoll(INFOPROT_TEXT,"",returnstring);
		return returnstring;
	}
#endif
	mboutput(prompt);
	do
	{
		zeichen = mbkey();
		if (zeichen < 256)
			zeichen = toupper(zeichen);

		/* history? */
		if (zeichen == C_UP || zeichen == C_DOWN)
		{
			box_cmd_t *akt;
			char *p;
			size_t l;

			if (zeichen == C_UP)
			{
				if (!hist_anz_lines || !hist_akt_line)
				{
					userglocke();
					continue;
				}
				/* aktuelle Zeile retten */
				if (hist_akt_line==hist_anz_lines)
					strcpy(history+hist_anz_lines*80,befehl);
				hist_akt_line--;
			}
			else
			{
				if (hist_akt_line >= hist_anz_lines)
				{
					userglocke();
					continue;
				}
				hist_akt_line++;
			}
			strcpy(befehl,history + 80*hist_akt_line);

			while (anz--)
				mboutput("\b \b");

			anz = strlen(befehl);
			zeichen = 0;

			mboutput(befehl);

			p=strchr(befehl,' ');
			if (!p)
				continue;
			l=p-befehl;

			brettexpansion=FALSE;
			for (akt=befehls_anker; akt ; akt=akt->next)
			{
				if (!COMMAND_ALLOWED(akt,lvl))
					continue;
				if (strnicmp(akt->name_ext,befehl,l))
					continue;
				brettexpansion=akt->parameter_hint & CMDPARA_BRETT;
				break;
			}
			continue;
		}

		if (zeichen>=128 && !strchr("",zeichen))
			continue;
		if (zeichen < 32 && !strchr("\r\n\b\t",zeichen))
			continue;

		if (zeichen == '\b' || zeichen==127)
		{
			if (anz)
			{
				befehl[--anz] = 0;
				mboutput("\b \b");
				if (!strchr(befehl,' '))
				{
					brettexpansion=FALSE;
				}
			}
			else
				userglocke();
			continue;
		}

		if (anz > 75)
		{
			userglocke();
			continue;
		}

		if (!user.autobefehl)
		{
			if (zeichen == '\r' || zeichen == '\n')
				mboutput("\r\n");
			else if (zeichen && (anz < 70))
			{
				befehl[anz++] = zeichen;
				befehl[anz  ] = 0;
				mboutput(befehl+anz-1);
			}
			continue;
		}

		if (isspace(zeichen))
		{
			if (!strchr(befehl,' '))
			{
				/* Befehlsvervollstndigung */
				box_cmd_t *akt;
				box_cmd_t *found=NULL;
				size_t l=strlen(befehl);
				/* erster durchlauf: Genau die Uppercase-Letters im Befehls-
				 * namen vergleichen 
				 */
				for (akt=befehls_anker; akt ; akt=akt->next)
				{
					char *p;
					size_t uc=0;
					if (!COMMAND_ALLOWED(akt,lvl))
						continue;
					for (p=akt->name_ext;*p && !islower(*p);p++)
						uc++;
					if (uc>l)
						continue;
					if (strncmp(akt->name_ext,befehl,l))
						continue;
					found=akt;
					break;
				}
				if (!found)
				{
					/* zweiter durchlauf: alles vergleichen */
					for (akt=befehls_anker; akt ; akt=akt->next)
					{
						if (!COMMAND_ALLOWED(akt,lvl))
							continue;
						if (strnicmp(akt->name_ext,befehl,l))
							continue;
						found=akt;
						break;
					}
				}
				if (found)
				{
					char buf[128];
					strcpy(befehl,akt->name_ext);
					strcpy(buf,akt->name_ext+l);
					mboutput(strupr(buf));
					anz+=strlen(buf);
					brettexpansion=found->parameter_hint & CMDPARA_BRETT;
				}
			}
			else if (brettexpansion)
			{	
				char *p;
				char *q;
				p=strchr(befehl,' ');
				do 
				{
					/* Leerzeichen bergehen */
					while (isspace(*p))
						p++;
					q=strchr(p,' ');
					if (q)
						p=q;
				} while (p && *p==' ');
				if (p && *p)
				{
					/* Dann Beginn eines Brettnamens */
					BRETTNR i;
					UWORD zahl=0;
					BRETTNR found=0;
					char block[128]="";
					size_t l;
					l=strlen(p);

					for (i = 1; i < max_brettnr; i++)
					{
						if (!br_ind[i].brett)
							continue;
						if (!strnicmp(br_ind[i].brett->name,p,l))
						{
							found=i;
							if (zahl==0)
								strcpy(block,br_ind[i].brett->name+l);
							else
							{
								char *s,*t;
								for (t=block,s=br_ind[i].brett->name+l;
									*t && tolower(*t)==tolower(*s);
									t++)
									s++;
								*t='\0';
							}
							zahl++;
						}
					}
					if (zahl==1)
					{
						char buf[128];
						strcpy(buf,br_ind[found].brett->name+l);
						strupr(buf);
						mboutput(buf);
						strcat(befehl,buf);
						anz+=strlen(buf);
					}
					else if (zahl>1)
					{
						/* mglichst weitgehend expandieren */
						if (*block)
						{
							strupr(block);
							mboutput(block);
							strcat(befehl,block);
							anz+=strlen(block);
							zeichen='\0'; /* nicht weitersetzen */
						}
						else if (zeichen=='\t')
						{
							/* matches ausgeben */
							char tmp[100];
							char *s;
							sprintf(tmp,"%s*",p);
							bef_brett(tmp);
							s=prepare_prompt();
							mboutput(s);
							mboutput(befehl);
							zeichen='\0'; /* nicht weitersetzen */
						}
					}
					/*
					 * else nicht meckern, wegen -L, 150 und so.
					 */
				}
			}
			if (zeichen == ' ' || zeichen == '\t')
			{
				befehl[anz++] = ' ';
				befehl[anz  ] = 0;
				mboutput(befehl+anz-1);
			}
			else if (isspace(zeichen)) /* bei \0 nicht ausgeben */
				mboutput("\r\n");
		}
#ifdef VORLAUTE_EXPANSION
		else	/* restliche Zeichen */
		{
			box_cmd_t *found=NULL;
			if (!befehl_rest_flagB && !strchr(befehl,' '))
			{
				box_cmd_t *akt;

				char test[128];
				size_t len;

				int zahl=0;

				sprintf(test,"%s%c",befehl,zeichen);
				len=strlen(test);

				for (akt=befehls_anker; akt; akt=akt->next)
				{
					if (!COMMAND_ALLOWED(akt,lvl))
						continue;

					if (!strnicmp(akt->name_ext,test,len))
					{
						found=akt;
						zahl++;
					}
				}
				if (zahl!=1)
					found=NULL;
			}
			if (found)
			{
				char buf[128];
				size_t l=strlen(befehl);
				strcpy(buf,found->name_ext+l);
				strcat(buf," ");
				mboutput(strupr(buf));
				anz+=strlen(buf);
				strcat(befehl,buf);
				brettexpansion=found->parameter_hint & CMDPARA_BRETT;
				befehl_rest_flagB = TRUE;
			}
			else
			{
				befehl[anz++] = zeichen;
				befehl[anz  ] = 0;
				mboutput(befehl+anz-1);
			}
		}
#else
		else	/* restliche Zeichen */
		{
			befehl[anz++] = zeichen;
			befehl[anz  ] = 0;
			mboutput(befehl+anz-1);
		}
#endif
	}
	while (zeichen != '\r' && zeichen != '\n' && logout_modus != 1);
	returnstring=XSTRDUP(befehl);

	count_crW = 0;

	if (loginB && user.protokoll)
		write_info_protokoll(INFOPROT_TEXT,"",returnstring);

	if (history)
	{
		/* aktuelle Eingabe von letzter verschieden? */
		if (!hist_anz_lines
		|| strcmp(returnstring, history+(hist_anz_lines - 1)*80))
		{
			strcpy(history+hist_anz_lines*80,returnstring);
			hist_anz_lines++;

			if (hist_anz_lines >= HISTORY_LINES)
			{
				hist_anz_lines--;
				memcpy(history,history+80,(HISTORY_LINES - 1) * 80);
				history[(HISTORY_LINES - 1) * 80] = 0;
			}
			else
				hist_akt_line++;
		}
		if (hist_akt_line <= hist_anz_lines)
			hist_akt_line = hist_anz_lines; /* war +1 */
	}

	return returnstring;
} /* befehl_eingabe */

/*
	16.11.92 Frank, auf C Strings umgeschrieben
	1.6.94, Uwe, "150 -> brett 150"
*/
box_cmd_t *befehl_find(const char *cmd, const char *par)
{
	WORD lvl;
	size_t cmd_len;
	box_cmd_t *akt;
	lvl = level();

	if (!init_done)
		befehl_do_init();

	cmd_len = strlen(cmd);

	for (akt=befehls_anker; akt!=NULL ; akt=akt->next)
	{
		if (strnicmp(akt->name_ext,cmd,cmd_len))
			continue;
		/* Falls kein Parameter angegeben ist, 
		 * und der Befehl in der Liste einen Parameter ungleich dem 
		 * Wildcardparameter hat, dann ist das nicht unser Befehl
		 */
		if ((!par || !*par) && akt->para_ext && !strstr(akt->para_ext,"{*}"))
			continue;
		if (par && *par)
		{
			/* Benutzer gab Parameter an */
			if (!akt->para_ext)	
				continue; /* Kein Para in der Liste -> nicht unser Befehl */

			/* Ist der Parameter korrekt? */
			if (!stricmp(par,akt->para_ext) || strstr(akt->para_ext,"{*}"))
			{
				const char *p=strstr(akt->para_ext,"{*}");
				if (p && p!=akt->para_ext) {
					if (0!=strnicmp(akt->para_ext,par,p-akt->para_ext))
						continue;
				}
				/* Ja */
				if (COMMAND_ALLOWED(akt,lvl))
					return akt;
				return NULL;
			}
			continue;
		}
		if (COMMAND_ALLOWED(akt,lvl))
			return akt;
	}
	return NULL;
} /* befehl_find */


/*
	16.11.92 Frank, auf C Strings umgeschrieben
	1.6.94, Uwe, "150 -> brett 150"
*/
void befehl_ausfuehrung(char *cmd)
{
	WORD lvl;
	unsigned int iW;
	char *par;
	char *tmpparstr=NULL;
	char *p;
	char str[256];
	box_cmd_t *found=NULL;

	lvl = level();

	/* 150 -> BRETT 150 Hack/Feature */
	strtol(cmd,&p,10);
	if (p && p != cmd && !*p)
	{
		strcpy(str,cmd);
		strcpy(cmd,"BRETT ");
		strcat(cmd,str);
	}

	/* Eingabe in Befehl und Parameter aufteilen */
	strtrim(cmd);
	while (*cmd == ' ')
		cmd++;
	par = NULL;
	if (strchr(cmd,' '))
	{
		par = strchr(cmd,' ');
		*par++ = 0;
		strtrim(par);
	}

	if (par && *cmd && (!strcmp(par,"HILFE") || !strcmp(par,"?")))
	{
		char *hlp;
		hlp = cmd;
		cmd = par;
		par = hlp;
	}
	found=befehl_find(cmd, par ? par : "");
	if (!found)
	{
		if (batchB)
			mbprintf(" BATCH-Befehl: <%s> unbekannt  ( bersicht unter 'BATch HILFE' )\r\n\n",cmd);
		else
			mboutput(" Befehlsliste mit '\?\'  /  Erklrungen mit 'HILFE'\r\n\n");
		return;
	}

	if (found->nur_zahler && !user.bezahlt)
	{
		mbprintf(" Befehl <%s> nur fr zahlende Benutzer\r\n\n",cmd);
		return;
	}
	if (batchB && !found->batch_freigabe)
	{
		mbprintf(" Befehl <%s> im Batch nicht erlaubt\r\n\n",found->name_ext);
		return;
	}

	if (!found->para_int)
		par=NULL;
	else if (!strstr(found->para_int,"{*}"))
	{
		par=found->para_int;
	}
	else
	{
		char *pr;
		char *pw;
			/* "*2", um zwei {*} zu ermglichen - Fehlerquelle, aber 
			 * sollte nicht auftreten */
		tmpparstr=XMALLOC(
			strlen(found->para_int) + (par ? strlen(par) : 0 ) * 2 + 129);
		pr=found->para_int;
		pw=tmpparstr;
		while (*pr)
		{
			if (!strncmp(pr,"{*}",3))
			{
				strcpy(pw,par ? par : "");
				pw+=strlen(pw);
				pr+=3;
			}
			else
				*pw++=*pr++;
		}
		par=tmpparstr;
	}

	if (befehl_anzeigeB)
		mbprintf("\r\n Befehls-Zuordnung: [%s] / [%s]  ->  [%s] / [%s]\r\n\n",
				found->name_ext,
				found->para_ext ? found->para_ext : "",
				cmd,par);

	for (iW = 0; iW < sizeof(sbef)/sizeof(struct cmd_typ); iW++)
	{
		if (!stricmp(found->name_int,sbef[iW].befehl))
		{
			char *modpara=NULL;
			const char *realpara=NULL;
			BOOLEAN errorflag=FALSE;
			BOOLEAN done=FALSE;

			realpara=par ? par : "";
#ifdef HAVE_LIBTCL
			TTCL_HOOK(found->name_int,realpara,modpara,
				errorflag=TRUE;, 
				done=TRUE)
			if (realpara==NULL)
				realpara=""; /* Befehle brauchen das so ... */
#endif
			if (errorflag)
				mboutput("Warnung: Fehler aufgetreten bei Hook-Ausfhrung\r\n");
			if (!done)
				(void) (sbef[iW].fn)(realpara);
			if (tmpparstr)
				free(tmpparstr);
			if (modpara)
				free(modpara);
			carrier_checkpoint(TRUE);
			return;
		}
	}

	free(tmpparstr);
	carrier_checkpoint(TRUE);
	mbprintf(" Befehl <%s> unbekannt\r\n\n",cmd);

} /* befehl_ausfuehrung */


/* 
	Prft, ob der Befehl fr den aktuellen User erlaubt ist.
	In:
	    for_group -> Welches Brett betroffen ist. NULL fr aktuelles Brett.
	    cmd       -> Der Kommandostring
	    para      -> eventueller Parameter. Kein Para -> "" oder NULL.
	Out:
	    TRUE  -> erlaubt
	    FALSE -> Pech.
	Uwe, 1.12.93, heavily rehacked.
*/
BOOLEAN befehl_check_befehl(BRETT *for_group,const char *cmd,const char *para)
{
	size_t paralen;
	size_t cmdlen;

	box_cmd_t *akt;

	if (para)
		paralen = strlen(para);
	else 
		paralen = 0;
	cmdlen = strlen(cmd);

	if (!for_group)
		for_group = &akt_brett;

	if (!init_done)
		befehl_do_init();

	for (akt=befehls_anker;akt;akt=akt->next)
	{
		if (!strnicmp(cmd,akt->name_int,cmdlen))
		{
			/* Befehl gefunden */
			BOOLEAN ok = TRUE;

			if (para && *para)
			{
				/* Wenn ein Parameter angegeben ist */
				if (strnicmp(para,akt->para_int,cmdlen)
					&& !strstr(akt->para_int,"{*}"))
					ok = FALSE;
			}
			/* Ist der Befehl fr den Level des Users freigegeben? */
			if (ok && !akt->level[user.level])
			{
				if (!akt->auch_chef 
					|| for_group->grp_chef!=user.nummer)
					ok = FALSE;
			}
			/* Ist der Befehl nur fr Brettverwalter erlaubt? */
			if (ok && akt->nur_chef
				&& user.nummer != for_group->grp_chef && user.level!=9)
				ok = FALSE;
			if (ok && akt->nur_zahler && !user.bezahlt)
				ok = FALSE;
			if (ok)
				return TRUE;
		}
	}
	return FALSE;
}



/*
	lrange: "1,2,5-10,6-2" -> 1,2,5,6,7,8,9,10,6,5,4,3,2 usw.

	(c) bei Frank Baumgart
	Version 1.0 vom 10.11.1993

	25.04.94 Frank, "*" wird zu "1-9999" expandiert, Aufrufer mu abbrechen
*/
int lrange(const char *vstring,long *wert)
{
	static const char *start;
	static char *next;
	static long von,bis;			/* Start- und Endnummer */
	static int bereich;			/* Flag, ob Bereichsangabe */
	static int vorwaerts;

	if (vstring)
	{
		start = vstring;
		if (!strcmp(vstring,"*"))
		{
			von = 0;					/* wird unten auf 1 erhht */
			bis = 9999;
			bereich = TRUE;
			vorwaerts = TRUE;
		}
		else
		{
			von = 0;
			bis = 0;
			bereich = FALSE;
		}
	}

	while (isspace(*start))
		start++;


	if (bereich)
	{
		if (vorwaerts && (von < bis))
		{
			*wert = ++von;

			return TRUE;
		}
		if (!vorwaerts && (von > bis))
		{
			*wert = --von;

			return TRUE;
		}
	}

	if (!*start)
		return 0;

	if (isdigit(*start))
	{
		*wert = strtol(start,&next,10);
		start = next;
		von = *wert;

		return TRUE;
	}

	if (*start == '-')
	{
		bereich = TRUE;
		start++;
		while (isspace(*start))
			start++;
		if (isdigit(*start))
			bis = strtol(start,&next,10);
		else if (!*start)
			bis=9999;
		else
			return FALSE;

		if (next)
			start = next;
		else
			start = "";

		if (von > bis)
		{
			vorwaerts = FALSE;
			*wert = --von;
			return TRUE;
		}
		if (von < bis)
		{
			vorwaerts = TRUE;
			*wert = ++von;
			return TRUE;
		}

		return lrange(NULL,wert);
	}

	if (*start == ',')
	{
		start++;
		von = bis = 0;

		return lrange(NULL,wert);
	}

	/* wenn man hier ankommt, gab es einen Fehler in Eingabe oder Parser */

	return FALSE;

} /* lrange */



/*
	larea: "1,2,5-10,6-2" -> "1","2","5-10","2-6" usw.

	(c) bei Frank Baumgart
	Version 1.0 vom 21.01.1997

	"*" wird zu "1-99999" expandiert, Aufrufer mu abbrechen
*/
int larea(const char *vstring,long *von,long *bis)
{
	static const char *start;
	static char *next;
	char *p_rng,*p_sep;

	if (vstring)
	{
		start = vstring;
		if (!strcmp(vstring,"*"))
		{
			*von = 1;
			*bis = 99999;
			start++;
			return TRUE;
		}
	}

	while (isspace(*start))
		start++;

	if (!*start)
		return 0;

	p_rng = strchr(start,'-');
	p_sep = strpbrk(start," ,");
	if ((p_rng && !p_sep)
	|| (p_rng && p_sep && p_rng < p_sep))
	{
		if (isdigit(*start))
			*von = atol(start);
		else
			*von = 1;
		if (!*von)
			*von = 1;
		if (!isdigit(p_rng[1]))
		{
			*bis = 99999;
			start += strlen(start);
		}
		else
		{
			*bis = strtol(p_rng+1,&next,10);
			start = next;
		}
		if (*von > *bis)
		{
			long hlp;
			hlp = *von;
			*von = *bis;
			*bis = hlp;
		}
	}
	else
	{
		*von = *bis = strtol(start,&next,10);
		start = next;
	}

	if (*start == ',')
		start++;

	return TRUE;

} /* larea */
