/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <errno.h>
#include <unistd.h>
#include <limits.h>

#ifdef HAVE_MMAP
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <fcntl.h>
#endif

#include "cslib.h"
#include "libquark.h"
#include "long-options.h"
#include "itbcompile.h"

static int option_version = 0;
static int option_verbose = 0;
static int option_list = 0;
static int option_freetext = 0;
static int option_from = 0;
static int option_to = 0;
static size_t option_umbruch = 0;
static size_t option_split_after =0;

static char *outputfilename = NULL;

static int keine_umlautwandlung;
#define LINEEND "\r\n"
#define LINEENDSIZE (sizeof(LINEEND)-1)

#ifdef HAVE_MMAP
caddr_t mm_addr;
caddr_t mm_read_addr;
size_t mm_size;
size_t mm_to_read=0;
int is_mmapped=0;
#endif

#define DEBUG(what) do { what ; } while(0);
#undef DEBUG
#define DEBUG(what) 

const char *program_name="cstrans";

struct
{
	char *start;
	char *next;
	size_t size; /* Nicht Laenge (next-start), sondern Puffergroesse */
	size_t calc_len; /* : nicht gezhlt, CR/LF als 1 Zeichen */
} bodybuf;

struct
{
	char *start;
	char *next;
	size_t size; /* Nicht Laenge (next-start), sondern Puffergroesse */
	long parts;
} headbuf;
char message_id[512];


void work (FILE * in, FILE * out);
void umbruchausgabe (FILE * out, const char *zeile, size_t l);
void usage (int exitcode);
void help (void);
void die (char *);
void put_headline(const char *s, size_t sl);
void put_bodyline(FILE *out, const char *s,size_t l);
void work_splitted(FILE *outf,int finish);

#ifdef HAVE_MMAP
size_t my_getline(char **inbuf, char delim);
#endif

int is_native=0;


void work_splitted(FILE *outf,int finish)
{
	if (bodybuf.start==bodybuf.next && headbuf.parts)
		return;
	/* da Spezialnachrichten leer sein knnen und sowieso nicht 
	 * gesplittet werden [hoffentlich], werden ihre IDs vorher
	 * ausgegeben 
	 */
	if (strchr(message_id,'@')) {
		if (finish && headbuf.parts==0) {
DEBUG(fprintf(outf,"!# by work-splitted #1"LINEEND);)
			fprintf(outf,"#%s"LINEEND,message_id);
		} else if (headbuf.parts==0) {
DEBUG(fprintf(outf,"!# by work-splitted #2"LINEEND);)
			fprintf(outf,"#%s"LINEEND,message_id);
			headbuf.parts++;
		} else {
DEBUG(fprintf(outf,"!# by work-splitted #3"LINEEND);)
			fprintf(outf,"#%s:%ld"LINEEND,message_id,++headbuf.parts);
		}
		fwrite(headbuf.start,1,headbuf.next-headbuf.start,outf);
		if (headbuf.parts>1 && *headbuf.start!='-' && !strstr(headbuf.start,"\r\n-")
			&& (*headbuf.start=='A' || strstr(headbuf.start,"\r\nG")))
			fprintf(outf,"-%s"LINEEND,message_id);
	} else if (headbuf.parts++==0) {
		fwrite(headbuf.start,1,headbuf.next-headbuf.start,outf);
	}
	/* Gute Idee, aber bei PMs gar nicht gut: Teil 2..n bekommen von
	 * der MAUS "?diese Nachricht ist nicht an Sie". Drfte eigentlich
	 * eher als MAUS-Bug gelten.
	 */
	if (bodybuf.start) /* NULL bei Statusmails usw. */
		fwrite(bodybuf.start,1,bodybuf.next-bodybuf.start,outf);
}

void put_headline(const char *line, size_t sl)
{
	size_t physlen;
	physlen=sl+LINEENDSIZE+1; /* +1 f. \0 */
	if (!headbuf.start || headbuf.next-headbuf.start+physlen>headbuf.size)
	{
		size_t newsize=headbuf.size*2;
		size_t offset=headbuf.next-headbuf.start;
		if (newsize<1024)
			newsize=1024;
		if (physlen+offset>newsize)
			newsize=physlen+offset;
		headbuf.start=XREALLOC(headbuf.start,newsize);
		headbuf.next=headbuf.start+offset;
		headbuf.size=newsize;
	}
	memcpy(headbuf.next,line,sl);
	headbuf.next+=sl;
	/* CR/LF ergnzen */
	*headbuf.next++='\r';
	*headbuf.next++='\n';
	/* sieht im Debugger besser aus und macht es mglich, nach
	 * \r\n- zu suchen
	 */
	*headbuf.next='\0';
}

void put_bodyline(FILE *outf, const char *line, size_t sl)
{
	size_t physlen;
	size_t calc_len=0;

	physlen=sl+2+LINEENDSIZE; /* sl: Stringlnge. 1: \0. 1: ":" */
	calc_len=sl+1;
	if (bodybuf.start)
	{
		if (option_split_after && bodybuf.calc_len + calc_len > option_split_after)
		{
			work_splitted(outf,0);
			bodybuf.next=bodybuf.start;
			*bodybuf.start=0;
			bodybuf.calc_len=0;
		}
	}
	if (!bodybuf.start || bodybuf.next-bodybuf.start+physlen>bodybuf.size)
	{
		size_t newsize=bodybuf.size*2;
		size_t offset=bodybuf.next-bodybuf.start;
		if (physlen+offset>newsize)
			newsize=physlen+offset;
		if (newsize <64*1024L)
			newsize=64*1024L;
		bodybuf.start=XREALLOC(bodybuf.start,newsize);
		bodybuf.next=bodybuf.start+offset;
		bodybuf.size=newsize;
	}
	*bodybuf.next=':';
	memcpy(bodybuf.next+1,line,sl);
	bodybuf.next+=sl+1;
	*bodybuf.next++='\r';
	*bodybuf.next++='\n';
	bodybuf.calc_len+=calc_len;
}


void 
umbruchausgabe (FILE * out, const char *zeile, size_t l)
{
	size_t restlen;
	const char *p;
	const char *pstart;

	restlen = l - 1;
	pstart = zeile + 1;

	do
	{
		int l1;

		p = pstart + option_umbruch;

		if (p >= zeile + l)
		{
			p = zeile + l;
			break;
		}

		while (*p != ' ' && *p != '\t' && p > pstart)
			p--;
		l1 = p - pstart;		/* Laenge Teilstck */
		if (!l1) /* kein Space gefunden? */
			l1=option_umbruch;
		put_bodyline(out, pstart,l1);
		pstart += l1;
		restlen -= l1;
		if (!restlen)
			return;
		/* Leerzeichen berspringen */
		while (*pstart == ' ' || *pstart == '\t')
		{
			restlen--;
			pstart++;
		}
	}
	while (restlen > (option_umbruch));
	put_bodyline(out, pstart,restlen);
}
#ifdef HAVE_MMAP
size_t
my_getline(char **inbuf, char delim)
{
	char *p;
	size_t ret;
	if (!mm_to_read)
	{
		*inbuf=NULL;
		return -1;
	}
	p=memchr(mm_read_addr,delim,mm_to_read);
	*inbuf=mm_read_addr;
	if (!p)
	{
		ret=mm_to_read;
		mm_read_addr=NULL;
	}
	else
	{
		ret=p-mm_read_addr+1;
		mm_read_addr=p+1;
	}
	mm_to_read-=ret;
	return ret;
}
#endif

void 
work (FILE * in, FILE * out)
{
	char *inbuf;
	char *outbuf;
	size_t inbuflen;
	size_t outbuflen;
	size_t l;
	const char *umbruchbuf = NULL;	/* keep gcc happy */
	char *p;
	int umbruch_erlaubt = 0;
	int do_unsplit=0;
	int got_m_zeile=0;
	int got_kludge_stop_line=0;
	char last_id[1024]="";
	int last_nr=1;
	char delim='\n';
	BOOLEAN first_line=TRUE;
	int header_umlautwandeln=1;

	/* Die sollte man nicht grer machen als die mir setvbuf 
	 * angelegten Puffer. Grund: Macintosh-Zeilenende-Hack (\r)
	 * mu rewind() aufrufen, und das funktioniert bei pipes
	 * nur, wenn der rewind innerhalb des Puffers bleibt.
	 */
	inbuflen=8192;
	inbuf=XMALLOC(inbuflen);
	outbuflen=16384;
	outbuf=XMALLOC(outbuflen);

/*
 * we *do* have getdelim in the lib-directory, but that 
 * implementation is slow (reads char by char. fgets 
 * usually work faster). So use getdelim only if we
 * get it from the system library.
 */
#ifndef HAVE_GETDELIM
#define getdelim(bufp,buflenp,delim,F) \
    (fgets (*(bufp),*(buflenp) , F)==NULL ? -1 : strlen(*(bufp)))
#endif

#define FN (getdelim(&inbuf,&inbuflen,delim,in))
#ifdef HAVE_MMAP
#define FN2 (is_mmapped ? my_getline(&inbuf,delim) : (FN))
#else
#define FN2 FN
#endif

	while ((l=(FN2))!=(size_t)-1)
	{
		p=inbuf+l-1;
		if (*p=='\n')
		{
			p--;
			l--;
			first_line=FALSE;
		} else if (first_line) {
			first_line=FALSE;
			if (memchr(inbuf,'\r',l))
			{
#ifdef HAVE_MMAP
				if (is_mmapped)
				{
					mm_to_read=mm_size;
					mm_read_addr=mm_addr;
					delim='\r';
					continue;
				}
#ifdef HAVE_GETDELIM
				else
#endif
#endif
#ifdef HAVE_GETDELIM
				if (0==fseek(in,0,SEEK_SET))
				{
					delim='\r';
					continue;
				}
#endif
			}
		}
		if (*p=='\r')
		{
			p--;
			l--;
		}
		/* M-Zeile beachten - dann keine Umlautwandlung im Body durchfhren */
		/* Header niemals umlautwandeln */
		if (l && !keine_umlautwandlung && *inbuf==':' && !got_m_zeile)
		{
			while (l*2+1>outbuflen) {
				outbuflen*=2;
				/* kein realloc - das kopiert unnoetig */
				free(outbuf);
				outbuf=XMALLOC(outbuflen);
			}
			if (is_native)
				l = block_to_foreign (outbuf, inbuf, outbuflen, l);
			else
				l = block_to_native (outbuf, inbuf, outbuflen, l);
			outbuf[l]=0;
			umbruchbuf = outbuf;
		}
		else
			umbruchbuf = inbuf;
		if (option_freetext)
		{
			if (l)
				fwrite (umbruchbuf, l, 1, out);
			fwrite (LINEEND, LINEENDSIZE,1,out);
			continue;
		}
		switch (l ? *umbruchbuf : '\0')
		{
		case '#':
			{
				char *dp;
				char *at;
				do_unsplit=0;
				at=memchr(umbruchbuf,'@',l);
				if (at && (dp=memchr(at,':',l-(at-umbruchbuf)))!=NULL)
				{
					char *next;
					long nr;
					nr=strtol(dp+1,&next,10);
					if (nr>0 && nr==last_nr+1)
					{
						if (0==strncmp(umbruchbuf+1,last_id,dp-umbruchbuf-1))
						{
							do_unsplit=1;
							last_nr=nr;
							continue;
						}
					}
				}
				else
				{
					memcpy(last_id,umbruchbuf+1,l-1);
					last_id[l-1]=0;
					last_nr=1;
				}

				/* 
				 * Wenn die letzte Nachricht "eventuell gesplittet werden mute",
				 * will sagen, wenn wir es nicht wuten, dann die Nachricht
				 * jetzt verarbeiten.
				 */
				work_splitted(out, 1);
				umbruch_erlaubt = 0;
				if (at)
					umbruch_erlaubt=1;
				*message_id=0;
				headbuf.next=headbuf.start;
				headbuf.parts=0;
				bodybuf.next=bodybuf.start;
				bodybuf.calc_len=0;
				got_m_zeile=0;
				header_umlautwandeln=1;

				/*
				 * Schau mer ma, ob diese Nachricht gesplittet werden mu:
				 */
				memcpy(message_id,umbruchbuf+1,l-1);
				message_id[l-1]=0;
				/* IDs von Spezialnachrichten jetzt ausgeben - die 
				 * haben ja moeglicherweise keinen Inhalt (leeres
				 * #LOG) und werden daher spaeter nicht immer
				 * ausgegeben 
				 */
				if (!strchr(message_id,'@')) {
					/* leere #LOG etc */ 
DEBUG(fprintf(out,"!# by work #1"LINEEND);)
					fprintf(out,"#%s\r\n",message_id);
				}
			}
			break;
		case 'M':
			got_m_zeile=1;
			got_kludge_stop_line=0;
			header_umlautwandeln=0;
			/* FALLTHROUGH */
		default:
			if (!do_unsplit)
			{
				put_headline(umbruchbuf,l);
			}
			break;
		case ':':
			if (header_umlautwandeln && !keine_umlautwandlung 
				&& headbuf.start)
			{
				size_t l1,l2;
				char *buf;
				l1=headbuf.next-headbuf.start;
				l2=l1*2+1;
				if (l2<headbuf.size)
					l2=headbuf.size;
				buf=XMALLOC(l2);
				if (is_native)
					l2 = block_to_foreign (buf, headbuf.start, l2, l1);
				else
					l2 = block_to_native (buf, headbuf.start, l2, l1);
				buf[l2]=0;
				free(headbuf.start);
				headbuf.start=buf;
				headbuf.next=buf+l2;
				header_umlautwandeln=0;
			}

			/*
			 * 
			 */
			if (umbruch_erlaubt && option_umbruch && l > option_umbruch
				&& (got_kludge_stop_line || !got_m_zeile))
				umbruchausgabe (out, umbruchbuf, l);
			else
				put_bodyline(out, umbruchbuf+1,l-1);
			break;
			if (got_m_zeile && !got_kludge_stop_line) {
				if (umbruchbuf[1]=='-' && !umbruchbuf[2]) {
					got_kludge_stop_line=1;
				}
			}
		}						/* switch */
	}
	work_splitted(out, 1);
	free(inbuf);
	free(outbuf);
}

void 
usage (int exitcode)
{
	if (exitcode) {
		fprintf (stderr, "Try `%s --help' for more information.\n",
				 program_name);
	} else {
		printf ("Usage: %s [OPTIONS] [FILES]\n",program_name);
		printf (" charset translator for maustausch files\n");
		printf ("\
  --help              display this help and exit\n\
  --version           output version information and exit\n\
  --from CS       -f  convert from charset CS to latin1\n\
  --to CS         -t  convert to charset CS from latin1\n\
  --output FI     -o  write output to FI (if converting stdin or one file)\n\
  --list          -l  list all known charsets\n\
  --linelength N  -L  break lines after N characters\n\
  --split-after N -s  split messages after N bytes\n\
\n\
short options use the same arguments as the long ones\n\
");
	}
	exit(exitcode);
}

void 
die (char *s)
{
	error(1,errno,s);
}

struct option opts[] =
{
	{"from", required_argument, NULL, 'f'},
	{"freetext", 0, NULL, 'F'},
	{"help", 0, NULL, 'h'},
	{"linelength", required_argument, NULL, 'L'},
	{"list", 0, &option_list, 'l'},
	{"output", required_argument, NULL, 'o'},
	{"split-after", required_argument, 0, 's'},
	{"to", required_argument, NULL, 't'},
	{"version", 0, NULL, 'v'},
	{NULL, 0, NULL, 0}
};

int 
main (int argc, char **argv)
{
	cstl_t *charset;
	char *charsetname=NULL;
	void *ibuf, *obuf;
	int option_index = 0;

	if (argv[0] && *argv[0])
		program_name = argv[0];		/* some silly environments don't provide this */

	parse_long_options (argc, argv, program_name, PACKAGE_VERSION, usage);

	if (argc == 1)
		usage (2);

	while (1)
	{
		int c;
		c = getopt_long (argc, argv, "Ff:t:hvL:lo:s:T:",
						 opts, &option_index);

		/* Detect the end of the options.   */
		if (c == -1)
			break;

		switch (c)
		{
		case 'f':
			option_from = 1;
			charsetname=optarg;
			break;
		case 't':
			option_to = 1;
			charsetname=optarg;
			break;
		case 'F':
			option_freetext = 1;
			break;
		case 'h':
			usage(0);
			break;
		case 'v':
			option_version = 1;
			break;
		case 'V':
			option_verbose = 1;
			break;
		case 'l':
			option_list = 1;
			break;
		case 'o':
			outputfilename = optarg;
			if (outputfilename == NULL)
				usage (2);
			break;
		case 'L':
			option_umbruch = (size_t) strtoul (optarg, NULL, 10);
			if (option_umbruch < 10)
			{
				fprintf (stderr, "%s: linelength too short\n", program_name);
				usage (2);
			}
			break;
		case 's':
			option_split_after = (size_t) strtoul (optarg, NULL, 10);
			if (option_split_after < 10)
			{
				fprintf (stderr, "%s: messagelength too short\n", program_name);
				usage (2);
			}
			break;
		case '?':
			usage (2);
			break;
		}
	}							/* while(1) */

	if (option_version)
	{
		printf("%s %s\n",program_name,VERSION);
		exit(0);
	}

	charset_init (printf);


	ibuf = malloc (32 * 1024);
	obuf = malloc (32 * 1024);

	if (option_list)
	{
		cst_list (1);
		exit (0);
	}

	if ((option_from == 0) == (option_to == 0) && option_umbruch == 0
		&& option_split_after==0)
	{
		fprintf (stderr, "use %s --help\n", program_name);
		exit (2);
	}

	if (!charsetname)
		keine_umlautwandlung = 1;
	else
	{
		charset = get_charset_from_name (charsetname);
		if (!charset)
		{
			error(1,0,"charset `%s' unknown",charsetname);
			/* NOTREACHED */
		}

		set_charset (charset);
	}

	is_native=option_to;

	if (optind == argc)
	{
		if (!outputfilename)
		{
			setvbuf (stdin, ibuf, _IOFBF, 32 * 1024);
			setvbuf (stdout, obuf, _IOFBF, 32 * 1024);
#ifdef __GO32__
			setmode(0,O_BINARY);
			setmode(1,O_BINARY);
#endif
			work (stdin, stdout);
		}
		else
		{
			FILE *f;

			f = fopen (outputfilename, "wb");
			if (!f)
				die (outputfilename);
#ifdef __GO32__
			setmode(0,O_BINARY);
#endif
			setvbuf (f, ibuf, _IOFBF, 32 * 1024);
			setvbuf (stdin, obuf, _IOFBF, 32 * 1024);
			work (stdin, f);
			fclose (f);
		}
	}
	else
	{
		if (outputfilename && optind != argc - 1)
		{
			error(1,0,"can only translate one file if -o is given");
			/* NOTREACHED */
		}
		while (optind < argc)
		{
			FILE *f;
			FILE *tmp = NULL;
			char name[PATH_MAX];
			char name2[PATH_MAX];

#ifdef HAVE_MMAP
	        int fd;
	        struct stat st;

	        fd = open (argv[optind], O_RDONLY);
	        if (fd < 0)
	        {
				error(1,errno,"cannot open %s for reading",argv[optind]);
				/* NOTREACHED */
	        }
	        if (fstat (fd, &st))
	        {
				error(1,errno,"cannot stat %s",argv[optind]);
				/* NOTREACHED */
	        }
	        mm_addr = mmap (0, st.st_size,
                       PROT_READ, MAP_PRIVATE, fd, 0);
			if (mm_addr == (caddr_t) - 1)
			{
				error(0,errno,"cannot mmap %s",argv[optind]);
				f = fdopen (fd, "rb");
				if (!f)
				{
					error(1,errno,"cannot fdopen %s for reading",argv[optind]);
					/* NOTREACHED */
				}
				mm_addr = (caddr_t) - 1;
			}
			else
			{
				mm_size = st.st_size;
				mm_to_read=st.st_size;
				mm_read_addr=mm_addr;
				close (fd);
				is_mmapped = 1;
				f=NULL;
			}
#else
			f = fopen (argv[optind], "rb");
			if (!f)
				die (argv[optind]);
#endif
			if (outputfilename)
			{
				tmp = fopen (outputfilename, "wb");
				if (!tmp)
					die (outputfilename);
			}
			else
			{
				char *p;

				p = strrchr (argv[optind], '/');
				if (p == NULL)
				{
					sprintf (name, "XXXXXX");
					sprintf (name2, "XXXXXX");
				}
				else
				{
					strcpy (name, argv[optind]);
					p = strrchr (name, '/');
					strcpy (p + 2, "XXXXXX");
					p[1]='a';

					strcpy (name2, argv[optind]);
					p = strrchr (name2, '/');
					strcpy (p + 2, "XXXXXX");
					p[1]='b';
				}
				if (option_verbose)
				{
					printf ("   ber %s und %s\n", name, name2);
				}
				if (!mktemp (name))
					die (name);
				tmp = fopen (name, "wb");
				if (!tmp)
					die (name);
				if (!mktemp (name2))
					die (name2);
			}
			if (f)
				setvbuf (f, ibuf, _IOFBF, 32 * 1024);
			setvbuf (tmp, obuf, _IOFBF, 32 * 1024);
			work (f, tmp);
			if (f)
				fclose (f);
#ifdef HAVE_MMAP
			else
				munmap (mm_addr, mm_size);
#endif
			if (!outputfilename)
			{
				fclose (tmp);
				if (rename (argv[optind], name2))
					die (argv[optind]);
				if (rename (name, argv[optind]))
				{
					error(0,errno,"%s",name);
					if (rename (name2, argv[optind]))
						die (name2);
					exit (1);
				}
				if (unlink (name2))
					die (name2);
			}
			else
				fclose (tmp);
			optind++;
		}
	}
	exit (0);
}
