/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
*/

/*
AUTO_NEED HEADER fcntl.h sys/file.h
AUTO_NEED HEADER unistd.h
AUTO_NEED HEADER string.h
AUTO_NEED HEADER memory.h 
AUTO_NEED MACRO AC_HEADER_STDC AC_FUNC_MMAP
 */
#include "config.h"

#include <stdio.h>

#ifdef HAVE_FCNTL_H
#include <fcntl.h>
#elif defined(HAVE_SYS_FILE_H)
#include <sys/file.h>
#endif

#ifdef STDC_HEADERS
#  include <stdlib.h>
#  include <string.h>
#else
#  if defined(HAVE_MEMORY_H)
#    include <memory.h>
#  endif
#endif

#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "portab.h"
#include "libquark.h"

/* #undef HAVE_MMAP */					/* wg. Fehler bei copy() */
#ifdef linux
#include <linux/version.h>
#if LINUX_VERSION_CODE < (1*65536 + 3 * 256 + 55)
/* too much trouble before */
#  define HALF_MMAP
#endif
#endif

#ifdef HAVE_MMAP
#include <sys/mman.h>
#endif

#ifndef O_BINARY
#define O_BINARY 0
#endif

/*
	29.01.93 Frank, toleranter, mit Rckgabe
	15.7.94 Uwe, beschleunigt

	sowohl src als auch dst mssen vollstndige Pfad+Dateiangaben enthalten
*/
#define COPYBUFFER 32768L

BOOLEAN file_copy(const char *src,const char *dst)
{
	long laenge;
	int save_errno=0;
	/* Normale Kopierroutine */
	int fdin=-1;
	int fdout=-1;
	struct stat st;

	if (!strcmp(src,dst))
		return TRUE;

	fdin = open(src, O_RDONLY | O_BINARY);
	if (fdin < 0)
		return FALSE;

	/* O_RDWR statt O_WRONLY, weil (linux) mmap das so will bei
	 * shared mappings.
	 */	
	fdout = open(dst, O_RDWR | O_CREAT | O_TRUNC | O_BINARY,0644);
	if (fdout < 0)
	{
		save_errno=errno;
		close(fdin);
		errno=save_errno;
		return FALSE;
	}
	
	if (fstat(fdin,&st))
	{
		save_errno=errno;
		close(fdin);
		close(fdout);
		errno=save_errno;
		return FALSE;
	}
	laenge=st.st_size;

#ifdef HAVE_MMAP
#ifndef HALF_MMAP
	/* diese Variante kann nur benutzt werden, wenn MAP_SHARED + PROT_WRITE +
	 * O_RDWR funktioniert, also nicht unter Linux <=1.?
	 */
	if (laenge*2 <= 2 * 1024 * 1024 && laenge > 0)
	{

		BOOLEAN ok=TRUE;
		caddr_t src_addr=(caddr_t) -1;
		caddr_t dst_addr=(caddr_t) -1;
		src_addr = mmap(0, laenge, PROT_READ, 
			MAP_FILE | MAP_SHARED, fdin, 0);
		if (src_addr == (caddr_t) -1)
		{
			save_errno=errno;
			goto mmap_reader;
		}
		close(fdin);

		/* set size of output file */
		if (ok && lseek(fdout, laenge - 1, SEEK_SET) == -1)
		{
			ok=FALSE;
			save_errno=errno;
		}
		if (ok && write(fdout, "", 1) != 1)
		{
			ok=FALSE;
			save_errno=errno;
		}

		if (ok)
		{
			/* PROT_READ is notwendig, weil i586-glibc-memcpy
			 * vom Ziel liesst um den Cache ausnutzen zu 
			 * koennen.
			 */
			dst_addr = mmap(0, laenge, PROT_READ | PROT_WRITE, 
				MAP_FILE | MAP_SHARED, fdout, 0);
			if (dst_addr == (caddr_t) -1)
			{
				save_errno=errno;
				ok=FALSE;
			}
		}
		if (ok)
		{
			/* die Datei kopieren */
			memcpy(dst_addr, src_addr, laenge);
		}
		if (src_addr!=(caddr_t)-1)
			munmap(src_addr,laenge);
		if (dst_addr!=(caddr_t)-1)
			munmap(dst_addr,laenge);
		close(fdout);
		errno=save_errno;
		return ok;
	}
mmap_reader:
#endif
	if (laenge <= 2 * 1024 * 1024)
	{
		caddr_t src_addr=(caddr_t) -1;
		BOOLEAN ok;
		src_addr = mmap(0, laenge, PROT_READ, 
			MAP_FILE | MAP_SHARED, fdin, 0);
		if (src_addr==(caddr_t)-1)
		{
			save_errno=errno;
			goto no_mmap;
		}
		/* mmap ok */

		close(fdin);

		if (laenge!= (long) xwrite(fdout,src_addr,laenge))
		{
			save_errno=errno;
			ok=FALSE;
		}
		else
			ok=TRUE;
		munmap(src_addr,laenge);
		close(fdout);
		errno=save_errno;
		return ok;
	}
no_mmap:
#endif
	{
		char *buffer;
		ssize_t got;
		ssize_t written=0;
		char *p;

		buffer = (char *) malloc(COPYBUFFER);

		if (buffer == NULL)
		{
			close(fdout);
			close(fdin);
			errno=ENOMEM;
			return FALSE;
		}

		while ((got=xread(fdin,buffer,
						  COPYBUFFER > laenge ? laenge : COPYBUFFER))>0)
		{
			p=buffer;
			laenge-=got;
			while (got > 0 && (written=xwrite(fdout,p,got))!=-1)
			{
				got-=written;
				p+=written;
			}
			if (written==-1)
			{
				save_errno=errno;
				break;
			}
		}
		if (got==-1)
			save_errno=errno;

		free(buffer);
		close(fdout);
		close(fdin);

		errno=save_errno;
		if (got==-1 || written==-1)
			return FALSE;
		return TRUE;
	}
} /* file_copy */
#ifdef TEST
int main(int argc,char **argv)
{
	if (!argv[1] || !argv[2])
		exit(2);
	if (file_copy(argv[1],argv[2]))
	{
		puts("ok");
		exit (0);
	}
	puts(strerror(errno));
	exit(1);
}
#endif
