/* file_sys.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Verzeichnis-Funktionen
	z.T. systemabhngig

	25.02.93 Frank
*/


#include <portab.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#if defined(__TOS__) && !defined(__MINT__)
#include <tos.h>
#else
#include <dirent.h>
#endif
#include <errno.h>

/*#include <filesys.h>*/

#include "prototyp.h"



/*
	Existenz einer Datei berprfen

	->			name	:	Name der Datei

	<-			TRUE	:	existiert
				FALSE	:	existiert nicht
*/
BOOLEAN fileexist(const char *name)
{
	struct stat statbuf;

	if (stat(name,&statbuf) < 0)
		return FALSE;
	else
		return TRUE;

} /* fileexist */



/*
	20.01.93 Frank, neu
*/
long len_datei(const char *datei)
{
	struct stat statbuf;

	if (stat(datei,&statbuf) < 0)
		return 0;
	else
		return (long) statbuf.st_size;

} /* len_datei */

/*
	Gre einer Datei ermitteln

	->			stream	:	FILE-Pointer auf geffneten stream

	<-			Dateigre in Bytes
*/
long filesize(FILE *stream)
{
	long save_pos;
	long laenge;

	save_pos = ftell(stream);
	fseek(stream,0,SEEK_END);
	laenge = ftell(stream);
	fseek(stream,save_pos,SEEK_SET);

	return laenge;

} /* filesize */


/*
 * Untersuchen, ob ein String auf den angegebenen Wildcard-Ausdruck mit den
 * Metazeichen '*', '?' pat.
 * Exit: TRUE/FALSE
 */
BOOLEAN wildcard(const char *vstring,const char *pattern)
{
    for (; *pattern != '\0'; pattern++, vstring++)
    {
        switch(*pattern)
        {
        case '*':
            if (*++pattern == '\0') /* '*' am Ende passt immer */
                return(TRUE);
            do  /* Sonst gucken, ob's irgendwo weiter hinten passt */
            {
                if (wildcard(vstring, pattern))   /* Rekursion */
                    return(TRUE);          /* Bis Match oder ... */
            } while(*vstring++ != '\0');    /* ... String-Ende */
            return(FALSE);
        case '?':
            if (*vstring == '\0')           /* Noch was da? */
                return(FALSE);
            break;
        default:                           /* Zeichenmenge? */
            if (*pattern != *vstring)       /* Direkter Vergleich */
                return(FALSE);
        }
    } /* for */
    return(*vstring == '\0');               /* Komplett gepasst? */
}



/*
	Anzahl Dateien eines Typs ermitteln

	->			pfad	:	Verzeichnis
				maske	:	Dateimaske

	<-			Anzahl passender Dateien

	19.01.93 Frank
	29.12.93 Uwe: Auch unter MiNT wird Fsfirst benutzt. Es ist auf einem 
	         Tosfilesystem massiv schnelles, auf einem Minix-FS nicht viel
	         langsamer.
*/
long anzahl_dateien(const char *pfad,const char *maske)
{
	long anzahl = 0;

#if defined(__TOS__) /* ### && !defined(__MINT__), falls readdir schneller */
	int fertig;

	fertig=Fsfirst(tprintf("%s%s",pfad,maske),64);
	while (!fertig)
	{
		anzahl++;
		fertig=Fsnext();
	}

#else

	DIR *dir;
	struct dirent *eintrag;

	if ((dir = opendir(pfad)) == NULL)
		return 0;
	while ((eintrag = readdir(dir)) != NULL)
	{
		if (wildcard(eintrag->d_name,maske))
			anzahl++;
	}
	closedir(dir);

#endif

	return anzahl;

} /* anzahl_dateien */

/*
	Zeile aus einem stream lesen
	das Zeilenende wird abgeschnitten
	maximal 100 Zeichen werden gelesen

	27.11.92 Frank
*/
char *tom_fgetline(char *buf,FILE *stream)
{
	char *anf,*pos;

	anf = fgets(buf,100,stream);
	if ((pos = strpbrk(buf,"\r\n")) != NULL)
		*pos = 0;

	return anf;

} /* tom_fgetline */
