/*
 * Copyright (C) 1995-1999 Uwe Ohse
 * 
 * This source is public domain: use it as you please.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
 */
#ifndef FILELOCK_H
#define FILELOCK_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <sys/types.h>
#if defined(HAVE_FCNTL_H) || !defined(HAVE_SYS_FILE_H)
#include <fcntl.h>
#else
#include <sys/file.h>
#endif

/*
 * we use POSIX fcntl file locking handling.
 * We do not use flock. It has different and interesting semantics
 * if fork()ed.
 * We do not use lockf, it's obsolete.
 */

#if defined(F_RDLCK) && defined(F_WRLCK) && defined(F_SETLK)
#define CAN_LOCK_FILES

/*
 * cmd: F_SETLK, F_GETLK, F_SETLKW
 * typ: F_RDLCK, F_WRLCK, F_UNLCK
 * offset: start of region to lock
 * len: length of lock: 0 means "to end of file"
 * whence: SEEK_SET, SEEK_END, SET_CUR
 */
#ifndef P__
#define P__(x) x /* assuming that applications default to prototypes */
#endif
int lock_region P__((int fd, int cmd, int typ, off_t offset, int whence, off_t len));

/* partial files: */
#define fd_read_lock(fd,offset,whence,len) \
	lock_region(fd,F_SETLK,F_RDLCK,offset,whence,len)
#define fd_readw_lock(fd,offset,whence,len) \
	lock_region(fd,F_SETLKW,F_RDLCK,offset,whence,len)
#define fd_write_lock(fd,offset,whence,len) \
	lock_region(fd,F_SETLK,F_WRLCK,offset,whence,len)
#define fd_writew_lock(fd,offset,whence,len) \
	lock_region(fd,F_SETLKW,F_WRLCK,offset,whence,len)
#define fd_un_lock(fd,offset,whence,len) \
	lock_region(fd,F_SETLK,F_UNLCK,offset,whence,len)

/* complete files: */
#define fd_read_lock0(fd) lock_region((fd),F_SETLK,F_RDLCK,0,SEEK_SET,0)
#define fd_readw_lock0(fd) lock_region((fd),F_SETLKW,F_RDLCK,0,SEEK_SET,0)
#define fd_write_lock0(fd) lock_region((fd),F_SETLK,F_WRLCK,0,SEEK_SET,0)
#define fd_writew_lock0(fd) lock_region((fd),F_SETLKW,F_WRLCK,0,SEEK_SET,0)
#define fd_un_lock0(fd) lock_region((fd),F_SETLK,F_UNLCK,0,SEEK_SET,0)
#endif

#endif
