/* getmodaddr.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/* schamelessly stolen from INN and heavily modified by Uwe Ohse */

#include "config.h"
#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <limits.h>
#include <stdlib.h>
#include <ctype.h>
#include <fnmatch.h>
#include "box_conf.h"
#include "tompaths.h"
#include "libquark.h"
#include "prototyp.h"

typedef struct mod_t
{
	char *pattern;
	char *address;
	struct mod_t *next;
} mod_t;

static mod_t *anker;

static void read_mod_file(const char *fname);

/*
**  Read the moderators file, looking for a moderator.
*/
static void
read_mod_file(const char *fname)
{
	FILE *F;
	mod_t *neu;
	mod_t *last=NULL;
	char buff[1024];

    F = fopen(fname, "r");
    if (!F)
    	return;

	while (fgets(buff, sizeof buff, F) != NULL) 
	{
		char *grp;
		char *adr;
		char *p;
	    /* Skip blank and comment lines. */
	    p=strpbrk(buff,"\r\n");
	    if (p)
	    	*p=0;
	    p=buff;
	    while (isspace(*p))
	    	p++;
	    if (*p==';' || *p=='#' || !*p)
	    	continue;

	    /* Snip off the first word. */
	    if ((p = strchr(buff, ':')) == NULL)
			/* Malformed line... */
			continue;
	    *p++ = '\0';
	    adr=strtrim(p);
	    grp=strtrim(buff);
	    if (!*adr || !*grp)
	    {
			/* Malformed line... */
			continue;
	    }

	    neu=malloc(sizeof(mod_t));
	    if (neu)
	    {
	    	neu->address=strdup(adr);
	    	neu->pattern=strdup(grp);
	    }
	    if (!neu || !neu->pattern || !neu->address)
	    {
	    	mod_t *next;
	    	if (neu)
	    	{
	    		free(neu->pattern);
	    		free(neu->address);
	    		free(neu);
	    	}
	    	for (;anker;anker=next)
	    	{
	    		next=anker->next;
	    		free(anker->pattern);
	    		free(anker->address);
	    		free(anker);
	    	}
	    	return;
	    }
	    if (last)
	    	last->next=neu;
	    else
	    	anker=neu;
	    neu->next=NULL;
	    last=neu;
	}

	(void)fclose(F);
}

char *
GetModeratorAddress(const char *group)
{
    char		address[1024];
    char *p;
    mod_t *akt;

#ifndef FNM_CASEFOLD
	char buf1[1024];
	entumlauten(group,buf1);
	strlwr(buf1);
#endif

    if (!anker)
    {
    	char fname[PATH_MAX];
    	sprintf(fname,"%smoderators.cnf",box_conf.config_dir);
		read_mod_file(fname);
    }

    for (akt=anker;akt;akt=akt->next)
    {
#ifdef FNM_CASEFOLD
    	if (0==fnmatch(akt->pattern,group,FNM_CASEFOLD))
#else
		char buf2[1024];
		entumlauten(akt->pattern,buf2);
		strlwr(buf2);
#endif
    	{
    		/* akt->address: %s gegen modifizierten Gruppennamen austauschen.
    		 * (alle . in - aendern). */
    		char *gdup=strdup(group);
    		XNULLCHECK(gdup);
    		for (p=gdup;*p;p++)
    		{
    			if (*p=='.')
    				*p='-';
    		}
    		sprintf(address,akt->address,gdup,NULL,NULL,NULL,NULL,NULL);
    		free(gdup);
    		return strdup(address);
    	}
    }

    /* If we don't have an address, see if the config file has a default. */
    if (!box_conf.default_moderator)
    	return NULL;

   	{
   		/* default_moderator: %s gegen modifizierten Gruppennamen austauschen.
   		 * (alle . in - aendern). */
   		char *gdup=strdup(group);
   		XNULLCHECK(gdup);
   		for (p=gdup;*p;p++)
   		{
   			if (*p=='.')
    				*p='-';
   		}
   		sprintf(address,box_conf.default_moderator,gdup,NULL,NULL,NULL,NULL,NULL);
   		free(gdup);
   		return strdup(address);
   	}
}
