/*
   Copyright (C) 1993,1994,1995,1996  Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   For questions contact:
   e-mail: (prefered)
   uwe@tirka.gun.de
   Uwe_Ohse@mausdu3.gun.de
   Uwe Ohse @ DU3 (inside MausNet)
 */

#include "tomsysdep.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <limits.h>
#include <errno.h>
#include <ctype.h>

#include "quarkoutfile.h"
#include "libquark.h"

grpconv_t *glanker;                         /* Liste, nach longnames sortiert */
grpconv_t *gsanker;                         /* Liste nach shortnames sort. */
BOOLEAN grplist_changed;                    /* Hat sich die Liste geaendert? */
grpconv_t *gshash[256];                     /* Gruppennamenhashtabelle */
grpconv_t *glhash[256];                     /* Gruppennamenhashtabelle */

/*****************************************************************************
 * Gruppennamenskonvertiercode. 
 * Arbeitet mit zwei alphabetisch sortierten Listen und einem einfachen
 * "Hashing", das die Zahl der Suchzugriffe schon mal deutlich senkt, ohne
 * allzu viel Logik zu brauchen.
 * Beide Listen zeigen auf dieselben Strings!
 *****************************************************************************
 */
grpconv_t *
find_shortname(unsigned char *sname)
{
	grpconv_t *akt;
	unsigned char c;
	c=tolower(*sname);
	akt=gshash[c];
	if (akt)
	{
		while (akt && tolower(*akt->shortname)==c)
		{
			if (!strcasecmp(akt->shortname,sname))
				return akt;
			akt=akt->next;
		}
	}
	/* nicht gefunden - aber getreu dem dont-trust-my-sources prinzip 
	 * wird doch noch mal die ganze Liste durchsucht.
	 */
	akt=gsanker;
	while (akt)
	{
		if (!strcasecmp(akt->shortname,sname))
			return akt;
		akt=akt->next;
	}
	return NULL;
}

grpconv_t *
find_longname(unsigned char *lname)
{
	grpconv_t *akt;
	unsigned char c;
	c=tolower(*lname);
	akt=glhash[c];
	if (akt)
	{
		while (akt && tolower(*akt->longname)==c)
		{
			if (!strcasecmp(akt->longname,lname))
				return akt;
			akt=akt->next;
		}
	}
	/* nicht gefunden - aber getreu dem dont-trust-my-sources prinzip 
	 * wird doch noch mal die ganze Liste durchsucht.
	 */
	akt=glanker;
	while (akt)
	{
		if (!strcasecmp(akt->longname,lname))
			return akt;
		akt=akt->next;
	}
	return NULL;
}

void 
grp_einfuegen(unsigned char *shortname, unsigned char *longname, 
	BOOLEAN import)
{
	unsigned char c;
	grpconv_t *akt,*last;

	grpconv_t *news, *newl;

	news=XMALLOC(sizeof(grpconv_t));
	newl=XMALLOC(sizeof(grpconv_t));
	shortname=strdup(shortname);
	longname=strdup(longname);
	if (!shortname || !longname)
	{
		fprintf(stderr,"%s: Kein Speicher mehr verfuegbar\n", program_name);
		exit(1);
	}
	news->longname=longname;
	news->shortname=shortname;
	news->import_flag=import;
	newl->longname=longname;
	newl->shortname=shortname;
	newl->import_flag=import;
	/*
	 * Alphabetisch sortiert in die beiden Listen einfuegen, Eintrag in der
	 * jeweiligen Hashtabelle anlegen.
	 */
	akt=glanker;
	c=tolower(longname[0]);
	last=NULL;
	while (akt)
	{
		if (strcasecmp(akt->longname,longname)>0)
		{
			newl->next=akt;
			if (last)
			{
				last->next=newl;
				/* Hashtabelle updaten */
				if (c != tolower(last->longname[0]))
					glhash[c]=newl;
			}
			else
			{
				glanker=newl;
				glhash[c]=newl;
			}
			break;
		}
		last=akt;
		akt=akt->next;
	}
	if (!akt)
	{
		/* nicht gefunden, also kommt also im Alphabet nach allen anderen */
		if (last)
		{
			last->next=newl;
			if (c != tolower(last->longname[0]))
				glhash[c]=newl;
		}
		else
		{
			glanker=newl;
			glhash[c]=newl;
		}
		newl->next=NULL; 
	}
	/* und nun dasselbe fuer den kurzen Namen */
	akt=gsanker;
	c=tolower(news->shortname[0]);
	last=NULL;
	while (akt)
	{
		if (strcasecmp(akt->shortname,news->shortname)>0)
		{
			news->next=akt;
			if (last)
			{
				last->next=news;
				/* Hashtabelle updaten */
				if (c != tolower(last->shortname[0]))
					gshash[c]=news;
			}
			else
			{
				gsanker=news;
				gshash[c]=news;
			}
			break;
		}
		last=akt;
		akt=akt->next;
	}
	if (!akt)
	{
		/* nicht gefunden, also kommt also im Alphabet nach allen anderen */
		if (last)
		{
			last->next=news;
			if (c != tolower(last->shortname[0]))
				gshash[c]=news;
		}
		else
		{
			gsanker=news;
			gshash[c]=news;
		}
		news->next=NULL; 
	}
	grplist_changed=TRUE;
}

void 
speichere_grplist(const char *fname)
{
	FILE *f;
	grpconv_t *akt;

	if (!grplist_changed)
		return;

	f=fopen(fname,"w");
	if (!f)
	{
		fprintf(stderr,"%s: kann %s nicht schreiben: %s",
			program_name,fname,strerror(errno));
		exit(1);
	}
	akt=glanker;
	while (akt)
	{
		fprintf(f,"%s%s = %s\n",
			akt->import_flag ? "+" : "" ,
			akt->longname,akt->shortname);
		akt=akt->next;
	}
	fclose(f);
}

void 
lade_grplist(const char *fname)
{
	FILE *f;
	unsigned char buf[1024];
	unsigned char *p;
	unsigned char *lname;
	unsigned char *sname;
	unsigned int zeile = 0;
	f=fopen(fname,"rb");
	if (!f)
	{
		fprintf(stderr,"%s: kann %s nicht lesen: %s",
			program_name,fname,strerror(errno));
		exit(1);
	}
	while (fgets(buf,sizeof(buf),f))
	{
		BOOLEAN import=FALSE;
		zeile++;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p='\0';
		p=strtrim(buf);
		if (*p==';' || *p=='#' || !*p)
			continue;
		sname=strchr(buf,'=');
		if (!sname)
		{
			/* Tsss ... */
			fprintf(stderr,"%s: Zeile %d in %s nicht parsbar",program_name,zeile,
				fname);
			fprintf(stderr,"%s\n",buf);
			continue;
		}
		*sname++='\0';
		lname=buf;
		if (*lname=='+')
		{
			lname++;
			import=TRUE;
		}
		lname=strtrim(lname);
		sname=strtrim(sname);
		if (!*lname || !*sname)
		{
			/* Tsss ... */
			fprintf(stderr,"%s: Gruppenname nicht angegeben in %s, Zeile %d\n",
				program_name,fname,zeile);
			continue;
		}
		grp_einfuegen(sname,lname,import);
	}
	fclose(f);
	grplist_changed=0;
}
