/* inquire.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/


#include "config.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <portab.h>
#include <errno.h>
#include <ctype.h>
#include <syslog.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/time.h>
#include <syslog.h>
#include <unistd.h>
#include <signal.h>
#include <fnmatch.h>
#ifdef HAVE_SOCKET
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#endif
#include <pwd.h>

#include "tomsysdep.h"
#include "user.h"
#include "aktdata.h"
#include "prototyp.h"
#include "box_conf.h"
#include "tompaths.h"
#include "inquire.h"
#include "authuser.h"
#include "io.h"        /* io_test_modem_lines */

#define LINEEND "\r\n"

BOOLEAN
inquire_remote (int fd)
{
#ifdef HAVE_SOCKET
	size_t raddrlen;
	struct sockaddr_in raddr;
#endif
	struct connectiondaten *fill=&akt_connection; /* Schreibhilfe */
	static int calls_to_ir=0;

	fill->startzeit=time(NULL);
#ifdef HAVE_SOCKET
	raddrlen = sizeof (raddr);
#endif

	fill->peer = FALSE;
	fill->is_tty = isatty(fd);

	/* Beim zweiten, dritten ... Aufruf sind einige Strings freizugeben. */
	if (calls_to_ir++!=0)
	{
		free(fill->ttyname);
		free(fill->rusername);
		free(fill->rhostname);
		free(fill->rhostaddr);
		free(fill->carrierstring);
		free(fill->carriercmd);
		free(fill->carrierstringraw);
	}
	fill->ttyname = NULL;
	fill->rusername = NULL;
	fill->rhostname = NULL;
	fill->rhostaddr = NULL;
	fill->carrierstring=NULL;
	fill->carriercmd=NULL;
	fill->carrierstringraw=NULL;
	fill->localport=0;
	fill->remoteport=0;

	fill->auth_ok = FALSE;
	fill->error_free = FALSE;
	fill->is_modem=FALSE;
	fill->is_console=FALSE;
	fill->has_modem_lines=FALSE;
	fill->carrier_baud=0;
	fill->carrier_cps=0;

	if (fill->is_tty)
	{
		char *s = ttyname (fd);
		if (s)
			fill->ttyname = XSTRDUP (s);
		else /* this should really not happen */
			fill->ttyname = XSTRDUP ("unknown");
		fill->has_modem_lines=io_test_modem_lines(fd);
		/*
		 * Der Umkehrschlu !fill->has_modem_lines --> is_console
		 * ist nicht korrekt, z.B. bei Verbindungen ber Terminalserver
		 * und rlogin
		 */
		if (fill->has_modem_lines)
			fill->is_modem=TRUE;
		/* console etc werden spter abgefragt, in inquire2 (box_conf
		 * ist hier noch nicht geladen */
	}

#ifdef HAVE_SOCKET
	if (getpeername (fd, (struct sockaddr *) &raddr, &raddrlen) != -1)
	{
		/* Das ist also eine Verbindung ber einen Socket. */
		unsigned long inlocal;
		unsigned long inremote;
		unsigned short local;
		unsigned short remote;
		struct hostent *hp;
		BOOLEAN might_be_console=FALSE;

		fill->peer = TRUE;
		fill->error_free = TRUE; /* i really hope so :-) */
		fill->is_console = FALSE; /* das kann nicht sein? Doch - s.o. */

		fill->rhostaddr= XSTRDUP(inet_ntoa (raddr.sin_addr));
		if (!strcmp(fill->rhostaddr,"127.0.0.1"))
			might_be_console=TRUE;

		hp = gethostbyaddr ((char *) &(raddr.sin_addr),
							sizeof (struct in_addr), raddr.sin_family);

		if (hp != NULL)
		{
			fill->rhostname = XSTRDUP (hp->h_name);
		}
		else 
		{
			char buf[256];

			sprintf (buf, "[%s]", fill->rhostaddr);
			fill->rhostname = XSTRDUP (buf);
		}
		if (!might_be_console)
		{
			char hn[256];
			if (!gethostname(hn,sizeof(hn)) && !strcasecmp(fill->rhostname,hn))
			{
				might_be_console=TRUE;
			}
		}
		if (auth_fd2 (fd, &inlocal, &inremote, &local, &remote) == 0)
		{
			char *u;
			fill->localport=local;
			fill->remoteport=remote;

			u = auth_tcpuser3 (inlocal, inremote, local, remote, 30);
			if (u)
			{
				fill->rusername = XSTRDUP (u);
				fill->auth_ok=TRUE;
				if (might_be_console)
				{
					struct passwd *pw;
					pw=getpwnam(u);
					if (pw && pw->pw_uid==getuid())
						fill->is_console=TRUE;
				}
			}
			else
				fill->rusername = XSTRDUP ("unknown");
		}
		else
			fill->rusername = XSTRDUP ("unknown");
	}
	else
#endif
	{
#ifdef HAVE_SOCKET
		if (errno==ENOTSOCK
#ifdef __MINT__
			/* Architekturfehler im Mintkernel */
			|| errno==EINVAL
#endif
			)
		{
#endif /* HAVE_SOCKET */
			/* The argument is a file, not a socket. */
			char u[256];
			char hn[256];
			fill->peer = FALSE;

			/* getlogin tut nur, wenn wir unter getty laufen, und ist
			 * auerdem teuer. passwd-Zugriff ist fr die Startupphase
			 * ebenfalls absoluter Overkill. Also Hack.
			 */
			sprintf(u,"[%ld]",(long) getuid());
			fill->rusername=XSTRDUP(u);

			if (!gethostname(hn,sizeof(hn)))
			{
				fill->rhostname=XSTRDUP(hn);
				fill->rhostaddr=XSTRDUP("127.0.0.1");
				fill->auth_ok=TRUE;
			}
			else
			{
				fill->rhostname=XSTRDUP("unknown");
				fill->auth_ok=FALSE;
			}
#ifdef HAVE_SOCKET
		}
		else
		{
            syslog(LOG_ERR,"getpeername returned %m %d",errno);
            return FALSE;
		}
#endif
	}
	return TRUE;
}

BOOLEAN
inquire_remote2 (int fd TOM_ATTRIB_UNUSED)
{
	struct connectiondaten *fill=&akt_connection; /* Schreibhilfe */
	/* dieser Teil wird nach Lesen der quark.cnf ausgefhrt. */
	if (fill->is_tty && fill->ttyname)
	{
		BOOLEAN found=FALSE;
		if (box_conf.modem_wildcard)
		{
			if (0==fnmatch(box_conf.modem_wildcard,fill->ttyname,0))
			{
				fill->is_modem=TRUE;
				found=TRUE;
			}
		}
		if (found && box_conf.console_wildcard)
		{
			if (0==fnmatch(box_conf.console_wildcard,fill->ttyname,0))
			{
				fill->is_console=TRUE;
			}
		}
	}
	return TRUE;
}
