/* interpret.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <syslog.h>
#include <errno.h>

#include "tinterp.h" /* damit sie in . liegen darf */

#include "tompaths.h"
#include "box_conf.h" /* MYDOMAINPART */
#include "prototyp.h"
#include "brett.h"
#include "user.h"
#include "modem.h"
#include "terminal.h"
#include "bin_msg.h"
#include "msg.h"
#include "befehle.h"				/* wg. bef_hilfe() */
#include "cslib.h"
#include "routing.h"
#include "aktdata.h"
#include "mtzeit.h"

#undef HAVE_MMAP					/* wg. Fehler bei copy() */

#ifdef HAVE_MMAP
#include <sys/mman.h>
#endif

extern USER user;
extern BRETT akt_brett;
extern BRETTNR brettnrW;

extern int col_ix[];

extern char *tau_ver;

extern WORD vtmodeW;

    /* Fr den Aufruf externer Programme und %e */
extern const char *external_parameter;
extern const char *protokoll_quelle;
extern const char *protokoll_ziel;
extern const char *packer_archive;
extern const char *packer_files;
extern const char *tausch_processed_file;
extern const char *benutzer_prompt_eingabe; /* Sind Sie eingetragener .. */

static size_t tom_tip_inetadr(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen);
static size_t tom_tip_lowerupper(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen);
static size_t tom_tip_ucanon(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen);
static size_t tom_tip_percent_fn(tip_t *tab, const char *this, 
	const char **next, char *to, size_t maxlen);
static size_t tom_tip_tmptoken_fn(tip_t *tab, const char *this, 
	const char **next, char *to, size_t maxlen);
static size_t tom_tip_datum_zeit(tip_t *tab, const char *this, 
	const char **next, char *to, size_t maxlen);


typedef struct tom_tip_tmptoken_t
{
	char *name;
	char *value;
	struct tom_tip_tmptoken_t *next;
} tom_tip_tmptoken_t;
static tom_tip_tmptoken_t *token_anker;


/* textinterpretationstabelle */
static tip_t tiptab[]=
{
	{ '\\', NULL,        0, tip_backslash},
	{ '~',  NULL,        0, tip_tilde},
	{ '\0', "%{",        0, tom_tip_tmptoken_fn},
	{ '%',  NULL,        0, tom_tip_percent_fn},
	{ '\0', "$[",        0, tip_conditional},
	{ '\0', "${strip",   0, tip_strip},
	{ '\0', "${macro",   0, tip_macro},
	{ '\0', "${literal", 0, tip_literal},
	{ '\0', "${force",   0, tip_force},

	{ '\0', "${inetadr",  0, tom_tip_inetadr},
	{ '\0', "${lower",  0, tom_tip_lowerupper},
	{ '\0', "${ucanon",  0, tom_tip_ucanon},
	{ '\0', "${u_canon",  0, tom_tip_ucanon},
	{ '\0', "${upper",  0, tom_tip_lowerupper},
	{ '\0', "${dd.mm.yy",  0, tom_tip_datum_zeit},
	{ '\0', "${yy_mm_dd",  0, tom_tip_datum_zeit},
	{ '\0', "${hh:mm:ss",  0, tom_tip_datum_zeit},

	{ '$',  NULL,        0, tip_dollar},
	{ '\0', NULL,        0, NULL}
};



#define PARANOIA_STRING(fn_name,start,this_magic) \
do { \
	if (strncmp(start,this_magic,strlen(this_magic))) \
	{ \
		tip_errtext = fn_name " called without proper magic " this_magic ; \
		return TIP_ERROR; \
	} \
} while (0)

#define PARANOIA_CHAR(fn_name,start,this_magic) \
do { \
	if (*start!=*this_magic) \
	{ \
		tip_errtext = fn_name " called without proper magic " this_magic ; \
		return TIP_ERROR; \
	} \
} while (0)
 

/* ========================  Utility - Routinen  ======================== */

#define COMPLETE(x) do {\
	l=strlen(x); \
	if (l>=maxlen) { tip_errtext="buffer shortage"; return TIP_ERROR;}; \
	strcpy(to,x); \
	return l; } while(0)
#define COMPLETENULL(x) do {if (x && *x) COMPLETE(x); else COMPLETE("?"); } \
	while(0)
#define COMPLETEC(x) do {\
	if (2>=maxlen) { tip_errtext="buffer shortage"; return TIP_ERROR;}; \
	*to++=x; \
	*to='\0'; \
	return 1; } while(0)
#define CONDCOMPLETEC(x,y,z) do {if (x) COMPLETEC(y); else COMPLETEC(z);} \
	while(0) 
#define CONDCOMPLETE(x,y,z) do {if (x) COMPLETE(y); else COMPLETE(z);} \
	while(0) 

static size_t 
tom_tip_percent_fn(tip_t *tab TOM_ATTRIB_UNUSED, 
				   const char *this, const char **next,
				   char *to, size_t maxlen)
{
	size_t l;
	char buf[512]="";

	PARANOIA_CHAR("tom_tip_%_fn", this, "%");

	switch(this[1])
	{
	case 'a':
		*next=this+3;
		/* Aktueller Anruf */
		switch(this[2])
		{
		case 'b':                     /* Connect_baud */
			sprintf(buf,"%lu",(ULONG) akt_connection.carrier_baud);
			COMPLETE(buf);
		case 'c':                     /* Connect_cps */
			sprintf(buf,"%lu",(ULONG) akt_connection.carrier_cps);
			COMPLETE(buf);
		case 'C':                     /* Connect_string */
			COMPLETENULL(akt_connection.carrierstring);
		case 'e':                     /* Endzeit */
			time_t_to_mtzeit(&akt_anruf.soft_limit,buf);
			COMPLETE(buf);
		case 'G':
			sprintf(buf,"%ld",(long) akt_connection.rgid);
			COMPLETE(buf);
		case 'n':                     /* Nummer */
			sprintf(buf,"%lu",(ULONG) akt_anruf.anrufnummer);
			COMPLETE(buf);
		case 'N':                     /* Name (mu nicht gleich Username sein) */
			COMPLETENULL(akt_anruf.name);
		case 'U':
			sprintf(buf,"%ld",(long) akt_connection.ruid);
			COMPLETE(buf);
		case 'w':                     /* Wurfzeit */
			time_t_to_mtzeit(&akt_anruf.hard_limit,buf);
			COMPLETE(buf);
		}
		return TIP_NOTDONE;
		break;
	case 'e':
		/*       "HALLO"                "" (leer)
		 * %ee    hallo                 ""
		 * %eE    HALLO                 ""
		 * %eu    hallo                 "unknown"
		 * %eU    HALLO                 "unknown"
		 */
		*next=this+3;
		if (tolower(this[2])=='e' || tolower(this[2]=='u'))
		{
			*buf='\0';
			if (external_parameter)
				strcpy(buf,external_parameter);
			else if (tolower(this[2])=='u')
				strcpy(buf,"unknown");
			if (islower(this[2]))
				strlwr(buf);
			COMPLETE(buf);
		}
		return TIP_NOTDONE;
	case 'P':
		*next=this+3;
		if (this[2]=='Q')
			COMPLETENULL(protokoll_quelle);
		if (this[2]=='Z')
			COMPLETENULL(protokoll_ziel);
		if (this[2]=='A')
			COMPLETENULL(packer_archive);
		if (this[2]=='F')
			COMPLETENULL(packer_files);
		if (this[2]=='B')
			COMPLETENULL(benutzer_prompt_eingabe);
		if (this[2]=='f')
			COMPLETENULL(tausch_processed_file);
		return TIP_NOTDONE;
	case 'u':
		*next=this+3;
		switch(this[2])
		{
		case 'A':                     /* A -> Anrufe */
			sprintf(buf,"%lu",(ULONG) user.anrufe);
			COMPLETE(buf);
		case 'c':                     /* c -> user.computertyp */
			sprintf(buf,"%ld",(long) user.computertyp);
			COMPLETE(buf);
		case 'C':                     /* C -> user.computer */
			COMPLETENULL(user.computer);
		case 'D':                     /* D -> Bezahlt-bis, MausTausch-Zeit */
			time_t_to_mtzeit(&user.bezahlt_bis,buf);
			COMPLETE(buf);
		case 'e':                     /* e -> erster Anruf, MausTausch-Zeit */
			time_t_to_mtzeit(&user.erster_login,buf);
			COMPLETE(buf);
		case 'g':                     /* g -> Geburtstag */
			{
				struct tm tm;
				memset(&tm,0,sizeof(tm));
				tm.tm_year=user.geburtsjahr;
				tm.tm_mon=user.geburtsmonat;
				tm.tm_mday=user.geburtstag;
				tm_to_mtzeit(&tm,buf);
				COMPLETE(buf);
			}
		case 'G':                     /* G -> N, W, G, je nach user.geschlecht */
			if (user.geschlecht==NEUTRUM)
				COMPLETEC('N');
			if (user.geschlecht==WEIBLICH)
				COMPLETEC('W');
			COMPLETEC('M');
		case 'L':                     /* L -> letzter Login, MausTausch-Zeit */
			time_t_to_mtzeit(&user.letzter_login,buf);
			COMPLETE(buf);
		case 'l':                     /* l -> level */
			sprintf(buf,"%ld",(long) level());
			COMPLETE(buf);
		case 'M':                     /* M -> Mehr-Zeilen */
			sprintf(buf,"%ld",(long) user.mehr);
			COMPLETE(buf);
		case 'm':                     /* m -> menue-Stil */
			sprintf(buf,"%d",(int) user.menue_stil);
			COMPLETE(buf);
		case 'N':                     /* N -> user.name */
			COMPLETENULL(user.name);
		case 'n':                     /* n -> user.nummer */
			sprintf(buf,"%ld",(long) user.nummer);
			COMPLETE(buf);
		case 'P':                     /* P -> Pflicht befreit ? J : N */
			CONDCOMPLETEC(user.pflicht_befreit,'J','N');
		case 'r':                     /* r -> restzeit in Sekunden */
			sprintf(buf,"%ld",(long) user.restzeit);
			COMPLETE(buf);
		case 'S':                     /* S -> BS-Spalten */
			sprintf(buf,"%ld",(long) user.bs_spalten);
			COMPLETE(buf);
		case 'T':                     /* S -> Terminaltyp */
			sprintf(buf,"%ld",(long) user.terminal_typ);
			COMPLETE(buf);
		case 'v':                     /* r -> vertreter */
			COMPLETENULL(user.vertreter);
		case 'z':                     /* z -> zeichensatz */
			{
				cstl_t *cs;
				const char *p;
				*buf='\0';
				cs=get_charset_from_nr(user.zeichensatz);
				p=(cs) ? cs->name : NULL;
				COMPLETENULL(p);
			}
		case 'Z':                     /* Z -> BS-Zeilen */
			sprintf(buf,"%ld",(long) user.bs_zeilen);
			COMPLETE(buf);
		case '$':                    /* $ -> bezahlt ? $ : N */
			CONDCOMPLETEC(user.bezahlt,'$','N');
		}
		return TIP_NOTDONE;
	case 'g':
		*next=this+3;
		switch(this[2])
		{
		case 'C':                     /* Chef?? */
			if (!brettnrW) 
				COMPLETEC('N');
			CONDCOMPLETEC(user.nummer==akt_brett.grp_chef,'J','N');
		case 'N':                     /* N -> brett.name */
			if (brettnrW)
				COMPLETENULL(akt_brett.name);
			else
				COMPLETE("Postfach");
		case 'n':                     /* n -> brett.nummer */
			sprintf(buf,"%lu",(ULONG) brettnrW);
			COMPLETE(buf);
		case 'r':                     /* r -> read-level (lesen_level) */
			sprintf(buf,"%lu",(ULONG) (brettnrW? akt_brett.lesen_level : 0));
			COMPLETE(buf);
		case 'w':                     /* w -> write-level (schreib_level) */
			sprintf(buf,"%lu",(ULONG) (brettnrW ? akt_brett.schreib_level : 0));
			COMPLETE(buf);
		case 'S':                     /* S -> status */
			if (brettnrW)
				brettflags_to_string(akt_brett.flags, buf, sizeof(buf));
			else
				*buf=0;
			COMPLETE(buf);
		case 'V':                     /* V -> Vernetzt*/
			if (brettnrW)
				CONDCOMPLETEC(akt_brett.netzstatus,'J','N');
			else
				COMPLETE("J");
		}
		return TIP_NOTDONE;
	case 't':
		*next=this+3;
		switch(this[2])
		{
		case 'v':                     /* %tv -> VERSION */
			COMPLETE(VERSION);
		case 'd':                     /* %td -> tom_dat */
			COMPLETE(vstr_date);
		case 'z':                     /* %tz -> tom_tim */
			COMPLETE(vstr_time);
		case 'c':                     /* %tz -> tom_tim */
			COMPLETE(vstr_compileversion);
		case 's':                     /* %ts -> sccs_box */
			/* cheat gegen schlaue Linker */
			COMPLETE("@(#) " PACKAGE_VERSION);
		case 't':                     /* %tt -> tau_ver */
			COMPLETE(tau_ver);
		}
		return TIP_NOTDONE;
	case 'p':
		*next=this+3;
		switch(this[2])
		{
		case 'b':                     /* %pb -> box_conf.bin_dir tom/bin */
			COMPLETE(box_conf.bin_dir);
		case 'B':                     /* %pB -> box_conf.batch_dir */
			COMPLETE(box_conf.batch_dir);
		/* case 'c' is reserved - see 's' */
		case 'C':                     /* %pC -> box_conf.tcl_dir */
			/* may be undefined if not having libtcl */
			COMPLETENULL(box_conf.tcl_dir);
		case 'f':                     /* %pf -> box_conf.binbrett_dir tom/db/binaer */
			COMPLETE(box_conf.binbrett_dir);
		case 'g':                     /* %pg -> box_conf.bretter_dir tom/db/text */
			COMPLETE(box_conf.bretter_dir);
		case 'h':                     /* %ph -> box_conf.hilfe_dir tom/hilfe */
			COMPLETE(box_conf.hilfe_dir);
		case 'i':                     /* %pi -> tom_info_lokal_dir tom/info/local */
			COMPLETE(tom_info_lokal_dir);
		case 'I':                     /* %pI -> tom_info_net_dir tom/info/net */
			COMPLETE(tom_info_net_dir);
		case 'l':                     /* %pl -> box_conf.log_dir */
			COMPLETE(box_conf.log_dir);
		case 'p':                     /* %pp -> tom_port_config_dir */
			CONDCOMPLETE(tom_port_config_dir,tom_port_config_dir,
						 box_conf.config_dir);
		case 'c':
		case 's':                     /* %ps -> box_conf.config_dir */
			COMPLETE(box_conf.config_dir);
		case 'S':                     /* %pS -> box_conf.spool_dir */
			COMPLETE(box_conf.spool_dir);
		case 'T':                     /* %pT -> box_conf.tausch_dir */
			COMPLETE(box_conf.tausch_dir);
		case 't':                     /* %pt -> tom_tmp_dir */
			COMPLETE(tom_tmp_dir);
		case 'u':                     /* %pu -> tom_utmp_dir */
			COMPLETE(tom_utmp_dir);
		case 'U':                     /* %pU -> box_conf.user_dir */
			COMPLETE(box_conf.user_dir);
		case 'v':                     /* %ps -> box_conf.var_dir */
			COMPLETE(box_conf.var_dir);
		case 'x':                     /* %px -> box_conf.texte_dir */
			COMPLETE(box_conf.texte_dir);
		}
		return TIP_NOTDONE;
	case '@':
		*next=this+2;
		COMPLETENULL(MYDOMAINPART);
	case '%':
		*next=this+2;
		COMPLETEC('%');
	default:
		/* nicht fuer uns */
		return TIP_NOTDONE;
	}
}
#undef COMPLETE
#undef COMPLETEC
#undef CONDCOMPLETEC

BOOLEAN
do_interpret(const char *format, char *to, size_t max)
{
	*to=0;
	if (tip_interpret(tiptab,format,to,max)!=TIP_ERROR)
		return TRUE;
	mbprintf("tip-error: %s\n",tip_errtext);
	return FALSE;
}

/*
 * ${inetadr  User Name } -> "user_name@hier.maus.de"
 * ${inetadr  User Name @ DU} -> "user_name@du.maus.de"
 */
static size_t 
tom_tip_inetadr(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen)
{
	char *ende=NULL;
	const char *start;
	int ok=0;
	char *duplicate=NULL;
	char *tmpbuf1=NULL;
	char *tmpbuf2=NULL;
	char *domain=NULL;
 	size_t len=0;
	char *domp;
	char *q;

#define MAGIC "${inetadr "

	PARANOIA_STRING("tom_tip_inetadr", format, MAGIC );

	*to='\0';
	start=format+sizeof(MAGIC)-1;
#undef MAGIC

	ende=tip_getpart(start,"}");
	if (!ende)
		return TIP_ERROR;

	duplicate=malloc(ende-start+1);
	if (!duplicate)
		goto abbruch;
	memcpy(duplicate,start,ende-start);
	duplicate[ende-start]=0;

	tmpbuf1=malloc(1024);
	if (!tmpbuf1)
		goto abbruch;
	len=tip_interpret(tab,duplicate,tmpbuf1,1024);
	if (len==TIP_ERROR)
		goto abbruch;

	domp=strchr(tmpbuf1,'@');
	if (domp)
	{
		*domp++=0;
		strtrim(domp);
	}
	strtrim(tmpbuf1);

	tmpbuf2=malloc(1024);
	if (!tmpbuf2)
		goto abbruch;

	ub_canonical(tmpbuf1,tmpbuf2);
	len=strlen(tmpbuf2);
	q=tmpbuf2;
	while (*q)
	{
		if (isspace(*q))
			*q='_';
		q++;
	}

	/* OK, Localpart ist fertig */
	domain=r_domainize(domp ? domp : MYDOMAINPART);
	len+=strlen(domain)+1;

	if (len+1>maxlen)
		goto abbruch;
	sprintf(to,"%s@%s",tmpbuf2,domain);

	ok=1;
	*next=ende+1;
abbruch:
	if (tmpbuf1)
		free(tmpbuf1);
	if (tmpbuf2)
		free(tmpbuf2);
	if (domain)
		free(domain);
 	if (duplicate)
		free(duplicate);
	if (ok)
		return len; /* Die +1 kommt hinterher von tip_interpret */
	else
		return TIP_ERROR;
}


/*
 * ${ucanon  User Name } -> "user name"
 */
static size_t 
tom_tip_ucanon(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen)
{
	char *ende=NULL;
	const char *start;
	int ok=0;
	char *duplicate=NULL;
	char *tmpbuf=NULL;
 	size_t lentmp=0;
 	USERNR unr;
 	USER udat;
	const char *p;
	char *q;
	BOOLEAN underscore=FALSE;

#define MAGIC1 "${ucanon "
#define MAGIC2 "${u_canon "
	if (!strncmp(format,MAGIC2,strlen(MAGIC2))) {
		underscore=TRUE;
		start=format+sizeof(MAGIC2)-1;
	} else if (strncmp(format,MAGIC1,strlen(MAGIC1))) {
		tip_errtext = "tom_tip_ucanon called without proper magic ucanon/u_canon" ; 
		return TIP_ERROR; 
	} else
		start=format+sizeof(MAGIC1)-1;

	*to='\0';
#undef MAGIC1
#undef MAGIC2

	ende=tip_getpart(start,"}");
	if (!ende)
		return TIP_ERROR;

	duplicate=malloc(ende-start+1);
	if (!duplicate)
		goto abbruch;
	memcpy(duplicate,start,ende-start);
	duplicate[ende-start]=0;

	tmpbuf=malloc(1024);
	if (!tmpbuf)
		goto abbruch;
	lentmp=tip_interpret(tab,duplicate,tmpbuf,1024);
	if (lentmp==TIP_ERROR)
		goto abbruch;

	if (ub_find_username(&unr,tmpbuf)
		&& ub_read_user(unr,&udat,FALSE))
		p=udat.name;
	else
		p="unknown";
	ub_canonical(p,tmpbuf);
	if (underscore) {
		for (q=tmpbuf;*q;q++) {
			if (isspace(*q))
				*q='_';
		}
	}
	lentmp=strlen(tmpbuf);

	if (lentmp+1>maxlen)
		goto abbruch;

	strcpy(to,tmpbuf);
	ok=1;
	*next=ende+1;
abbruch:
	if (tmpbuf)
		free(tmpbuf);
 	if (duplicate)
		free(duplicate);
	if (ok)
		return lentmp; /* Die +1 kommt hinterher von tip_interpret */
	else
		return TIP_ERROR;
}

/*
 * ${lower  User Name } -> " user name "
 * ${upper  User Name } -> " USER NAME "
 */
static size_t 
tom_tip_lowerupper(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen)
{
	char *ende=NULL;
	const char *start;
	char *duplicate=NULL;
	char *tmpbuf=NULL;
 	size_t lentmp=0;
	BOOLEAN upper=FALSE;

#define MAGIC1 "${upper "
#define MAGIC2 "${lower "
	if (!strncmp(format,MAGIC1,strlen(MAGIC1)))
		upper=TRUE;
	else if (strncmp(format,MAGIC2,strlen(MAGIC2))) 
	{ 
		tip_errtext = "tom_tip_lowerupper called without proper magic upper/lower" ; 
		return TIP_ERROR; 
	} 
	/* else: tolower() */
	
	*to='\0';
	start=format+sizeof(MAGIC1)-1;
#undef MAGIC1
#undef MAGIC2

	ende=tip_getpart(start,"}");
	if (!ende)
		return TIP_ERROR;

	duplicate=malloc(ende-start+1);
	if (!duplicate)
		goto abbruch;
	memcpy(duplicate,start,ende-start);
	duplicate[ende-start]=0;

	tmpbuf=malloc(1024);
	if (!tmpbuf)
		goto abbruch;
	lentmp=tip_interpret(tab,duplicate,tmpbuf,1024);
	if (lentmp==TIP_ERROR)
		goto abbruch;

	if (upper)
		strupr(tmpbuf);
	else
		strlwr(tmpbuf);
	lentmp=strlen(tmpbuf);

	if (lentmp+1>maxlen)
		goto abbruch;

	strcpy(to,tmpbuf);
	*next=ende+1;
abbruch:
	if (tmpbuf)
		free(tmpbuf);
 	if (duplicate)
		free(duplicate);
	return lentmp; /* Die +1 kommt hinterher von tip_interpret */
}

/*
 * ${dd.mm.yy YYMMDDHHMM[SS] } -> "DD.MM.YY"
 * ${yy_mm_dd YYMMDDHHMM[SS] } -> "YY_MM_DD"
 * ${hh:mm:ss YYMMDDHHMM[SS] } -> "HH:MM:SS"
 */
static size_t 
tom_tip_datum_zeit(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen)
{
	char *ende=NULL;
	const char *start;
	char *duplicate=NULL;
	char *tmpbuf=NULL;
 	size_t lentmp=0;
	struct tm tm;
	const char *ftim_format;

#define MAGIC1 "${dd.mm.yy "
#define MAGIC2 "${yy_mm_dd "
#define MAGIC3 "${hh:mm:ss "
	if (strncmp(format,MAGIC1,strlen(MAGIC1))==0)
	{
		ftim_format="%d.%m.%y";
		start=format+sizeof(MAGIC1)-1;
	}
	else if (strncmp(format,MAGIC2,strlen(MAGIC2))==0)
	{
		ftim_format="%y_%m_%d";
		start=format+sizeof(MAGIC2)-1;
	}
	else if (strncmp(format,MAGIC3,strlen(MAGIC3))==0)
	{
		ftim_format="%H:%M:%S";
		start=format+sizeof(MAGIC3)-1;
	}
	else
	{ 
		tip_errtext = "tom_tip_datum_zeit called without proper magic upper/lower" ; 
		return TIP_ERROR; 
	} 
	*to='\0';
#undef MAGIC1
#undef MAGIC2
#undef MAGIC3

	ende=tip_getpart(start,"}");
	if (!ende)
		return TIP_ERROR;

	duplicate=malloc(ende-start+1);
	if (!duplicate)
		goto abbruch;
	memcpy(duplicate,start,ende-start);
	duplicate[ende-start]=0;

	tmpbuf=malloc(1024);
	if (!tmpbuf)
		goto abbruch;
	lentmp=tip_interpret(tab,duplicate,tmpbuf,1024);
	if (lentmp==TIP_ERROR)
		goto abbruch;

	if (!mtzeit_to_tm(tmpbuf,&tm))
	{
		lentmp=TIP_ERROR;
		goto abbruch;
	}
	if (NULL==fast_strftime(to,maxlen-1,ftim_format,&tm))
	{
		lentmp=TIP_ERROR;
		goto abbruch;
	}
	lentmp=strlen(to);

	*next=ende+1;
abbruch:
	if (tmpbuf)
		free(tmpbuf);
 	if (duplicate)
		free(duplicate);
	return lentmp; /* Die +1 kommt hinterher von tip_interpret */
}

BOOLEAN
define_itoken(const char *name, const char *value)
{
    tom_tip_tmptoken_t *akt=token_anker;
    tom_tip_tmptoken_t *last=NULL;
    tom_tip_tmptoken_t *neu;

    if (!value)
    {
        /* undef ... */
        akt=token_anker;
        while (akt)
        {
            if (!strcmp(akt->name,name))
            {
                if (last)
                    last->next=akt->next;
                else
                    token_anker=akt->next;
                free(akt->name);
                free(akt->value);
                free(akt);
                return TRUE;
            }
            last=akt;
            akt=akt->next;
        }
        return TRUE;
    }

    akt=token_anker;
    while (akt)
    {
        int vgl;
        vgl=strcmp(akt->name,name);
        if (vgl == 0)
        {
            char *n=strdup(value);
            if (!n)
                return FALSE;
            free(akt->value);
            akt->value=n;
            return TRUE;
        }
        else if (vgl > 0)
        {
            neu=malloc(sizeof(tom_tip_tmptoken_t));
            if (!neu)
                return FALSE;
            neu->name=strdup(name);
            if (!neu->name)
            {
                free(neu);
                return FALSE;
            }
            neu->value=strdup(value);
            if (!neu->value)
            {
                free(neu->name);
                free(neu);
                return FALSE;
            }
            neu->next=akt;
            if (!last)
                token_anker=neu;
            else
                last->next=neu;
            return TRUE;
        }
        last=akt;
        akt=akt->next;
    }
    neu=malloc(sizeof(tom_tip_tmptoken_t));
    if (!neu)
        return FALSE;
    neu->name=strdup(name);
    if (!neu->name)
    {
        free(neu);
        return FALSE;
    }
    neu->value=strdup(value);
    if (!neu->value)
    {
        free(neu->name);
        free(neu);
        return FALSE;
    }
    neu->next=NULL;
    if (last)
        last->next=neu;
    else
        token_anker=neu;
    return TRUE;
}

/*
 * ${macro any} -> "macro value", interpretiert!
 */
size_t
tom_tip_tmptoken_fn(tip_t *tab, const char *format,
    const char **next, char *to, size_t maxlen)
{
    const char *ende=NULL;
    const char *start=NULL;
    tom_tip_tmptoken_t *akt;
    size_t vgllen;
#define MAGIC "${"

    PARANOIA_STRING("tom_tip_tmptoken_fn",format, MAGIC );

    *to='\0';
    start=format+sizeof(MAGIC)-1;
#undef MAGIC

    ende=strchr(start,'}');
    if (!ende)
        return TIP_ERROR;

    vgllen=ende-start;
    *next=ende+1;

    akt=token_anker;
    while (akt)
    {
        int di;
        di=strncmp(akt->name,start,vgllen);
        if (di>0)
            break;
        if (di==0)
        {
			size_t len=strlen(akt->value);
			if (len+1 >= maxlen)
				return TIP_ERROR;
			strcpy(to,akt->value);
			return len;
        }
        akt=akt->next;
    }
    return tip_interpret(tab,"undefined",to,maxlen);
}

#if 0
	template
static size_t 
tom_tip_XXXXXX(tip_t *tab, const char *format, 
	const char **next, char *to, size_t maxlen)
{
	char *ende=NULL;
	const char *start;
	int ok=0;
	char *duplicate=NULL;
	char *tmpbuf=NULL;
 	size_t lentmp=0;

	char *p;
#define MAGIC "${XXXXXX "

	PARANOIA_CHAR("tom_tip_ucanon", format, MAGIC );

	*to='\0';
	start=format+sizeof(MAGIC)-1;
#undef MAGIC

	ende=tip_getpart(start,"}");
	if (!ende)
		return TIP_ERROR;

	duplicate=malloc(ende-start+1);
	if (!duplicate)
		goto abbruch;
	memcpy(duplicate,start,ende-start);
	duplicate[ende-start]=0;

	tmpbuf=malloc(1024);
	if (!tmpbuf)
		goto abbruch;
	lentmp=tip_interpret(tab,duplicate,tmpbuf,1024);
	if (lentmp==TIP_ERROR)
		goto abbruch;

	/* work */

	ok=1;
	*next=ende+1;
abbruch:
	if (tmpbuf)
		free(tmpbuf);
 	if (duplicate)
		free(duplicate);
	if (ok)
		return lentmp; /* Die +1 kommt hinterher von tip_interpret */
	else
		return TIP_ERROR;
}
#endif

