
#include "config.h"

#include <sys/types.h>
#include <unistd.h>

#include "jobs.h"

#ifdef WITH_JOBCONTROL
job_t *jc_first_job;
pid_t jc_shell_pgid;
struct termios jc_shell_tmodes;
int jc_shell_terminal;
int jc_shell_is_interactive;



/* Find the active job with the indicated PGID.  */
job_t *
jc_find_job (pid_t pgid)
{
	job_t *j;

	for (j = jc_first_job; j; j = j->next)
		if (j->pgid == pgid)
			return j;
	return NULL;
}

	 /* Return true if all processes in the job have stopped or completed.  */
int
jc_job_is_stopped (job_t * j)
{
	if (!j->completed && !j->stopped)
		return 0;
	return 1;
}

	 /* Return true if all processes in the job have completed.  */
int
jc_job_is_completed (job_t * j)
{
	if (!j->completed)
		return 0;
	return 1;
}

	 /* Make sure the shell is running interactively as the foreground job
	  * before proceeding. */

void
jc_init_shell (void)
{

	/* See if we are running interactively.  */
	jc_shell_terminal = STDIN_FILENO;
	jc_shell_is_interactive = isatty (jc_shell_terminal);

	if (jc_shell_is_interactive) {
		/* Loop until we are in the foreground.  */
		while (tcgetpgrp (jc_shell_terminal) != (jc_shell_pgid = getpgrp ()))
			kill (-jc_shell_pgid, SIGTTIN);

		/* Ignore interactive and job-control signals.  */
		signal (SIGINT, SIG_IGN);
		signal (SIGQUIT, SIG_IGN);
		signal (SIGTSTP, SIG_IGN);
		signal (SIGTTIN, SIG_IGN);
		signal (SIGTTOU, SIG_IGN);
		signal (SIGCHLD, SIG_IGN);

		/* Put ourselves in our own process group.  */
		jc_shell_pgid = getpid ();
		if (setpgid (jc_shell_pgid, jc_shell_pgid) < 0) {
			perror ("Couldn't put the shell in its own process group");
			exit (1);
		}
		/* Grab control of the terminal.  */
		tcsetpgrp (jc_shell_terminal, jc_shell_pgid);

		/* Save default terminal attributes for shell.  */
		tcgetattr (jc_shell_terminal, &jc_shell_tmodes);
	}
}

#endif
