/* main.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
*/

/**************************************************************************
*                                                                         *
*                            TOM Mailboxsystem                            *
*                                                                         *
*-------------------------------------------------------------------------*
*                                                                         *
*                  Programmierer    :  Frank Baumgart    (FB)             *
*                                   :  Uwe Ohse          (UO)             *
*                                                                         *
* Projektstart: 14.10.1992 - 22:41:12                                     *
**************************************************************************/

#include "tomsysdep.h" 

#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <errno.h>
#include <syslog.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <signal.h>
#include <fnmatch.h>
#include <conflib.h>
#ifdef HAVE_LOCALE_H
# ifndef hpux
#  include <locale.h>
# else
   /* seems not to compile */
#  undef HAVE_LOCALE_H
# endif
#endif
#include <fcntl.h>
#include <pwd.h>

#include "tom_termcap.h"
/* manche curses definieren TRUE/FALSE unbedingt */
#include <portab.h>

#include "getopt.h"
#include "tompaths.h"
#include "prototyp.h"
#ifdef OFFLINE2
#include "offline/inc/mdb_global.h"
#else
#include "offline/mdb_global.h"
#endif
#include "io.h"
#include "modem.h"
#include "terminal.h"
#include "user.h"
#include "brett.h"
#include "event.h"
#include "bin_msg.h"
#include "business.h"
#include "box_conf.h"
#include "log.h"
#include "getty.h"
#include "tausch.h"
#include "cslib.h"
#include "protokoll.h"
#include "aktdata.h"
#include "routing.h"
#include "tausch.h"
#include "msg.h"
#include "queues.h"
#include "lockfile.h" /* libquark */
#include "owrite_cmdl.h"
#include "tsignal.h"
#include "argument.h"
#ifdef HAVE_LIBTCL
#include "tomtcl.h"
#endif
#include "xstrtoul.h"

USER user;

int user_in_fd=STDIN_FILENO;
int user_out_fd=STDOUT_FILENO;
int sysop_in_fd=-1;
int sysop_out_fd=-1;

extern char **environ;

#ifdef HAVE_DOSDRIVES
#ifdef __OS2__
#  define DRIVE        "C:" PS "look" PS
#else
#  define DRIVE        "A:" PS
#endif
#else
#  define DRIVE
#endif

/* char *tom_port_config_dir = NULL; */
const char *program_name;

BOOLEAN modem_vorhanden;
static BOOLEAN mailbox(char *, const char *, const char *);

BYTE
	batchB = FALSE,
	pm_schutzB,								/* keine Cons.ausgabe bei DOWNL,LES,SEND */
	loginB = FALSE,
	offlineB = FALSE;

WORD
	anz_lineW = 24,
	anz_spalten = 80,
	temp_levelW = -1,
	count_crW = 0,							/* Zeilenzhler fr <MEHR> */
	msgtyp = 0,
	vtmodeW = T_ASCII;

long
	actual_uploadL = 0,
	actual_downloadL = 0;

/* dubiose globale Variablen (durch Basic eingeschlichen) */

BYTE
	tausch_auto_loginB,
	tausch_outB;

int logout_modus;

BOOLEAN mehr_disabled;
BOOLEAN mitten_im_chat;

char *device_lockfile=NULL;
BOOLEAN device_locked=FALSE;

char *global_event_hack;

extern volatile int got_sigusr1;
extern volatile int got_sigusr2;

#ifdef HAVE_DOSDRIVES
char known_drivesS[30];
#endif

struct anrufdaten akt_anruf;
struct connectiondaten akt_connection;


static void setup_environment(void);
static void cmdline_bootstrap(void);
static void init_databases(void);
static int cmdline_batch(void);
#ifdef HAVE_LIBTCL
static int cmdline_tclscript(void);
#endif
static int cmdline_queue_run(void);
static int cmdline_tausch(void);
static void init_files(void);
static void do_exec(char *cmd);
static BOOLEAN do_cmd(void);

BOOLEAN tom_locked;

	/* 
	 * Aufzurufen nach Programmfehlern, wenn zu befrchten ist, da
	 * Speicherverwaltung oder sonstige wichtige Strukturen der
	 * Standardlib nicht mehr funktionsfhig sind. 
	 */
void 
tom_leave_after_error(int exitcode)
{
	if (chdir(box_conf.quark_dir)==0)
	{
		if (device_lockfile && device_locked)
			unlink(device_lockfile);
		if (tom_locked)
			unlink("LCK..tom");
	}
	_exit(exitcode);
}

void 
tom_leave(enum tomexitcodes exitcode, const char *eventid)
{
	char fname[PATH_MAX];
	if (tom_locked)
	{
		strcpy(fname,"LCK..tom");
		lock_delete(fname);
	}
	if (device_lockfile && device_locked)
		lock_delete(device_lockfile);
#ifdef LEAK_CHECK
	GC_gcollect();
#endif
	if (eventid)
	{
		int ret;
		int fd;
		const char *new_argv[4];
		struct stat st;
		io_restore_stdin();
		/* wozu der Aufwand? Wenn wir hier ankommen, sind mglicherweise
		 * stdin/out/err geschlossen (z.B. beim manuellen Auslsen von
		 * Events - da ist die Verbindung ja weg), und stdin/out/err
		 * sollten irgendwo verbunden sein!
		 */
		if (fstat(0,&st))
		{
			fd=open("/dev/null",O_RDONLY);
			if (fd==-1)
			{
				syslog(LOG_ERR,"cannot open /dev/null: %s",
					strerror(errno));
				exit(1);
			}
			else if (fd!=0)
			{
				dup2(fd,0);
				close(fd);
			}
		}
		if (fstat(1,&st))
		{
			fd=open("/dev/null",O_WRONLY);
			if (fd==-1)
			{
				syslog(LOG_ERR,"cannot open /dev/null: %s",
					strerror(errno));
				exit(1);
			}
			else if (fd!=1)
			{
				dup2(fd,1);
				close(fd);
			}
		}
		if (fstat(2,&st))
		{
			fd=open("/dev/null",O_RDWR);
			if (fd==-1)
			{
				syslog(LOG_ERR,"cannot open /dev/null: %s",
					strerror(errno));
				exit(1);
			}
			else if (fd!=2)
			{
				dup2(fd,2);
				close(fd);
			}
		}
		new_argv[0]="sh";
		new_argv[1]="-c";
		new_argv[2]=concat(box_conf.bin_dir, "do_event.sh \"",eventid,"\" \"",
			akt_connection.ttyname,"\"",NULL);
		new_argv[3]=0;

		/* diese Queue lschen vor einem Exec */
		dqueue_delete_queue(QUEUE_EXIT_NOT_ON_EXEC);
		/* Alle Cleanupfunktionen abarbeiten */
		dqueue_work_all();

#define HACK
	/* Profilerversion ist ELF, Normalversion aout */
#ifdef PROFILING
		moncontrol(0);
		_mcleanup();
#endif

		ret=execv ("/bin/sh", (char *const *) new_argv);
		syslog(LOG_ERR,"cannot execute event %s, execv failed with %d",
			eventid,ret);
		exit(1);
	}
	/* um dem, der an die Box will, eine Chance zu geben */
	if (got_sigusr1 || got_sigusr2)
		sleep(3);

	io_restore_stdin();
	dqueue_work_all();

	exit((signed int) exitcode);
}

static void 
cleanup_tmpdirs(void *dummy TOM_ATTRIB_UNUSED)
{
	char buf[128];
	sprintf(buf,"%s/tom.%lu",box_conf.tmp_dir,(ULONG) getpid());
	rm_dir_content(buf);
	rmdir(buf);
}

static void 
cleanup_stdin(void *dummy TOM_ATTRIB_UNUSED)
{
	io_restore_stdin();
}

int 
main(int argc,char **argv)
{
	int ret=LEAVE_OK;
	char *eventid=NULL;
	BOOLEAN inquire_ok;
	char *pwd_realname=NULL; /* Name in der Passwd fr Shelluser */
	char *pwd_shortname=NULL; /* Name in der Passwd fr Shelluser */
	char *hint_user=NULL;

#ifdef HAVE_SIGINTERRUPT
	/* BSD braindamage */
	siginterrupt(SIGALRM,1);
	siginterrupt(SIGHUP,1);
#endif

	setup_SIGHUP(TRUE);
	io_save_stdin();
	if (argv[0]) {
		program_name=strrchr(argv[0],'/');
		if (program_name)
			program_name++;
		else
			program_name=argv[0];
		program_name=XSTRDUP(program_name); /* we modify argv! */
	} else
		program_name="tom";


	/* Zum berschreiben mu cmdline_overwrite Anfang und Ende wissen */
	cmdline_save_param(argc,argv);
	/* Kommandozeilenauswertung jetzt vornehmen */
	{
		uid_t e,r;
		BOOLEAN is_setuid;
		r=getuid();
		e=geteuid();
		is_setuid=(r!=e && r!=0);
		cmdline_evaluation(argc,argv, is_setuid);
	}

	/* Konfigdatei auslesen */
	/* das mu vor getty_init geschehen, da dort auf eine Configvariable
	 * console_init_prg zugegriffen wird 
	 */
	if (!read_quark_configuration(
		argument_cnffile ? argument_cnffile : SYSCONFDIR "/quark.cnf"))
	{
		fprintf(stderr,"Auswertung von %s fehlgeschlagen\n",
			argument_cnffile ? argument_cnffile : SYSCONFDIR "/quark.cnf");
		syslog(LOG_ERR,"Auswertung von %s fehlgeschlagen\n",
			argument_cnffile ? argument_cnffile : SYSCONFDIR "/quark.cnf");
		exit(1);
	} else if (argument_check_configuration) {
		fprintf(stderr,"%s scheinbar ok\n",
			argument_cnffile ? argument_cnffile : SYSCONFDIR "/quark.cnf");
		exit(0);
	}

	if (argument_convdate) {
		char buf[129];
		char *s;
		char *dopp;
		const char *sc;
		time_t t;
		s=XSTRDUP(argument_convdate);
		dopp=strchr(s,':');
		if (dopp) {
			*dopp++=0;
			if (!*dopp)
				dopp=NULL;
		}
		if (!mtzeit_to_time_t(s,&t))
			error(1,0,"kann Datum nicht konvertieren");
		switch ((dopp ? *dopp : 'c')) {
		case 'e': /* epoch */
			sprintf(buf,"%lu",(unsigned long) t);
			puts(buf);
			break;
		case 'm': /* mtzeit */
			time_t_to_mtzeit_sec(&t,buf);
			puts(buf);
			break;
		case 'd': /* datum_zeit_string */
			sc=datum_zeit_string(&t);
			puts(sc);
			break;
		default: /* ctime */
			fputs(ctime(&t),stdout);
			break;
		}
		exit(0);
	}


#ifndef THIS_IS_TOMSHELL
	/* das mu vor dem inquire_remote geschehen */
	if (argument_getty)
	{
		/* TERM mu wegen des Consolen-Init-Prg gesetzt sein */
		if (argument_term)
		{
			char *p;
			p=concat("TERM=",argument_term,NULL);
			XNULLCHECK(p);
			putenv(p);
		}
		getty_init(argv[0]);
	}
#endif
	inquire_ok=inquire_remote(0);

	if (argument_versioncode)
	{
		puts(VERSION);
		exit(0);
	}
	if (argument_version)
	{
		if (!inquire_remote2(0))
		{
			fprintf(stderr,"cannot get further details on stdin\n");
#ifdef LEAK_CHECK
	GC_gcollect();
#endif
			exit(1);
		}

		print_version(printf);

		printf(" peer %s\n", akt_connection.peer ? "TRUE" : "FALSE");
		printf(" console %s\n", akt_connection.is_console ? "TRUE" : "FALSE");
		printf(" modem %s\n", akt_connection.is_modem ? "TRUE" : "FALSE");
		printf(" modemlines %s\n", 
			akt_connection.has_modem_lines ? "TRUE" : "FALSE");
		printf(" tty %s\n", 
			akt_connection.is_tty ? akt_connection.ttyname : "none");
		if (akt_connection.rusername)
			printf(" remoteuser %s\n",akt_connection.rusername);
		if (akt_connection.rhostname)
			printf(" remotehost %s\n",akt_connection.rhostname);
		if (akt_connection.rhostaddr)
			printf(" remoteaddr %s\n",akt_connection.rhostaddr);
		printf(" authentication %s\n", akt_connection.auth_ok ? "true" : "false");
		printf(" errorfree %s\n", akt_connection.error_free ? "true" : "false");
		if (akt_connection.localport)
			printf(" localportnr %u",(unsigned) akt_connection.localport);
		if (akt_connection.remoteport)
			printf(" remoteportnr %u",(unsigned) akt_connection.remoteport);

		/* okay, a little hack to make the linker include some thinhs */
		if (time(0)==27) /* yes, we don't really use that function */
			sym_junk_function();

		puts("");
		printf("sizeof(BRETT,USER)=(%ld,%ld)\n",
			(long) sizeof(BRETT), (long) sizeof(USER));
		fflush(stdout);
#ifdef LEAK_CHECK
		GC_gcollect();
#endif
		exit(0);
	}
	if (!argument_auto && !inquire_ok)
	{
		fprintf(stderr,"cannot get details on stdin\n");
		exit(1);
	}

	/* Dinge erledigen, die jetzt getan werden mssen, mit 
	 * unvernderter Umgebung und Verzeichnis.
	 */

	/* Alle locale-Kategorien auf die aus dem Environment gelesenen Werte
	 * setzen. *Vor* etwaigen Ausgaben (aber nur, weil es schner ist,
	 * nicht weil es wichtig ist).
	 */
	setlocale (LC_ALL, "");

	openlog("tom",LOG_PID | LOG_CONS| LOG_ODELAY,LOG_LOCAL0);

	/* User- und Gruppen-IDs merken */
	akt_connection.ruid=getuid();
	akt_connection.euid=geteuid();
	akt_connection.rgid=getgid();
	akt_connection.egid=getegid();
	/* Falls wir der realen uid 0 laufen, vergessen wir diese Sonderrechte 
	 * nun (setuid als root setzt euid,ruid und saved uid). Unbedingt ntig 
	 * ist das nicht, aber es gibt auch keinen Grund warum die Box unter 
	 * Rootrechten laufen sollte.
	 */
	if (akt_connection.ruid == 0)
	{
		if (akt_connection.euid==0)
		{
			/* hmm - *ganz* root? Das mssen wir ndern. */
			struct passwd *pwd;
			if (box_conf.system_user_name)
				pwd=getpwnam(box_conf.system_user_name);
			else
				pwd=getpwnam("quark");
			if (!pwd)
			{
				fprintf(stderr,
					"%s: kann User-ID fr die Box nicht ermitteln\n",
						argv[0]);
				exit(1);
			}
			akt_connection.euid=pwd->pw_uid;
			if (akt_connection.egid==0 && akt_connection.rgid==0)
				akt_connection.egid=pwd->pw_gid;
		}

		/* Root werden */
		setuid(0);
		/* Rootstatus loswerden: setuid als root setzt ruid, euid und 
		 * saved uid. */
		set_box_ids();
		akt_connection.ruid=akt_connection.euid;
		akt_connection.rgid=akt_connection.egid;
	}

	/* Trojanische Pferde vermeiden */
	if (akt_connection.euid!=akt_connection.ruid)
	{
		struct passwd *pwd;
		akt_connection.shelluser=TRUE;

		pwd=getpwuid(akt_connection.ruid);
		if (pwd)
		{
			/* pwd->pw_gecos enthlt oft nicht nur den Usernamen, sondern
			 * durch Kommas getrennt auch noch anderes Zeug 
			 */
			char *duplicate;
			char *p;
			duplicate=XSTRDUP(pwd->pw_gecos);
			p=strchr(duplicate,',');
			if (p)
				*p=0;
			if (*duplicate)
				pwd_realname=XSTRDUP(duplicate);
			free(duplicate);
			pwd_shortname=XSTRDUP(pwd->pw_name);

			/* Batch, Kommando und Tausch vie Kommandozeile erwarten,
			 * da argument_user gesetzt ist.
			 */
			if (argument_auto)
			{
				if (pwd_realname)
					argument_user=XSTRDUP(pwd_realname);
				else
					argument_user=XSTRDUP(pwd_shortname);
			}
		}
		else
		{
			/* Hm ... */
			printf("Sie sind nicht in der Passwd eingetragen!\n");
			fflush(stdout);
		}
	}
	else
	{
		akt_connection.user_environ=NULL;
		akt_connection.shelluser=0;
	}


	if (akt_connection.shelluser)
	{
		char *save[20];
		int zahl=0;
		int savezahl=0;
#define SAVE(s) do { const char *q=getenv(s); if (q) { \
			save[savezahl]=concat(s,"=",q,NULL); \
			XNULLCHECK(save[savezahl]); \
			savezahl++; \
		} } while (0)
		/* Den Environment des Users ist nicht zu trauen, wenn wir
		 * setuid laufen. No way, sorry. 
		 * Wir bernehmen, was bekannt und ungefhrlich ist
		 */ 
		SAVE("COLUMNS");
		SAVE("LANG");
		SAVE("LC_ALL");
		SAVE("LC_COLLATE");
		SAVE("LC_CTYPE");
		SAVE("LC_MESSAGES");
		SAVE("LC_MONETARY");
		SAVE("LC_NUMERIC");
		SAVE("LC_TIME");
		SAVE("LINES");
		/* TMPDIR ist *nicht* ungefhrlich: config/... */
		SAVE("TERM");
		/* TERMCAP ist nicht ungefhrlich, kann bufferoverflows bewirken */
		SAVE("TZ");
		SAVE("TOM");
#undef SAVE
		akt_connection.user_environ=environ;
		environ=XMALLOC((3+savezahl)*sizeof(char *));
		environ[zahl++]=XSTRDUP("PATH=/usr/local/bin:/usr/bin:/bin");
		environ[zahl++]=XSTRDUP("SHELL=/bin/sh");
		while (savezahl) 
		{
			savezahl--;
			environ[zahl++]=save[savezahl];
		}
		environ[zahl]=NULL;
	}

	if (!box_conf.default_chef)
		box_conf.default_chef = "Sysop";

	if (argument_dump_cnf)
	{
		dump_quark_configuration(stdout);
#ifdef LEAK_CHECK
		GC_gcollect();
#endif
		exit(0);
	}
	if (argument_dump_shell_vars) {
		dump_shell_vars(stdout);
		exit(0);
	}

	/* reale uid auf effektive setzen (damit geforkte Prozesse nur
	 * unter unserer eigenen id laufen)
	 */
	set_box_ids();

	/* 
	 * Owner: Alles
	 * Group: Lesen + Ausfhren
	 * Other: Nichts.
	 */
	umask(022);

	/* Am Programmende alle Warteschlangen abarbeiten */
	atexit(dqueue_work_all);

	/* Environment auswerten, Arbeitsverzeichnis wechseln */
	setup_environment();

	/* Boxinternes Environment ergnzen */
	if (!akt_connection.shelluser)
	{
		char *p=concat("HOME=",box_conf.quark_dir,NULL);
		putenv(p);
	}

	/* Die Temporrverzeichnisse am Programmende lschen */
	dqueue_add(QUEUE_EXIT,cleanup_tmpdirs,NULL);

	/* Fr sauberes Cleanup von stdin sorgen */
	dqueue_add(QUEUE_EXIT,cleanup_stdin,NULL);

	if (akt_connection.is_tty && isatty(1) && !argument_auto)
		io_resize_terminal(STDIN_FILENO,STDOUT_FILENO,FALSE);

	/* USR1 sendet ein anderer Prozess, der an die Box will */
	signal(SIGUSR1,SIG_IGN);
	/* USR2 ist "urgent" - jemand mu unbedingt an die Box (Event).
	 * Kein Grund sich *sofort* zu beenden.
	 */
	signal(SIGUSR2,SIG_IGN);
	/* TERM ist "sofortiges Ende" */
	signal(SIGTERM,SIG_IGN);

	/* Lockfile anlegen - erst jetzt, *nach* dem chdir */
#ifndef THIS_IS_TOMSHELL
	if (!argument_getty) /* bei getty wird erst spter gelockt */
#endif
	{
		int max_tries=-1; /* unbegrenzt */

		if (!argument_auto)
		{
			/* Ein User will hinein. Solche Leute warten nicht gerne. 
			 */
			max_tries=9;
		}
		else
			max_tries=-1; /* unbegrenzt */
		if (!lock_tom(max_tries))
		{
			/* Meldung und syslog schon ausgegeben */
			tom_leave(LEAVE_ERROR,NULL);
		}
	}

#ifndef THIS_IS_TOMSHELL
	/* Bootstraping - neue User/Brettdaten anlegen (aber niemals 
	 * berschreiben). Die Routine beendet das Programm.
	 */
	if (argument_bootstrap)
		cmdline_bootstrap();
#endif

	/* Brett/User/Nachrichtendatenbank initialisieren */
#ifndef THIS_IS_TOMSHELL
	if (!argument_getty)
#endif
		init_databases();

	/* jetzt knnen wir fr Shelluser die Userdaten prfen */
	if (akt_connection.shelluser 
		&& ((pwd_realname && *pwd_realname) || pwd_shortname))
	{
		USERNR unr;
		if (pwd_shortname && ub_find_username(&unr, pwd_shortname))
		{
			/* Dem Loginnamen in /etc/passwd kann man immer trauen */
			akt_connection.shellusernummer=unr;
			hint_user=XSTRDUP(pwd_shortname);
		}
		else if (pwd_realname && ub_find_username(&unr, pwd_realname))
		{
			/* Dem Realname in /etc/passwd kann man nicht trauen,
			 * oft knnen User chfn aufrufen ... */
			USER u;
			if (!ub_read_user(unr, &u,FALSE))
			{
				fprintf(stderr,"Benutzerdaten nicht lesbar\n");
				ret=LEAVE_ERROR;
				goto go_out;
			}
			if ((uid_t) u.assigned_uid!=akt_connection.ruid)
			{
				fprintf(stderr,
"Sie sind diesem Benutzer nicht zugeordnet, Autologin unmglich\n");
				pwd_realname=NULL;
				pwd_shortname=NULL;
			}
			else
			{
				akt_connection.shellusernummer=u.nummer;
				hint_user=XSTRDUP(pwd_realname);
			}
		}
	}
	else if (!akt_connection.shelluser && argument_user && *argument_user)
	{
		hint_user=XSTRDUP(argument_user);
	}
	else
		hint_user=NULL;

	if (box_conf.catch_signals)
		catch_fault_signals(TRUE);

	/* diverse Statusdateien einlesen */
	init_files();

	/* USR2 ist "urgent" - jemand mu unbedingt an die Box (Event) */
	catch_SIGUSR2(TRUE);
	/* TERM ist "sofortiges Ende" */
	catch_SIGTERM(TRUE);

	/* Zeichensatzwandlungsroutinen initialisieren */
	charset_init(mbprintf);

	if (argument_search_id) {
		char g[256],u[256],a[256];
		char *p;
		ret=0;
		for (p=argument_search_id;*p;p+=strlen(p)+1) {
			printf("<%s>:\n",p);
			if (msgid_find_infos(p,g,u,a))
			{
				if (*g)
					printf("  Gruppe: %s\r\n",g);
				if (*u)
					printf("  User  : %s\r\n",u);
				if (*a)
					printf("  Autor : %s\r\n",a);
			}
			else 
			{
				puts("  nicht vorhanden");
				ret=1;
			}
		}
		goto go_out;
	}
	/*
	 * Gettyroutinen sollen den Port und das Modem initialisieren.
	 */
#ifndef THIS_IS_TOMSHELL
	if (argument_getty)
		getty_devinit();
#endif
	if (argument_logentry) {
		anrufdaten_init(ANRUF_CCOMMAND);
		if (argument_seit) {
			time_t t;
			if (mtzeit_to_time_t(argument_seit,&t)) {
				akt_anruf.startzeit=t;
				akt_anruf.userstartzeit=t;
			}
		}
		strncpy(akt_anruf.name,argument_user,sizeof(akt_anruf.name)-1);
		akt_anruf.name[sizeof(akt_anruf.name)-1]=0;
		strncpy(akt_anruf.free_string,argument_logentry,sizeof(akt_anruf.free_string)-1);
		akt_anruf.free_string[sizeof(akt_anruf.free_string)-1]=0;
	}

	if (argument_command||argument_batch)
		ret=cmdline_batch();
	else if (argument_tclscript) {
#ifdef HAVE_LIBTCL
		ret=cmdline_tclscript();
#else
		error(0,0,"no tcl support compiled in");
		ret=LEAVE_ERROR;
#endif
	} else if (argument_tausch) {
		ret=cmdline_tausch();
	} else if (argument_queue_run) {
		ret=cmdline_queue_run();
	} else if (argument_logentry) {
		login_protokoll_write();
	} else {
		/* Irgendeine Sorte von Login (interaktiv) */
		int ctyp;
#ifndef THIS_IS_TOMSHELL
		if (argument_getty)
		{
			cmdline_overwrite("TOM: getty waiting");
			io_mode_work(STDIN_FILENO,TRUE);
			ret = getty_wait(&eventid,&ctyp);
			if (ret==LEAVE_LOGIN)
				init_databases();
		}
		else
#endif
		{
			/* Das ist also eine Verbindung, bei der wir nicht
			 * selbst das Modem abnehmen knnen. Ergo bekommen
			 * wir keine "akt_connection.carrier*"-Werte.
			 */
			if (!inquire_remote2(0))
			{
				fprintf(stderr,"cannot get further details on stdin\n");
				exit(1);
			}
			anrufdaten_init(ANRUF_USER);
			ctyp=CARRIER_LOGIN;
			ret=LEAVE_LOGIN;
			if (akt_connection.is_tty && akt_connection.ttyname)
			{
				if (box_conf.console_wildcard && 
					0==fnmatch(box_conf.console_wildcard,
						akt_connection.ttyname,0))
				{
					akt_connection.is_console=TRUE;
				}
				if (box_conf.modem_wildcard && 
					0==fnmatch(box_conf.modem_wildcard,
						akt_connection.ttyname,0))
				{
					akt_connection.is_modem=TRUE;
				}
			}

			io_mode_work(STDIN_FILENO, FALSE);

			if (akt_connection.is_tty)
			{
				const char *p;
				size_t l;
				p=strrchr(akt_connection.ttyname,'/');
				if (p)
					p++;
				else
					p=akt_connection.ttyname;
				l=strlen(p);
				if (l>4)
					p+=l-4;
				akt_connection.carrierstring=XSTRDUP(p);
			}
			else
			{
				/* kein tty - raw tcp connection? */
				akt_connection.carrierstring=XSTRDUP("raw");
			}
		}

		if (ret==LEAVE_LOGIN)
		{
			/* soweit alles OK */
			signal(SIGUSR1,SIG_IGN);
			/* tom_port_config_dir-Pfad zusammensetzen */
			make_port_sys_path();
#ifndef THIS_IS_TOMSHELL
			if (ctyp==CARRIER_SPEC)
			{
				if (tom_locked)
				{
					char fname[PATH_MAX];
					strcpy(fname,"LCK..tom");
					lock_delete(fname);
					tom_locked=FALSE;
				}
				do_cmd();
			}
			else 
#endif
			if (argument_as_login)
				mailbox(hint_user,argument_user,argument_prompt);
			else
				mailbox(hint_user,NULL,argument_prompt);
			io_set_CLOCAL(user_in_fd,TRUE);
#ifndef THIS_IS_TOMSHELL
			if (argument_getty)
				getty_deinit();
#endif
			if (global_event_hack)
			{
				eventid=global_event_hack;
				ret=LEAVE_EVENT;
			}
			else
				ret=LEAVE_RESTART;
		}
	}

go_out:
	bb_exit();								/* Brettverwaltung schlieen */

	closelog();

	tom_leave(ret,eventid);
#ifdef LEAK_CHECK
	GC_gcollect();
#endif
	exit (0);								/* keep compiler quiet */
	fprintf(stderr, "exit call failed. Program will continue\n");
	/* -- Nach einer Diskussion in de.alt.folklore.computer */
}


/* ==========================  Start - Routinen  ======================== */

/*
	08.12.92 Frank
*/
static void init_files(void)
{
	read_business();

	rm_dir_content(tom_tmp_dir);
	rm_dir_content(tom_utmp_dir);

} /* init_files */


static BOOLEAN 
do_cmd(void)
{
	char *call_cmd;
	char *buf;
	BOOLEAN ret=FALSE;
	const char *incmd;
	time_t timeout=0;

	akt_anruf.anruftyp = ANRUF_SPEC;
	incmd=akt_connection.carriercmd;
	if (!incmd)
	{
		strcpy(akt_anruf.zuletzt,"NINS"); /* No FAX installed */
		syslog(LOG_ERR,"Kein Programm fr %s",akt_connection.carrierstring);
		goto out;
	}
	if (akt_connection.carrierstring)
	{
		strncpy(akt_anruf.free_string,akt_connection.carrierstring,
			sizeof(akt_anruf.free_string)-1);
		akt_anruf.free_string[sizeof(akt_anruf.free_string)-1]=0;
	}
	{
		const char *q;
		q=strpbrk(incmd," \t");
		if (!q)
			q=incmd;
		else
		{
			while (q>incmd && *q!='/')
				q--;
			if (*q=='/')
				q++;
		}

		strncpy(akt_anruf.name,q,sizeof(akt_anruf.name)-1);
		akt_anruf.name[sizeof(akt_anruf.name)-1]=0;
	}

	/* strncpy(akt_anruf.zuletzt, akt_connection.carrierstring,	 
		sizeof(akt_anruf.zuletzt)-1);
	akt_anruf.zuletzt[sizeof(akt_anruf.zuletzt)-1]=0;
	*/
	if (0==strnicmp(akt_connection.carriercmd,"timeout=",8)) {
		char *pp;
		char *qq;
		timeout=time(NULL)+ strtoul (akt_connection.carriercmd+8, &pp, 10);
		while (isspace(*pp))
			pp++;
		qq=akt_connection.carriercmd; /* ignore this warning. C is missing `friend' */
		akt_connection.carriercmd=XSTRDUP(pp);
		free(qq);
	}
#define CALL_CMD_SIZE (PATH_MAX+2048)
	call_cmd=XMALLOC(CALL_CMD_SIZE);
	if (!do_interpret(akt_connection.carriercmd,call_cmd,CALL_CMD_SIZE))
	{
		syslog(LOG_INFO,"kann Fax/Externkommando nicht interpretieren");
		goto out;
	}
	buf=concat("TOM: calling ",call_cmd,NULL);
	if (buf)
	{
		cmdline_overwrite(buf);
		free(buf);
	}
	if (call_extern(call_cmd,NULL,timeout, TRUE, FALSE))
	{
		syslog(LOG_INFO,"kann Fax/Externkommando nicht ausfuehren");
		goto out;
	}
	free(call_cmd);
#ifdef EXPLIZITE_CARRIERABFRAGE
	if (carrier_still_up())
		strcpy(akt_anruf.zuletzt,"BOX");
#endif
	ret=TRUE;
out:
	login_protokoll_write();
	return ret;
}



/* ======================  Mailbox - Hauptroutine  ====================== */


static BOOLEAN 
mailbox(char *force_user, const char *hint_user, const char *fake_prompt)
{
	BOOLEAN ret=FALSE;
	BOOLEAN carrier_lost=FALSE;
	BOOLEAN no_login=FALSE;
	BOOLEAN first=TRUE;

	io_tty_test(user_out_fd);
#ifdef WITH_JOBCONTROL
	jc_init_shell();
	if (jc_shell_is_interactive) {
		/* ISIG muss fuer Jobcontrol gesetzt werden! */
		io_set_ISIG(STDIN_FILENO,TRUE);
	}
#endif

	do
	{
		BOOLEAN tauschlogin=FALSE;
		BOOLEAN multitausch=FALSE;
		BOOLEAN ist_gast=FALSE;
		BOOLEAN sysopschutzzeichen=FALSE;
		char *cmd=NULL;

		/* Manche Modems senden mehrere Zeilen bei Connect. Weg damit */
		if (akt_connection.is_modem)
		{
			io_drain(user_in_fd);
			io_flush(user_in_fd);
		}
		logout_modus = 0;
		count_crW = 0;
		tausch_outB = 0;

		akt_anruf.chat_status=TRISTATE_NOT_TRIED;
		akt_anruf.tausch_status=TRISTATE_NOT_TRIED;
		akt_anruf.batch_status=TRISTATE_NOT_TRIED;
		akt_anruf.neuuser=FALSE;
		akt_anruf.userstartzeit=time(NULL);

		strcpy(akt_anruf.zuletzt,"LOG");

		loginB = FALSE;
		unset_charset();
		akt_anruf.interaktiv=FALSE;
		impressum(&tauschlogin,&multitausch,&ist_gast,&sysopschutzzeichen,
			force_user, 
			first ? hint_user : NULL,
			first ? fake_prompt : NULL, 
			&cmd);

		user.nummer=0; /* siehe user/ub_read_user.c wg. Sonderbehandlung */

		if (cmd && *cmd)
		{
			char fname[PATH_MAX];
			bb_exit();
			if (tom_locked)
			{
				strcpy(fname,"LCK..tom");
				lock_delete(fname);
			}
			do_exec(cmd);
			/* NOTREACHED */
		}
		else if (login(force_user,
			first ? hint_user : NULL, 
			tauschlogin, &ist_gast,sysopschutzzeichen))
		{
			USERNR old_unr=user.nummer;
			
			/* das ist im Zweifelsfall fr den Gast ntig */
			USER *backup=XMALLOC(sizeof(USER));
			*backup=user;

			loginB = TRUE;
			first=FALSE;
			if (!tauschlogin)
				akt_anruf.interaktiv=TRUE;
			communication(tauschlogin, ist_gast);

			ret=TRUE;
			/* Sortierte Liste der Brettnamen lschen, da nur fr den
			 * jeweiligen User gltig.
			 */
			b_free_sortierung();

			/* Man *mu* nicht Gast bleiben ... */
			if (ist_gast && user.nummer==old_unr)
			{
				backup->erster_login=user.erster_login;
				backup->anrufe=user.anrufe;
				backup->geschriebene_mails=user.geschriebene_mails;
				backup->uploads=user.uploads;
				backup->ul_bytes=user.ul_bytes;
				backup->geschriebene_mails=user.geschriebene_mails;
				backup->downloads=user.downloads;
				backup->dl_bytes=user.dl_bytes;
				backup->txt_bytes=user.txt_bytes;
				ub_write_user(backup,TRUE);
			}
			else
			{
				if (akt_anruf.did_reset==1)
				{
					user.letzter_login=backup->letzter_login;
					user.letzter_login2=backup->letzter_login2;
					user.letzter_login3=backup->letzter_login3;
					ub_write_user(&user,TRUE);
				}
				else if (akt_anruf.did_reset==0)
					ub_write_user(&user,TRUE);
			}

			online_protokoll_close();
			/* Nix Hangup mehr */
			if (logout_modus == 1)
				io_set_CLOCAL(user_in_fd,TRUE);
			if (multitausch)
				logout_modus = 2;

			if (logout_modus == 2)
			{
				/* Relogin im Protokoll vermerken */
				strcpy(akt_anruf.zuletzt,"REL");
				akt_anruf.ist_relogin=TRUE;
			}
#ifdef EXPLIZITE_CARRIERABFRAGE
			else if (carrier_still_up())
				*akt_anruf.zuletzt='\0';
			else
				carrier_lost=TRUE;
#else
			*akt_anruf.zuletzt='\0';
#endif
			/* setzt Endzeit selbst */
			login_protokoll_write();
		}
		else
		{
			strcpy(akt_anruf.name,"No Login");
			login_protokoll_write();
			no_login=TRUE;
			carrier_lost=TRUE;
		}
		force_user=NULL; /* fr Relogins */
	}
	while (logout_modus == 2);

	/* weg vom Modem ... */
	if (!argument_getty) {
		io_restore_stdin();
		close(0);
		open("/dev/null",O_RDONLY,0);
		close(1);
		open("/dev/null",O_WRONLY);
		close(2);
		open("/dev/null",O_RDWR);
		user_in_fd=-1;
		user_out_fd=-1;
	}

	if (akt_connection.is_modem && box_conf.log_quality_command
           && box_conf.log_quality_level > 0 && akt_connection.ttyname)
    {
    	if (box_conf.log_quality_level > 1 || carrier_lost)
    	{
    		if (no_login)
    		{
    			/* Bei Carrier Lost vor oder whrend des Einloggens
    			 * kommt keine Uhrzeit ins Infoprotokoll. Daher
    			 * holen wir das hier nach.
    			 */
				time_t t;
				char *p,*q;

				time(&t);
				p=ctime(&t);
				q=strpbrk(p,"\r\n");
				if (q)
					*q=0;

				write_info_protokoll(INFOPROT_FREE,NULL,p);
			}
    		write_info_protokoll(INFOPROT_FREE,NULL,
    			carrier_lost 
    				? "Verbindungsqualitt nach Carrier Lost"
    				: "Verbindungsqualitt");
    		if (carrier_lost)
    			sleep(2);
			get_connection_quality( akt_connection.ttyname, 
				box_conf.log_quality_command);
		}
	}

	/* Async-Infiles verarbeiten */
	ta_run_infiles(TRUE);

	loginB = FALSE;

	return ret;

} /* mailbox */


static void
setup_environment(void)
{
	const char *env;

#ifdef __TOS__
	int org_drive;
#endif
	/* Initialisierung */
	env=getenv("TERM");
	if (!env)
		env="vt100";
#ifdef TGETENT_NULL
	tgetent(0,env);
#else
	{
		void *ptr=XMALLOC(2048);
		tgetent(ptr,env);
	}
#endif

	if (-1==chdir(box_conf.quark_dir))
	{
		fprintf(stderr, "cannot chdir to %s: %s\n",
				box_conf.quark_dir, strerror(errno));
		exit(1);
	}

	/* Das Temporrverzeichnis bestimmen und anlegen */
	{
		char buf[100];
		char *base;
		char *zwi;
		struct stat st;
		sprintf(buf,"%s/tom.%lu",box_conf.tmp_dir,(ULONG) getpid());

		/* zuerst mal eine existierende Verzeichnishierarchie lschen */
		if (0==stat(buf,&st))
			rm_dir_content(buf);

		if (*buf!='/')
			base=concat(box_conf.quark_dir,buf,NULL);
		else
			base=strdup(buf);
		XNULLCHECK(base);
		mkdir(base,0750);

		zwi=concat(base,"/box",NULL);
		XNULLCHECK(zwi);
		mkdir(zwi,0750);
		free(zwi);

		zwi=concat(base,"/user",NULL);
		XNULLCHECK(zwi);
		mkdir(zwi,0750);
		free(zwi);

		tom_tmp_dir=concat(base,"/box/",NULL);
		tom_utmp_dir=concat(base,"/user/",NULL);
		XNULLCHECK(tom_tmp_dir);
		XNULLCHECK(tom_utmp_dir);
		free(base);
	}

#ifdef HAVE_DOSDRIVES
	get_drives(known_drivesS);
#endif
}

static void 
cmdline_bootstrap(void)
{
	if (ub_init(box_conf.var_dir))
		fprintf(stderr,"%suser.dat existiert bereits\n",box_conf.var_dir);
	else
	{
		USER antrag;
		if (!ub_create_user_dat())
		{
			fputs("Anlegen der Userdatenbank fehlgeschlagen\n",stderr);
			tom_leave(LEAVE_ERROR,NULL);
		}
		if (!ub_init(box_conf.var_dir))
			fprintf(stderr,"%suser.dat angelegt, aber nicht zu ffnen?\n",
				box_conf.var_dir);
		memset(&antrag,0,sizeof(USER));
		strcpy(antrag.name,"sysop");
		antrag.level=ULV_MAX;
		antrag.restzeit=86400L; /* ein Tag ... */
		antrag.bezahlt_bis=time(NULL)+365*86400L;
		strcpy(antrag.ansage,"Dummyeintrag fr den Sysop");
		antrag.pw_encoding=PW_UNENCODED;
		antrag.menue_stil=1; /* lang */
		strcpy(antrag.passwort,"geheim");
		if (!ub_create_user(&antrag))
		{
			mboutput(" Leider ist ein Fehler aufgetreten!\r\n");
			tom_leave(LEAVE_ERROR,NULL);
		}

		memset(&antrag,0,sizeof(USER));
		strcpy(antrag.name,"Gast");
		antrag.level=ULV_MIN;
		antrag.restzeit=30*60; /* ein halbe Stunde ... */
		antrag.pw_encoding=PW_UNENCODED;
#ifdef HAVE_LIBTCL
		antrag.box_stil=MAUS_STYLE;
#endif
		antrag.menue_stil=1; /* lang */
		if (!ub_create_user(&antrag))
		{
			mboutput(" Leider ist ein Fehler aufgetreten!\r\n");
			tom_leave(LEAVE_ERROR,NULL);
		}
	}
	if (bb_init(box_conf.var_dir,box_conf.bretter_dir,
				box_conf.binbrett_dir))
		fprintf(stderr,"%sbrett.dat existiert bereits\n",box_conf.var_dir);
	else
	{
		if (!bb_create_db(box_conf.var_dir))
		{
			fputs("Anlegen der Brettdatenbank fehlgeschlagen\n",stderr);
			tom_leave(LEAVE_ERROR,NULL);
		}
	}
	tom_leave(LEAVE_OK,NULL);
}



static void
init_databases(void)
{
	if (!ub_init(box_conf.var_dir))
	{
		fputs("Initialisierung der Userdatenbank fehlgeschlagen\n",stderr);
		tom_leave(LEAVE_ERROR,NULL);
	}

	if (!bb_init(box_conf.var_dir,box_conf.bretter_dir,
				 box_conf.binbrett_dir))
	{
		fputs("Initialisierung der Brettdatenbank fehlgeschlagen\n",stderr);
		tom_leave(LEAVE_ERROR,NULL);
	}

	if (box_conf.offline_cnf && *box_conf.offline_cnf)
	{
		if (!mdb_init(box_conf.offline_cnf))
		{
			fputs("Initialisierung der Messagedatenbank fehlgeschlagen\n",stderr);
			tom_leave(LEAVE_ERROR,NULL);
		}
	}
	else if (argument_cnffile)
	{
		char *d=XMALLOC(strlen(argument_cnffile)+strlen("offline.cnf")+2);
		char *p;
		strcpy(d,argument_cnffile);
		p=strrchr(d,'/');
		if (p)
			strcpy(p+1,"offline.cnf");
		else
			strcpy(d,"offline.cnf"); /* aktuelles Verzeichnis? */
		if (!mdb_init(d))
		{
			fputs("Initialisierung der Messagedatenbank fehlgeschlagen\n",stderr);
			tom_leave(LEAVE_ERROR,NULL);
		}
	}
	else
	{
		if (!mdb_init(SYSCONFDIR "/offline.cnf"))
		{
			fputs("Initialisierung der Messagedatenbank fehlgeschlagen\n",stderr);
			tom_leave(LEAVE_ERROR,NULL);
		}
	}
}

#ifdef HAVE_LIBTCL
static int
cmdline_tclscript(void)
{
	USERNR unr;

	mehr_disabled=TRUE;
	akt_anruf.batch_status=TRISTATE_FAILURE;

	if (!argument_user) {
		/* Usernamen aus TCL-Script lesen */
		FILE *f=fopen(argument_tclscript,"r");
		int i=0;
		char *p;
		char buf[1024];
		while (fgets(buf,sizeof(buf),f)) {
			i++;
			if (i>5)
				break;
			if (0==strncmp(buf,"#user ",6)) {
				p=strpbrk(buf,"\r\n");
				if (p)
					*p=0;
				p=buf+6;
				strtrim(p);
				argument_user=XSTRDUP(p);
			}
		}
		fclose(f);
	}
	if (!argument_user) {
		fprintf(stderr,"Kein Benutzername angegeben/gefunden\r\n");
		return LEAVE_ERROR;
	}
	if (!ub_find_username (&unr, argument_user))
	{
		fprintf(stderr,"%s ist leider unbekannt\r\n",argument_user);
		return LEAVE_ERROR;
	}
	else if (!ub_read_user (unr, &user, TRUE))
	{
		syslog(LOG_ERR,"kann Benutzerdaten nicht lesen: %s",argument_user);
		fprintf(stderr,"kann Benutzerdaten nicht lesen\r\n");
		return LEAVE_ERROR;
	}
	/* TCL-Scripten normaler Benutzer kann man nicht trauen. */
	else if (user.level!=ULV_MAX)
	{
		syslog(LOG_ERR,"cmdline-tclscript called from %s",argument_user);
		fprintf(stderr,"nicht erlaubt.\r\n");
		return LEAVE_ERROR;
	}
	{
		char *buf;
		char *p;
		box_conf.view_user = TRUE;
		user_in_fd=STDIN_FILENO;
		user_out_fd=STDOUT_FILENO;
		anrufdaten_init(ANRUF_CCOMMAND);
		offlineB=TRUE;
		buf=concat("TOM: tclscript fr ",user.name,NULL);
		if (buf)
		{
			cmdline_overwrite(buf);
			free(buf);
		}
		b_set_flags (&user);
		buf=NULL;
		p=strchr(argument_tclscript,' ');
		if (!p) {
			if (!ttcl_script(argument_tclscript,&buf))
				return LEAVE_ERROR;
		} else {
			char *para;
			*p++=0;
			para=strchr(p,' ');
			if (para)
				*para++=0;
			if (!ttcl_call_func(argument_tclscript,p,para,&buf))
				return LEAVE_ERROR;
		}
		ub_write_user(&user,TRUE);
		if (!argument_no_log_entry)
			login_protokoll_write();
		if (buf && *buf)
			return LEAVE_ERROR;
		return LEAVE_OK;
	}
}
#endif

static int
cmdline_batch(void)
{
	USERNR unr;

	mehr_disabled=TRUE;
	akt_anruf.batch_status=TRISTATE_FAILURE;

	if (!ub_find_username (&unr, argument_user))
	{
		fprintf(stderr,"%s ist leider unbekannt\r\n",argument_user);
		return LEAVE_ERROR;
	}
	else if (!ub_read_user (unr, &user, TRUE))
	{
		syslog(LOG_ERR,"kann Benutzerdaten nicht lesen: %s",argument_user);
		fprintf(stderr,"kann Benutzerdaten nicht lesen\r\n");
		return LEAVE_ERROR;
	}
	else
	{
		char *buf;
		box_conf.view_user = TRUE;
		user_in_fd=STDIN_FILENO;
		user_out_fd=STDOUT_FILENO;
		anrufdaten_init(ANRUF_CCOMMAND);
		offlineB=TRUE;
		buf=concat("TOM: batch fr ",user.name,NULL);
		if (buf)
		{
			cmdline_overwrite(buf);
			free(buf);
		}
		b_set_flags (&user);
		if (argument_command)
		{
			/*
			 * Kommandos mssen als Batches ausgefhrt werden, um die
			 * Kommandos durch batch_ausfuehrung ausfuehren zu koennen,
			 * was noetig ist, um die Verwendung interaktiver Kommandos
			 * zu verhindern (-c "lesen 1").
			 */
			char pfad[PATH_MAX];
			FILE *f;
			sprintf(pfad,"%stmp.bat",tom_tmp_dir);
			f=xfopen(pfad,"w");
			fprintf(f,"%s\n",argument_command);
			fclose(f);
			akt_anruf.batch_status=TRISTATE_OK;
			batch_ausfuehrung(pfad);
			unlink(pfad);
		}
		else if (fileexist(argument_batch))
		{
			akt_anruf.batch_status=TRISTATE_OK;
			batch_ausfuehrung(argument_batch);
		}
		else
		{
			char pfad[PATH_MAX];
			sprintf(pfad,"%s%s",box_conf.batch_dir,argument_batch);
			if (fileexist(pfad))
			{
				akt_anruf.batch_status=TRISTATE_OK;
				batch_ausfuehrung(pfad);
			}
			else
			{
				fprintf(stderr,"Batch %s nicht gefunden\r\n",
					argument_batch);
				return LEAVE_ERROR;
			}
		}
		if (!ub_write_user (&user, TRUE))
		{
			syslog(LOG_ERR,"kann Benutzerdaten nicht schreiben: %s",argument_user);
			fprintf(stderr,"kann Benutzerdaten nicht schreiben\r\n");
			return LEAVE_ERROR;
		}
		sprintf(akt_anruf.name,user.name);
		if (!argument_no_log_entry)
			login_protokoll_write();
	}
	return LEAVE_OK;
}

static int cmdline_tausch(void)
{
	char *buf;
	int ret=LEAVE_OK;
	box_conf.view_user = TRUE;
	mehr_disabled=TRUE;

	user_in_fd = STDIN_FILENO;
	user_out_fd = STDOUT_FILENO;

	anrufdaten_init(ANRUF_CTAUSCH);
	tausch_auto_loginB=TRUE;

	buf=concat("TOM: tausch fr ",argument_user,NULL);
	if (buf)
	{
		cmdline_overwrite(buf);
		free(buf);
	}

	if (argument_infile && argument_outfile)
	{
		/* Nicht ta_modus_normal, wegen Downloadbatch u.s.w. */
		if (!ta_run_tausch(ta_modus_inout,argument_user,
			argument_infile,argument_outfile,argument_seit))
			ret=LEAVE_ERROR;
	}
	else if (argument_infile)
	{
		if (!ta_run_tausch(ta_modus_in,argument_user,
			argument_infile,NULL,argument_seit))
			ret=LEAVE_ERROR;
	}
	else if (argument_outfile)
	{
		if (ta_run_tausch(ta_modus_out,argument_user,NULL,
			argument_outfile,argument_seit))
			ret=LEAVE_OK;
		else
			ret=LEAVE_ERROR;
	}
	else
	{
		puts(" Wie wre es, wenn sie -i oder -o benutzten?");
		return LEAVE_PARA;
	}
	sprintf(akt_anruf.name,argument_user);
	if (!argument_no_log_entry)
		login_protokoll_write();
	return ret;
}

static int
cmdline_queue_run(void)
{
	BOOLEAN ret;
	mehr_disabled=TRUE;
	box_conf.view_user = TRUE;
	user_in_fd=STDIN_FILENO;
	user_out_fd=STDOUT_FILENO;
	anrufdaten_init(ANRUF_CCOMMAND);
	offlineB=TRUE;

	cmdline_overwrite("TOM: quererun");

	ret=r_queuerun();
	if (!argument_no_log_entry)
		login_protokoll_write();
	if (!ret)
		return LEAVE_ERROR;
	if (!ta_run_infiles(FALSE))
		return LEAVE_ERROR;
	return LEAVE_OK;
}

static void
do_exec(char *cmd)
{
	char *le; /* logentry */
	char *sp; /* space */
	char *buf;
	int i;
	size_t len;

#ifndef THIS_IS_TOMSHELL
	if (argument_getty) /* sonst ist es schon geschehen! */
		anrufdaten_init(ANRUF_USER);
#endif

	for (i=0,len=PATH_MAX*4+1024;i<3;i++,len*=2)
	{
		buf=malloc(len);
		if (!buf)
		{
			syslog(LOG_ERR,"out of memory");
			fprintf(stderr,"tom: out of memory before exec\r\n");
			tom_leave(LEAVE_ERROR,NULL);
		}
		if (do_interpret(cmd,buf,len))
			break;
		free(buf);
		buf=NULL;
	}
	if (!buf)
	{
		syslog(LOG_ERR,"kann kommando nicht interpretieren");
		fprintf(stderr,"tom: kann Kommando nicht interpretieren\r\n");
		tom_leave(LEAVE_ERROR,NULL);
	}

	sp=strchr(buf,' ');
	if (sp)
	{
		char *p;
		*sp=0;
		p=strrchr(buf,'/');
		*sp=' ';
		if (p)
			le=XSTRDUP(p+1);
		else
			le=XSTRDUP(buf);
	}
	else
		le=XSTRDUP(buf);
	if (strlen(le)>=sizeof(akt_anruf.name))
		le[sizeof(akt_anruf.name)-1]=0;
	strcpy(akt_anruf.name,le);
	{
		char *p=concat("TOM: exec ", le,NULL);
		if (p)
		{
			cmdline_overwrite(p);
			free(p);
		}
	}

	/* effektiv kein exec - erstens wollen wir noch protokolle schreiben,
	 * zweitens braucht das Loopscript bestimmte Exitcodes 
	 */
#ifndef THIS_IS_TOMSHELL
	if (argument_getty)
	{
		int ret;

		syslog(LOG_DEBUG,"system(%s)",buf);
		
		ret=system(buf);
		if (ret)
		{
			syslog(LOG_ERR,"system(%s) return %d",buf,ret);
			login_protokoll_write();
			tom_leave(LEAVE_ERROR,NULL);
		}
		login_protokoll_write();
		tom_leave(LEAVE_RESTART,NULL);
	}
	else
#endif
	{
		/* mit -l gestartet, nicht aus loopscript */
#define MAXARG 32
		char *argv[MAXARG];
		char *p;
		const char *pn="(null)";

		/* Carrier Lost knnen wir nicht mehr feststellen */
		akt_anruf.zuletzt[0]=0;
		login_protokoll_write();

		i=0;
		do
		{
			p=strtok(i ? NULL : buf,"\t ");
			if (!p)
				break;
			if (i==0)
			{
				pn=p;
				argv[i]=strrchr(p,'/');
				if (argv[i])
					argv[i]++;
				else
					argv[i]=p;
				i++;
			}
			else
				argv[i++]=p;
			if (i>=MAXARG)
			{
				syslog(LOG_ERR,"to many arguments on exec %s",argv[0]);
				tom_leave(LEAVE_ERROR,NULL);
			}
		} while (TRUE);
		argv[i]=NULL;
		syslog(LOG_DEBUG,"exec %s",pn);
		/* Alle Cleanupfunktionen abarbeiten */
		dqueue_work_all();
		execv(pn,argv);
		syslog(LOG_ERR,"cannot exec %s: %s",pn,strerror(errno));
		tom_leave(LEAVE_ERROR,NULL);
	}
}

