/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <setjmp.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <syslog.h>
#include <setjmp.h>

#include "io.h"
#include "modem.h"
#include "prototyp.h"
#include "terminal.h"
#include "user.h"					/* nur fr die User-Umlaute */
#include "brett.h"
#include "box_conf.h"
#include "cslib.h"
#include "aktdata.h"
#include "log.h"
#include "tompaths.h"

extern USER user;

extern volatile int got_sigterm;
extern volatile int got_sigusr2;

extern int user_in_fd;
extern int user_out_fd;
extern int sysop_in_fd;
extern int sysop_out_fd;

extern BYTE batchB;
extern BYTE loginB;

extern WORD count_crW;

extern int logout_modus;

extern BOOLEAN mehr_disabled;
extern BOOLEAN mitten_im_chat;

static const char *urgent_text=
"\r\n"
"Die Box mu leider sofort beendigt werden. Vielleicht mu der Sysop\r\n"
"dringende Arbeiten vornehmen oder es mu ein Autoevent ausgefhrt\r\n"
"werden.\r\n";
static int urgent_printed=0;


extern char handle_escape(char pressed_key);

/*
	Zeichen von remote Port oder Konsole holen
	Bei remote-Logins Consolen-Funktionen ausfhren, whrend der User online 
	ist (Zwang zum Chat, Userlevelnderung usw.)

	21.10.93 Frank, stark berarbeitet, select
	                bernahme der Usertimeout-Funktionalitt
	07.01.94 Uwe, Umlautwandlung mit Pufferung
	21.04.94 Frank, sofortige Rckkehr bei Batch-Betrieb
	13.07.94 Frank, berarbeitung zur alleinigen Eingaberoutine
*/
int mbchar(void)
{
	struct timeval zeit;
	fd_set set;
	unsigned char pressed_key = 0;
	static unsigned char last=0;

	/* kein Tastendruck bei Batch-Ablufen */
	if (batchB)
		return 0;

	if (logout_modus)
		return 0;

	/* timeout-Wert in Sekunden fr Loginphase */
	if (!box_conf.login_timeout)
		box_conf.login_timeout = 30;

	count_crW = 0;

	do
	{
		int st;						/* select-Status */

		/* max. Onlinezeit berschritten? */
		if (loginB && !mitten_im_chat)
		{
			check_timeout();		/* ### CPU-Ressourcenfresser */
			if (logout_modus == 1)
				return 0;
		}

#ifdef EXPLIZITE_CARRIERABFRAGE
		/*
			workaround wg. Funktionsuntchtigkeit von SIGHUP generell 
			sowie speziell MiNT auf Atari:
			explizite Carrier Detect Abfrage
		*/
		if (!carrier_still_up())
		{
			logout_modus=1;
			return 0;
		}
#else
		if (logout_modus == 1)
			return 0;
#endif

		/*
		 * Urgent Shutdown?
		 */
		if (got_sigusr2 || got_sigterm)
		{
			if (!urgent_printed++) mboutput(urgent_text);
			logout_modus=1;
			return 0;
		}

		FD_ZERO(&set);
		if (user_in_fd >= 0)
			FD_SET(user_in_fd,&set);
		if (sysop_in_fd >= 0)
			FD_SET(sysop_in_fd,&set);

		/*
			whrend der Loginphase ist loginB auf FALSE und wird erst 
			nach Paworteingabe TRUE, daher wird fr das Impressum sowie 
			das Login ein separater (default: 30 s) timeout-Wert gewhlt
		*/
#ifdef __TOS__
		zeit.tv_sec = 0;
		zeit.tv_usec = 200;
#else
		/* Im Gedenken an den GCC-Fragezeichen-Bug */
		/* diesen Teil NICHT ndern! */
		if (loginB)
			zeit.tv_sec = box_conf.user_timeout;
		else
			zeit.tv_sec = box_conf.login_timeout;
		zeit.tv_usec = 0;
#endif
		/* Die Carrierabfrage mu in der Schleife geschehen, um 
		 * nach Erhalt eines SIGHUP sofort reagieren zu knnen
		 */
		do
		{
			int max;

#ifdef EXPLIZITE_CARRIERABFRAGE
			if (!carrier_still_up() || logout_modus)
#else
			if (logout_modus)
#endif
				return 0;
			/*
			 * Urgent Shutdown?
			 */
			if (got_sigusr2 || got_sigterm)
			{
				if (!urgent_printed++) mboutput(urgent_text);
				logout_modus=1;
				return 0;
			}

			max = user_in_fd;
			if (sysop_in_fd > max)
				max = sysop_in_fd;

			st = select(max+1,&set,NULL,NULL,&zeit);
		}
		while (st == -1);

		/*
		 * Urgent Shutdown?
		 */
		if (got_sigusr2 || got_sigterm)
		{
			if (!urgent_printed++) mboutput(urgent_text);
			logout_modus=1;
			return 0;
		}
#ifdef EXPLIZITE_CARRIERABFRAGE
		if (!carrier_still_up() || logout_modus)
#else
		if (logout_modus)
#endif
			return 0;

		if (!st)
		{
#ifndef __TOS__
			/* timeout */
			logout_modus = 1;
			mehr_disabled=TRUE;
			mboutput("\r\n");
			set_status_line("Timeout");
			mboutput(" User inaktiv -> Timeout ... Rauswurf\r\n");
			strcpy(akt_anruf.zuletzt,"INA");
			mboutput("\r\n");

			return 0;
#endif
		}

#ifdef __TOS__
		if ((user_in_fd >= 0 && FD_ISSET(user_in_fd, &set) && !port_status(user_in_fd)))
			msleep(100);
#endif

		/* Zwischengetippe von der Console? */
		if (sysop_in_fd>=0 && FD_ISSET(sysop_in_fd,&set))
		{
			static time_t last_escape;
			if (read(sysop_in_fd,&pressed_key,1) <= 0)
			{
				kill(getpid(),SIGHUP);
				logout_modus=1;
				syslog(LOG_ERR,"IO Error mbchar(sysop_in_fd)");
				return 0;
			}
            if (last_escape)
            {
                if (time(NULL) - last_escape >=3)
                    last_escape=0;
            }
            if (pressed_key==25) /* ^Y */
            {
                if (last_escape)
                    last_escape=0;
                else
                    last_escape=time(NULL);
            }
            else if (last_escape)
			{
                pressed_key=handle_escape(pressed_key);
				last_escape=0;
			}
		}
		if (!pressed_key && user_in_fd >= 0 && FD_ISSET(user_in_fd,&set))
		{
			if (read(user_in_fd,&pressed_key,1) <= 0)
			{
				kill(getpid(),SIGHUP);
				logout_modus=1;
				return 0;
			}
			if (!akt_connection.is_console && pressed_key)
			{
				char in_puffer[5];
				char_to_native((unsigned char *)in_puffer,pressed_key);
				pressed_key=*in_puffer;
			}
			if (!akt_connection.is_tty && loginB) {
				/* Jetzt wird es unschoen. In Abhaengigkeit vom gesendeten
				 * Zeilenende die Zeichen auffressen.
				 */
				if (akt_connection.sends_cr && akt_connection.sends_lf) {
					if (pressed_key=='\r')
						pressed_key=0;
				}
			}
			if (last=='\r' && pressed_key=='\n' && loginB)
				pressed_key='\0'; /* grr, damit kommt die Quark nicht klar */
			last=pressed_key;
			/* ### ASCII-Abhngigkeit! */
			if (pressed_key > 0  && pressed_key < 32)
			{
				switch(pressed_key)
				{
				case '\n':
					if (loginB)
						pressed_key='\r';
					break;
				case '\r':
				case '\b':
				case '\t':
				case 27:
					break;
				default:
					/* Controlcodes filtern */
					pressed_key=0;
					break;
				}
			}
		}
	}
	while (!pressed_key);

	return pressed_key;

} /* mbchar */
