/* mbreadline.c : still no description */
/*
    Copyright (C) 1994 1995 1996 1997 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#ifdef WANT_READLINE
#include <readline/readline.h>
#endif

#include "io.h"
#include "modem.h"
#include "prototyp.h"
#include "terminal.h"
#include "user.h"					/* nur fr die User-Umlaute */
#include "aktdata.h"
#include "libquark.h"
#include "log.h"

extern USER user;

extern int user_in_fd;

int keine_ausgaben;

extern BYTE loginB;

extern WORD count_crW, vtmodeW;

extern int logout_modus;

/*
 * Input:
	laenge:	          max. Eingabelnge
	mode:    2345   = toupper
	           45   = '*'
	       0 2 4    = nach <CR> wird Zeilenrest gelscht, Linefeed
	       0123     = wird protokolliert
	linie:            Unterstrichlinie zeichnen
	frage:            Fragetext
	vorgabe:          Default-Antworttext
<-	text:             Hier wird der Text zurckgeliefert

	05.02.93 Frank
*/
BOOLEAN mbquestion(size_t laenge,int mode,BOOLEAN linie,const char *frage,
					const char *vorgabe,char *text)
{
	int flags=0;
	if (vorgabe && *vorgabe)
		strcpy(text,vorgabe);
	else
		*text=0;
	switch (mode) {
	case 0: flags|=0; break;
	case 1: flags|=MBF_NOLF; break;
	case 2: flags|=MBF_TOUPPER; break;
	case 3: flags|=MBF_NOLF|MBF_TOUPPER; break;
	case 4: flags|=MBF_TOUPPER|MBF_STAR|MBF_NOPROT; break;
	case 5: flags|=MBF_NOLF|MBF_TOUPPER|MBF_STAR|MBF_NOPROT; break;
	}
	if (linie)
		flags|=MBF_LINE;
	return mbfrage(laenge,flags,frage,text);
} /* mbquestion */

BOOLEAN mbfrage(size_t laenge,int flags,const char *frage,char *text)
{
	size_t anzahl = 0;
	int zeichen;
	static BOOLEAN firstline_cr;        /* erste Zeile endete mit CR */
	static BOOLEAN secondline_nl;       /* zweite Zeile begann mit LF */
	static unsigned int question_lines=0;

	char *vorgabe;

	BOOLEAN check_cr=FALSE;
	BOOLEAN check_nl=FALSE;
	BOOLEAN got_first=FALSE;

	if (text && *text)
		vorgabe=XSTRDUP(text);
	else
		vorgabe=NULL;

	if (question_lines==0) {
		/* we want it raw */
		io_set_ICRNL(user_in_fd,FALSE);
		io_set_IGNCR(user_in_fd,FALSE);
		io_set_INLCR(user_in_fd,FALSE);
		check_cr=TRUE;
	}
	else if (question_lines==1)
		check_nl=TRUE; 
	question_lines++;

#ifdef WANT_READLINE
	if ((user.flags & UB_FLAG_READLINE)
		&& !(flags & (MBF_TOUPPER|MBF_TOLOWER)))
	{
		for (;;)
		{
			char *s;
			size_t l;
			if (vorgabe && *vorgabe)
				rl_insert_text(vorgabe);
			s=do_readline(frage,'?');
			if (!s)
				return FALSE;
			if (logout_modus)
			{
				free(s);
				free(vorgabe);
				return FALSE;
			}
			l=strlen(s);
			if (l<=laenge)
			{
				if (flags & MBF_TRIM)
					strtrim(s);
				strcpy(text,s);
				free(s);
				free(vorgabe);
				return TRUE;
			}
			free(s);
			mbprintf("Zu lang. Maximallnge %ld Zeichen\r\n",(long) laenge);
		}

	}
#endif
	
	if (frage && *frage)
		mboutput(frage);
	if (vorgabe && *vorgabe)
	{
		strncpy(text,vorgabe,laenge);
		text[laenge] = 0;
		if (*vorgabe)
			mboutput(text);
	}
	else 
		*text=0;

	if (flags & MBF_LINE)
	{
		char zeile[80];
		size_t len;

		len = strlen(text);
		if (len < laenge)
		{
			if (vtmodeW == T_VT100 || vtmodeW == T_VT52 || vtmodeW == T_ANSI)
				mboutput(vt_S(STORE_CUR));

			memset(zeile,'_',laenge-len);
			zeile[laenge-len] = 0;
			mboutput(zeile);

			if (vtmodeW == T_VT100 || vtmodeW == T_VT52 || vtmodeW == T_ANSI)
				mboutput(vt_S(RESTORE_CUR));
			else
			{
				len = strlen(text);
				while (len++ < laenge)
					mboutput(vt_S(CUR_LEFT));
			}
		}
	}

	count_crW = 0;

	do
	{
		zeichen = mbchar();

		if (!got_first && check_nl && firstline_cr==TRUE && zeichen=='\n')
		{
			secondline_nl=TRUE;
			zeichen = mbchar();
		}
		got_first=TRUE;

		if (!zeichen || logout_modus) {
			free(vorgabe);
			return FALSE;
		}

		if (zeichen>31 || zeichen=='\b' || zeichen=='\r' || zeichen=='\n')
		{
			if (zeichen=='\b' || zeichen==127)
			{
				if (*text)
				{
					text[strlen(text)-1] = 0;
					if (flags & MBF_LINE)
						mboutput("\b_\b");
					else
						mboutput("\b \b");
					anzahl--;
				}
			}
			else
			{
				if (zeichen != '\r' && zeichen != '\n' && strlen(text) < laenge)
				{
					if (flags & MBF_TOUPPER)
						zeichen = toupper(zeichen);
					else if (flags & MBF_TOLOWER)
						zeichen = tolower(zeichen);
					anzahl = strlen(text);
					text[anzahl++] = (char) zeichen;
					text[anzahl]   = 0;
					if (flags & MBF_STAR)
						mboutput("*");
					else
						mboutput(tprintf("%c",zeichen));
				}
				if ((zeichen=='\r' || zeichen=='\n') && !(flags & MBF_NOLF))
				{
					if (flags & MBF_LINE)
					{
						char tmp[200];

						memset(tmp,' ',laenge-anzahl);
						tmp[laenge-anzahl] = 0;
						mboutput(tmp);
					}
					mboutput("\r\n");
				}
			}
		}
	}
	while (zeichen != '\r' && zeichen != '\n');
	if (check_cr && zeichen=='\r')
		firstline_cr=TRUE;

	/* keine Paworte protokollieren */
	if (!(flags & MBF_NOPROT) && level() < 9 && loginB && user.protokoll)
		write_info_protokoll(INFOPROT_TEXT,frage,text);
		
	if (check_nl)
	{
		/* Nun knnen wir entscheiden, welches Mapping gentigt wird:
		 * user\npasswort\n  (!firstline_cr) -> IGNCR (just for safety)
		 * user\rpasswort\?  (firstline_cr,!secondline_nl) -> ICRNL -> 1
		 * user\r\npasswort.. (beides TRUE)  -> IGNCR
		 * user\rpasswort\r
		 */
		if (!firstline_cr)
		{
			/* LF wandeln nach CR. */
			akt_connection.sends_cr=FALSE;
			akt_connection.sends_lf=TRUE;
			io_set_IGNCR(user_in_fd,TRUE); /* just for safety */
			io_set_INLCR(user_in_fd,TRUE);
		}
		else if (secondline_nl)
		{
			/* CR/LF wandeln nach CR */
			akt_connection.sends_cr=TRUE;
			akt_connection.sends_lf=TRUE;
			io_set_IGNCR(user_in_fd,TRUE);
			io_set_INLCR(user_in_fd,TRUE);
		}
		else
		{
			/* nur CR */
			akt_connection.sends_cr=TRUE;
			akt_connection.sends_lf=FALSE;
		}
	}
	free(vorgabe);
	if (flags & MBF_TRIM)
		strtrim(text);
	return TRUE;

} /* mbfrage */


