/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>

#include "prototyp.h"
#include "user.h"
#include "modem.h"
#include "md5file.h"
#include "box_conf.h"
#include "tompaths.h"

extern USER user;

/*
	->
	            work            :   das PW dieses Users prfen
	            passwort		:	zu berprfendes Pawort

	<-			TRUE			:	Passwort in mistpwd enthalten
				FALSE			:	dito nicht enthalten

	Unerwnschte Paworte anhand der Datei "/tom/system/mistpwd.cnf"
	berprfen. Jede Zeile in der Datei steht dabei fr einen zu
	filternden Eintrag und kann auch die Wildcards * und ? enthalten.

	10.06.92 Frank
	18.03.93 Frank, fast neu
*/
BOOLEAN 
mistpasswort(USER *work, const char *passwort, BOOLEAN warnen)
{
	char fname[PATH_MAX];
	char eintrag[100];
	char kopie[100];
	char un[UDAT_NAME_LEN];
	char pw[UDAT_NAME_LEN];
	FILE *mist_f;
	BOOLEAN ret=TRUE; /* Passwort ist Mist */

	strcpy(kopie,passwort);
	strlwr(kopie);

	if (strlen(kopie)<6)
	{
		if (warnen)
			mboutput(" Das Passwort ist zu kurz, "
				"6 Zeichen sollten es schon sein!\r\n");
		goto cleanup;
	}

	if (!stricmp(kopie,work->name))
	{
		if (warnen)
			mboutput(" Passwort und Username sind gleich!\r\n");
		goto cleanup;
	}

	sprintf(fname,"%s%s",box_conf.config_dir,"mistpwd.cnf");
	if (fileexist(fname))
	{
		if ((mist_f = fopen(fname,BINREAD)) == NULL)
			return FALSE;
		while (tom_fgetline(eintrag,mist_f))
		{
			if (*eintrag && wildcard(passwort,eintrag))
			{
				fclose(mist_f);
				if (warnen)
					mboutput(" Das Passwort ist verboten!\r\n");
				goto cleanup;
			}
		}
		fclose(mist_f);
	}
	if (box_conf.bad_password_prg && *box_conf.bad_password_prg)
	{	
		char *cmd=NULL;
		char *p=strchr(passwort,'\'');
		if (p) {
			/* Sicherheitsloch! */
			if (warnen)
				mboutput(" Das Passwort enthlt ein illegales Zeichen: \"'\"!\r\n");
			goto cleanup;
		}
		asprintf(&cmd,"%s '%s'",box_conf.bad_password_prg,passwort);
		XNULLCHECK(cmd);
		if (0==system(cmd))
		{
			if (warnen)
				mboutput(" Das Passwort ist leider unzureichend!\r\n");
			goto cleanup;
		}
	}
	/* 
	 * So, alle Standardtests sind durch - und jetzt legen wir noch einen
	 * drauf, nmlich die Prfung, ob Passwort und Username zu hnlich sind
	 */
	{
		char *p;
		char *s;
		strcpy(un,work->name);
		strlwr(un);
		s=un;
		/* Username "thomas otto", Passwort "thomasotto"? Nicht mit mir! */
		while (s && *s)
		{
			p=strchr(s,' ');
			if (p)
			{
				*p++='\0';
				while (isspace(*p))
					p++;
			}
			if (strlen(s)>2 && strstr(kopie,s))
			{
				mboutput(" Teil des Usernamens im Passwort erkannt!\r\n");
				goto cleanup;
			}
			s=p;
		}
		/* Username "otto kaufmichmann", Passwort "kaufmich"? Nicht mit mir! */
		strcpy(pw,kopie);
		strcpy(un,work->name);
		strlwr(un);
		s=pw;
		while (s && *s)
		{
			p=strchr(s,' ');
			if (p)
			{
				*p++='\0';
				while (isspace(*p))
					p++;
			}
			if (strlen(s)>2 && strstr(un,s))
			{
				mboutput(" Teil des Passworts im Namen erkannt!\r\n");
				goto cleanup;
			}
			s=p;
		}
	}
	ret=FALSE; /* passwort ist OK; */

cleanup:
	memset(kopie,0,sizeof(kopie));
	memset(pw,0,sizeof(pw));
	return ret;

} /* mistpasswort */
