/*
 * Copyright (C) 1994 1995 1996 1997 Frank Baumgart, Uwe Ohse
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Contact1: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
 * Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
 * Contact3: MausNet Newsgroups Quark & Q-Sysops.
 */
/*
 * Modemkommunikation: gemeinsame Routinen fr Getty- und Nichtgettyversion.
 * 
 */

#include "tomsysdep.h"

#include <portab.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <syslog.h>
#include <signal.h>
#include <ctype.h>

#include "io.h"
#include "modem.h"
#include "log.h"
#include "box_conf.h"
#include "prototyp.h"
#include "box_conf.h"

extern BOOLEAN modem_vorhanden;

/* 
 * Luft am Ende von mailbox() und hat eine Menge nur deshalb 
 * unbedenklicher Seiteneffekte.
 */
void
get_connection_quality (const char *tty, const char *command)
{
	BOOLEAN t_o = FALSE;
	char *buf;
	BOOLEAN old_modem_vorhanden = modem_vorhanden;
	int fd;
	int leer = 0;
	int bytes = 0;

	if (!command || !tty) /* simple sanity check */
		return;

	fd = open (tty, O_RDWR
#ifdef O_NDELAY
				| O_NDELAY
#elif defined(O_NONBLOCK)
                | O_NONBLOCK
#endif
#ifdef O_NOCTTY
			   | O_NOCTTY
#endif
		);
	if (fd == -1)
		return;
	fcntl(fd,F_SETFL, O_RDWR);

	modem_vorhanden = TRUE;

	hangup_modem (fd);
	send_modem (fd, command);
	send_modem (fd, "\r\n");

	while ((buf = modem_readline (fd, 2, &t_o)) != NULL) {
		BOOLEAN go_out = FALSE;
		char *p;
		bytes += strlen (buf);
		if (bytes > 5000) {
syslog(LOG_INFO,"breakout gcq: >5000");
			break;
		}
		p = strpbrk (buf, "\r\n");
		if (p)
			*p = 0;
		if (!*buf) {
			/* vier Leerzeilen hintereinander sind Signal zum Abbruch.
			 * Es knnte hier durchaus mal Mll vom Modem kommen.
			 * vier weil: \r\n mglicherweise als \n\n ankommt (die
			 * Schnittstelle ist kaum im normalen workmode 
			 */
			if (leer++ > 2) {
syslog(LOG_INFO,"breakout gcq: leer");
				break;
			}
		} else
			leer = 0;
		/* Bei Controlcodes abbrechen - zu oft kommt hier nur noch
		 * Mll an (warum? ungeklrt). */
		for (p = buf; *p; p++) {
			if (((unsigned char) *p) < ' ' && !isspace (*p)) {
				go_out = TRUE;
syslog(LOG_INFO,"breakout gcq: controlcodes");
				break;
			}
		}
		if (go_out) {
			write_info_protokoll (INFOPROT_FREE, NULL, "Abbruch: Modemmll?");
			break;
		}
		if (box_conf.modem_tst_rcv
			&& !strcmp (box_conf.modem_tst_rcv, buf)) {
syslog(LOG_INFO,"breakout gcq: tst_rcv received");
			break;
		}
		/* Just for Safety */
		if (!box_conf.modem_tst_rcv && !strcmp ("OK", buf)) {
syslog(LOG_INFO,"breakout gcq: OK received");
			break;
		}
		if (*buf)
			write_info_protokoll (INFOPROT_FREE, NULL, buf);
		free (buf);
		buf = NULL;
	}
	if (buf)
		free (buf);

	modem_vorhanden = old_modem_vorhanden;

	close (fd);
}

void 
hangup_modem (int fd)
{
	if (modem_vorhanden) {
		io_flush (fd);

		if (box_conf.modem_drop_dtr) {
			io_set_DTR (fd, FALSE);
			sleep (1);
			io_set_DTR (fd, TRUE);
			io_flush (fd);
		} else {
			send_modem (fd, box_conf.modem_active);		/* 5 */
			msleep (200);
/*          modem_ok(fd); */
			send_modem (fd, box_conf.modem_hangup);		/* 3 */
			send_modem (fd, "\r");
			modem_ok (fd);
		}
	}
}								/* hangup_modem */


/*
 * max. 3 s auf OK-Antwort des Modems warten
 * 12.10.93 Frank
 * 26.06.94 Frank
 * 
 * <-           TRUE    :   "OK" empfangen
 * FALSE    :   kein "OK" empfangen, Modem evtl. ausgeschaltet
 */
#define OKDEBUG(x) x
/* #define OKDEBUG(x)  */

BOOLEAN 
modem_ok (int fd)
{
	char *antwort;
	time_t ende;
	BOOLEAN t_o;

	ende = time (NULL) + 3;

	do {
		antwort = modem_readline (fd, 3, &t_o);

		if (!antwort)
			return FALSE;
		if (t_o) {
			if (antwort)
				free (antwort);
			return FALSE;
		}
		OKDEBUG (printf ("%s\n", antwort));
		if (!strcmp (antwort, box_conf.modem_tst_rcv)) {
			free (antwort);
			return TRUE;
		}
		/* msleep(200); */
		free (antwort);
	}
	while (ende >= time (NULL));
	OKDEBUG (printf ("(failed)\n"));

	return FALSE;

}								/* modem_ok */

/*
 * Modem abheben
 */
void 
takeoff_modem (int fd)
{
	if (modem_vorhanden) {
#if 0
		io_send_break (fd);
		io_set_baud (fd, box_conf.modem_speed);
		send_modem (fd, tprintf ("%s\r", box_conf.modem_reset));	/* 4 */
		modem_ok (fd);
#endif
		send_modem (fd, tprintf ("%s\r", box_conf.modem_takeoff));	/* 10 */
		modem_ok (fd);
		io_flush (fd);
	}
}								/* takeoff_modem */



/*
	22.11.92 Frank, wird nur in Initialisierungsphase benutzt
*/
extern void check_modem(int fd);
void check_modem(int fd)
{
	int try=0;

	modem_vorhanden = FALSE;

	/*
		vermeiden, da sich die Box nach Konfigurationsfehlern im Modembereich
		nicht mehr remote warten lt
	*/
	if (!*box_conf.modem_tst_snd || !*box_conf.modem_tst_rcv)
	{
		modem_vorhanden = TRUE;
		return;
	}

#define HACK_WEGEN_NICHT_ERKANNTEM_MODEM
	while (TRUE)
	{

		io_set_baud(fd,box_conf.modem_speed);
		io_flush(fd);

		if (!*box_conf.modem_tst_snd || !*box_conf.modem_tst_rcv)
			return;

		send_modem(fd,box_conf.modem_tst_snd);
		send_modem(fd,"\r");

		if (modem_ok(fd))
		{
			modem_vorhanden = TRUE;
			return;
		}
		else if (try>=3)
		{
			/* display_status("Modem antwortet nicht"); */
			break;
		}
		else
		{
			int i;
			int sl;
#if 0
			char buf[128];
#endif
			try++;
#if 0
			sprintf(buf,"Modem antwortet nicht (%d. Versuch)",try);
			display_status(buf);
#endif
			io_set_DTR(fd, FALSE);
			for (i=1,sl=10;i<=try;i++,sl*=2)
			{
				io_send_break(fd);
				msleep(sl);
			}
			io_set_DTR(fd,TRUE);
			io_flush(fd);
		}
	}
	if (try)
		sleep(2);
	if (!modem_vorhanden)
	{
		io_set_DTR(fd, FALSE);
		syslog(LOG_ERR,"kann Modem nicht ansprechen");
		tom_leave(LEAVE_RESTART,"");
	}
} /* check_modem */



extern void init_modem(int fd);
void init_modem(int fd)
{
	if (modem_vorhanden)
	{
		char *tmp;
		io_send_break(fd);
		io_set_baud(fd,box_conf.modem_speed);

		asprintf(&tmp,"%s\r",box_conf.modem_reset);
		XNULLCHECK(tmp);
		send_modem(fd,tmp);
		free(tmp);
		modem_ok(fd);

		asprintf(&tmp,"%s\r",box_conf.modem_init);
		XNULLCHECK(tmp);
		send_modem(fd,tmp);
		free(tmp);
		modem_ok(fd);

		io_flush(fd);
	}
} /* init_modem */

/*
	Anruf annehmen
*/
extern void answer_modem(int fd);
void answer_modem(int fd)
{
	send_modem(fd,box_conf.modem_answer);		/* ATA+CR */
	send_modem(fd,"\r");
} /* answer_modem */


/*
	Nummer anwhlen
*/
extern void dial_modem(int fd,const char *nummer);
void dial_modem(int fd,const char *nummer)
{
	io_flush(fd);
	send_modem(fd,"\r");
	send_modem(fd,box_conf.modem_dial);
	send_modem(fd,nummer);
	send_modem(fd,"\r");
} /* dial_modem */

