/* nntp_abgleich.c: brett-db und nntp-db vergleichen und nderungskommandos
 * als MausTausch-Kommandos ausgeben.
 * Copyright (C) 1998 Uwe Ohse
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
 */

#include "tomsysdep.h"
#include "libquark.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <syslog.h>
#include <assert.h>
#include <errno.h>
#include <signal.h>
#include <getopt.h>
#include <ctype.h>
#include <fnmatch.h>

#include "argument.h"
#include "box_conf.h"

#include "config.h"
#include "libquark.h"
#include "long-options.h"

#include "brett_db.h"
#include "open_service.h"

/* the good */
static int got_alarm = 0;
static char nntp_buf[2048];
static char *nntp_ptr;
static int nntp_fd;
static int nntp_maxread;
static int nntp_offset;
const char *program_name;

/* and the good, but ugly */
#define SENDLINE(str) do { \
		if (!nntp_sendline(str)) { \
			syslog(LOG_NOTICE,"can't write to nntp server: %s",strerror(errno)); \
			fprintf(stderr,"can't write to nntp server: %s\r\n",strerror(errno)); \
			close(nntp_fd); \
			nntp_fd=-1; \
			return FALSE; \
		} \
	} while (0)
#define READLINE(failstart,resvar) do { \
		ssize_t sgr=nntp_getline(); \
		if (sgr<=0) { \
			syslog(LOG_NOTICE,"can't read nntp answer"); \
			fprintf(stderr,"can't read nntp answer\r\n"); \
			close(nntp_fd); \
			nntp_fd=-1; \
			return FALSE; \
		} \
		resvar=strtol(nntp_ptr,NULL,10); \
		if (resvar>=failstart) { \
			syslog(LOG_NOTICE,"nntp answered us with %s",nntp_ptr); \
			fprintf(stderr,"nntp answered us with %s\r\n",nntp_ptr); \
			close(nntp_fd); \
			nntp_fd=-1; \
			return FALSE; \
		} \
	} while (0)
#define FOREACHNNTPLIST_START while(1) { \
		ssize_t sgr=nntp_getline(); \
		if (sgr<=0) { \
			syslog(LOG_NOTICE,"can't read nntp answer"); \
			fprintf(stderr,"can't read nntp answer\r\n"); \
			close(nntp_fd); \
			nntp_fd=-1; \
			return FALSE; \
		} \
		if (nntp_ptr[0]=='.' && !nntp_ptr[1]) \
			break;
#define FOREACHNNTPLIST_END }
#define SENDGET(str,failstart,resvar) do { \
		SENDLINE(str); \
		READLINE(failstart,resvar); \
	} while (0)

static void
handler (int x)
{
	got_alarm = x;
}

static ssize_t
nntp_getline (void)
{
	if (nntp_maxread) {
		char *p;
		for (p = nntp_buf + nntp_offset; *p && p != nntp_buf + nntp_maxread; p++) {
			if (*p == '\n') {
				nntp_ptr = nntp_buf + nntp_offset;
				nntp_offset = (p - nntp_buf) + 1;
				*p = 0;
				if (p[-1] == '\r') {
					p--;
					*p = 0;
				}
				return p - nntp_ptr;
			}
		}
		/* ok, unvollstndige Zeile im Puffer */
		memmove (nntp_buf, nntp_buf + nntp_offset, nntp_maxread - nntp_offset);
		nntp_maxread = nntp_maxread - nntp_offset;
	}
	while (1) {
		ssize_t this;
		char *p;
		alarm (30);
		this = read (nntp_fd, nntp_buf + nntp_maxread, sizeof (nntp_buf) - nntp_maxread - 1);
		if (this == 0 || this == -1) {
			alarm (0);
			return this;
		}
		for (p = nntp_buf + nntp_maxread; p != nntp_buf + nntp_maxread + this; p++) {
			if (*p == '\n') {
				nntp_offset = (p - nntp_buf) + 1;
				*p = 0;
				if (p[-1] == '\r') {
					p--;
					*p = 0;
				}
				nntp_ptr = nntp_buf;
				alarm (0);
				nntp_maxread += this;
				return p - nntp_ptr;
			}
		}
		nntp_maxread += this;
	}
}

static BOOLEAN
nntp_sendline (const char *line)
{
	ssize_t l = strlen (line);
	return (l == xwrite (nntp_fd, line, l));
}


static BOOLEAN
nntp_abgleich (const char *nntpserver,const char *port, char **match)
{
	char *err;
	ssize_t got;
	BRETTNR bnr;
	nntp_fd = open_service (port, nntpserver, &err);
	if (nntp_fd == -1) {
		syslog (LOG_NOTICE, "can't open nntp service: %s", err);
		fprintf (stderr, "can't open nntp service: %s\r\n", err);
		return FALSE;
	}
	signal (SIGALRM, handler);
	nntp_maxread = 0;
	nntp_offset = 0;
	READLINE (300, got);
	SENDGET ("mode reader\r\n", 300, got);
	SENDGET ("list active *\r\n", 300, got);
	printf("#CMD\n");

	FOREACHNNTPLIST_START {
		char *name = nntp_ptr;
		char *min, *max, *status;
		max = strchr (name, ' ');
		if (!max)
			continue;
		*max++ = 0;
		if (match) {
			int i;
			for (i=0;match[i];i++) {
				if (0==fnmatch(match[i],name,0))
					break;
			}
			if (!match[i])
				continue;
		}
		min = strchr (max, ' ');
		if (!min)
			continue;
		*min++ = 0;
		status = strchr (min, ' ');
		if (!status)
			continue;
		*status++ = 0;
		bnr = brettnr (name);
		if (!bnr) {
			/* a new group */
			printf(":GA%s\n",name);
			switch(*status) {
			case 'y': /* normale Gruppe */
				break;
			case 'n': /* keine lokalen Postings erlaubt, nur remote darf schreiben */
				printf(":GZ%s=R0W7\n",name);
				break;
			case 'x': /* keine Postings erlaubt */
				printf(":GZ%s=R0W9\n",name);
				break;
			case 'm': /* moderiert */
				printf(":GZ%s=O+\n",name);
				break;
			case 'j': /* not kept, but passed on */
				/* unsupported */
				break;
			case '=': /* =news.announce.important -> umlenken in andere Gruppe zur Speicherung */
				/* unsupported */
				break;
			}
		} else {
			int mod=0;
			switch(*status) {
			case 'y': /* normale Gruppe */
				break;
			case 'n': /* keine lokalen Postings erlaubt, nur remote darf schreiben */
				break;
			case 'x': /* keine Postings erlaubt */
				break;
			case 'm': /* moderiert */
				mod=1;
				if (!(br_ind[bnr].brett->flags & B_MODERIERT)) 
					printf(":GZ%s=O+\n",name);
				break;
			case 'j': /* not kept, but passed on */
				break;
			case '=': /* =news.announce.important -> umlenken in andere Gruppe zur Speicherung */
				break;
			}
			if (!mod) {
				if (br_ind[bnr].brett->flags & B_MODERIERT) 
					printf(":GZ%s=O-\n",name);
			}
			br_ind[bnr].brett->tmpflags|=1; /* done */
		}
	} FOREACHNNTPLIST_END
	/* ok. Nun durch die Liste gehen um zu prfen was wir zuviel haben */
	FOR_EACH_BRETTNR(bnr) {
		if (!br_ind[bnr].brett)
			continue;
		if (br_ind[bnr].brett->tmpflags)
			continue;
		if (match) {
			int i;
			char buf[BDAT_GRP_NAME_LEN*2];
			bb_canonical(br_ind[bnr].brett->name,buf);
			for (i=0;match[i];i++) {
				if (0==fnmatch(match[i],buf,0))
					break;
			}
			if (!match[i])
				continue;
		}
		printf(":GL%s\n",br_ind[bnr].brett->name);
		br_ind[bnr].brett->tmpflags|=2; /* deleted */
	}
	SENDGET ("list newsgroups\r\n", 300, got);

	FOREACHNNTPLIST_START {
		char *text;
		text=strpbrk(nntp_ptr,"\t ");
		if (text)
			*text++=0;
		else
			continue;
		while (isspace(*text))	
			text++;
		if (!*text)
			continue;
		if (match) {
			int i;
			for (i=0;match[i];i++) {
				if (0==fnmatch(match[i],nntp_ptr,0))
					break;
			}
			if (!match[i])
				continue;
		}
		bnr = brettnr (nntp_ptr);
		if (!bnr)
			continue; /* removed groups are never remove from newsgroups, only from active */
		if (!br_ind[bnr].brett->userinfo
			|| 0!=strcmp(text,br_ind[bnr].brett->userinfo)) {
			printf(":GB%s=0:%s\n",nntp_ptr,text);
		}
	} FOREACHNNTPLIST_END

	SENDGET ("QUIT\r\n", 300, got);
	close (nntp_fd);
	
	nntp_fd = -1;
	return TRUE;
}

static void 
usage (int ecode)
{	
	FILE *f;
	if (ecode==0)
		f=stdout;
	else
		f=stderr;
	fprintf(f,"usage: %s [options]\n",program_name);
	fprintf(f,"  options include:\n");
	fprintf(f,"  -C --cnffile FILENAME   where to find the quark config file\n");
	fprintf(f,"  -p --port PORT          name or number of the 'nntp' port to use\n");
	fprintf(f,"  -s --server NAME        name or address of the nntp server to use\n");
	fprintf(f,"  -m --match WILDCARDS    wildcards separated with ,; space or tab\n");
	fprintf(f,"wildcards can be used to determine which hierarchies are checked\n"); 
	exit (ecode);
}

struct option opts[] =
{
    {"help", 0, NULL, 'h'},
    {"cnffile",required_argument,NULL,'C'},
    {"port", required_argument, NULL, 'p'},
    {"server", required_argument, NULL, 's'},
    {"version", 0, NULL, 'v'},
    {"match", 0, NULL, 'm'},
    {NULL, 0, NULL, 0}
};

int
main (int argc, char **argv)
{
	const char *port="nntp";
	const char *server="localhost"; /* best default */
	char **match=NULL;
	int matchcount=0;
	int matchsize=0;
	int option_index = 0;

	if (argv[0]) {
		program_name = strrchr (argv[0], '/');
		if (program_name)
			program_name++;
		else
			program_name = argv[0];
		program_name = XSTRDUP (program_name);	/* we modify argv! */
	} else
		program_name = "tom";

	parse_long_options (argc, argv, program_name, PACKAGE_VERSION, usage);

	while (1) {
		int c;
		c = getopt_long (argc, argv, "C:p:s:hm:", opts, &option_index);

		/* Detect the end of the options.   */
		if (c == -1)
			break;
		switch(c) {
		case 'c': argument_cnffile=optarg; break;
		case 'h': usage(0); break;
		case 'p': port=optarg; break;
		case 's': server=optarg; break;
		case 'm':
			{
				char *next,*s=optarg;
				for (s=optarg;s && *s;s=next) {
					next=strpbrk(s,";, \t");
					if (next) {
						*next++=0;
						while (strchr(";, \t",*next))
							next++;
					}
					if (matchsize==0) {
						matchsize=10;
						match=XMALLOC(matchsize*sizeof(char *));
					} else if (matchcount>=matchsize-1) {
						matchsize*=2;
						match=XREALLOC(match,matchsize*sizeof(char *));
					}
					match[matchcount++]=XSTRDUP(s);
				}
				match[matchcount]=NULL;
			}
			break;
		default:
			usage(2);
			break;
		}
	}

	if (!read_quark_configuration(
			argument_cnffile? argument_cnffile : SYSCONFDIR "/quark.cnf")) {
		fprintf(stderr,"Auswertung von %s fehlgeschlagen\n",
		argument_cnffile? argument_cnffile : SYSCONFDIR "/quark.cnf");
		exit(1);
	}

	if (!bb_init(box_conf.var_dir,box_conf.bretter_dir,
		box_conf.binbrett_dir))
	{
		fprintf(stderr,"cannot open group database\n");
		exit(1);
	}

	nntp_abgleich (server,port,match);
	exit (0);
}
