/* readkonf.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Netzprogramm

	Konfigurationsdatei-Verarbeitung

	(c) Frank Baumgart

	begonnen am 27.09.1992
	Version vom 08.08.1994
	Version 2.03

	Konfigurationsdatei einlesen und Variablen auswerten
*/

#include "portab.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <unistd.h>
#ifdef __UNIX__
#include <pwd.h>
#include <sys/types.h>
#endif

#include "system.h"
#include "readkonf.h"

#ifdef BOX

#define DEFAULT_CONFIG		"quark.cnf"
#define DEFAULT_RC			".quarkrc"

#else

#define DEFAULT_CONFIG		"offline.cnf"
#define DEFAULT_RC			".offlinerc"

#endif



/* Schlsselworte case-sensitiv ? */
#ifdef NOCASE
#define str_comp(s,t,l)		strnicmp(s,t,l)
#else
#define str_comp(s,t,l)		strncmp(s,t,l)
#endif



extern long filesize(FILE *stream);



/* leerer String als Default fr Stringvariablen */
static char leer[] = "";



/*
	Konfigurationsdatei auswerten

	->			cnf_name	:	Name der Konfigurationsdatei bzw. NULL
				cmds		:	Zeiger auf Konfigurationsvariablen

	<-			TRUE		:	ok
				FALSE		:	Fehler
*/
BOOLEAN cnf_read(const char *cnf_name,cmd_typ cmds[])
{
	FILE *cnf_f;				/* stream fr Konfigurationsdatei */
	char zeile[1024];			/* eingelesene Konfigurationszeile */
	char *pos;					/* Zeiger darauf */
	long zeilennr = 0;		/* aktuelle Zeilennummer */
	int nr = 0;					/* aktuell untersuchter String */
	BOOLEAN found;				/* Flag, ob Kommando erkannt */
	BOOLEAN ok = TRUE;		/* Flag, ob Konfigdatei fehlerlos */
	const char *home;

	home = getenv("HOME");

	/* smtliche internen Konfigurationseintrge initialisieren */
	while (cmds[nr].name != NULL)
	{
		switch (cmds[nr].typ)
		{
			case 's' :
			case 'p' :
				*(char **)cmds[nr].par = leer;
				break;

			case 'n' :
				*(long *)cmds[nr].par = cmds[nr].init;
				break;

			case 'b' :
				*(BOOLEAN *)cmds[nr].par = FALSE;
				break;

			case 'B' :
				*(BOOLEAN *)cmds[nr].par = TRUE;
				break;
		}
		nr++;
	}

	/* Lesen im Textmodus */
	if (cnf_name)
		cnf_f = fopen(cnf_name,"r");
	else
	{
		if ((cnf_f = fopen(DEFAULT_RC,"r")) == NULL)
		{
#ifdef __UNIX__
			char tmp[PATH_MAX];

			if (home)
			{
				strcpy(tmp,home);
				strcat(tmp,"/");
				strcat(tmp,DEFAULT_RC);
				cnf_f = fopen(tmp,"r");
			}
#endif
			if (!cnf_f)
				cnf_f = fopen(DEFAULT_CONFIG,"r");
		}
	}
	if (!cnf_f)
	{
		fputs("keine Konfigurationsdatei\r\n",stderr);
		return TRUE;
	}

	while (fgets(zeile,(int)(sizeof(zeile)-1),cnf_f))
	{
		char *p;
		size_t len;

		zeilennr++;

		/* Kommentarblock ? */
		if (!strncmp(zeile,"###",3))
		{
			long start = zeilennr;

			while (1)
			{
				if (!fgets(zeile,(int)(sizeof(zeile)-1),cnf_f))
				{
					fprintf(stderr,"Kommentarblock (ab Zeile %ld) nicht beendet\r\n",start);
					ok = FALSE;
					break;
				}
				zeilennr++;
				if (!strncmp(zeile,"###",3))
					break;
			}
		}

		pos = zeile;

		/* Leerzeichen usw. berlesen */
		while (isspace(*pos))
			pos++;
		/* Kommentar- oder Leerzeile ? */
		if (strpbrk(pos,";#\r\n") == pos  ||  !(*pos))
			continue;

		/* Text mit den verfgbaren Kommandos vergleichen */
		found = FALSE;
		nr = 0;
		while (cmds[nr].name != NULL)
		{
			len = strlen(cmds[nr].name);
			if (!str_comp(cmds[nr].name,pos,len))
			{
				pos += len;
				found = TRUE;
				break;
			}
			nr++;
		}

		if (!found)
		{
			fprintf(stderr,"Fehler in Konfigurationszeile %ld:\n",zeilennr);
			fprintf(stderr,"%s\n",zeile);
			ok = FALSE;
			continue;
		}

		/* Auswertung der Parameter */
		/* erstmal wieder Leerzeichen berlesen */
		while (isspace(*pos))
			pos++;
		/* optional kann ein '=' angegeben werden */
		if (*pos == '=')
		{
			pos++;
			/* dann wieder mal Leerzeichen berlesen */
			while (isspace(*pos))
				pos++;
		}

		/* Zeilenende entfernen */
		if ((p = strpbrk(pos,"\r\n")) != NULL)
		{
			*p-- = 0;
			while (p > pos && isspace(*p))
				*p-- = 0;
		}

		/* folgt rechts von einem ';' kein Quote, ist es ein Kommentar */
		while ((p = strrchr(pos,';'))!=NULL && !strrchr(p,'\"'))
		{
			*p-- = 0;
			while (p > pos && isspace(*p))
				*p-- = 0;
		}

		/* durch Anfhrungszeichen eingeschlossen ? */
		if (*pos == '\"')
		{
			pos++;
			if (strrchr(pos,'\"'))
				*strrchr(pos,'\"') = 0;
		}
		else
		/* ggf. Tilde-Expansion */
		if (*pos == '~')
		{
#ifdef __UNIX__
			/* expliziter Username ? */
			if (isalpha(*(pos+1)))
			{
				char tmp[1024];
				char name[30];
				size_t name_len;
				struct passwd *pwd;

				pos++;
				name_len = strcspn(pos,"/");
				strncpy(name,pos,name_len);
				name[name_len] = 0;
				if ((pwd = getpwnam(name)) == NULL)
				{
					fputs("unbekannter Username",stderr);
					return FALSE;
				}
/*				puts(pwd->pw_dir);*/
				strcpy(tmp,pwd->pw_dir);
				while (isalpha(*pos) || isdigit(*pos))
					pos++;
				strcat(tmp,pos);
				strcpy(pos,tmp);
			}
			else
#endif
			if (home)
			{
				char tmp[1024];
				strcpy(tmp,home);
				strcat(tmp,pos+1);
				strcpy(pos,tmp);
			}
			else
				*pos = '.';
		}

		/* numerischer, boolescher oder String/Pfadparameter ? */
		switch (cmds[nr].typ)
		{
			case 's' :
				if ((*(char **)cmds[nr].par = strdup(pos)) == NULL)
					ok = FALSE;
				break;

			case 'p' :
				len = strlen(pos);
				if (len && pos[len-1] != '/' && pos[len-1] != '\\')
					strcat(pos,PS);
				if ((*(char **)cmds[nr].par = strdup(pos)) == NULL)
					ok = FALSE;
				break;

			case 'n' :
				*(long *)cmds[nr].par = atol(pos);
				break;

			case 'b' :
			case 'B' :
				if (!strncmp(pos,"FALSE",5) || !strncmp(pos,"false",5))
					*(BOOLEAN *)cmds[nr].par = FALSE;
				else
				if (!strncmp(pos,"TRUE",4)  || !strncmp(pos,"true",4))
					*(BOOLEAN *)cmds[nr].par = TRUE;
				break;
		}
	}

	fclose(cnf_f);

	return ok;

} /* cnf_read */



/*
	aktuelle Konfiguration exportieren

	->			cnf_name	:	Dateiname bzw. NULL fr stdout
				init		:	1 = inkl. aktueller Variableninhalte , 0 = leer

	<-			TRUE		:	ok
				FALSE		:	Datei konnte nicht geffnet werden

	Diese Funktion kann auch ntzlich sein, um umfangreiche CNF-Dateien in 
	bereits ausgewerteter Form und ohne Kommentare etc. als INF zu speichern 
	und anstelle der CNF-Datei zu verwenden, sofern die CNF-Datei nicht neuer 
	als die INF-Datei ist.
*/
BOOLEAN cnf_export(const char *cnf_name,cmd_typ cmds[],BOOLEAN init)
{
	FILE *cnf_f;
	int nr;

	if (!cnf_name)
		cnf_f = stdout;
	else
		if ((cnf_f = fopen(cnf_name,"w")) == NULL)
			return FALSE;

	/* Text mit den verfgbaren Kommandos vergleichen */
	nr = 0;
	while (cmds[nr].name != NULL)
	{
		fprintf(cnf_f,"%s = ",cmds[nr].name);
		if (init)
		/* numerischer, boolescher oder String/Pfadparameter ? */
		switch (cmds[nr].typ)
		{
			case 's' :
				fputs(*(char **)cmds[nr].par,cnf_f);
				break;

			case 'p' :
				fputs(*(char **)cmds[nr].par,cnf_f);
				break;

			case 'n' :
				fprintf(cnf_f,"%ld",*(long *)cmds[nr].par);
				break;

			case 'b' :
			case 'B' :
				if (*(BOOLEAN *)cmds[nr].par)
					fputs("TRUE",cnf_f);
				else
					fputs("FALSE",cnf_f);
				break;
		}

		fputs("\r\n",cnf_f);
		nr++;
	}

	if (cnf_name)
		fclose(cnf_f);

	return TRUE;

} /* cnf_export */



#ifdef BOX
/*
	Konfigurationsvariable auslesen bzw. setzen
	spez. fr externe Module
	Fr Strings ist noch etwas auszuarbeiten (Puffer)

	->			s		:	Bezeichnung der Konfigurationsvraiablen
				var	:	Puffer fr bzw. Zeiger auf Wert der Variablen
				set	:	TRUE = Wert setzen, FALSE = Wert abfragen

	23.02.94 Frank
*/
BOOLEAN cnf_getset(cmd_typ cmds[],const char *s,void *var,BOOLEAN set)
{
	cmd_typ *pc = cmds;

	while (pc->name)
	{
		if (!strcmp(s,pc->name))
		{
			switch (pc->typ)
			{
				case 's' :
				case 'p' :
					if (set)
/*						strcpy((char *)pc->par,(char *)var);*/
						;
					else
						strcpy((char *)var,(char *)pc->par);
					return TRUE;

				case 'n' :
					if (set)
						*(long *)pc->par = *(long *)var;
					else
						*(long *)var = *(long *)pc->par;
					return TRUE;

				case 'b' :
				case 'B' :
					if (set)
						*(BOOLEAN *)pc->par = *(BOOLEAN *)var;
					else
						*(BOOLEAN *)var = *(BOOLEAN *)pc->par;
					return TRUE;
			}
		}
		pc++;
	}

	return FALSE;

} /* cnf_getset */
#endif
