/* gruppen.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	Gruppen- & Userlisten-Verwaltung
	(c) 1992-95 bei Frank Baumgart

	begonnen am 02.04.1992
	Version vom 25.01.1995
*/

#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <limits.h>
#include <unistd.h>

#include "system.h"
#include "mdb_conf.h"
#include "struktur.h"
#include "mdb_intr.h"
#include "mdb_glob.h"
#include "mdb/msgid.h"
#include "message.h"
#include "screen/io.h"
#include "tool.h"
#include "fehler.h"
#include "missing/missing.h"
#include "frontend/frontend.h"


#define GRUPPEN			"gruppen"
#define USER				"user"

#define GRP_NAME			"grp%05ld"
#define USR_NAME			"usr%05ld"

#define TRENNZEICHEN		','		/* Trennzeichen zwi. Gruppenname und Pfad */

extern CONFIG config;				/* globale Einstellungen (Verkettung...) */
extern KENNUNG leere_kennung;		/* zum Anlegen neuer Gruppen */


typedef struct
{
	char *name;				/* Zeiger auf Gruppen/Username */
	char *pfad;				/* Zeiger auf Pfadname bzw. NULL */
	char *info;				/* Kurzinfo aus der ITI (unbenutzt) */
	CRC crc;					/* CRC ber Gruppen/Usernamen (berechnet, unbenutzt) */
	MESSAGE neue;			/* Anzahl ungelesene Nachrichten (berechnet, unbenutzt) */
	MESSAGE anzahl;		/* Anzahl aller Nachrichten (berechnet, unbenutzt) */
	BOOLEAN locked;		/* Gruppe exklusiv geffnet ? */

	/* ### wenn hier "berechnet" steht, gilt das nur f. bereits ex. Gruppen! */

} GRP_CACHE;

/* max. Tabelleneintrge fr Gruppen/User */
static size_t maximum[2] = { 1000, 1000 };

/* Indizes fr lexikographisch sortierte Gruppen/User-Eintrge */
static size_t *sort_name[2] = { NULL, NULL };

static GRP_CACHE *puffer[2] = { NULL, NULL };
static GRUPPE anzahl[2] = { 0, 0} ;	/* Anzahl Gruppen/User */

/* dirty-Flag fr Gruppen/User-nderungen */
static BOOLEAN dirty[2] = { FALSE, FALSE };


static BOOLEAN grp_expand(BOOLEAN pm);
static BOOLEAN grp_init(BOOLEAN pm);



/*
	Gruppen- bzw. Userliste erweitern

	<-			TRUE	:	ok
				FALSE	:	zuwenig Speicher
*/
static BOOLEAN grp_expand(BOOLEAN pm)
{
	GRP_CACHE *t1;
	size_t *t2;

	t1 = (GRP_CACHE *) realloc(puffer[pm],(maximum[pm] + 1000) * sizeof(GRP_CACHE));
	if (!t1)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}

	memset(t1 + maximum[pm],0,sizeof(GRP_CACHE) * 1000);
	maximum[pm] += 1000;
	puffer[pm] = t1;

	t2 = (size_t *) realloc(sort_name[pm],maximum[pm] * sizeof(size_t));
	if (!t2)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}
	sort_name[pm] = t2;

	return TRUE;

} /* grp_expand */



static int srt_pm;

static int srt_groups(const void *a,const void *b)
{
	return stricmp(puffer[srt_pm][*(size_t *)a].name,puffer[srt_pm][*(size_t *)b].name);

} /* srt_groups */



static void grp_sort(BOOLEAN pm)
{
	long i,n;

	i = n = 0;
	while (i < anzahl[pm])
	{
		while (!puffer[pm][n].name)
			n++;
		sort_name[pm][i++] = n++;
	}

	srt_pm = pm;
	qsort(sort_name[pm],anzahl[pm],sizeof(size_t),srt_groups);

#if 0
for (i = 0; i < anzahl[pm]; i++)
	puts(puffer[pm][sort_name[pm][i]].name);
taste();
#endif

} /* grp_sort */



/*
	Gruppen- und Userlisten intern aufbauen

	->			pm		:	TRUE = User, FALSE = Gruppen

	<-			FALSE	:	keine Gruppenlistendatei vorhanden
							(also unabhngig von der Pufferung)
							bzw. kein Speicher mehr
				TRUE	:	sonst ok
*/
static BOOLEAN grp_init(BOOLEAN pm)
{
	char datei[PATH_MAX];
	FILE *lst_f;
	GRUPPE nr = 0;

	/* bereits gepuffert ? */
	if (puffer[pm])
		return TRUE;

	puffer[pm] = (GRP_CACHE *) calloc(maximum[pm],sizeof(GRP_CACHE));
	if (puffer[pm] == NULL)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}

	anzahl[pm] = 0;

	strcpy(datei,config.system);
	strcat(datei,(pm) ? USER : GRUPPEN);

	/* Gruppen/Userlistendatei ffnen */
	if ((lst_f = fopen(datei,"r")) != NULL)
	{
		char zeile[MAX_CPG + PATH_MAX];

		pufferung(lst_f,NULL,_IOFBF,4096);

		while (fgets(zeile,MAX_CPG + PATH_MAX,lst_f))
		{
			char *pos;
			size_t len;

			if ((pos = strpbrk(zeile,"\r\n")) != NULL)
				*pos = 0;
			len = strlen(zeile);
			if (len && (nr <= maximum[pm]))
			{
				if ((nr == maximum[pm]) && !grp_expand(pm))
					return FALSE;

				/* Gruppenname und Pfadname werden zusammen verwaltet */
				if ((puffer[pm][nr].name = (char *) malloc(len+1)) == NULL)
				{
					fehler(NO_MEMORY);
					return FALSE;
				}

				/* Eintrag bernehmen */
				strcpy(puffer[pm][nr].name,zeile);

				/* Pfad-Trennzeichen suchen */
				if ((pos = strchr(puffer[pm][nr].name,TRENNZEICHEN)) != NULL)
				{
					puffer[pm][nr].pfad = pos + 1;
					*pos = 0;
				}
				else
					puffer[pm][nr].pfad = NULL;

				/* CRC-Berechnung ber den Namen */
				puffer[pm][nr].crc = crc32ansi(puffer[pm][nr].name,strlen(puffer[pm][nr].name));
anzahl[pm]++;
			}
			else
			{
				puffer[pm][nr].name = NULL;
			}

/*			anzahl[pm]++;*/
/*printf("anz[pm] = %ld\r\n",(long) anzahl[pm]);*/
			nr++;
		}
		fclose(lst_f);
	}

	/* Array mit sortierten Gruppen/Usernamen aufbauen */
	if ((sort_name[pm] = (size_t *) calloc(maximum[pm],sizeof(size_t))) == NULL)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}
	grp_sort(pm);

#if 0 /*ndef BOX*/
	/* Anzahl neuer und gesamter Nachrichten ermitteln, langsam */
	{
		KENNUNG kennung;
		long i;

		for (i = 0; i < anzahl[pm]; i++)
		{
			if (!group_info(puffer[pm][i].name,pm,&kennung))
				puffer[pm][i].neue = puffer[pm][i].anzahl = 0;
			else
			{
				puffer[pm][i].neue = kennung.neue;
				puffer[pm][i].anzahl = kennung.anzahl;
			}
		}
	}
#endif

	/*
		kurze Gruppenbeschreibungen eintragen

		Diese werden aus der ITG ermittelt und zum schnelleren Zugriff in 
		derselben Reihenfolge wie in "gruppen" in der Datei "gruppen.inf" 
		gespeichert.
		Letztere wird erneuert, wenn "gruppen" neuer als "gruppen.inf" ist.

		todo:
		-"gruppen.inf" erneuern, falls ITG neuer

		bugs:
		- wenn whrend der Laufzeit von Offline neue Gruppen importiert werden, 
		  wird dafr kein Eintrag erzeugt

		Ziel:
		wenn in eine leere DB importiert wird, mssen danach die Zhler fr 
		neue/gesamte Mails auf akt. Stand sein, ebenso die akt.(!) ITG-Eintrge 
		Weg:
		beim Startup smtliche ITG-Eintrge merken, bei Import einer neueren 
		ITG verwerfen und neu aufbauen
	*/
#if 0
	if (!pm)
	{
		char inf[PATH_MAX];
		char lst[PATH_MAX];
		struct stat inf_stat,lst_stat;

		strcpy(inf,config.system);
		strcat(inf,"gruppen.inf");
		strcpy(lst,config.system);
		strcat(lst,"gruppen");
		if (stat(inf,&lst_stat) == -1)
			...
		if (stat(inf,&inf_stat) == -1)
			refresh = TRUE;
		else
		{
			
		...
	}
#endif

	return TRUE;

} /* grp_init */



/*
	kpl. Gruppen/Userliste in Datei sichern

	<-			TRUE	:	Update ok
				FALSE	:	Fehler (beim Dateihandling)

	Diese Fn. beendet das Programm bei schweren Fehlern selbst
*/
BOOLEAN grp_update(BOOLEAN pm)
{
	char datei[PATH_MAX];
	char backup[PATH_MAX];
	FILE *lst_f;					/* Gruppen- bzw. Userlistendatei */
	GRUPPE nr = 0;
	GRUPPE rest;					/* Anz. Eintrge, um Leerzeilen zu vermeiden */

	/* Aktualisierung aufgrund von nderungen berhaupt notwendig ? */
	if (!dirty[pm])
		return TRUE;

	/* altes Backup der Gruppen/Userlistendatei lschen */
	strcpy(datei,config.system);
	strcat(datei,(pm) ? USER : GRUPPEN);
	strcpy(backup,datei);
	strcat(backup,"~");
	(void) unlink(backup);

	/* alte Datei umbenennen */
	(void) rename(datei,backup);

	if ((lst_f = fopen(datei,"w")) == NULL)
	{
		perror("grp_update (file creation)");
		fehler((pm) ? NO_USRLST : NO_GRPLST);
		if (rename(backup,datei) == -1)
			perror("re-rename");
		exit(1);
	}

	/* Gruppen/Userdatei mit 4 KB puffern */
	pufferung(lst_f,NULL,_IOFBF,4096);

	rest = anzahl[pm];

	while ((nr < maximum[pm]) && rest)
	{
		if (puffer[pm][nr].name)
		{
			fwrite(puffer[pm][nr].name,strlen(puffer[pm][nr].name),1,lst_f);
			if (puffer[pm][nr].pfad)
			{
				fputc(TRENNZEICHEN,lst_f);
				fwrite(puffer[pm][nr].pfad,strlen(puffer[pm][nr].pfad),1,lst_f);
			}
			rest--;
		}
		fputc('\n',lst_f);
		nr++;
	}
	fclose(lst_f);

	dirty[pm] = FALSE;

	return TRUE;

} /* grp_update */



#if 0
/*
	Gruppen/Userliste freigeben
*/
BOOLEAN grp_close(BOOLEAN pm)
{
	GRUPPE nr = 0;

	if (dirty[pm])
		grp_update(pm);

	/* erst Eintrge, dann Puffer selber freigeben */
	while (nr < maximum[pm])
	{
		if (puffer[pm][nr].name)
			free(puffer[pm][nr].name);
		nr++;
	}

	return FALSE;

} /* grp_close */
#endif



/*
	Datei mit allen Gruppen (opt. +User) in sortierter Form liefern

	->			datei	:	Zieldatei, ein Name je Zeile
				pm		:	TRUE = auch Usernamen
			<-	anz	:	Anzahl Namen (0 - n)

	<-			TRUE	:	immer
				FALSE	:	fataler Fehler in Gruppenverwaltung

	todo:
	- Erweitern mit group_info() fr nntp-Format
	- opt. letzten Importzeitpunkt ausgeben
*/
BOOLEAN mdb_grouplist(const char *datei,BOOLEAN pm,long *anz)
{
	KENNUNG k;
	BOOLEAN zeit = FALSE;
	FILE *lst_f;
	char *name;
	long i;

	if (!puffer[0] && !grp_init(0))
		return FALSE;
	if (!puffer[1] && !grp_init(1))
		return FALSE;

	*anz = 0;

	if ((lst_f = fopen(datei,"w")) == NULL)
		return TRUE;

	if (pm)
	{
		for (i = 0; i < anzahl[1]; i++)
		{
			name = puffer[1][sort_name[1][i]].name;
			fprintf(lst_f,"*%s",name);
			if (zeit && group_info(name,TRUE,&k))
				fprintf(lst_f,",%lu\r\n",k.changed);
			else
				fputs("\r\n",lst_f);
		}
		*anz += anzahl[1];
	}

	for (i = 0; i < anzahl[0]; i++)
	{
		name = puffer[0][sort_name[0][i]].name;
		fputs(name,lst_f);
		if (zeit && group_info(name,TRUE,&k))
			fprintf(lst_f,",%lu\r\n",k.changed);
		else
			fputs("\r\n",lst_f);
	}
	*anz += anzahl[0];

	fclose(lst_f);

	return TRUE;

} /* mdb_grouplist */



/*
	Datenbank-Pfade ermitteln

	->			name			:	Name der Gruppe/User (ASCII-String + CR/LF)
			<-	basepath		:	Speicher fr Pfad- und Datei-Basename
				pm				:	PM (TRUE) oder Gruppe (FALSE)

	<-			-1				:	Fehler
				0 - x			:	Nummer der Gruppe

	Diese Funktion ordnet dem Namen einer Gruppe/User die entsprechenden 
	Pfade, wo Index- und Datendatei zu finden sind, zu.

	Wichtig ist, da innerhalb des gesamten Gruppen-Moduls eine getrennte 
	Behandlung von Gruppennamen und Usernamen stattfindet, so da sowohl 
	getrennte Listen fr Gruppen und User gefhrt als auch nur die Gruppen-
	listendatei gepuffert werden kann.
*/
long group_path(const char *name,char *basepath,BOOLEAN pm)
{
	long nr = 0;

	/* Userauswahl ? */
	if (pm && *name == '*')
		name++;

	if (!grp_init(pm))
		return -1;

	/*
		Index der Gruppe suchen
	*/
	while (nr < maximum[pm])
	{
		/* Gruppen/Uservergleiche nicht case-sensitiv */
		if (puffer[pm][nr].name 
			&& tolower((unsigned char) puffer[pm][nr].name[0])
				== tolower((unsigned char) name[0])
			&& !stricmp(puffer[pm][nr].name+1,name+1))
		{
			/* Zeiger auf Pfad setzen, falls vorhanden */
			char *pfad = puffer[pm][nr].pfad;
			char *p;

			if (!pfad)
				pfad = (pm) ? config.new_us : config.new_gr;
			else
			if ((p = strpbrk(pfad,"\r\n")) != NULL)
				*p = 0;

			/* Dateibasispfad erstellen, z.B. "msgbase/groups/grp00678." */
			if (basepath)
				sprintf(basepath,"%s%s%05ld.",pfad,(pm) ? "usr" : "grp",nr+1);

			return nr;
		}
		nr++;
	}

	return -1;

} /* group_path */



#if !defined(CLIENT)
/*
	neue Gruppe/User anlegen

	->				name	:	Gruppen/Username
					pm		:	TRUE = User, FALSE = Gruppe
					pfad	:	Pfad bzw. NULL fr default

	<-				TRUE	:	ok, Gruppe/User angelegt
					FALSE	:	Fehler (Speichermangel)

	Achtung:
	Es wird keine "dat"-Datei angelegt.
	Dazu besteht innerhalb von Offline auch keine Notwendigkeit, auerdem 
	legt der Import diese selbst an. Allerdings meckert die Box gelegentlich.
*/
BOOLEAN mdb_create_group(const char *name,BOOLEAN pm,const char *pfad)
{
	BOOLEAN ok = TRUE;
	KENNUNG kennung;
	char ind_pfad[PATH_MAX];
	FILE *ind_f;
	long nr = 0;
	size_t len;

	/* ex. die Gruppe bereits? */
	if (group_path(name,ind_pfad,pm) >= 0)
	{
		fehler(DUP_GROUP);
		return FALSE;
	}

	/* freien Eintrag suchen */
	while ((nr < maximum[pm]) && puffer[pm][nr].name)
		nr++;

	/* Tabelle erweitern, falls freie Eintrge fehlen */
	if ((nr == maximum[pm]) && !grp_expand(pm))
		return FALSE;

	len = strlen(name);
	if (pfad)
		len += strlen(pfad)+1;

	/* neue Gruppe/User eintragen */
	if ((puffer[pm][nr].name = (char *) malloc(len+1)) == NULL)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}
	strcpy(puffer[pm][nr].name,name);
	if (!pfad)
		puffer[pm][nr].pfad = NULL;
	else
	{
		puffer[pm][nr].pfad = puffer[pm][nr].name + strlen(puffer[pm][nr].name) +1;
		strcpy(puffer[pm][nr].pfad,pfad);
	}

	dirty[pm] = TRUE;
	anzahl[pm]++;

	/* Indexpfad erstellen */
	sprintf(ind_pfad,"%s%s%05ld.ind",
	       (pm) ? config.new_us : config.new_gr,
	       (pm) ? "usr" : "grp",nr+1);

	if ((ind_f = fopen(ind_pfad,BINWRITE)) == NULL)
		return FALSE;

	/*
		Kennung initialisieren
	*/
	kennung = leere_kennung;
	kennung.major = DB_MAJOR;
	kennung.minor = DB_MINOR;

	/* Status: PM oder ffentlich */
	kennung.pm = pm;

	/* Lnge der Datenstrukturen KENNUNG und INDEX eintragen */
	kennung.kn_size = (UWORD) sizeof(KENNUNG);
	kennung.ix_size = (UWORD) sizeof(INDEX);

	/* Gruppenname */
	strncpy(kennung.name,name,sizeof(kennung.name));

	/* aktuelles Datum und Uhrzeit */
	kennung.created = time(NULL);

	/* Standardkonfigurationswerte bernehmen */
	kennung.max_msgs = config.max_msgs;
	kennung.max_size = config.max_size;
	kennung.max_time = config.max_time;
/*	kennung.min_putz = config.min_putz;*/

	/* und Kennung schreiben */
	fwrite(&kennung,sizeof(KENNUNG),1,ind_f);
	fclose(ind_f);

	/* falls ein Pfad angegeben ist, wird die DB jetzt verschoben */
	if (pfad && *pfad)
		ok = mdb_move_group(name,pm,pfad);

	/* Gruppen-Datei updaten; ### evtl. VOR mdb_move_group ? */
#if 1
	/* paranoid, jedenfalls beim Import nicht notwendig */
	grp_update(pm);
#endif
	grp_sort(pm);

	return ok;

} /* mdb_create_group */



/*
	Gruppe zwischen Laufwerken/Pfaden verschieben

	->				name	:	Gruppenname
					pm		:	TRUE = User, FALSE = Gruppe
					dest	:	Zielpfad bzw. NULL

	<-				TRUE	:	ok
					FALSE	:	Fehler

	Wenn als Zielpfad NULL angegeben ist, wird die Gruppe in das Standard-
	gruppenverzeichnis (CNF: GroupPath) verschoben, ansonsten mu der Pfad 
	durch Slash bzw. Backslash abgeschlossen sein.
*/
BOOLEAN mdb_move_group(const char *name,BOOLEAN pm,const char *dest)
{
	char dat_pfad[PATH_MAX],
		  ind_pfad[PATH_MAX],
		  neu_pfad[PATH_MAX] = { 0 },
		  dat_neu_pfad[PATH_MAX],
		  ind_neu_pfad[PATH_MAX];
	char cmd[PATH_MAX];
	long nr;
	size_t len;
	char *p1,*p2;

	if ((nr = group_path(name,dat_pfad,pm)) < 0)
	{
		fehler(NO_GROUP);
		return FALSE;
	}
	strcpy(ind_pfad,dat_pfad);
	strcat(ind_pfad,"ind");
	strcat(dat_pfad,"dat");

	/* bisherigen Pfad ausgeben */
	if (puffer[pm][nr].pfad)
		mprintf("alter Pfad: %s\r\n",puffer[pm][nr].pfad);
	if (dest)
		strcpy(neu_pfad,dest);
	else
		strcpy(neu_pfad,config.new_gr);

	/* (Back)slash anhngen, falls nicht vorhanden */
	len = strlen(neu_pfad);
	if (len && neu_pfad[len-1] != '/' && neu_pfad[len-1] != '\\')
		strcat(neu_pfad,PS);

	/* Dateinamen = Basename anhngen */
	strcpy(ind_neu_pfad,neu_pfad);
	p1 = strrchr(ind_pfad,'/');
	p2 = strrchr(ind_pfad,'\\');
	if (p1 || p2)
		strcat(ind_neu_pfad,(p1 > p2) ? p1+1 : p2+1);
	else
		strcat(ind_neu_pfad,ind_pfad);

	/* neuer Pfad gleich altem ? dann einfach abbrechen */
	if (!strcmp(ind_neu_pfad,ind_pfad))
		return TRUE;

	/* Dateinamen = Basename anhngen */
	strcpy(dat_neu_pfad,neu_pfad);
	p1 = strrchr(dat_pfad,'/');
	p2 = strrchr(dat_pfad,'\\');
	if (p1 || p2)
		strcat(dat_neu_pfad,(p1 > p2) ? p1+1 : p2+1);
	else
		strcat(dat_neu_pfad,dat_pfad);

	sprintf(cmd,"mv %s %s",ind_pfad,ind_neu_pfad);
	if (shell(cmd))
	{
		mputs("Fehler beim Kopieren der Indexdatei");
		return FALSE;
	}
	sprintf(cmd,"mv %s %s",dat_pfad,dat_neu_pfad);
	if (shell(cmd))
	{
		mputs("Fehler beim Kopieren der Datendatei");
		return FALSE;
	}

	/* Pfadeintrag ndern */
	puffer[pm][nr].pfad = (char *) malloc(strlen(neu_pfad+1));
	if (!puffer[pm][nr].pfad)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}
	strcpy(puffer[pm][nr].pfad,neu_pfad);

	return TRUE;

} /* mdb_move_group */



/*
	Gruppe lschen bzw. komplett entfernen

	->			name	:	Name der zu entfernenden Gruppe
				pm		:	Typ, PM (TRUE) oder ffentlich (FALSE)

	<-			TRUE = ok , FALSE = Fehler

	Ablauf:
	- ID-CRCs in Killdatei merken
	- Lschen der Datendatei
	- Lschen der Indexdatei bzw. Reduktion auf Kennung
	- Entfernen der Gruppe aus der Gruppenlistendatei bei keep = FALSE
*/
BOOLEAN mdb_remove_group(const char *name,BOOLEAN pm)
{
	char id[MAX_HDR];
	char dat_pfad[PATH_MAX];
	char ind_pfad[PATH_MAX];
	FILE *ind_f;
	FILE *dat_f;
	KENNUNG kennung;
	INDEX ix;
	long nr;

	if ((nr = group_path(name,dat_pfad,pm)) < 0)
	{
		fehler(NO_GROUP);
		return FALSE;
	}
	strcpy(ind_pfad,dat_pfad);
	strcat(ind_pfad,"ind");
	strcat(dat_pfad,"dat");

	if (!group_info(name,pm,&kennung))
		return FALSE;

	if ((ind_f = fopen(ind_pfad,BINREAD)) == NULL)
	{
		fehler(IND_OPEN);
		return FALSE;
	}
	if ((dat_f = fopen(dat_pfad,BINREAD)) == NULL)
	{
		fclose(ind_f);
		fehler(DAT_OPEN);
		return FALSE;
	}

	/* IDs aus Ref-DB lschen */
	lies_kennung(ind_f,NULL);
	while (fread(&ix,sizeof(INDEX),1,ind_f))
	{
		fseek(dat_f,ix.pos,SEEK_SET);
		fgets(id,sizeof(id),dat_f);
		msgid_delete(id+1);
	}

	fclose(dat_f);
	fclose(ind_f);

	unlink(dat_pfad);
	unlink(ind_pfad);

	/* Eintrag aus der Gruppen/Userlistendatei entfernen */
	free(puffer[pm][nr].name);
	puffer[pm][nr].name = NULL;
	memset(&puffer[pm][nr],0,sizeof(GRP_CACHE));

	/* Datei aktualisieren */
	anzahl[pm]--;
	dirty[pm] = TRUE;
	grp_update(pm);
	grp_sort(pm);

	return TRUE;

} /* mdb_remove_group */



/*
	Gruppe umbenennen

	->			oldname	:	alter Name der Gruppe
				newname	:	neuer Name der Gruppe
				pm			:	Typ, PM (TRUE) oder ffentlich (FALSE)

	<-			TRUE		:	ok
				FALSE		:	Fehler bzw. Gruppe mit Zielnamen ex. bereits

	Sowohl in den Verwaltungsdaten ("gruppen"-Datei) als auch in den Nachrich-
	ten selbst wird eine Gruppenumbenennung durchgefhrt, wie sie i.A. durch 
	den REN-Block bekanntgegeben wird.
	Alter und neuer Name drfen auch gleich sein, damit auch Msgs. gewandelt 
	werden knnen, die bei einer manuellen Umbenennung in der "gruppen"-Datei 
	ihren alten Namen ja beibehalten haben.

	Nur whrend der Umbenennung der temp. Dateien wird exklusiver Zugriff auf 
	die Gruppe bentigt, da bis dahin alles eben in den temp. Dateien statt-
	findet und auf die Quelldateien nur Lesezugriffe erfolgen.
	evtl. mu der Import auch Lesezugriffe kurzfristig sperren; dabei mu 
	beachtet werden, da diese Funktion meist durch den Import selbst (bei 
	Auftreten eines REN-Blockes) aufgerufen wird.

	Optimierungsmglichkeiten:
	- wenn keine Umbenennungen stattfanden, auf rename() verzichten und tmp. 
	  Dateien nur lschen
*/
BOOLEAN mdb_rename_group(const char *oldname,const char *newname,BOOLEAN pm)
{
	char dat_pfad[PATH_MAX],
		  ind_pfad[PATH_MAX];
	char dat_tmp[PATH_MAX],
	     ind_tmp[PATH_MAX];
	FILE *dat_f = NULL,*dtmp_f = NULL;
	FILE *ind_f = NULL,*itmp_f = NULL;
	char *buf = NULL;
	MESSAGE msg_nr = 0;
	KENNUNG kennung;
	INDEX ix;
	long nr;
	long offset = 0;

	if (!strlen(oldname))
	{
		mputs("(rename) kein gltiger alter Gruppenname");
		return FALSE;
	}
	if (!strlen(newname))
	{
		mputs("(rename) kein gltiger neuer Gruppenname");
		return FALSE;
	}

	if ((nr = group_path(oldname,dat_pfad,pm)) < 0)
	{
		fehler(NO_GROUP);
		return FALSE;
	}
	strcpy(ind_pfad,dat_pfad);
	strcat(ind_pfad,"ind");
	strcat(dat_pfad,"dat");

	/* abbrechen, falls bereits eine Gruppe mit dem neuen Namen existiert */
	if (group_path(newname,NULL,pm) >= 0)
	{
		fehler(DUP_GROUP);
		return FALSE;
	}

	if ((buf = (char *) malloc(MSG_SIZE)) == NULL)
	{
		fehler(NO_MEMORY);
		return FALSE;
	}

	if ((dat_f = fopen(dat_pfad,BINREAD)) == NULL)
		goto abbruch;
	if ((ind_f = fopen(ind_pfad,BINREAD)) == NULL)
		goto abbruch;

	strcpy(dat_tmp,dat_pfad);
	strcpy(dat_tmp + strlen(dat_pfad)-3,"tmd");
	if ((dtmp_f = fopen(dat_tmp,BINWRITE)) == NULL)
		goto abbruch;

	strcpy(ind_tmp,ind_pfad);
	strcpy(ind_tmp + strlen(ind_pfad)-3,"tmi");
	if ((itmp_f = fopen(ind_tmp,BINWRITE)) == NULL)
		goto abbruch;

	/*
		nderung in Kennung der Indexdatei
	*/
	lies_kennung(ind_f,&kennung);
	memset(kennung.name,0,sizeof(kennung.name));
	strcpy(kennung.name,newname);
	fwrite(&kennung,sizeof(KENNUNG),1,itmp_f);

	/*
		jeweils einen Datensatz lesen, ggf. Header und Indexdaten anpassen
	*/
	while (fread(&ix,sizeof(INDEX),1,ind_f))
	{
		size_t bytes;
		size_t x;
		char *pos;

		ix.pos += offset;

		fread(buf,ix.t_text,1,dat_f);
		buf[ix.t_text] = 0;

		x = 0;
		while ((pos = strstr(buf + x,"\r\nG")) != NULL)
		{
			size_t len;

			len = strcspn(pos+3,"\r\n");
			x = pos - buf + len+3;
			if (len != strlen(oldname))
				continue;
			if (!strnicmp(pos+3,oldname,len))
				break;
		}

		if (pos)
		{
			fwrite(buf,(size_t)(pos - buf),1,dtmp_f);
			fwrite("\r\nG",3,1,dtmp_f);
			fwrite(newname,strlen(newname),1,dtmp_f);
			fwrite("\r\n",2,1,dtmp_f);
			pos += 5 + strlen(oldname);		/* 5 = strlen("\r\nG\r\n") */
			if (pos - buf < ix.t_text)
				fwrite(pos,ix.t_text - (size_t)(pos - buf),1,dtmp_f);
			/* Differenz immer neu berechnen, mglw. ex. mehrere alte Namen */
			bytes = strlen(newname) - strlen(oldname);
			ix.len += bytes;
			ix.t_text += bytes;
			offset += bytes;
		}
		else
			fwrite(buf,ix.t_text,1,dtmp_f);

		/* body lesen und schreiben */
		bytes = ix.len - ix.t_text;
		while (bytes)
		{
			size_t block;

			block = (bytes > MSG_SIZE) ? MSG_SIZE : bytes;
			fread(buf,block,1,dat_f);
			fwrite(buf,block,1,dtmp_f);
			bytes -= block;
		}

		fwrite(&ix,sizeof(INDEX),1,itmp_f);

		msg_nr++;
	}

	free(buf);
	fclose(itmp_f);
	fclose(dtmp_f);
	fclose(ind_f);
	fclose(dat_f);


	protect(TRUE);

	rename(dat_tmp,dat_pfad);
	rename(ind_tmp,ind_pfad);

	/*
		nderung in interner Gruppenverwaltung
	*/
	if (strlen(newname) > strlen(puffer[pm][nr].name))
	{
		free(puffer[pm][nr].name);
		puffer[pm][nr].name = strdup(newname);
	}
	else
		strcpy(puffer[pm][nr].name,newname);

	/* Datei aktualisieren */
	dirty[pm] = TRUE;
	grp_update(pm);
	grp_sort(pm);

	protect(FALSE);

	return TRUE;


abbruch:
	if (buf)
		free(buf);
	if (itmp_f)
		fclose(itmp_f);
	if (dtmp_f)
		fclose(dtmp_f);
	if (ind_f)
		fclose(ind_f);
	if (dat_f)
		fclose(dat_f);

	return FALSE;

} /* mdb_rename_group */
#endif /* !CLIENT */



/*
	den zur Gruppen/Usernummer passenden Namen liefern

	->			nr		:	Nummer der Gruppe/User (>= 0)
				pm		:	TRUE = User, FALSE = Gruppe
			<-	name	:	Puffer fr Gruppen/Username bzw. NULL

	<-			TRUE	:	ok
				FALSE	:	Gruppe/User ex. nicht bzw. Fehler

	Diese Funktion ist v.a. fr DB-interne Zwecke gedacht.
*/
BOOLEAN gruppen_nr2name(GRUPPE nr,BOOLEAN pm,char *name)
{
	if (!puffer[pm] && !grp_init(pm))
		return FALSE;

	if (nr > anzahl[pm])
		return FALSE;

	if (!puffer[pm][nr].name)
		return FALSE;

	if (name)
		strcpy(name,puffer[pm][nr].name);

	return TRUE;

} /* gruppen_nr2name */



/*
	den zur Gruppen/Usernummer passenden Dateibasispfad liefern

	->			nr		:	Nummer der Gruppe/User (>= 0)
				pm		:	TRUE = User, FALSE = Gruppe
			<-	name	:	Puffer fr Dateibasispfad

	<-			TRUE	:	passenden Basispfad gefunden
				FALSE	:	Fehler

	Diese Funktion ist v.a. fr DB-interne Zwecke gedacht.
*/
BOOLEAN gruppen_nr2pfad(GRUPPE nr,BOOLEAN pm,char *name)
{
	char tmp[20];

	if (!puffer[pm] && !grp_init(pm))
		return FALSE;

	if (nr > anzahl[pm])
		return FALSE;

	if (!puffer[pm][nr].name)
		return FALSE;

	if (puffer[pm][nr].pfad)
		strcpy(name,puffer[pm][nr].pfad);
	else
		strcpy(name,(pm) ? config.new_us : config.new_gr);

	sprintf(tmp,(pm) ? USR_NAME "." : GRP_NAME ".",(long)(nr+1));
	strcat(name,tmp);

	return TRUE;

} /* gruppen_nr2pfad */



/*
	Funktionen zum Durchlaufen der Gruppen/Userliste
*/



static BOOLEAN user_liste;				/* TRUE = User, FALSE = Gruppen */
static GRUPPE akt_nr = 0;
static GRUPPE akt_index = 0;

/*
	durchlaufen der Gruppen/User
*/
BOOLEAN open_grplst(BOOLEAN pm)
{
	user_liste = pm;

	if (!grp_init(pm))
		return FALSE;

	akt_nr = 0;
	akt_index = 0;

	return TRUE;

} /* open_grplst */



/*
	durchlaufen der Gruppen/User

	->		<-	name	:	Puffer fr einen Gruppen/Usernamen
*/
BOOLEAN read_grplst(char *name)
{
	while (akt_nr < anzahl[user_liste])
	{
		if (puffer[user_liste][akt_index].name)
		{
			strcpy(name,puffer[user_liste][akt_index].name);
			akt_nr++;
			akt_index++;

			return TRUE;
		}
		akt_index++;
	}

	return FALSE;

} /* read_grplst */



/*
	durchlaufen der Gruppen/User
*/
void close_grplst(void)
{
} /* close_grplst */



/*
	exklusiven Schreibzugriff fr Gruppe/User anfordern

	->			name		:	Name der Gruppe/User
				pm			:	PM (TRUE) oder Gruppe (FALSE)
				warten	:	warten, falls Zugriff nicht sofort mglich

	<-			-1			:	Lock noch nicht erhalten (im warte-Modus)
				0			:	Lock nicht (sofort) erhalten oder Gruppe ex. nicht
				1			:	ok, Lock erhalten

	todo:
	- zustzlich filelocking, damit auch Zugriffe fremder Prozesse 
	  bercksichtigt werden
*/
int group_lock(const char *name,BOOLEAN pm,BOOLEAN warten)
{
	long nr;

	if ((nr = group_path(name,NULL,pm)) < 0)
		return 0;

	if (!puffer[pm][nr].locked)
	{
		puffer[pm][nr].locked = TRUE;
		return 1;
	}

	if (!warten)
		return 0;
	else
		return -1;

	/*
		zum Thema deadlocks:
		jede globale DB-Funktion darf max. 1 Gruppe lock'en, bis auf den Putz, 
		der darf beliebig viele.
		Aber:
		Der Putz darf wg. crosspostings nur ein Mal laufen, Import(?), Export 
		Test/Korrekt dagegen mehrmals parallel.

		Wie macht man das ?
		Es gibt einen Scheduler, der von den (TCP/IP-)Ports von allen Clients 
		Anfragen entgegennimmt und diese parallel ausfhrt, sofern mglich.
		Dazu mu er sich merken, ob insb. ein Putz luft.
		Wenn das Locking nicht klappt, wird die Anfrage um eine Position zu-
		rckgesetzt (oder evtl. intern mit group_unlock synchronisiert, 
		durchbricht aber die Ebenen)
	*/

} /* group_lock */



/*
	exklusiven Schreibzugriff fr Gruppe/User zurckgeben

	->			name		:	Name der Gruppe/User
				pm			:	PM (TRUE) oder Gruppe (FALSE)
				warten	:	warten, falls Zugriff nicht sofort mglich

	<-			TRUE		:	ok
				FALSE		:	Fehler (Gruppe/User war nicht gelocked)

	todo:
	- zustzlich filelocking, damit auch Zugriffe fremder Prozesse 
	  bercksichtigt werden
*/
BOOLEAN group_unlock(const char *name,BOOLEAN pm)
{
	long nr;

	if ((nr = group_path(name,NULL,pm)) < 0)
		return FALSE;

	if (!puffer[pm][nr].locked)
		return FALSE;

	puffer[pm][nr].locked = FALSE;

	return TRUE;

} /* group_unlock */
