/* eingabe.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB3 (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
	Offline - MausTausch-FrontEnd

	universelle Eingaberoutine
	(c) 1994-95 bei Frank Baumgart

	begonnen am 03.08.1994
	Version vom 28.01.1995
*/


#include "portab.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <limits.h>
#include <unistd.h>

#include "system.h"
#include "mdb_glob.h"
#include "mdb_conf.h"
#include "mdb/mt_cmd.h"
#include "message.h"
#include "io.h"
#include "frontend/frontend.h"
#include "missing/missing.h"
#include "termcap.h"

extern CONFIG config;

extern int bs_zeilen;
extern int bs_spalten;


static void print_eintrag(int nr,BOOLEAN show);
static void update_eingabe(void);
static void update_zeile(BOOLEAN show);
static void gruppen_parameter(const char *name);

static FILE *ref_f = NULL;		/* Infodatei */
static int eingabetyp;
static const SEL_EINTRAG *entry;

static size_t anz_texte;

static int top;					/* Nr. der obersten dargestellten Zeile */
static int cursor;				/* aktuelle Zeilennummer innerhalb der Seite */
static int i_anz;					/* Anzahl auszugebender Zeilen/Seite */


/*
	universelle Eingaberoutine

	->				titel		:	berschrift
					typ		:	INPUT_GRUPPE / INPUT_USER / INPUT_INFOFILE
					eintrag	:	Array mit den Auswahleintrgen, sortiert
					anzahl	:	Anzahl der Texte
					start		:	ggf. darzustellender Eintrag
					frei		:	freie Eingabe erlauben
				<-	eingabe	:	Benutzereingabe

	<-				-1			:	Abbruch
					-2			:	freie Eingabe ok
					>= 0		:	Nr. des gewhlten Eintrages

	- explizite Untersttzung fr Gruppen- und Infofileeingabe
	- vt52, vt100 oder vergleichbar mchtiger Terminaltyp Vorraussetzung
	- erste Zeile kann fr eine Titelzeile genutzt werden

	todo:
	anfangs nachsehen, wie lang der lngste Gruppenname ist und dement-
	sprechend die Infozeilenlnge anpassen (krzen)
*/
int
uni_eingabe(const char *titel,int typ,SEL_EINTRAG *eintrag,size_t anzahl,
            int start,BOOLEAN frei,char *eingabe)
{
	char trenner[200];
	char in[80];
	BOOLEAN abbruch = FALSE;
	BOOLEAN refresh = TRUE;				/* Bildschirmneuaufbau erzwingen */
	char *line_del;
	char *line_ins;
	size_t max;

	entry = eintrag;
	anz_texte = anzahl;
	eingabetyp = typ;

	get_screen();
	i_anz = bs_zeilen -2;
	if (typ == INPUT_GRUPPE)
		i_anz -= 5;
	if (i_anz > anzahl)
		i_anz = anzahl;

	max = (bs_spalten >= 200) ? 200-1 : bs_spalten-1;
	memset(trenner,'=',max);
	trenner[max] = 0;

	*in = 0;
	if (start && start < anzahl)
	{
		top = start / i_anz;
		cursor = start % i_anz;
	}
	else
		top = cursor = 0;

	/* Codes/Fhigkeit zum einfgen/lschen einzelner Zeilen */
	line_del = tgetstr("dl",NULL);
	line_ins = tgetstr("al",NULL);


	while (!abbruch)
	{
		int key;
		int old_top;
		int old_cursor;
		const char *akt_sel;
		int len;					/* Lnge der akt. Eingabe */

		if (refresh)
		{
			tprint("cl");
			mputs(titel);
			gotoxy(bs_spalten-19,0);
			puts("[Abbruch = Ctrl-X]");
			puts(trenner);
			gotoxy(0,i_anz +2);
			puts(trenner);

			gotoxy(0,i_anz +4);
			if (typ == INPUT_GRUPPE)
			{
				puts("Mit CTRL: e(x)it, (a)b(b)estellen, (d)elete, (p)arameter, i(n)fo, (t)esten");
				printf("p(u)tzen, (e)xportieren, (v)erschieben");
			}
			else
			if (typ == INPUT_INFOFILE)
				puts("Mit CTRL: e(x)it, (a)b(b)estellen, (d)elete, (p)arameter, (v)ergleichen");
			gotoxy(0,i_anz +3);
			update_eingabe();
			gotoxy(0,i_anz +3);
			printf("%s",in);

			refresh = FALSE;
		}

		old_top = top;
		old_cursor = cursor;

		key = taste();

		len = (int) strlen(in);

		/*
			normale Zeichen, die in Gruppen- oder Usernamen (auch nicht-lokal) 
			vorkommen knnen, sowie TAB und Backspace/Delete
		*/
		if ((key < 256) && (isalnum(key) || strchr("!@%*./ &+-_",key)
		|| key == '\b' || key == 127 || key == '\t'))
		{
			int nr = top + cursor;
			int match;
			int next = 0;

			if (key == '\b' || key == 127)
			{
				if (len)
				{
					in[--len] = 0;
					putchar('\b');
					putchar(' ');
					putchar('\b');
				}
				else
					do_beep();
			}
			else
			if (key != '\t')
			{
				in[len++] = key;
				in[len] = 0;
			}

			update_zeile(FALSE);
			gotoxy(len,i_anz+3);

			/*
				Eingabe completion
			*/
			do
			{
				/* Match(es) suchen */
				const char *ref;
				match = 0;
				next = 0;
				nr = 0;
				while (nr < anzahl)
				{
					ref = eintrag[nr].text;

					if ((frei && !strnicmp(in,ref,(size_t)len))
					|| (!frei &&  !strncmp(in,ref,(size_t)len)))
					{
						if (!match)
						{
							if (nr >= top && nr < top + i_anz)
								cursor = nr - top;
							else
							{
								top = nr;
								cursor = 0;
							}
						}
						match++;
						if (key != '\t')
							break;

						/*
							feststellen, ob in allen Matches auch das nchste 
							Zeichen bereinstimmt
						*/
						if ((frei && !strnicmp(ref,eintrag[top + cursor].text,(size_t)len+1))
						|| (!frei &&  !strncmp(ref,eintrag[top + cursor].text,(size_t)len+1)))
							next++;
					}
					nr++;
				}

				if (key != '\t')
					break;

				/* erster Komplettmatch beendet completion */
				if (len == strlen(eintrag[top + cursor].text))
					break;

				if (match > 1 && next == match)
				{
					in[len] = eintrag[top + cursor].text[len];
					in[++len] = 0;
					putchar(in[len-1]);
				}
				else
				if (match == 1 && next == match)
				{
					if (!strcmp(in,eintrag[top + cursor].text))
						do_beep();
					else
					{
						/* vervollstndigen */
						strcpy(in,eintrag[top + cursor].text);
						printf("%s",in + len);
						len = (int) strlen(in);
					}
					break;
				}
				else
				if (next == 1)
				{
					do_beep();
					break;
				}
				else
					break;
			}
			while (1);

			/* Match oder freie Eingabe erlaubt ? */
			if (match || (!match && frei))
			{
#if 0
				if (key != '\t' && key != '\b' && key != 127)
					putchar(key);
#endif
			}
			else
			{
				in[--len] = 0;
				do_beep();
			}

/*			gotoxy(0,i_anz + 3);*/
			printf("\r%s",in);

			key = 0;
		}

		/* Name der gerade vom Cursor ausgewhlten Gruppe */
		akt_sel = eintrag[top + cursor].text;

		switch (key)
		{
			case C_UP :
				update_zeile(FALSE);
				gotoxy(len,i_anz+3);
				if (cursor)
					--cursor;
				else
				if (top)
					top--;
				else
					do_beep();
				break;

			case C_DOWN :
				update_zeile(FALSE);
				gotoxy(len,i_anz+3);
				if (top + cursor +1 >= anzahl)
					do_beep();
				else
				if (cursor + 1 == i_anz)
					top++;
				else
					cursor++;
				break;

			case C_PGDN :
				update_zeile(FALSE);
				gotoxy(len,i_anz+3);
				if (top + cursor +1 >= anzahl)
				{
					do_beep();
					break;
				}
				if (top + i_anz < anzahl)
				{
					top += i_anz;
					if (top + cursor +1 >= anzahl)
						cursor = anzahl - top -1;
				}
				else
					cursor = anzahl - top -1;
				break;

			case C_PGUP :
				update_zeile(FALSE);
				gotoxy(len,i_anz+3);
				if (!top && !cursor)
					do_beep();
				else
				if (top > i_anz)
					top -= i_anz;
				else
				if (top)
					top = 0;
				else
					cursor = 0;
				break;

			case '\r' :
			case '\n' :
				if (*in && frei)
					abbruch = TRUE;
				else
				{
					if (!stricmp(in,akt_sel))
						abbruch = TRUE;
					else
					{
						strcpy(in,akt_sel);
						gotoxy(0,i_anz +3);
						printf("%s",in);
						/*
							### hier mu noch der Bereich rechts von der aktuellen 
							Selektion gelscht werden, denn u.U. stand dort vorher 
							ein lngeres Wort
						*/
					}
				}
				break;

#define CTRL(n)		((n) - 'a' +1)

			case CTRL('b') :
				if (typ == INPUT_GRUPPE)
					bestelle_gruppe(akt_sel,TRUE);
				else
				if (typ == INPUT_INFOFILE)
					infile_cmd(akt_sel," -1");
				else
					do_beep();
				break;

			case CTRL('a') :
				if (typ == INPUT_GRUPPE)
					bestelle_gruppe(akt_sel,FALSE);
				else
				if (typ == INPUT_INFOFILE)
					;
				else
					do_beep();
				break;

			case CTRL('d') :
				if (typ == INPUT_GRUPPE)
				{
					int c;
					int i;

					mputs("");
					mprintf("Gruppe %s wirklich lschen? (j/n)",akt_sel);
					c = taste();
					if (toupper(c) == 'J')
					{
						mdb_remove_group(akt_sel,akt_sel[0] == '*');

						for (i = top+cursor; i < anzahl; i++)
							eintrag[i] = eintrag[i+1];
						anzahl--;
						anz_texte--;
						if (i_anz >= anzahl)
							i_anz--;
						refresh = TRUE;
					}
				}
				else
					do_beep();
				break;

			case CTRL('e') :
				if (typ == INPUT_GRUPPE)
				{
					char exp_lst[PATH_MAX];
					char outfile[PATH_MAX];
					FILE *exp_f;
					EXPORT opt;

					opt.box = FALSE;
					opt.dist = V_LOKAL;		/* Alle Distributionen */
					opt.zeit = 0;				/* alle Mails */
					opt.one_group = FALSE;	/* richtige crosspostings */
					opt.site = NULL;			/* site */
					opt.noidfname = NULL;	/* keine IDs bergehen */
					opt.maximum = 0;			/* kein Limit */

					strcat(strcpy(exp_lst,config.export),"export.lst");
					strcat(strcpy(outfile,config.export),"outfile.txt");
					if ((exp_f = fopen(exp_lst,"w")) == NULL)
						break;
					fputs(akt_sel,exp_f);
					fclose(exp_f);

					mdb_export(exp_lst,outfile,&opt);
					refresh = TRUE;
				}
				break;

			case CTRL('n') :
				if (typ == INPUT_GRUPPE)
				{
					/* evtl. in ... und gruppen_traffic(NULL,FALSE) aufteilen */
					gruppeninfo(akt_sel,akt_sel[0] == '*');
					refresh = TRUE;
				}
				else
					do_beep();
				break;

			case CTRL('p') :
				if (typ == INPUT_GRUPPE)
				{
					gruppen_parameter(akt_sel);
					refresh = TRUE;
				}
				else
					do_beep();
				break;

			case CTRL('t') :
				if (typ == INPUT_GRUPPE)
				{
					mdb_check_group(akt_sel,akt_sel[0] == '*',FALSE);
					refresh = TRUE;
				}
				else
					do_beep();
				break;

			case CTRL('u') :
				if (typ == INPUT_GRUPPE)
				{
					mdb_putze_gruppe(akt_sel,akt_sel[0] == '*',FALSE);
					refresh = TRUE;
				}
				else
					do_beep();
				break;

			case CTRL('v') :
				if (typ == INPUT_INFOFILE)
				{
					if (show_infofilediff(akt_sel))
						refresh = TRUE;
					else
						do_beep();
				}
				else
				if (typ == INPUT_GRUPPE)
				{
					char dest[PATH_MAX];

					mprintf("neuer Pfad: ");
					*dest = 0;
					mgets(dest,PATH_MAX);
					if (*dest)
						(void) mdb_move_group(akt_sel,akt_sel[0] == '*',dest);
				}
				else
					do_beep();
				break;

#if 0
			case CTRL('u')
				in[0] = 0;
				break;
#endif
			case 'q' :
			case 24 :
				/* Ctrl-X */
				*in = 0;
				abbruch = TRUE;
				break;

			case 0 :
				break;
		}

		if (top != old_top)
		{
			/* nur eine Zeile nach unten gescrollt ? */
			if ((top - old_top) == 1 && line_del && line_ins)
			{
				gotoxy(0,2);
				tprint("dl");				/* Zeile entfernen */
				gotoxy(0,i_anz+1);
				tprint("al");				/* Zeile einfgen */
				update_zeile(TRUE);
				gotoxy(len,i_anz+3);
			}
			else
			/* nur eine Zeile nach oben gescrollt ? */
			if ((old_top - top) == 1 && line_del && line_ins)
			{
				gotoxy(0,i_anz+1);
				tprint("dl");
				gotoxy(0,2);
				tprint("al");
				update_zeile(TRUE);
				gotoxy(len,i_anz+3);
			}
			else
			{
				update_eingabe();
				gotoxy(len,i_anz+3);
			}
		}
		else
		{
			update_zeile(TRUE);
			gotoxy(len,i_anz+3);
		}
	}

	strcpy(eingabe,in);

	if (line_ins)
		free(line_ins);
	if (line_del)
		free(line_del);

	if (ref_f)
		fclose(ref_f);

	if (!*in)
		return -1;
	if (frei)
		return -2;
	else
		return top + cursor;

} /* uni_eingabe */



/*
	Eingabeseite neu aufbauen
*/
static void update_eingabe(void)
{
	int i = top;
	int printed = 0;

	tprint("ho");
	puts("");
	puts("");

	while (i < anz_texte  &&  printed < i_anz)
	{
		print_eintrag(i,(i - top == cursor));
		printed++;
		i++;
	}
	while (printed < i_anz)
	{
		tprint("ce");
		puts("");
		printed++;
	}
} /* update_eingabe */



/*
	Cursorzeile neu aufbauen
*/
static void update_zeile(BOOLEAN show)
{
	gotoxy(0,cursor+2);
	print_eintrag(top+cursor,show);

} /* update_zeile */



/*
	einzelne Zeile ausgeben

	->			nr		:	Nummer des Eintrags
				show	:	TRUE = inverse Darstellung, sonst normal
*/
static void print_eintrag(int nr,BOOLEAN show)
{
	static char *fett_an = NULL;
	static char *fett_aus = NULL;
	int text_len,info_len;

	if (!fett_an)
		fett_an = tgetstr("md",NULL);
	if (!fett_aus)
		fett_aus = tgetstr("se",NULL);

	if (show)
		tprint("so");

	if (eingabetyp == INPUT_GRUPPE)
		text_len = 20,info_len = bs_spalten -23;
	else
	if (eingabetyp == INPUT_USER)
		text_len = 47,info_len = bs_spalten -50;
	else
	if (eingabetyp == INPUT_INFOFILE)
		text_len =  8,info_len = bs_spalten -11;
	else
		return;

	printf("%s%-*.*s",
		(entry[nr].fett && fett_an) ? fett_an : "",
		text_len,text_len,
		entry[nr].text);

	if (fett_aus)
	{
		printf("%s",fett_aus);
		if (show)
			tprint("so");		/* wurde durch fett_aus auch ausgeschaltet */
	}

	printf("  %-*.*s\r\n",
		info_len,info_len,
		(entry[nr].info) ? entry[nr].info : "");

	if (show)
		tprint("se");

} /* print_eintrag */



/*
	Parameter einstellen
	z.B. Messagehaltezeiten
*/
static void gruppen_parameter(const char *name)
{
	BOOLEAN pm;
	KENNUNG k,old;
	char eingabe[10];

	if (*name == '*')
		pm = TRUE;
	else
		pm = FALSE;

	/* wg. interakt. Eingabe Gruppe erst spter MIT Schreibzugriff ffnen */
	if (!group_info(name + pm,pm,&k))
		return;

	old = k;

	tprint("cl");
	puts(name);

	printf("Max. Msgs.: ");
	sprintf(eingabe,"%lu",k.max_msgs);
	mgets(eingabe,8);
	if (*eingabe)
		k.max_msgs = atol(eingabe);
	mputs("");

	printf("Max. Gre: ");
	sprintf(eingabe,"%lu",k.max_size);
	mgets(eingabe,8);
	if (*eingabe)
		k.max_size = atol(eingabe);
	mputs("");

	printf("Max. Tage : ");
	sprintf(eingabe,"%lu",k.max_time);
	mgets(eingabe,8);
	if (*eingabe)
		k.max_time = atol(eingabe);
	mputs("");

	if (memcmp(&k,&old,sizeof(KENNUNG)))
	{
		int gh;

		if ((gh = group_open(getpid(),name + pm,pm,TRUE)) < 0)
			return;
		index_write_kennung(gh,&k);
		group_close(gh,getpid());
	}
} /* gruppen_parameter */
