/* 
   Copyright (C) 1994 1995 1998 Uwe Ohse

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

   open_service.c: internet service handling 
   Hauptsaechlich entstanden, um andere Module sauberer zu halten.
   Kommt nur mit Streamsockets klar.
*/

#include "config.h"

#ifdef HAVE_SOCKET
#include "portab.h"

#include <sys/types.h>
#ifdef HAVE_SYS_STAT_H
#  include <sys/stat.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <syslog.h>
#include <unistd.h>
#include <errno.h>
#include <ctype.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "open_service.h"

int
open_service (const char *servicename, const char *host, char **errtext)
{
	int socketfd;
	struct protoent *protocol;
	struct servent *service;
	struct sockaddr_in name;
	struct hostent *hostinfo;
	struct servent dummy_service;
	char dummy_proto[20];
	char dummy_name[20];

	hostinfo = gethostbyname (host);
	if (!hostinfo)
	{
		struct in_addr ina;
		int err=errno;
		if (0==inet_aton(host,&ina))
		{
			errno=err;
			hostinfo=gethostbyaddr((char *) &ina,sizeof(struct in_addr),AF_INET);
		}
		else
			errno=err;
	}
	if (!hostinfo)
	{
		asprintf(errtext,"Cannot get host %s: %s", host,strerror(errno));
		return -1;
	}

	service = getservbyname (servicename, "tcp");
	if (!service && isdigit(*servicename)) {
		dummy_service.s_port=strtol(servicename,NULL,10);
		strcpy(dummy_proto,"tcp");
		dummy_service.s_aliases=NULL;
		dummy_service.s_proto=dummy_proto;
		strncpy(dummy_name,servicename,sizeof(servicename));
		dummy_name[sizeof(dummy_name)-1]=0;
		dummy_service.s_name=dummy_name;
		service=&dummy_service;
	}
	if (!service) {
		asprintf (errtext, "Cannot get service %s: %s", servicename,strerror(errno));
		return -1;
	}

	protocol = getprotobyname (service->s_proto);
	if (!protocol)
	{
		asprintf (errtext, "Cannot get protocol %s: %s",
			 service->s_proto,strerror(errno));
		return -1;
	}

	socketfd = socket (PF_INET, SOCK_STREAM, protocol->p_proto);
	if (socketfd < 0)
	{
		asprintf (errtext, "Cannot open socket: %s",strerror(errno));
		return -1;
	}
	/* Give the socket a name. */
	name.sin_family = AF_INET;
	name.sin_port = service->s_port;	/* s_port is in network byte order */
	name.sin_addr.s_addr = htonl (INADDR_ANY);
	name.sin_addr = *(struct in_addr *) hostinfo->h_addr;
	if (connect (socketfd, (struct sockaddr *) &name, sizeof (name)) < 0)
	{
		asprintf (errtext, "Cannot connect socket: %m",strerror(errno));
		return -1;
	}
	return socketfd;
}
#endif /* HAVE_SOCKET */
