/* domainroute.c : Funktionen zum Routing nach Domains */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
#include <ctype.h>
#include <limits.h>

#include "prototyp.h"
#include "modem.h"
#include "tompaths.h"
#include "box_conf.h"
#include "user.h"
#include "routing.h"
#include "route_int.h"
extern USER user;

int
r_localhost_aufloeser(r_adr_t *adr TOM_ATTRIB_UNUSED, r_a_work_t *work)
{
	if (!r_hostnamecmp(box_conf.mausnet_kennung,work->domain))
	{
		free(work->domain);
		work->domain=NULL;
		return R_A_DONE;
	}
	if (!r_hostnamecmp(box_conf.domainname,work->domain)
		|| !r_hostnamecmp(box_conf.msgid_domain,work->domain))
	{
		free(work->domain);
		work->domain=NULL;
		return R_A_DONE;
	}
	return R_A_NOTDONE;
}

static BOOLEAN
r_dom_finalize(r_gateway_t *gate, r_adr_t *adr, 
			   r_a_work_t *work TOM_ATTRIB_UNUSED)
{
	if (adr->level<gate->ab_level)
	{
		r_writecomment(R_LOG_ERROR,adr,
			"Benutzung von %s nicht erlaubt", gate->path_entry);
		R_PRINTF(2,
("r_dom_finalize: Benutzung von %s nicht erlaubt\r\n", gate->path_entry));
		adr->errorflags |= R_E_ACCESS;
		adr->dsn=r_get_dsn("4.7.1 (not allowed to use %s)",gate->path_entry);
		return R_A_ERROR;
	}
	else if (gate->nur_zahler && !adr->bezahlt)
	{
		r_writecomment(R_LOG_ERROR,adr,
			"Benutzung von %s nur fr Zahler", gate->path_entry);
		R_PRINTF(2,
("r_dom_finalize: Benutzung von %s nur fr Zahler\r\n", gate->path_entry));
		adr->errorflags |= R_E_ACCESS;
		adr->dsn=r_get_dsn("4.7.1 (use of %s only for paying users)",gate->path_entry);
		return R_A_ERROR;
	}
	adr->user_gate_nr=gate->via;
	adr->flags &= ~R_F_LOKAL;
	adr->transport=r_t_maustausch;
	return R_A_DONE;
}

int 
r_fido_aufloeser(r_adr_t *adr, r_a_work_t *work)
{
	r_domain_t *akt;
	char *p;
	char *kopie;

	/* Fido-Style-Adressen haben immer ein Space im Hostnamen, a la
	 * User @ FIDO 1:2/3.4. Davon sehen wir hier "FIDO 1:2/3.4"
	 */

	kopie=malloc(strlen(work->domain)*2+1);
	if (!kopie)
	{
		r_writecomment(R_LOG_FATAL,adr,"out of memory");
		R_PRINTF(0,("r_fido_aufloeser: out of memory\r\n"));
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
		adr->dsn=r_dsn_out_of_memory;
		return R_A_ERROR;
	}

	r_hcanonical(work->domain,kopie);

	p=strchr(kopie,' ');
	if (!p)
	{
		free(kopie);
		return R_A_NOTDONE;
	}

	*p='\0';
	for (akt=r_domain_anker;akt;akt=akt->next)
	{
		if (akt->fidostyle)
		{
			if (!strcmp(akt->domain,kopie))
			{
				free(kopie);
				return r_dom_finalize(akt->via,adr,work);
			}
		}
	}
	free(kopie);
	return R_A_NOTDONE;
}

int 
r_fqdn_aufloeser(r_adr_t *adr, r_a_work_t *work)
{
	char *kopie;
	r_domain_t *akt;
	char buf[128]="";
	char buf_dsn[128]="";
	BOOLEAN has_dot;

	size_t longest=0;
	size_t hostlen;
	ULONG maxkosten=0;
	r_domain_t *best=NULL;

	if (strchr(work->domain,' '))
		return R_A_NOTDONE;

	kopie=malloc(strlen(work->domain)*2+1);
	if (!kopie)
	{
		r_writecomment(R_LOG_FATAL,adr,"out of memory");
		R_PRINTF(0,("r_fqdn_aufloeser: out of memory\r\n"));
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
		adr->dsn=r_dsn_out_of_memory;
		return R_A_ERROR;
	}
	r_hcanonical(work->domain,kopie);

	hostlen=strlen(kopie);
	has_dot=strchr(kopie,'.')!=NULL;

	for (akt=r_domain_anker;akt;akt=akt->next)
	{
		if (!akt->fidostyle)
		{
			const char *p=NULL;
			BOOLEAN wc=FALSE;
			size_t l;

			l=strlen(akt->domain);
			if (l <= hostlen)
			{
				if (!strcmp(kopie+hostlen-l,akt->domain))
					p=kopie+hostlen-l;
			}
			if (!p)
			{
				/* * matcht alles, .* nur alles mit . darin */
				if (0==strcmp(akt->domain,"*"))
					 wc=TRUE;
				else
					 wc=FALSE;
				if (!wc && has_dot && 0==strcmp(akt->domain,".*"))
					 wc=TRUE;
			}
			if (!p && !wc && l==hostlen-1)
			{
				/* user@maus.de ?
				 * hostlen == 7
				 * l       == 8 (.maus.de)
				 */
				if (!strcmp(akt->domain+1,kopie))
					wc=TRUE;
			}
			if ((p && p[strlen(akt->domain)]=='\0')  || wc)
			{
				if (l>longest
					|| (l==longest && akt->kosten+akt->via->kosten<maxkosten))
				{
					BOOLEAN darf=TRUE;
					if (akt->via->nur_zahler)
					{
						darf=adr->bezahlt;
						if (!darf)
						{
							LISTUSER *lu;
							lu=ub_find_usernummer(akt->via->via);
							sprintf(buf,"Benutzung von %s nur fr Zahler",
								lu ? lu->name : "???" );
							sprintf(buf_dsn,
									"4.7.1 (use of %s only for paying users)",
									lu ? lu->name : "???");
						}
					}
					else if (adr->level < akt->via->ab_level)
					{
						LISTUSER *lu;
						darf=FALSE;
						lu=ub_find_usernummer(akt->via->via);
						sprintf(buf,"Benutzung von %s nur ab Level %d",
								lu ? lu->name : "???", akt->via->ab_level);
						sprintf(buf_dsn,"4.7.1 (use of %s only for paying users)",
								lu ? lu->name : "???");
					}
					if (darf)
					{
						longest=strlen(akt->domain);
						maxkosten=akt->kosten+akt->via->kosten;
						best=akt;
					}
				}
			}
		}
	}

	free(kopie);

	if (best)
	{
		return r_dom_finalize(best->via,adr,work);
	}
	if (*buf)
	{
		/* ein anderes Gateway knnte noch den Transport erlauben. Die
		 * Fehlermeldung mssen wir aber irgendwie durchschleifen.
		 */
		r_writecomment(R_LOG_ERROR,adr,"%s",buf);
		R_PRINTF(2,("r_fqdn_aufloeser: %s\r\n",buf));
		/* die drfen wir hier nicht setzen, weil sie sonst spter 
		 * mglicherweise verodert werden ...
		 */
#if 0
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
#endif
	}
	if (*buf_dsn)
		adr->dsn=r_get_dsn("%s",buf_dsn);

	return R_A_NOTDONE;
}

int
r_bindestrich_aufloeser(r_adr_t *adr, r_a_work_t *work)
{
	/* 
	 * MAUS-Hack: 
	 * "user @ FIDO-AC2 1:2/3.4" geht ber AC2 an "user @ FIDO 1:2/3.4"
	 * "user @ any.kiste.com-AC2" geht via AC2 an "user @ any.kiste.com"
	 * Im Envelope mu die originale Adresse stehen bleiben, um den
	 * anderen Boxen im MausNet mitzuteilen, da die Nachricht 
	 * explizit ber eine bestimmte Box gehen soll.
	 * Es sei denn natrlich, wir selbst sind diese Box.
	 */
	char *p,*q;
	char *kopie;
	char *neulocalpart=NULL;
	char *neudomainpart=NULL;

	kopie=malloc(strlen(work->domain)*2+1);
	if (!kopie)
	{
		r_writecomment(R_LOG_FATAL,adr,"out of memory");
		R_PRINTF(0,("r_bindestrich_aufloeser: out of memory\r\n"));
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
		adr->dsn=r_dsn_out_of_memory;
		return R_A_ERROR;
	}

	r_hcanonical(work->domain,kopie);

	p=strchr(kopie,' ');
	if (p)
	{
		*p++='\0';
		q=strchr(kopie,'-');
		if (!q)
		{
			free(kopie);
			return R_A_NOTDONE;
		}
		*q++='\0';
		/* 
		 * Variante: @ FIDO-AC2  1:2/3.4
		 * kopie enthlt "FIDO"
		 * p zeigt auf "1:2/3.4"
		 * q auf  "AC2".
		 *
		 * Der folgende Code ist ein ganz bser Hack.
		 */
		neulocalpart=malloc(strlen(work->local) + strlen(kopie) + strlen(p) +5);
		neudomainpart=strdup(q);
		if (!neulocalpart || !neudomainpart)
		{
			r_writecomment(R_LOG_FATAL,adr,"out of memory");
			R_PRINTF(0,("r_bindestrich_aufloeser: out of memory\r\n"));
			adr->errorflags|=R_E_TEMP;
			adr->errorflags|=R_E_INTERN;
			free(kopie);
			if (neudomainpart)
				free(neudomainpart);
			if (neulocalpart)
				free(neulocalpart);
			adr->dsn=r_dsn_out_of_memory;
			return R_A_ERROR;
		}

		sprintf(neulocalpart,"%s %% %s %s",work->local, kopie, p);
		if (work->local)
			free(work->local);
		if (work->domain)
			free(work->domain);
		work->local=neulocalpart;
		work->domain=neudomainpart;

		free(kopie);
		return R_A_DONE;
	}

	p=strrchr(kopie,'-');
	if (!p)
	{
		free(kopie);
		return R_A_NOTDONE; /* enthaelt keinen Bindestrich */
	}
	if (strchr(p,'.'))
	{
		/* "@ kiste.do.main-du3.maus.de" ist nicht erlaubt, da - ein
		 * erlaubtes oder zumindest verbreitetes Zeichen in Hostnamen ist.
		 */
		free(kopie);
		return R_A_NOTDONE; 
	}

	*p++='\0';
	/* 
	 * Variante:  @ irgendwo.domain.com-AC2
	 * kopie enthaelt "irgendwo.domain.com"
	 * p zeigt auf "AC2"
	 * Der folgende Code ist ein ganz bser Hack.
	 */

	neulocalpart=malloc(strlen(work->local) + strlen(kopie) + 4);
	neudomainpart=strdup(p);
	if (!neulocalpart || !neudomainpart)
	{
		r_writecomment(R_LOG_FATAL,adr,"out of memory");
		R_PRINTF(0,("r_bindestrich_aufloeser: out of memory\r\n"));
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
		free(kopie);
		if (neudomainpart)
			free(neudomainpart);
		if (neulocalpart)
			free(neulocalpart);
		adr->dsn=r_dsn_out_of_memory;
		return R_A_ERROR;
	}

	sprintf(neulocalpart,"%s %% %s",work->local, kopie);
	if (work->local)
		free(work->local);
	if (work->domain)
		free(work->domain);
	work->local=neulocalpart;
	work->domain=neudomainpart;
	free(kopie);
	return R_A_DONE;
}

int
r_821sourceroute_aufloeser(r_adr_t *adr, r_a_work_t *work)
{
	/*
	 * Ein eher berflssiges Mifeature aus der RFC-Welt:
	 *     @host1,@host2:user@host3
	 * Es sollte verstanden werden als:
	 *     user%host3%host2@host1
	 * und genau dahin bersetzt dieser Auflser auch.
	 * Wenn obige Adresse hier ankommt, sieht sie so aus:
	 *     host1,@host2:user@host3
	 * Sendmail wertet seit 8.6.x nur noch "user@host3" aus. Angeblich
	 * legt RFC 1123 das nahe (das sehe ich etwas anders - es *erlaubt*
	 * dieses Verhalten allerdigns).
	 */
	char *writepos;
	const char *suchpos;
	char *p,*at;
	char *doppel;

	char *kopie;
	char *neu;

	/* 
	 * Spaces kommen nie vor, auer in Adressen von Netzen, die nach dem
	 * FIDO-Schema (@ GATECODE Adresse-im-anderen-Netz) gebildet werden,
	 * und bei denen dieser RFC-Hack gar nicht sinnvoll ist.
	 */
	if (strchr(work->domain,' '))
		return R_A_NOTDONE;

	/*
	 * Ein Doppelpunkt mu sein, der trennt Pfad vom Rest.
	 */
	doppel=strchr(work->domain,':');
	if (!doppel)
		return R_A_NOTDONE;

	kopie=strdup(work->domain);
	neu=malloc(strlen(work->domain)+2);
	if (!kopie || !neu)
	{
		r_writecomment(R_LOG_FATAL,adr,"out of memory");
		R_PRINTF(0,("r_821_aufloeser: out of memory\r\n"));
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
		if (neu)
			free(neu);
		if (kopie)
			free(kopie);
		adr->dsn=r_dsn_out_of_memory;
		return R_A_ERROR;
	}

	doppel=kopie+(doppel-work->domain);

	strcpy(neu,doppel+1);
	p=strchr(neu,'@');
	if (p)
	{
		/* 
		 * Das ist: "@irgendwo:user@host" (legal)
		 * Die andere Form "@irgendwo:user" ist nicht legal, aber wir
		 * kommen auch damit klar.
		 */
		*p='%';
	}
	writepos=neu+strlen(neu);

	suchpos=kopie;
	*doppel=0;

	at=strrchr(suchpos,'@');
	while (at)
	{
		size_t l;
		if (!at || at<=suchpos)
		{
			r_writecomment(R_LOG_ERROR,adr,"unparsbare RFC821 Adresse");
			R_PRINTF(0,("r_821_aufloeser: unparsbare Adresse: %s\r\n",suchpos));
			free(neu);
			free(kopie);
			adr->errorflags|=R_E_ADR;
			adr->dsn=r_get_dsn("5.1.2 (can't parse address)");
			return R_A_ERROR;
		}
		p=at-1; /* mindestens suchpos */
		if (*p==',')
			*p=0;
		at++;
		if (!*at)
		{
			r_writecomment(R_LOG_ERROR,adr,"unparsbare 821 Adresse");
			R_PRINTF(0,("r_821_aufloeser: unparsbare Adresse: %s\r\n",suchpos));
			free(neu);
			free(kopie);
			adr->errorflags|=R_E_ADR;
			adr->dsn=r_get_dsn("5.1.2 (can't parse address)");
			return R_A_ERROR;
		}
		*writepos++='%';
		l=strlen(at);
		strncpy(writepos,at,l);
		writepos+=l;
		*writepos='\0';
		at=strrchr(suchpos,'@');
	}
	/* Jetzt ist noch ein Host brig.
	 * war:       host1,@host2:user@host3
	 * temporr:  host1,@host2
	 * nun suchpos:      host2
	 */

	if (work->local)
		free(work->local);
	work->local=neu;
	if (work->domain)
		free(work->domain);
	work->domain=strdup(suchpos);
	free(kopie);

	if (!work->domain)
	{
		r_writecomment(R_LOG_FATAL,adr,"out of memory");
		R_PRINTF(0,("r_821_aufloeser: out of memory\r\n"));
		adr->errorflags|=R_E_TEMP;
		adr->errorflags|=R_E_INTERN;
		adr->dsn=r_dsn_out_of_memory;
		return R_A_ERROR;
	}
	
	return R_A_DONE;
}

