/*
    Copyright (C) 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <dirent.h>
#include <syslog.h>
#include <ctype.h>

#include "prototyp.h"
#include "modem.h"
#include "offline/mdb_glob.h"
#include "msg.h"
#include "routing.h"
#include "route_int.h"
#include "tompaths.h"
#include "box_conf.h"
#include "lockfile.h" /* libquark */
#include "address_hash.h"

static r_transport_t *transports;


static BOOLEAN init_done;
extern USER user;

static r_transport_t *
r_find_transport(const char *name)
{
	r_transport_t *akt;
	if (!init_done)
		r_do_init();
	init_done=TRUE;
	akt=transports;
	while(akt)
	{
		if (!stricmp(akt->name,name))
			return akt;
		akt=akt->next;
	}
	return NULL;
}

BOOLEAN 
r_t_add_to_transport(char *str)
{
	/* Syntax: "transport test test_user1 test-xy*"
	 * wir sehen          ^^^^^^^^^^^^^^^^^^^^^^^^
	 */
	char *space;
	r_transport_t *tr;
	space=strpbrk(str,"\t ");
	if (space)
	{
		*space++=0;
		while (isspace(*space))
			space++;
	}
	if (!space || space==str || !*space)
	{
		syslog(LOG_WARNING,"r_t_add_to_transport: illegal transport line %s",
			   str);
		return FALSE;
	}
	strtrim(space);
	strtrim(str);
	tr=r_find_transport(str);
	if (!tr)
	{
		syslog(LOG_WARNING,"r_t_add_to_transport: unbekannter transport %s",
			   str);
		return FALSE;
	}
	str=space;
	while (str && *str)
	{
		USERNR unr;
		r_t_zuord_t *zu;
		space=strpbrk(str," \t");
		if (space)
		{
			*space++=0;
			while (isspace(*space))
				space++;
		}
		if (!ub_find_username(&unr,str))
		{
			syslog(LOG_WARNING,"r_t_add_to_transport: unbekannter benutzer %s",
			   str);
			str=space;
			continue;
		}
		/* In die Liste hngen */
		zu=XMALLOC(sizeof(r_t_zuord_t));
		zu->transport=tr;
		zu->user_nr=unr;
		zu->next=r_transport_zuord_anker;
		r_transport_zuord_anker=zu;
		
		str=space;
	}
	return TRUE;
}

BOOLEAN 
r_transport(message_t *msg, r_adr_t *adr, BOOLEAN do_errors)
{
	BOOLEAN ok=TRUE;
	r_adr_t *akt;

	if (!init_done)
		r_do_init();

	if (msg->flags_zeile) { /* check for expire */
		char *E=strstr(msg->flags_zeile,"E+");
		/* safeguard against things like "XAE+-" */
		if (E && ((E-msg->flags_zeile) & 1) == 0) {
			/* OK */
			/* Throw away this flag */
			memmove(E,E+2,strlen(E+2)+1);
			if (!msg->flags_zeile[0]) {
				/* get rid of the whole line */
				free(msg->flags_zeile);
				msg->flags_zeile=NULL;
			}
			if (user.address_hash_expire) {
				time_t expire=time(NULL)+user.address_hash_expire*86400L;
				strcpy(msg->absender,address_hash_create(&user,expire));
				strcat(msg->absender," @ ");
				strcat(msg->absender,MYDOMAINPART);
				r_tauschuserlog(NULL,":!Absender %s\r\n",msg->absender);
				strcpy(msg->realname,user.name);
			}
		}
	}

	if (!adr || msg->msgtyp==gruppen_msg)
	{
		return r_t_maustausch->deliver(msg,NULL);
	}

	if (do_errors)
	{
		static BOOLEAN recurse=FALSE;
		if (recurse)
			return FALSE;
		recurse=TRUE; 
		/* #LOG *jetzt* schreiben */
		for (akt=adr; akt ; akt=akt -> next)
		{
			if (akt->flags & R_F_ZUGESTELLT)
				continue;
			if (!akt->error)
				continue;
			if (!(akt->flags & R_F_TAUSCHLOG))
				continue;
			r_tauschuserlog(akt,":?%s\r\n",akt->logstr);
		}

		for (akt=adr; akt ; akt=akt -> next)
		{
			if (akt->flags & R_F_ZUGESTELLT)
				continue;
			if (!akt->error)
				continue;

			if (!r_t_error->deliver(msg,akt))
				ok=FALSE;
		}
		recurse=FALSE;
		return ok;
	}


	for (akt=adr; akt ; akt=akt -> next)
	{
		if (akt->flags & R_F_ZUGESTELLT)
			continue;
		if (akt->error)
			continue;
		if (!akt->transport)
			continue;

		if (!akt->transport->deliver(msg,akt))
			ok=FALSE;
	}

	{
		/* Alle Empfnger der Nachricht (aber nicht der Fehlermeldungen -
		 * da macht das ziemlich wenig Sinn?)
		 */
		simplelist_t *li=NULL;
		for (akt=adr; akt ; akt=akt -> next)
		{
			if (akt->resultadr)
			{
				simplelist_t *n=simple_alloc(akt->resultadr);
				if (n)
				{
					n->next=li;
					li=n;
				}
			}
		}
		if (li)
		{
			if (!r_st_newmail(msg->msg_id,
				msg->absender, li, msg->eingabezeit_tt))
			{
				syslog(LOG_WARNING,"can't save %s to mailstatus database",
					msg->msg_id);
			}
			simple_free_chain(li);
			/* haha */
			r_st_set_status(msg->msg_id,
				li->data, /* from ... */
				msg->absender, /* to - umgedreht! */
				"N",
				time(NULL));
			for (akt=adr; akt ; akt=akt -> next)
			{
				if (adr->flags & R_F_LOKAL)
				{
					r_st_set_status(msg->msg_id,
						akt->resultadr, /* from ... */
						msg->absender, /* to - umgedreht! */
						"N",
						time(NULL));
				}
				else
				{
					r_st_set_status(msg->msg_id,
						akt->resultadr, /* from ... */
						msg->absender, /* to - umgedreht! */
						"M", /* im MausNet - naja, aber besser ist das ... */
						time(NULL));
				}
			}
		}
	}

	for (akt=adr; akt ; akt=akt -> next)
	{
		if (akt->flags & R_F_ZUGESTELLT)
			continue;
		if (akt->error)
			continue;
		msglog(
			"** TSNH: message not delivered, but no errorflag\n"
			"    !erroradr: %s\n" 
			"    !to (input): %s\n"
			"    !to (result): %s\n"
			"    !comment: %s\n"
			"    !flags: %lx\n"
			"    !errorflags: %lx\n",
				akt->erroradr ? akt->erroradr : "not given",
				akt->inputadr ? akt->inputadr : "not given",
				akt->resultadr ? akt->resultadr : "none???",
				akt->logstr ? akt->logstr : "none",
				akt->flags,
				akt->errorflags
			);
		r_tauschuserlog(adr,"?TSNH: message not delivered, but no errorflag\r\n");
		ok=FALSE;
	}
	return ok;
}

/*
 * durch alle Transports laufen und die flush-Funktionen aufrufen, falls
 * sie welche haben.
 */
BOOLEAN 
r_flush(void)
{
	BOOLEAN ok=TRUE;
	r_transport_t *akt;
	if (!init_done)
		r_do_init();
	for (akt=transports; akt; akt=akt->next)
	{
		if (akt->flush && !akt->flush(akt))
			ok=FALSE;
	}
	return ok;
}

BOOLEAN 
r_queuerun(void)
{
	DIR *dir;
	struct dirent *dp;
	char lname[PATH_MAX];
	char fname[PATH_MAX];
	FILE *f;
	BOOLEAN ok=TRUE;

	if (!init_done)
		r_do_init();

    dir = opendir(box_conf.spool_dir);
    if (!dir)
    	return FALSE;

	for (dp = readdir(dir); dp != NULL; dp = readdir(dir))
	{
		if (dp->d_name[0]!='_' || dp->d_name[1]!='i')
			continue;
		if (strstr(dp->d_name,".LCK"))
			continue;
		sprintf(lname,"%s%s.LCK",box_conf.spool_dir,dp->d_name);
		sprintf(fname,"%s%s",box_conf.spool_dir,dp->d_name);
		if (lock_create(lname,NULL)!=0)
			continue;
		f=fopen(fname,BINAPPEND);
		if (!f)
		{
			lock_delete(lname);
			continue;
		}
		fprintf(f,"#\r\n");
		fclose(f);
		if (mdb_import(fname,TRUE))
		{
			unlink(fname);
			ok=FALSE;
		}
		else
		{
			syslog(LOG_ERR,"cannot import %s",fname);
			mbprintf("kann %s nicht importieren",fname);
		}
		lock_delete(lname);
	}
	closedir(dir);
	return ok;
}

void
r_t_chain_transport(r_transport_t *n)
{
	n->next=transports;
	transports=n;
} 
