/* route_int.h : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef ROUTE_INT_H
#define ROUTE_INT_H

/*
 * Interne Daten des Routers, fr die Auenwelt nicht von Belang.
 */

extern void r_t_maustausch_define(char *str);
extern void r_t_intern_define(char *str);
extern void r_t_error_define(char *str);
extern void r_t_extern_define(char *str);

extern r_transport_t *r_t_maustausch;
extern r_transport_t *r_t_error;

	/* 
	 * Beschreibung der Nachbarsysteme
	 */
typedef struct r_gateway_t 
{
	USERNR via;       /* Nummer des Postfachs, bzw. Pseudousers */
	const char *path_entry; /* Patheintrag */
	BOOLEAN nur_zahler;
	UWORD   ab_level;
	ULONG kosten;     /* ungenutzt */
	struct r_gateway_t *next;
} r_gateway_t;
extern r_gateway_t *r_gateway_anker;

#ifdef notyet
typedef struct r_database_t
{
	const char *fname;
	enum 
	{ 
		R_DB_SEARCH
	} type;
	void *private;
} r_database_t;
#endif

/* 
 * Listeneintrag fr jede Domain und Site, die bekannt ist.
 */
typedef struct r_domain_t
{
	char *domain;      /* Fuer welche Domain/site? */
	BOOLEAN fidostyle; /* @ FIDO 1:2/3.4 */
	r_gateway_t *via;  /* ist ueber ... anzusprechen */
	ULONG kosten;
	struct r_domain_t *next;
} r_domain_t;
extern r_domain_t *r_domain_anker;

	/*
	 * Umschreiben von Adressen.
	 */
typedef struct r_alias_t
{
	char *from;
	size_t fromlen; /* strlen(from) */
	char *to;
	struct r_alias_t *next;
} r_alias_t;
extern r_alias_t *r_rewrite_anker;
extern r_alias_t *r_useralias_anker;
extern r_alias_t *r_hostalias_anker;
extern r_alias_t *r_domainbounce_anker;

typedef struct r_t_zuord_t 
{
	USERNR user_nr;
	r_transport_t *transport;
	struct r_t_zuord_t *next;
} r_t_zuord_t;
extern r_t_zuord_t *r_transport_zuord_anker;

/* Falls der Aufloeser Erfolg meldet, dann mit dem Resultat weiter
 * suchen. 
 */
#define AUFLOESER_CONTINUE 1

#define R_A_DONE      0     /* aufloeser hat gearbeitet */
#define R_A_NOTDONE   1     /* Aufloeser nicht zustndig */
#define R_A_ERROR     2     /* irgendein Fehler ist aufgetreten */

typedef struct r_a_work_t 
{
	char *local;
	char *domain;
} r_a_work_t;


typedef int (*aufloeser_fn_t) (r_adr_t *adr, r_a_work_t *work);
typedef void (*aufloeser_cleanup_t) (void);

typedef struct r_aufloeser_t 
{
	const char *name;
		/* Kennung fr Debugzwecke */
	char kenn;
	const char *magic;
	aufloeser_fn_t call;
    ULONG flags;
    struct r_aufloeser_t *next; /* naechster aufloeser */
} r_aufloeser_t;
/* Liste der Domainauflser */
extern r_aufloeser_t *r_dom_aufloeser_anker;
/* Liste der Lokalauflser. */
extern r_aufloeser_t *r_loc_aufloeser_anker;


	/* Domainpart-Router - also solche, die aufgerufen werden, um
	 * festzustellen, ob die Adresse lokal ist 
	 */
int r_domainbounce_aufloeser(r_adr_t *, r_a_work_t *);  /* ich -> ich.maus.de */
int r_rewriter_aufloeser(r_adr_t *, r_a_work_t *);  /* ich -> ich.maus.de */
int r_hostalias_aufloeser(r_adr_t *, r_a_work_t *); /* ich -> ich.maus.de */
int r_fqdn_aufloeser(r_adr_t *, r_a_work_t *);      /* kiste.do.main */
int r_bindestrich_aufloeser(r_adr_t *, r_a_work_t *); /* @ FIDO-AC2 */
int r_fido_aufloeser(r_adr_t *, r_a_work_t *);      /* @ FIDO irgendwo */
int r_localhost_aufloeser(r_adr_t *host, r_a_work_t *result);
int r_821sourceroute_aufloeser(r_adr_t *host, r_a_work_t *result);



	/* Router, die aufgerufen werden, wenn die Adresse lokal ist */
int r_bang_aufloeser(r_adr_t *, r_a_work_t *);     /* kiste!user */
int r_percent_aufloeser(r_adr_t *, r_a_work_t *);  /* user % kiste */
int r_useralias_aufloeser(r_adr_t *, r_a_work_t *);    /* Alias */
int r_chef_aufloeser(r_adr_t *, r_a_work_t *);     /* Chef Gruppe */
int r_user_aufloeser(r_adr_t *, r_a_work_t *);    /* Username */


extern BOOLEAN r_init_done;
void r_do_init(void);
BOOLEAN r_transport_init(void);

BOOLEAN r_t_define_transport(char *str);
BOOLEAN r_t_add_to_transport(char *str);


	/*
	 * Ins Log fr den User (#LOG, wenn im Tausch, sonst screen).
	 */
void r_userlog (const char *format,...) TOM_ATTRIB_PRINTF(1,2);

#define R_ULOG(adr,command) \
	if (!adr || adr->no_user_log) command

	/*
	 * Panicmessages schreiben.
	 */
void r_paniclog (const char *format,...) TOM_ATTRIB_PRINTF(1,2);

	/*
	 * Diese Funktion ruft der transport auf, um zu loggen, da die
	 * Nachricht ok ist.
	 */
BOOLEAN r_t_writelogs(message_t *head, r_adr_t *adr, const char *id);

	/*
	 * Macht aus 
	 *  "Benutzer ", "NixIs", "unbekannt!", NULL
	 * ->
	 *  "Benutzer NixIs unbekannt"
	 * Mit anderen Worten, \r\n ist *oben* anzuhngen, auerdem ist
	 * ? oder ! zu ergnzen.
	 */
void r_writecomment(r_loggrade_t, r_adr_t *, ...);

void r_printf(const char *format, ...) TOM_ATTRIB_PRINTF(1,2);

extern int r_verbose_grade;
#define R_PRINTF(grade,paras) \
	do { if (r_verbose_grade>=grade) r_printf paras;} while (0)

void r_t_chain_transport(r_transport_t *n);

#endif
