/* shellvars.c : print config variables useful for shellscripts */
/*
    Copyright (C) 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "config.h"

#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <limits.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "aktdata.h"
#include "box_conf.h"

static void
print_varname(FILE *f, const char *s)
{
	unsigned char c;
	fputs("TOM_",f);
	while (*s)
	{
		c=*s;
		if (isalnum(c))
			putc(toupper(c),f);
		else
			putc('_',f);
		s++;
	}
}

static void
print_val(FILE *f, const char *s)
{
	unsigned char c;
	putc('\'',f);
	while (*s)
	{
		c=*s;
		if (c=='\'')
			putc('\\',f);
		putc(c,f);
		s++;
	}
	putc('\'',f);
}

static void
dump_one(FILE *f, const char *vn, const char *val) 
{
	if (!val)
		return;
	print_varname(f,vn);
	putc('=',f);
	print_val(f,val);
	printf("\nexport ");
	print_varname(f,vn);
	putc('\n',f);
}
/*
	15.12.92 Frank
	07.10.93 Frank
*/
BOOLEAN dump_shell_vars(FILE *f)
{
	dump_one(f,"Mausnetkennung",box_conf.mausnet_kennung);
	dump_one(f,"Systemname",box_conf.systemname);
	dump_one(f,"domainname",box_conf.domainname);
	dump_one(f,"batch-dir",box_conf.batch_dir);
	dump_one(f,"bin-dir",box_conf.bin_dir);
	dump_one(f,"binbrett-dir",box_conf.binbrett_dir);
	dump_one(f,"bretter-dir",box_conf.bretter_dir);
	dump_one(f,"config-dir",box_conf.config_dir);
	dump_one(f,"hilfe-dir",box_conf.hilfe_dir);
	dump_one(f,"infofile-dir",box_conf.infofile_dir);
	dump_one(f,"log-dir",box_conf.log_dir);
	dump_one(f,"modules-dir",box_conf.modules_dir);
	dump_one(f,"spool-dir",box_conf.spool_dir);
	dump_one(f,"tausch-dir",box_conf.tausch_dir);
	dump_one(f,"texte-dir",box_conf.texte_dir);
	dump_one(f,"tmp-dir",box_conf.tmp_dir);
	dump_one(f,"tcl-dir",box_conf.tcl_dir);
	dump_one(f,"var-dir",box_conf.var_dir);

	dump_one(f,"cnf-file-name",box_conf.cnf_file_name);
	dump_one(f,"quark-dir",box_conf.quark_dir);
	dump_one(f,"offline.cnf",box_conf.offline_cnf);
	dump_one(f,"system-user-name",box_conf.system_user_name);
	return TRUE;
}
