/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
/*
 * Deliver via external programs.
 */
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <ctype.h>
#include <errno.h>
#include <syslog.h>
#include <time.h>
#include <sys/stat.h>

#include "box_conf.h"
#include "offline/mdb_glob.h"
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "modem.h"
#include "tompaths.h"
#include "routing.h"
#include "route_int.h"
#include "msg.h"				/* write_msg fr #LOG */

#include "lockfile.h"				/* libquark */

#ifndef LINEEND
#define LINEEND "\r\n"
#endif

extern BRETTINFO *br_ind;

static BOOLEAN r_t_extern_deliver (message_t * head, r_adr_t * adr);
static BOOLEAN r_t_extern_flush (struct r_transport_t *);


struct extern_priv_data {
	const char *cmd;
	int counter;
	int max;
	FILE *fhandle;
};
static BOOLEAN r_t_extern_open (struct extern_priv_data *priv);

/*
 * "deftransport extern blah ; para1=...;para2=... ; cmd=..."
 * wir sehen:           ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 */
void
r_t_extern_define (char *str)
{
	r_transport_t *n;
	char *name;
	struct extern_priv_data *priv;
	char *semi = strchr (str, ';');
	char *cmd=NULL;

	if (!semi)
	{
		syslog (LOG_WARNING, "illegal line in routing database: %s", str);
		return;
	}
	*semi++ = 0;
	while (isspace (*semi))
		semi++;
	strtrim (str);
	strtrim (semi);
	if (!*semi)
	{
		syslog (LOG_WARNING, 
			"illegal extern definition in routing database: %s", str);
		return;
	}
	name=XSTRDUP(str);
	strlwr(name);

	while (semi && *semi)
	{
		char *eq=strchr(semi,'=');
		char *next=strchr(semi,';');
		/* Nur Gleichheitszeichen vor dem nchsten Semicolon nehmen */
		if (eq && next && eq>next)
			eq=NULL;
		if (!eq)
		{
			syslog (LOG_WARNING, 
				"illegal attribute %s in definition of %s", semi, str);
			free(name);
			return;
		}
		*eq++=0;
		while (isspace(*eq))
			eq++;
		strtrim(semi);
		if (0==stricmp(semi,"cmd"))
		{
			cmd=XSTRDUP(eq);
			break;
		}
		/* andere Attribute gibt es noch nicht */
		if (next)
		{
			*next++=0;
			while (isspace(*next))
				next++;
		}
		semi=next;
	}
	if (!cmd)
	{
		syslog (LOG_WARNING, 
			"definition of %s lacks command", str);
		free(name);
		return;
	}

	n = XMALLOC (sizeof (r_transport_t));
	n->name = name;
	n->deliver = r_t_extern_deliver;
	n->flush = r_t_extern_flush;
	priv=XMALLOC(sizeof(struct extern_priv_data));
	priv->cmd=cmd;
	priv->max=1;
	priv->counter=0;
	priv->fhandle=NULL;
	n->priv_data=priv;
	r_t_chain_transport(n);
}

static BOOLEAN
r_t_extern_open (struct extern_priv_data *priv)
{
	priv->fhandle=popen(priv->cmd,"w");
	if (!priv->fhandle)
	{
		syslog(LOG_ERR,"cannot open pipe to %s: %s",priv->cmd,
			strerror(errno));
		return FALSE;
	}
	return TRUE;
}

static BOOLEAN
r_t_extern_deliver_one (message_t * head, r_adr_t * adr, const char *id)
{
	FILE *in = NULL;			/* keep gcc quiet */
	struct extern_priv_data *priv=adr->transport->priv_data;

	if (head->fname)
	{
	   /* Text aus der Datei lesen */
		in = fopen (head->fname, "r");
		if (!in)
		{
			const char *s = strerror (errno);

			r_paniclog ("cannot open inputfile %s: %s\n", head->fname, s);
			r_tauschuserlog (adr, ":?TSNH: kann Nachricht nicht ffnen: %s\r\n", s);
			msglog ("   ?TSNH: kann Nachricht nicht ffnen: %s\r\n", s);
			if (adr)
			{
				adr->errorflags |= R_E_INTERN;
				adr->flags |= R_F_TO_POSTMASTER;
				adr->flags |= R_F_TO_SENDER;
				adr->error = TRUE;
			}
			return FALSE;
		}
	}

   /* printf und Abarten sind in allen Abkmmlingen der GNU-Library
    * extrem langsam - und vielleicht auch noch in anderen Systemen.
    * Deshalb werden sie hier vermieden. Das Zeilenende einer Zeile
    * wird immer vor der Ausgabe der nchsten Zeile geschrieben!
    */
#define ONELINE(chr,x) do { fputs(LINEEND chr,priv->fhandle); \
	fputs(x,priv->fhandle); } while(0)
#define IFEX(chr,x) if (head->x && *head->x) ONELINE(chr,head->x)

	putc ('#', priv->fhandle);
	fputs (id, priv->fhandle);
	fputs (LINEEND "W", priv->fhandle);
	fputs (head->betreff, priv->fhandle);
	fputs (LINEEND "E", priv->fhandle);
	fputs (head->eingabezeit, priv->fhandle);
	IFEX ("-", ref_id);
	IFEX ("V", absender);		/* anonym? */
	IFEX ("S", sender);
	IFEX ("T", reply_to);
	IFEX ("O", organisation);
	IFEX ("N", realname);
	IFEX ("Y", gateway_zeile);
	if (head->msgtyp == pers_msg)
	{
		const char *p_A_name;

		ONELINE ("B", head->bearbeitungsstatus);

	   /*
	    * Header/Envelope-Trennung lt sich mit MausTausch nicht machen,
	    * es klappt nur in der lokalen Datenbank, nicht jedoch bei 
	    * Nachrichten an andere Boxen.
	    */

		if (adr->flags & R_F_LOKAL)
			p_A_name = head->to->data;
		else
			p_A_name = adr->resultadr;

	   /* das sollte zwar weiter oben erledigt sein, aber sicher ist 
	    * sicher.
	    */
		if (strpbrk (p_A_name, "@!%"))
			ONELINE ("A", p_A_name);
		else
			fprintf (priv->fhandle, LINEEND "A%s @ %s", p_A_name, MYDOMAINPART);

		msglog ("  Textern/%s\n",adr->transport->name);
		msglog ("  >U%lu %s\n", adr->user_gate_nr, p_A_name);
	}
	else
	{
		simplelist_t *akt;

	   /*
	    * Gruppenzeilen schreiben.
	    */
		for (akt = head->gruppen; akt; akt = akt->next)
		{
			ONELINE ("G", akt->data);
			if (head->junk_group)
				msglog ("  >G%s (junked)\n", akt->data);
			else
				msglog ("  >G%s\n", akt->data);
		}

	   /*
	    * Followup-To:
	    */
		for (akt = head->followup; akt; akt = akt->next)
			ONELINE ("F", akt->data);

		if (head->msgtyp==gruppen_msg)
		{
			fputs(LINEEND "D",priv->fhandle);
			fputc(head->distribution_byte,priv->fhandle);
		}

		if (head->junk_group)
			ONELINE ("g", box_conf.junk_group);
	}

	IFEX ("M", mimeinfo);
	IFEX ("X", flags_zeile);
   /* 
    * Erst nach allen anderen Headerzeilen, zur Optimierung von Offline. 
    */
	if (head->msgtyp == gruppen_msg)
		IFEX ("p", path_zeile);

	if (adr && adr->logstr && *adr->logstr)
		ONELINE (">", adr->logstr);

   /* die letzte Headerzeile mit CR/LF abschliessen */
	fputs (LINEEND, priv->fhandle);

	if (head->fname)
	{
		char buf[1024];

		while (fgets (buf, sizeof (buf), in))
		{
			char *p = strpbrk (buf, "\r\n");

			if (p)
				*p = 0;
			fprintf (priv->fhandle, ":%s" LINEEND, buf);
		}
		fclose (in);
	}
	else
		fputs (head->body.buf, priv->fhandle);

	if (head->msgtyp == pers_msg || ++priv->counter >= priv->max)
	{
		priv->counter = 0;
		if (fflush (priv->fhandle) != 0)
		{
		   /* AUA! */
			const char *s = strerror (errno);

			r_paniclog ("error flushing outputfile: %s", s);
			r_tauschuserlog (adr, ":?Fehler bei fflush(): %s\r\n", s);
			msglog ("   ?Fehler bei fflush(): %s\r\n", s);
			return FALSE;
		}
	}
	msglog ("  =%s\n", id);
	r_tauschuserlog (adr, ":=%s\r\n", id);
	return TRUE;
}

/* 
 * Hierhin kommen nur vollstndige Nachrichten, keine Spezialnachrichten,
 * wohl aber Statusmeldungen.
 */
BOOLEAN
r_t_extern_deliver (message_t * head, r_adr_t * adr)
{
	BOOLEAN ok = TRUE;
	struct extern_priv_data *priv=adr->transport->priv_data;

	if (!priv->fhandle)
	{
		if (!r_t_extern_open (adr->transport->priv_data))
		{
			adr->errorflags |= R_E_INTERN;
			adr->flags |= R_F_TO_POSTMASTER;
			adr->flags |= R_F_TO_OWNER;
			adr->error = TRUE;
			return FALSE;
		}
	}

	if (head->msgtyp == pers_msg)
	{
		if (TRANSPORT_ALLOWED (adr))
		{
			char idbuf[256];
			char *id;

			if (adr->flags & R_F_NEUE_ID)
			{
				make_msg_id (idbuf, TRUE, NULL, NULL);
				id = idbuf;
			}
			else
				id = head->msg_id;

			if (!r_t_extern_deliver_one (head, adr, id))
			{
				r_writecomment (R_LOG_ERROR, adr, "Transport gescheitert");
				adr->error = TRUE;
				adr->errorflags |= R_E_INTERN;
				ok = FALSE;
			}
			else
				adr->flags |= R_F_ZUGESTELLT;
		}
	}
#if 0
	/* ### extern-transport kann weder OM noch Status */
	else if (head->msgtyp == status_msg)
	{
		r_t_extern_deliver_status (head, adr);
	}
	else
	{
		if (!r_t_offl_deliver_one (head, NULL, head->msg_id))
			ok = FALSE;
	}
#endif
	return ok;
}

BOOLEAN
r_t_extern_flush (r_transport_t *t)
{
	struct extern_priv_data *priv=t->priv_data;
	if (priv && priv->fhandle)
	{
		pclose(priv->fhandle);
		priv->fhandle=NULL;
		priv->counter=0;
	}

	return TRUE;
}

