#line 1 "ta_gcmd.c4"
/* hi emacs, this is -*- C -*- */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#line 22 "ta_gcmd.c4"
/*
 * Tausch - Gruppenbezogene Kommandos
 *
 * Uwe Ohse, 29.11.1993
 * Frank Baumgart, 20.05.1997 Anpassungen einiger Texte
 */
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fnmatch.h>

#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/mdb_global.h"
#else
#include "offline/version.h"
#include "offline/mdb_global.h"
#endif
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "modem.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_cmd.h"

#define UMASK           (S_IRUSR | S_IWUSR | S_IXUSR)
#define STRING2(x) #x
#define STRING(x) STRING2(x)

static BOOLEAN ta_gcmd_set_chef_sub(BRETTNR bnr, USER *u, USERNR unr);

static void ta_gcmd_mitglied (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_mitgliederliste (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_bestellen (char *cmd, const char *dummy, LONG set);
static unsigned int ta_gcmd_bestellen_sub(BOOLEAN set, const char *gruppe, BOOLEAN mult);
static void ta_gcmd_ren(char *cmd, const char *dummy1, LONG dummy2);
static void ta_gcmd_loeschen(char *cmd, const char *dummy1, LONG dummy2);
static void ta_gcmd_anlegen(char *cmd, const char *dummy1, LONG dummy2);
static void ta_gcmd_chef (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_status (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_rename (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_beschreibung (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_zugriff (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_followup (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_sprache (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_herkunft (char *cmd, const char *dummy, LONG dummy2);
static void ta_gcmd_alias (char *cmd, const char *dummy, LONG dummy2);

/*
 * GQMgruppe : Mitglieder einer Gruppe ausgeben.
 */
#line 107

#line 107
ta_command_t ta_gcmd_desc_mitgliederliste = {
#line 107
	/* Magic 1       */ 	"GQM",
#line 107
	/* Magic 2       */ 	"",
#line 107
	/* Funktionsname */ 	ta_gcmd_mitgliederliste,
#line 107
	/* Min. Level    */ 	0,
#line 107
	/* Textpara      */ 	NULL,
#line 107
	/* Num. Parameter*/ 	0,
#line 107

#line 107
	/* ITK-ID-Nummer */ 	26,
#line 107
	/* Gruppierung   */ 	"G",
#line 107
	/* ITK-Version   */ 	0,
#line 107
	/* ITK-Typ       */ 	"B", 
#line 107
	/* ITK-Name      */ 	"Mitgliederliste anfordern", 
#line 107
	/* ITK-Beschrei. */ 
#line 107
		
#line 107
	"Hiermit knnen Sie eine Liste der Mitglieder einer Gruppe, deren"
#line 107
	"Chef Sie sind, anfordern."
#line 107
	, 
#line 107

#line 107
	/* ITK-Syntaxdefinition*/
#line 107
		
#line 107
	":CGQM\r\n"
#line 107
	":FG\r\n"
#line 107
	":KGruppe\r\n"
#line 107
	":LWhlen Sie eine Gruppe.\r\n" 
#line 107
	,
#line 107

#line 107
	/* Verkettungsptr*/ NULL
#line 107
};
#line 107


static void
ta_gcmd_mitgliederliste (char *cmd, 
						 const char *dummy TOM_ATTRIB_UNUSED,
						 LONG dummy2 TOM_ATTRIB_UNUSED)
{
	char *p;					/* Hilfsvar. */
	char *gname;				/* Name der betroffenen Gruppe */
	BRETTNR bnr;
	BOOLEAN is_mitglied = FALSE;
	char listfname[PATH_MAX];
	char outfname[PATH_MAX];
	FILE *in,*out;

	gname = cmd + 3;			/* GQM berspringen */
	/* kein strtrim(gname): wir brauchen das Originalkommando, und 
	 * ta_cmd_get_group kommt auch mit Spaces klar 
	 */

	/* Brettdaten lesen, keinen Lock setzen */
	if (!ta_cmd_get_group (gname, &bnr, FALSE))
		return;					/* log done */

	is_mitglied = b_tst_access (bnr, user.nummer);
	if (!is_mitglied && level()<ULV_MAX)
	{
		/* Keine Mitgliedschaft in Gruppe */
		ta_do_log_error ( "Zugriff verweigert");
		return;
	}

	if (!B_MITLISTE(br_ind[bnr].brett->flags))
	{
		/* unlock */
		ta_do_log_error ( "In dieser Gruppe gibt es keine Mitgliedschaft");
		return;
	}
	b_export_access(bnr);
	sprintf(listfname,"%s%s",brettpfad(bnr),"zugriff.lst");
	sprintf(outfname, "%s" KOMMANDO_OUT_F , tom_tmp_dir );
	out=ta_open_out_block(cmd);
	in=fopen(listfname,"r");
	if (in)
	{
		char buf[256];
		while (fgets(buf,sizeof(buf),in))
		{
			p=strpbrk(buf,"\r\n");
			if (p) *p=0;
			fprintf(out,"%s\r\n",buf);
		}
		fclose(in);
	}
	else
		fprintf(out,":Mitgliederliste leer / nicht vorhanden\r\n.");
	fclose(out);
	return;
}

/*
 * GUuser[<>=]gruppe : Kommando fuer Mitgliedschaft und Gruppenchef.
 */
#line 205

#line 205
ta_command_t ta_gcmd_desc_mitglied = {
#line 205
	/* Magic 1       */ 	"GU",
#line 205
	/* Magic 2       */ 	"",
#line 205
	/* Funktionsname */ 	ta_gcmd_mitglied,
#line 205
	/* Min. Level    */ 	0,
#line 205
	/* Textpara      */ 	NULL,
#line 205
	/* Num. Parameter*/ 	0,
#line 205

#line 205
	/* ITK-ID-Nummer */ 	25,
#line 205
	/* Gruppierung   */ 	"G",
#line 205
	/* ITK-Version   */ 	0,
#line 205
	/* ITK-Typ       */ 	"B", 
#line 205
	/* ITK-Name      */ 	"Mitgliedschaft setzen", 
#line 205
	/* ITK-Beschrei. */ 
#line 205
		
#line 205
	"In den Gruppen, in denen Sie Chef sind, knnen Sie hiermit Mitglieder "
#line 205
	"ein- und austragen. Mitglieder knnen sich austragen." 
#line 205
	, 
#line 205

#line 205
	/* ITK-Syntaxdefinition*/
#line 205
		
#line 205
	":CGU\r\n"
#line 205
	":FU\r\n"
#line 205
	":KBenutzer\r\n"
#line 205
	":LGeben Sie den Namen des Benutzers ein.\r\n"
#line 205
	":FA\r\n"
#line 205
	":KAktion\r\n"
#line 205
	":LWhlen Sie, was mit dem Benutzer in der Gruppe geschehen soll.\r\n"
#line 205
	":A>\r\n"
#line 205
	":Bals Mitglied eintragen\r\n"
#line 205
	":A<\r\n"
#line 205
	":Baus Gruppe austragen\r\n"
#line 205
	":A=\r\n"
#line 205
	":Bzum Gruppenchef machen\r\n"
#line 205
	":FG\r\n"
#line 205
	":KGruppe\r\n"
#line 205
	":LWhlen Sie eine Gruppe.\r\n" 
#line 205
	,
#line 205

#line 205
	/* Verkettungsptr*/ NULL
#line 205
};
#line 205

static void
ta_gcmd_mitglied (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	char *p;					/* Hilfsvar. */
	const char *uname;			/* Name des betroffenen Users */
	char *gname;				/* Name der betroffenen Gruppe */
	char c;						/* Hilfsvar. */
	char *tmpuname;
	BOOLEAN set_chef = FALSE;	/* Chef setzen? */
	BOOLEAN eintragen = FALSE;	/* Chef austragen? */
	USER u;						/* betroffener User */
	USERNR unr;
	BRETTNR bnr;
	BOOLEAN is_mitglied = FALSE;
	BOOLEAN changed = FALSE;
	BOOLEAN allowed = FALSE;

	p = strpbrk (cmd, "<>=");
	if (!p)
	{
		ta_do_log_error ("unbekanntes Kommando!");
		return;
	}
	c = *p;
	*p = '\0';
	tmpuname = cmd + 2;			/* GU berspringen */
	gname = p + 1;
	switch (c)
	{
	case '>':
		eintragen = TRUE;
		break;
	case '<':
		break;
	case '=':
		set_chef = TRUE;
		break;
	default:
		ta_do_log_error ("unbekanntes Kommando!");
		return;
	}

	/* Spaces rund um den Usernamen weg */
	*p='\0';
	strtrim(tmpuname);
	if (!*tmpuname)
		uname=user.name;
	else
		uname=tmpuname;

	if (!ta_cmd_get_user (uname, &unr, &u, FALSE))
		return;					/* log done */

	/* Spaces rund um den Gruppennamen weg */
	strtrim(gname);

	/* Brettdaten lesen, einen Lock setzen */
	if (!ta_cmd_get_group (gname, &bnr, TRUE))
		return;					/* log done */

	if (set_chef)
	{
		/* setzt auch Configaenderungsflag */
		ta_gcmd_set_chef_sub(bnr, &u, unr);
		bb_write_brett (bnr, TRUE, FALSE);
		return;
	}

	/* Gruppenchef ist abgehandelt, bleibt noch die Mitgliedschaft */

	if (!B_MITLISTE(br_ind[bnr].brett->flags))
	{
		/* Keine Mitgliedschaft in Gruppe */
		ta_do_log_error ( "In dieser Gruppe gibt es keine Mitgliedschaft");
		/* unlock */
		bb_write_brett (bnr, TRUE, FALSE);
		return;
	}

	/* Hier hat sich mir die Frage gestellt, was man Level 8 verbieten
	 * sollte. 
	 * - Eintrge fr LV-9-Bretter
	 * - nderungen an den Daten von LV-9-Usern.
	 */
	is_mitglied = b_tst_access (bnr, unr);

	if (level() == 9 || user.nummer == br_ind[bnr].brett->grp_chef)
		allowed = TRUE;
	else if (level() == 8)
	{
		/* Level 8 darf schon entschieden weniger */
		allowed = TRUE;			/* Logik umkehren ist hier einfacher! */

		/* Level 9 Bretter nicht! */
		if (br_ind[bnr].brett->lesen_level == 9 || br_ind[bnr].brett->schreib_level == 9)
			allowed = FALSE;
		/* User mit Level 9 ebenfalls nicht */
		else if (u.level == 9)
			allowed = FALSE;
	}
	if (!allowed && !eintragen && unr == user.nummer)
	{
		/* Jeder darf sich selbst berall austragen */
		allowed = TRUE;
	}

	if (!allowed)
	{
		ta_do_log_error ("Wie wollen SIE denn das machen?");
	}
	else if (eintragen)
	{
		/* jemanden eintragen */
		if (is_mitglied)
		{
			ta_do_log (":!%s bleibt Mitglied in %s!\r\n",
					   uname, gname);
		}
		else
		{
			if (b_add_access (bnr, unr))
			{
				ta_do_log (":!%s ist nun Mitglied in %s!\r\n",
						   uname, gname);
				changed = TRUE;
			}
			else
				ta_do_log_error ("Eintrag gescheitert");
		}
	}
	else
		/* dann austragen */
	{
		/* Level 9 und Gruppenchef drfen alle austragen, 
		   User nur sich selbst
		 */
		if (!is_mitglied)
			ta_do_log (":! %s bleibt Nichtmitglied in %s!\r\n",
					   uname, gname);
		else
		{
			if (b_del_access (bnr, unr))
			{
				ta_do_log ( ":!%s ist nun kein Mitglied mehr in %s!\r\n",
						   uname, gname);
				changed = TRUE;
			}
			else
				ta_do_log_error ("Austrag gescheitert");
		}
	}
	bb_write_brett (bnr, TRUE, FALSE);
	return;
}

#line 392

#line 392
ta_command_t ta_gcmd_desc_bestellen = {
#line 392
	/* Magic 1       */ "G+",
#line 392
	/* Magic 2       */ "G-",
#line 392
	/* Funktionsname */ ta_gcmd_bestellen,
#line 392
	/* Min. Level    */ 0,
#line 392
	/* Textpara      */ NULL,
#line 392
	/* Num. Parameter*/ 0,
#line 392

#line 392
	/* ITK-ID-Nummer */ 	36,
#line 392
	/* Gruppierung   */ "G",
#line 392
	/* ITK-Version   */ 	0,
#line 392
	/* ITK-Typ       */ "B", 
#line 392
	/* ITK-Name      */ "Gruppe bestellen", 
#line 392
	/* ITK-Beschrei. */ 
#line 392
		
#line 392
	"Mit diesem Kommando knnen Sie eine Gruppe bestellen oder abbestellen." 	
#line 392
	, 
#line 392

#line 392
	/* ITK-Syntaxdefinition*/
#line 392
		
#line 392
	":CG\r\n"
#line 392
	":FA\r\n"
#line 392
	":KAktion\r\n"
#line 392
	":LWhlen Sie, ob die Gruppe bestellt oder abbestellt werden soll.\r\n"
#line 392
	":A+\r\n"
#line 392
	":Bbestellen\r\n"
#line 392
	":A-\r\n"
#line 392
	":Babbestellen\r\n"
#line 392
	":FG\r\n"
#line 392
	":KGruppe\r\n"
#line 392
	":Lzu (ab)bestellende Gruppe\r\n" 
#line 392
	,
#line 392

#line 392
	/* Verkettungsptr*/ NULL
#line 392
};
#line 392


static void
ta_gcmd_bestellen (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				   LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BOOLEAN set;
	char *gruppe;

	if (cmd[1]=='+')
		set=TRUE;
	else
		set=FALSE;

	gruppe = cmd + 2;			/* G+ berspringen */

	/* Spaces rund um den Gruppennamen weg */
	strtrim(gruppe);

	if (!*gruppe)
	{
		ta_do_log_error ("Keine Gruppe angegeben");
		return;
	}
	if (strpbrk(gruppe,"*?")) {
		/* durch die ganze Liste gehen und passende Gruppen durch
		   die Subroutine jagen. */
		char *g;
		unsigned int count=0;
		BRETTNR i;
		g=entumlauten1(gruppe);
		XNULLCHECK(g);
		for (i=1;i<max_brettnr; i++) {
			char *buf;
			if (!br_ind[i].brett)
				continue;
			buf=entumlauten1(br_ind[i].brett->name);
			XNULLCHECK(buf);
			strlwr(g);
			if (0 == fnmatch(g,buf,0)) {
				count+=ta_gcmd_bestellen_sub(set,buf,TRUE);
			}
			free(buf);
		}
		ta_do_log (":!%u Gruppen %sbestellt\r\n", count,
				   set ? "" : "ab");
	} else {
		ta_gcmd_bestellen_sub(set,gruppe,FALSE);
	}
}

static unsigned int
ta_gcmd_bestellen_sub(BOOLEAN set, const char *gruppe, BOOLEAN mult)
{
	BRETTNR bnr;
	BOOLEAN ok;
	char fehltext[128];
	BOOLEAN darf;
	const char *err=NULL;

	/* Brettdaten lesen, keinen Lock setzen */
	if (!ta_cmd_get_group (gruppe, &bnr, FALSE))
		return 0;					/* log done */

	darf = ta_check_readperm(bnr, &err);
	if (!darf)
	{
		if (mult)
			return 0;
		/* 
		 * Keinen Lesezugang fr die Gruppe.
		 * Dann mal eine passende Fehlermeldung generieren.
		 */
		if (!err)
		{
			if (B_MITLISTE(br_ind[bnr].brett->flags) 
				&& !b_tst_access (bnr, user.nummer))
			{
				ta_do_log (
				 ":? Sie sind kein Mitglied. Wenden Sie sich an 'Chef %s'!\r\n",
					   gruppe);
				return 0;
			}
			err="Zugriff nicht erlaubt";
		}
		ta_do_log_error ( err );
		return 0;
	}


	/* Ok, der User darf lesen. */
	if (set)
		ok = ta_brett_bestellen (bnr, fehltext);
	else
		ok = ta_brett_abbestellen (bnr, fehltext);
	if (!ok) {
		if (!mult)
			ta_do_log (":!%s\r\n",fehltext); /* !, da auf jeden Fall Erfolg */
		return 0;
	} else {
		if (!mult)
			ta_do_log (":!Die Gruppe %s ist nun %sbestellt!\r\n",
					   gruppe, (set) ? "" : "ab");
		ta_opts.changed=TRUE;
		return 1;
	}
}

#line 521

#line 521
ta_command_t ta_gcmd_desc_anlegen = {
#line 521
	/* Magic 1       */ "GA",
#line 521
	/* Magic 2       */ "",
#line 521
	/* Funktionsname */ ta_gcmd_anlegen,
#line 521
	/* Min. Level    */ 8,
#line 521
	/* Textpara      */ NULL,
#line 521
	/* Num. Parameter*/ 0,
#line 521

#line 521
	/* ITK-ID-Nummer */ 28,
#line 521
	/* Gruppierung   */ "G",
#line 521
	/* ITK-Version   */ 	0,
#line 521
	/* ITK-Typ       */ "B", 
#line 521
	/* ITK-Name      */ "Gruppe einrichten", 
#line 521
	/* ITK-Beschrei. */ 
#line 521
	
#line 521
	"Hiermit knnen Sie eine neue Gruppe lokal einrichten" 
#line 521
	, 
#line 521

#line 521
	/* ITK-Syntaxdefinition*/
#line 521
		
#line 521
	":CGA\r\n"
#line 521
	":FS" STRING(BDAT_GRP_NAME_LEN) "\r\n"
#line 521
	":KName\r\n"
#line 521
	":LGeben Sie einen Namen fr die neu einzurichtende Gruppe ein.\r\n" 
#line 521

#line 521
	,
#line 521

#line 521
	/* Verkettungsptr*/ NULL
#line 521
};
#line 521


void
ta_gcmd_anlegen (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				 LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETT b;
	BRETTNR bnr;
	char *gruppe;

	gruppe = cmd + 2;			/* GA berspringen */

	/* Spaces rund um den Gruppennamen weg */
	strtrim(gruppe);

	if (!*gruppe)
	{
		ta_do_log_error ("Keinen Gruppenname angegeben");
		return;
	}

	bnr=brettnr(gruppe);
	if (bnr)
	{
		ta_do_log_error ("Gruppe schon vorhanden");
		return;
	}
	b_default_brett(&b);
	b.flags|=B_READ|B_WRITE;
	strcpy(b.name,gruppe);
	b.configaenderung=time(NULL);

	if (!b_create_brett(&b))
		ta_do_log_error ("Kann Gruppe nicht anlegen");
	else
	{
		ta_do_log (":!Die Gruppe %s ist nun angelegt!\r\n", gruppe);
		br_ind[b.nr].brett->flags|=B_READ|B_WRITE;
	}
}

#line 583

#line 583
ta_command_t ta_gcmd_desc_loeschen = {
#line 583
	/* Magic 1       */ "GL",
#line 583
	/* Magic 2       */ "",
#line 583
	/* Funktionsname */ ta_gcmd_loeschen,
#line 583
	/* Min. Level    */ 8,
#line 583
	/* Textpara      */ NULL,
#line 583
	/* Num. Parameter*/ 0,
#line 583

#line 583
	/* ITK-ID-Nummer */ 30,
#line 583
	/* Gruppierung   */ "G",
#line 583
	/* ITK-Version   */ 	0,
#line 583
	/* ITK-Typ       */ "B", 
#line 583
	/* ITK-Name      */ "Gruppe lschen", 
#line 583
	/* ITK-Beschrei. */ 
#line 583
	
#line 583
	"Mit diesem Kommando knnen Sie eine Gruppe lschen." 
#line 583
	, 
#line 583

#line 583
	/* ITK-Syntaxdefinition*/
#line 583
		
#line 583
	":CGL\r\n"
#line 583
	":FG\r\n"
#line 583
	":KGruppe\r\n"
#line 583
	":LWhlen Sie die zu lschende Gruppe.\r\n" 
#line 583

#line 583
	,
#line 583

#line 583
	/* Verkettungsptr*/ NULL
#line 583
};
#line 583


void
ta_gcmd_loeschen (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *gruppe;

	gruppe = cmd + 2;			/* GA berspringen */

	/* Spaces rund um den Gruppennamen weg */
	strtrim(gruppe);

	if (!*gruppe)
	{
		ta_do_log_error ("Keinen Gruppenname angegeben");
		return;
	}

	if (!ta_cmd_get_group (gruppe, &bnr, TRUE))
	{
		ta_do_log_error ("Kann Gruppendaten nicht lesen");
		return;
	}

	if (!b_remove_brett(bnr))
		ta_do_log_error ("Kann Gruppe nicht loeschen");
	else
		ta_do_log (":!Die Gruppe %s ist geloescht!\r\n", gruppe);
}
#line 639

#line 639
ta_command_t ta_gcmd_desc_ren = {
#line 639
	/* Magic 1       */ "REN",
#line 639
	/* Magic 2       */     "",
#line 639
	/* Funktionsname */     ta_gcmd_ren,
#line 639
	/* Min. Level    */     0,
#line 639
	/* Textpara      */     NULL,
#line 639
	/* Num. Parameter*/     0,
#line 639

#line 639
	/* ITK-ID-Nummer */     6,
#line 639
	/* Gruppierung   */     "G",
#line 639
	/* ITK-Version   */     0,
#line 639
	/* ITK-Typ       */     "K", 
#line 639
	/* ITK-Name      */     "Umbenennungen anfordern", 
#line 639
	/* ITK-Beschrei. */ 
#line 639
	    
#line 639
    "Geben Sie an, ob Sie Informationen zu umbenannten Gruppen bekommen "
#line 639
	"mchten."     
#line 639
	, 
#line 639

#line 639
	/* ITK-Syntaxdefinition*/
#line 639
	    
#line 639
    ":CREN\r\n"
#line 639
    ":FDo\r\n"
#line 639
    ":KDatum\r\n"
#line 639
    ":Lber Umbenennungen ab angegebenem Datum informieren. Ohne Datum nur"
#line 639
    "neue Informationen liefern.\r\n"  
#line 639
	,
#line 639

#line 639
	/* Verkettungsptr*/ NULL
#line 639
};
#line 639


static void 
ta_gcmd_ren(char *cmd, const char *dummy1 TOM_ATTRIB_UNUSED, 
			LONG dummy2 TOM_ATTRIB_UNUSED)
{
	time_t start;
	char *p=cmd+3;
	while (isspace(*p))
		p++;
	if (!*p)
	{
		ta_passinfo.send_ren=TRUE;
		return;
	}
	if (strtol(p,NULL,10)==-1)
	{
		ta_passinfo.send_ren=TRUE;
		ta_opts.last_ren=(time_t)0;
		return;
	}
	if (mtzeit_to_time_t(p,&start))
	{
		ta_passinfo.send_ren=TRUE;
		ta_opts.last_ren=start;
		return;
	}
	ta_do_log_error ("unparsbares Datum!");
}

static BOOLEAN
ta_gcmd_set_chef_sub(BRETTNR bnr, USER *u, USERNR unr)
{
	BRETT *b;
	if (u->level < 2)
	{
		ta_do_log_error ("Userlevel zu niedrig!");
		return FALSE;
	}

	/* das sollte hier natrlich nicht vorkommen!
	 * Ja, die ganze Welt verfolgt mich.
	 */
	CHECK_BRETTNR(bnr,return FALSE);

	b=br_ind[bnr].brett;

	if (!befehl_check_befehl (b, "ED", "BRETT"))
	{
		/* Aha - nicht erlaubt. */
		ta_do_log_error (
						 "Auf welche Ideen die Leute doch kommen ...");
		return FALSE;
	}
	if (b->grp_chef == unr)
	{
		ta_do_log (
				   ":! Bitteschn: %s bleibt Chef der Gruppe %s\r\n",
				   u->name, b->name);
		return FALSE;
	}
	b->grp_chef = unr;
	ta_do_log (":! Bitteschn: %s ist Chef der Gruppe %s\r\n",
			   u->name, b->name);
	b->configaenderung=time(NULL);
	return TRUE;
}


/*
 * GCgruppe=user : Kommando zum Setzen des Gruppenchefs.
 */
#line 739

#line 739
ta_command_t ta_gcmd_desc_chef = {
#line 739
	/* Magic 1       */ "GC",
#line 739
	/* Magic 2       */     "",
#line 739
	/* Funktionsname */     ta_gcmd_chef,
#line 739
	/* Min. Level    */     8,
#line 739
	/* Textpara      */     NULL,
#line 739
	/* Num. Parameter*/     0,
#line 739

#line 739
	/* ITK-ID-Nummer */     1002,
#line 739
	/* Gruppierung   */     "G",
#line 739
	/* ITK-Version   */     0,
#line 739
	/* ITK-Typ       */     "B", 
#line 739
	/* ITK-Name      */     "Gruppenchef setzen", 
#line 739
	/* ITK-Beschrei. */ 
#line 739
	    
#line 739
    "Mit dieser Funktion knnen Sie den Gruppenchef setzen"     
#line 739
	, 
#line 739

#line 739
	/* ITK-Syntaxdefinition*/
#line 739
	    
#line 739
    ":CGC\r\n"
#line 739
    ":FG\r\n"
#line 739
	":KGruppe\r\n"
#line 739
	":LName der Gruppe, deren Chef zu ndern ist\r\n"
#line 739
	":C=\r\n"
#line 739
    ":FU\r\n"
#line 739
	":KBenutzer\r\n"
#line 739
	":LName des neuen Chefs\r\n"
#line 739
    
#line 739

#line 739
	,
#line 739

#line 739
	/* Verkettungsptr*/ NULL
#line 739
};
#line 739


static void
ta_gcmd_chef (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
			  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	char *p;					/* Hilfsvar. */
	char *uname;				/* Name des betroffenen Users */
	char *gname;				/* Name der betroffenen Gruppe */
	USER u;						/* betroffener User */
	USERNR unr;
	BRETTNR bnr;

	p = strrchr (cmd, '='); /* str*r*chr, weil = mglicherweise in g-namen */
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando!");
		return;
	}
	*p = '\0';
	gname = cmd + 2;			/* GU berspringen */
	uname = p + 1;

	/* Spaces rund um den Usernamen weg */
	strtrim(uname);
	if (!*uname)
	{
		unr=user.nummer;
		u=user;
	}
	else
	{
		if (!ta_cmd_get_user (uname, &unr, &u, FALSE))
			return;					/* log done */
	}

	/* Spaces rund um den Gruppennamen weg */
	strtrim(gname);

	/* Brettdaten lesen, einen Lock setzen */
	if (!ta_cmd_get_group (gname, &bnr, TRUE))
		return;					/* log done */

	ta_gcmd_set_chef_sub(bnr, &u, unr);
	bb_write_brett (bnr, TRUE, FALSE);
}


/* 
 * GSname=status, status=[+|-]
 */
#line 822

#line 822
ta_command_t ta_gcmd_desc_status = {
#line 822
	/* Magic 1       */ "GS",
#line 822
	/* Magic 2       */     "",
#line 822
	/* Funktionsname */     ta_gcmd_status,
#line 822
	/* Min. Level    */     8,
#line 822
	/* Textpara      */     NULL,
#line 822
	/* Num. Parameter*/     0,
#line 822

#line 822
	/* ITK-ID-Nummer */     33,
#line 822
	/* Gruppierung   */     "G",
#line 822
	/* ITK-Version   */     0,
#line 822
	/* ITK-Typ       */     "B", 
#line 822
	/* ITK-Name      */     "Vernetzungsstatus", 
#line 822
	/* ITK-Beschrei. */ 
#line 822
	    
#line 822
    "Mit dieser Funktion knnen Sie den Vernetzungsstatus einer Gruppe" 
#line 822
    "ndern (das wirkt nur auf die Gruppenlisten)." 
#line 822
	, 
#line 822

#line 822
	/* ITK-Syntaxdefinition*/
#line 822
		
#line 822
	":CGS\r\n"
#line 822
	":FG\r\n"
#line 822
	":KGruppe\r\n"
#line 822
	":LWhlen Sie eine Gruppe.\r\n"
#line 822
	":C=\r\n"
#line 822
	":FA\r\n"
#line 822
	":A+\r\n"
#line 822
	":Bvernetzen\r\n"
#line 822
	":A-\r\n"
#line 822
	":Bentnetzen\r\n"
#line 822
	":KVernetzungsstatus\r\n"
#line 822
	":LWhlen Sie, ob die Gruppe vernetzt oder nicht vernetzt werden soll.\r\n"
#line 822
    
#line 822

#line 822
	,
#line 822

#line 822
	/* Verkettungsptr*/ NULL
#line 822
};
#line 822

void
ta_gcmd_status (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *altgruppe;
	char *p;
	UWORD on;

	altgruppe = cmd + 2;			/* GS berspringen */
	p=strchr(altgruppe,'=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++=0;
	strtrim(altgruppe);
	strtrim(p);

	if (!*altgruppe)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (*p=='-')
		on=FALSE;
	else if (*p=='+')
		on=1;
	else
	{
		ta_do_log_error ("Flag fehlt");
		return;
	}
	if (!ta_cmd_get_group (altgruppe, &bnr, TRUE))
	{
		ta_do_log_error ("Kann Gruppendaten nicht lesen");
		return;
	}
	if ((on && br_ind[bnr].brett->netzstatus) 
		|| (!on && !br_ind[bnr].brett->netzstatus))
	{
		ta_do_log (":!Netzstatus unverndert (gleich altem Status)" LINEEND);
		bb_unlock_brett(bnr);
	}
	else if (on)
	{
		ta_do_log (":!Netzstatus ist nun '+', war `-'" LINEEND);
		br_ind[bnr].brett->configaenderung=time(NULL);
		br_ind[bnr].brett->netzstatus=on;
		bb_write_brett (bnr, TRUE, FALSE);
	}
	else
	{
		ta_do_log (":!Netzstatus ist nun '-', war `+'" LINEEND);
		br_ind[bnr].brett->configaenderung=time(NULL);
		br_ind[bnr].brett->netzstatus=on;
		bb_write_brett (bnr, TRUE, FALSE);
	}

}


/* 
 * GNalter_name=neuer_name
 */
#line 917

#line 917
ta_command_t ta_gcmd_desc_rename = {
#line 917
	/* Magic 1       */ "GN",
#line 917
	/* Magic 2       */     "",
#line 917
	/* Funktionsname */     ta_gcmd_rename,
#line 917
	/* Min. Level    */     8,
#line 917
	/* Textpara      */     NULL,
#line 917
	/* Num. Parameter*/     0,
#line 917

#line 917
	/* ITK-ID-Nummer */     32,
#line 917
	/* Gruppierung   */     "G",
#line 917
	/* ITK-Version   */     0,
#line 917
	/* ITK-Typ       */     "B", 
#line 917
	/* ITK-Name      */     "Gruppe umbenennen", 
#line 917
	/* ITK-Beschrei. */ 
#line 917
	    
#line 917
    "Mit dieser Funktion knnen Sie eine Gruppe umbenennen"     
#line 917
	, 
#line 917

#line 917
	/* ITK-Syntaxdefinition*/
#line 917
	    
#line 917
    ":CGN\r\n"
#line 917
    ":FG\r\n"
#line 917
    ":KAlter Name\r\n"
#line 917
    ":LWhlen Sie die Gruppe, die Sie umbenennen mchten.\r\n"
#line 917
    ":C=\r\n"
#line 917
    ":FS" STRING(BDAT_GRP_NAME_LEN) "\r\n"
#line 917
    ":KNeuer Name\r\n"
#line 917
    ":LGeben Sie den neuen Namen fr die Gruppe ein.\r\n"     
#line 917

#line 917
	,
#line 917

#line 917
	/* Verkettungsptr*/ NULL
#line 917
};
#line 917


void
ta_gcmd_rename (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *altgruppe;
	char *p;
	BRETTNR n;
	char buf[BDAT_GRP_NAME_LEN*2+1];

	altgruppe = cmd + 2;			/* Gn berspringen */
	p=strchr(altgruppe,'=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++=0;
	strtrim(altgruppe);
	strtrim(p);

	if (!*altgruppe || !*p)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (!strcmp(altgruppe,p))
	{
		ta_do_log (":!Nichts gendert!\r\n");
		return;
	}
	if (!ta_cmd_get_group (altgruppe, &bnr, TRUE))
	{
		ta_do_log_error ("Kann Gruppendaten nicht lesen");
		return;
	}
	/*
	 * Wenn ein Brett dieses Namens existiert und es nicht das 
	 * umzubenennende Brett ist.
	 */
	n=brettnr(p);
	if (n!=bnr)
	{
		bb_unlock_brett(bnr);
		ta_do_log_error ("Eine Gruppe dieses Namens existiert bereits");
		return;
	}
	strcpy(br_ind[bnr].brett->name,p);
	if (br_ind[bnr].canon)
		free(br_ind[bnr].canon);
	bb_canonical(p,buf);
	br_ind[bnr].canon=XSTRDUP(p);

#ifdef OFFLINE_VERSION
	mdb_rename_group(altgruppe,p);
#else
	mdb_rename_group(altgruppe,p,FALSE);
#endif
	ta_add_rename_info(time(NULL),altgruppe,p);

	br_ind[bnr].brett->configaenderung=time(NULL);
	bb_write_brett (bnr, TRUE, FALSE);
	ta_do_log (":!Gruppe umbenannt!\r\n");
}

/* 
 * GFname=followup
 */
#line 1016

#line 1016
ta_command_t ta_gcmd_desc_followup = {
#line 1016
	/* Magic 1       */ "GF",
#line 1016
	/* Magic 2       */     "",
#line 1016
	/* Funktionsname */     ta_gcmd_followup,
#line 1016
	/* Min. Level    */     8,
#line 1016
	/* Textpara      */     NULL,
#line 1016
	/* Num. Parameter*/     0,
#line 1016

#line 1016
	/* ITK-ID-Nummer */     35,
#line 1016
	/* Gruppierung   */     "G",
#line 1016
	/* ITK-Version   */     0,
#line 1016
	/* ITK-Typ       */     "B", 
#line 1016
	/* ITK-Name      */     "Default-Followup-To", 
#line 1016
	/* ITK-Beschrei. */ 
#line 1016
	    
#line 1016
    "Hier stellen Sie ein, in welche Gruppe Kommentare gehen sollen."     
#line 1016
	, 
#line 1016

#line 1016
	/* ITK-Syntaxdefinition*/
#line 1016
	    
#line 1016
    ":CGF\r\n"
#line 1016
    ":FG\r\n"
#line 1016
    ":KGruppe\r\n"
#line 1016
    ":LWhlen Sie die Gruppe, deren Followupgruppe Sie setzen mchten.\r\n"
#line 1016
    ":C=\r\n"
#line 1016
    ":FGo\r\n"
#line 1016
    ":KDefault-Followup-Gruppe\r\n"
#line 1016
    ":LGeben Sie den Namen der Gruppe an, in die Kommentare gehen sollen. "
#line 1016
    	"Wird hier kein Name angegeben, wird der Followup gelscht.\r\n"     
#line 1016

#line 1016
	,
#line 1016

#line 1016
	/* Verkettungsptr*/ NULL
#line 1016
};
#line 1016


void
ta_gcmd_followup (char *cmd, 
				  const char *dummy TOM_ATTRIB_UNUSED, 
				  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *gruppe;
	char *p;
	BRETTNR fol;

	gruppe = cmd + 2;			/* Gn berspringen */
	p=strchr(gruppe,'=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++=0;
	strtrim(gruppe);
	strtrim(p);

	if (!*gruppe)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (!ta_cmd_get_group (gruppe, &bnr, TRUE))
	{
		ta_do_log (":?Kann Gruppendaten von %s nicht lesen\r\n",gruppe);
		return;
	}

	if (!*p)
	{
		if (br_ind[bnr].brett->followup_to_default==0)
		{
			bb_unlock_brett(bnr);
			ta_do_log (":!keine nderung\r\n");
		}
		else
		{
			br_ind[bnr].brett->followup_to_default=0;
			br_ind[bnr].brett->configaenderung=time(NULL);
			ta_do_log (":!gelscht\r\n");
			bb_write_brett (bnr, TRUE, FALSE);
		}
		return;
	}

	fol=brettnr(p);
	if (!fol)
	{
		bb_unlock_brett(bnr);
		ta_do_log (":?unbekannte Gruppe: %s\r\n",p);
		return;
	}
	else if (fol==bnr)
	{
		bb_unlock_brett(bnr);
		ta_do_log_error ("Gruppe gleich Followupgruppe");
		return;
	}

	br_ind[bnr].brett->followup_to_default=fol;
	br_ind[bnr].brett->configaenderung=time(NULL);
	bb_write_brett (bnr, TRUE, FALSE);
	ta_do_log (":!Followup gesetzt!\r\n");
}

/* 
 * GPname=sprachcode
 */
#line 1126

#line 1126
ta_command_t ta_gcmd_desc_sprache = {
#line 1126
	/* Magic 1       */ "GP",
#line 1126
	/* Magic 2       */     "",
#line 1126
	/* Funktionsname */     ta_gcmd_sprache,
#line 1126
	/* Min. Level    */     8,
#line 1126
	/* Textpara      */     NULL,
#line 1126
	/* Num. Parameter*/     0,
#line 1126

#line 1126
	/* ITK-ID-Nummer */     37,
#line 1126
	/* Gruppierung   */     "G",
#line 1126
	/* ITK-Version   */     0,
#line 1126
	/* ITK-Typ       */     "B", 
#line 1126
	/* ITK-Name      */     "Benutzte Sprache", 
#line 1126
	/* ITK-Beschrei. */ 
#line 1126
	    
#line 1126
    "Hier stellen Sie ein, welche Sprache in der Gruppe benutzt wird."     
#line 1126
	, 
#line 1126

#line 1126
	/* ITK-Syntaxdefinition*/
#line 1126
	    
#line 1126
    ":CGP\r\n"
#line 1126
    ":FG\r\n"
#line 1126
    ":KGruppe\r\n"
#line 1126
    ":LWhlen Sie die Gruppe, deren Sprache Sie setzen mchten.\r\n"
#line 1126
    ":C=\r\n"
#line 1126
    ":FA\r\n"
#line 1126
    ":KSprache\r\n"
#line 1126
    ":LSprache, auf die die Gruppe gesetzt werden soll\r\n"
#line 1126
    ":A0\r\n"
#line 1126
    ":BUnbekannt\r\n"
#line 1126
    ":A1\r\n"
#line 1126
    ":BDeutsch\r\n"
#line 1126
    ":A2\r\n"
#line 1126
    ":BEnglisch\r\n"
#line 1126
    ":A3\r\n"
#line 1126
    ":BFranzsisch\r\n"
#line 1126
    
#line 1126

#line 1126
	,
#line 1126

#line 1126
	/* Verkettungsptr*/ NULL
#line 1126
};
#line 1126


void
ta_gcmd_sprache (char *cmd, 
				 const char *dummy TOM_ATTRIB_UNUSED, 
				 LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *gruppe;
	char *p;
	int i_spr;
	const char *s_spr;

	gruppe = cmd + 2;			/* Gn berspringen */
	p=strchr(gruppe,'=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++=0;
	strtrim(gruppe);
	strtrim(p);
	if (!*p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	i_spr=(int) strtol(p,NULL,10);
	s_spr=bb_sprache(i_spr);
	if (!s_spr)
	{
		ta_do_log_error ("unbekannte Sprache");
		return;
	}

	if (!*gruppe)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (!ta_cmd_get_group (gruppe, &bnr, TRUE))
	{
		ta_do_log (":?Kann Gruppendaten von %s nicht lesen\r\n",gruppe);
		return;
	}

	if (br_ind[bnr].brett->sprache==i_spr)
	{
		bb_unlock_brett(bnr);
		ta_do_log (":!keine nderung\r\n");
		return;
	}
	br_ind[bnr].brett->sprache=i_spr;
	br_ind[bnr].brett->configaenderung=time(NULL);
	ta_do_log (":!Sprache nun %s\r\n",s_spr);
	bb_write_brett (bnr, TRUE, FALSE);
	return;
}

/* 
 * GHname=herkunftsnetz
 */
#line 1229

#line 1229
ta_command_t ta_gcmd_desc_herkunft = {
#line 1229
	/* Magic 1       */ "GH",
#line 1229
	/* Magic 2       */     "",
#line 1229
	/* Funktionsname */     ta_gcmd_herkunft,
#line 1229
	/* Min. Level    */     8,
#line 1229
	/* Textpara      */     NULL,
#line 1229
	/* Num. Parameter*/     0,
#line 1229

#line 1229
	/* ITK-ID-Nummer */     38,
#line 1229
	/* Gruppierung   */     "G",
#line 1229
	/* ITK-Version   */     0,
#line 1229
	/* ITK-Typ       */     "B", 
#line 1229
	/* ITK-Name      */     "Herkunftsnetz", 
#line 1229
	/* ITK-Beschrei. */ 
#line 1229
	    
#line 1229
    "Hier stellen Sie das Herkunftsnetz der Gruppe ein."     
#line 1229
	, 
#line 1229

#line 1229
	/* ITK-Syntaxdefinition*/
#line 1229
	    
#line 1229
    ":CGH\r\n"
#line 1229
    ":FG\r\n"
#line 1229
    ":KGruppe\r\n"
#line 1229
    ":LWhlen Sie die Gruppe, deren Herkunftsnetz Sie setzen mchten.\r\n"
#line 1229
    ":C=\r\n"
#line 1229
    ":FA\r\n"
#line 1229
    ":KNetz\r\n"
#line 1229
    ":LHerkunftsnetz\r\n"
#line 1229
    ":A0\r\n"
#line 1229
    ":BUnbekannt\r\n"
#line 1229
    ":A1\r\n"
#line 1229
    ":BMausNet\r\n"
#line 1229
    ":A2\r\n"
#line 1229
    ":BUseNet\r\n"
#line 1229
    ":A3\r\n"
#line 1229
    ":BFidoNet\r\n"
#line 1229
    ":A4\r\n"
#line 1229
    ":BZ-Netz\r\n"
#line 1229
    ":A5\r\n"
#line 1229
    ":BMailingListe\r\n"
#line 1229
    
#line 1229

#line 1229
	,
#line 1229

#line 1229
	/* Verkettungsptr*/ NULL
#line 1229
};
#line 1229


void
ta_gcmd_herkunft (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *gruppe;
	char *p;
	int i_herkunft;
	const char *s_herkunft;

	gruppe = cmd + 2;			/* Gn berspringen */
	p=strchr(gruppe,'=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++=0;
	strtrim(gruppe);
	strtrim(p);
	if (!*p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	i_herkunft=(int) strtol(p,NULL,10);
	s_herkunft=bb_ursprungsnetz(i_herkunft);
	if (!s_herkunft)
	{
		ta_do_log_error ("unbekannte Sprache");
		return;
	}

	if (!*gruppe)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (!ta_cmd_get_group (gruppe, &bnr, TRUE))
	{
		ta_do_log (":?Kann Gruppendaten von %s nicht lesen\r\n",gruppe);
		return;
	}

	if (br_ind[bnr].brett->ursprungsnetz==i_herkunft)
	{
		bb_unlock_brett(bnr);
		ta_do_log (":!keine nderung\r\n");
		return;
	}
	br_ind[bnr].brett->ursprungsnetz=i_herkunft;
	br_ind[bnr].brett->configaenderung=time(NULL);
	ta_do_log (":!Sprache nun %s\r\n",s_herkunft);
	bb_write_brett (bnr, TRUE, FALSE);
	return;
}
/*
 * GBgruppe=0:kurzinfo
 * GBgruppe=n:langinfo     n Element 1..
 */
#line 1322

#line 1322
ta_command_t ta_gcmd_desc_beschreibung = {
#line 1322
	/* Magic 1       */ "GB",
#line 1322
	/* Magic 2       */     "",
#line 1322
	/* Funktionsname */     ta_gcmd_beschreibung,
#line 1322
	/* Min. Level    */     0,
#line 1322
	/* Textpara      */     NULL,
#line 1322
	/* Num. Parameter*/     0,
#line 1322

#line 1322
	/* ITK-ID-Nummer */     29,
#line 1322
	/* Gruppierung   */     "G",
#line 1322
	/* ITK-Version   */     0,
#line 1322
	/* ITK-Typ       */     "B", 
#line 1322
	/* ITK-Name      */     "Beschreibung", 
#line 1322
	/* ITK-Beschrei. */ 
#line 1322
	    
#line 1322
    "Mit dieser Funktion kann die kurze oder lange Beschreibung einer Gruppe," 
#line 1322
    "deren Chef Sie sind, gendert werden." 
#line 1322
	, 
#line 1322

#line 1322
	/* ITK-Syntaxdefinition*/
#line 1322
	    
#line 1322
    ":CGB\r\n"
#line 1322
    ":Fg\r\n"
#line 1322
    ":KGruppe\r\n"
#line 1322
    ":LWhlen Sie eine Gruppe.\r\n"
#line 1322
    ":C=\r\n"
#line 1322
    ":FI0,99\r\n"
#line 1322
    ":KZeilennummer\r\n"
#line 1322
    ":LGeben Sie die Nummer der zu ndernden Zeile ein (0: Kurzbeschreibung)\r\n"
#line 1322
    ":C:\r\n"
#line 1322
    ":FS78\r\n"
#line 1322
    ":KText\r\n"
#line 1322
    ":LGeben Sie den Beschreibungstext ein.\r\n" 
#line 1322

#line 1322
	,
#line 1322

#line 1322
	/* Verkettungsptr*/ NULL
#line 1322
};
#line 1322

void
ta_gcmd_beschreibung (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
					  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	char *p;					/* Hilfsvar. */
	char *dp;
	char *gname;				/* Name der betroffenen Gruppe */
	long zeile;
	BRETTNR bnr;

	p = strchr (cmd, '='); /* str*r*chr, weil = mglicherweise in g-namen */
	if (!p || (dp=strchr(p+1,':'))==NULL)
	{
		ta_do_log_error ("unvollstndiges Kommando!");
		return;
	}
	*p++ = 0;
	zeile=strtol(p,NULL,10);
	if (zeile<0)
	{
		ta_do_log_error ("Illegale Zeilennummer!");
		return;
	}
	dp++;

	gname = cmd + 2;			/* GU berspringen */
	strtrim(gname);  /* Gruppenname */
	strtrim(dp);     /* Beschreibungszeile */

	/* Brettdaten lesen, Lock setzen */
	if (!ta_cmd_get_group (gname, &bnr, TRUE))
		return;					/* log done */

	if (level() < 8 && br_ind[bnr].brett->grp_chef!=user.nummer)
	{
		ta_do_log_error ("Sie sind weder Gruppenchef noch Sysop!");
		return;					/* log done */
	}

	if (zeile == 0)
	{
		free(br_ind[bnr].brett->userinfo);
		br_ind[bnr].brett->userinfo=XSTRDUP(dp);
	}
	else
	{
		int zahl,i;
#define SIZE 26
		char *li[SIZE];
		size_t l;
		zahl=0;
		if (br_ind[bnr].brett->lokalinfo)
		{
			for (p=br_ind[bnr].brett->lokalinfo; p && *p ;)
			{
				char *q=strpbrk(p,"\r\n");
				if (q)
					*q=0;
				li[zahl++]=XSTRDUP(p);
				if (q)
					p=q+2;
				if (zahl==SIZE-2)
				{
					ta_do_log (":!alter Infotext zu lang, gekrzt" LINEEND);
					break;
				}
			}
		}
		li[SIZE-1]=0;
		if (zeile>zahl+1)
		{
			ta_do_log_error ("Zeilennummer zu hoch!");
			return;
		}

		if (zeile<=zahl)
			free(li[zeile-1]);
		else
			zahl++;
		li[zeile-1]=XSTRDUP(dp);

		for (i=0,l=0;i<zahl+1;i++)
		{
			if (li[i])
				l+=2+strlen(li[i]);
		}
		br_ind[bnr].brett->lokalinfo=malloc(l+2);
		if (br_ind[bnr].brett->lokalinfo)
		{
			br_ind[bnr].brett->lokalinfo[0]=0;
			for (i=0;i<zahl;i++)
			{
				if (li[i])
				{
					strcat(br_ind[bnr].brett->lokalinfo,li[i]);
					free(li[i]);
				}
				strcat(br_ind[bnr].brett->lokalinfo,"\r\n");
			}
		}
		else
		{
			for (i=0;i<zahl;i++)
			{
				if (li[i])
					free(li[i]);
			}
			bb_unlock_brett(bnr);
			ta_do_log_error ("out of memory");
			return;
		}
	}
	
	br_ind[bnr].brett->configaenderung=time(NULL);
	bb_write_brett (bnr, TRUE, TRUE);
}


/* 
 * GZname=zugriffsrechte
 * Zugriffsrechte: String [X?Y?Z?], mit ? == + oder -.
 *  A Anonym
 *  G Geheim
 *  M Mitglied
 *  P Passwort
 *  Z Zwang
 *  $ Zahler
 *  D Default
 *  O Moderiert
 *  Lnn Level
 */
#line 1508

#line 1508
ta_command_t ta_gcmd_desc_zugriff = {
#line 1508
	/* Magic 1       */ "GZ",
#line 1508
	/* Magic 2       */     "",
#line 1508
	/* Funktionsname */     ta_gcmd_zugriff,
#line 1508
	/* Min. Level    */     8,
#line 1508
	/* Textpara      */     NULL,
#line 1508
	/* Num. Parameter*/     0,
#line 1508

#line 1508
	/* ITK-ID-Nummer */     34,
#line 1508
	/* Gruppierung   */     "G",
#line 1508
	/* ITK-Version   */     0,
#line 1508
	/* ITK-Typ       */     "B", 
#line 1508
	/* ITK-Name      */     "Zugriffsrechte", 
#line 1508
	/* ITK-Beschrei. */ 
#line 1508
	    
#line 1508
    "Mit Hilfe dieser Funktion knnen Sie die Zugriffsrechte auf eine "
#line 1508
    "Gruppe verndern."     
#line 1508
	, 
#line 1508

#line 1508
	/* ITK-Syntaxdefinition*/
#line 1508
	    
#line 1508
	":CGZ\r\n"
#line 1508
	":FG\r\n"
#line 1508
	":KGruppe\r\n"
#line 1508
	":LWhlen Sie eine Gruppe.\r\n"
#line 1508
	":C=\r\n"
#line 1508
	":FM\r\n"
#line 1508
	":KZugriffsrechte\r\n"
#line 1508
	":Lndern Sie hier die Zugriffsrechte\r\n"
#line 1508
	":AA+:A-:\r\n"
#line 1508
	":BAnonym\r\n"
#line 1508
	":AG+:G-:\r\n"
#line 1508
	":BGeheim\r\n"
#line 1508
	":AM+:M-:\r\n"
#line 1508
	":BMitglied\r\n"
#line 1508
	":AP+:P-:\r\n"
#line 1508
	":BPasswort (nur Onlinezugriff mglich)\r\n"
#line 1508
	":AZ+:Z-:\r\n"
#line 1508
	":BZwang (Plicht)\r\n"
#line 1508
	":A$+:$-:\r\n"
#line 1508
	":BNur Zahler drfen lesen\r\n"
#line 1508
	":AD+:D-:\r\n"
#line 1508
	":BDefault\r\n"
#line 1508
	":AO+:O-:\r\n"
#line 1508
	":BModeriert (noch unvollstndig/unbrauchbar)\r\n"
#line 1508
	":AC+:C-:\r\n"
#line 1508
	":BACL (noch unvollstndig/unbrauchbar)\r\n"
#line 1508
	":AL+:L-:\r\n"
#line 1508
	":BMitgliedschaft zum Lesen notwendig\r\n"
#line 1508
	":AY+:Y-:\r\n"
#line 1508
	":BSchreiben nur fr Zahler\r\n"
#line 1508
	":AR0:R1:R2:R3:R4:R5:R6:R7:R8:R9:\r\n"
#line 1508
	":BLeselevel\r\n"
#line 1508
	":AW0:W1:W2:W3:W4:W5:W6:W7:W8:W9:\r\n"
#line 1508
	":BSchreiblevel\r\n" 
#line 1508

#line 1508
	,
#line 1508

#line 1508
	/* Verkettungsptr*/ NULL
#line 1508
};
#line 1508


void
ta_gcmd_zugriff (char *cmd, const char *dummy TOM_ATTRIB_UNUSED, 
				 LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *altgruppe;
	char *p;
	ULONG flags;
	BOOLEAN ok;
	int rlv;
	int wlv;

	altgruppe = cmd + 2;		/* Gn berspringen */
	p = strchr (altgruppe, '=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++ = 0;
	strtrim (altgruppe);
	strtrim (p);

	if (!*altgruppe || !*p)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (!ta_cmd_get_group (altgruppe, &bnr, TRUE))
	{
		ta_do_log_error ("Kann Gruppendaten nicht lesen");
		return;
	}
	flags = br_ind[bnr].brett->flags;
	rlv = br_ind[bnr].brett->lesen_level;
	wlv = br_ind[bnr].brett->schreib_level;
	ok = TRUE;
	while (*p)
	{
		if (!p[1])
		{
			ta_do_log (":?unvollstndiges Flag %c" LINEEND, *p);
			ok = FALSE;
			break;
		}
#define FLAG(which) \
if (p[1]=='+') flags |=(which) ; \
else if (p[1]=='-') flags &= ~(which); \
else { ok=FALSE; ta_do_log(":?illegaler Wert '%c' fr Flag %c"LINEEND, \
	p[1],*p); }
		switch (*p)
		{
		case 'A':
			FLAG (B_ANONYM);
			break;
		case 'G':
			FLAG (B_GEHEIM);
			break;
		case 'M':
			FLAG (B_MITGLIED);
			break;
		case 'P':
			FLAG (B_PASSWORT);
			break;
		case 'Z':
			FLAG (B_ZWANG);
			break;
		case 'D':
			FLAG (B_DEFAULT);
			break;
		case '$':
			FLAG (B_ZAHLER);
			break;
		case 'O':
			FLAG (B_MODERIERT);
			break;
		case 'C':
			FLAG (B_ACL);
			break;
		case 'L':
			FLAG (B_LESMITGLIED);
			break;
		case 'Y':
			FLAG (B_PAYWRITE);
			break;

		case 'R':
			if (!p[1])
			{
				ta_do_log (":?unvollstndige Levelangabe" LINEEND);
				ok = FALSE;
			}
			else if (!isdigit (p[1]))
			{
				ta_do_log (":?illegale Levelangabe" LINEEND);
				ok = FALSE;
			}
			else
			{
				rlv = p[1] - '0';
			}
			break;
		case 'W':
			if (!p[1])
			{
				ta_do_log (":?unvollstndige Levelangabe" LINEEND);
				ok = FALSE;
			}
			else if (!isdigit (p[1]))
			{
				ta_do_log (":?illegale Levelangabe" LINEEND);
				ok = FALSE;
			}
			else
			{
				wlv = p[1] - '0';
			}
			break;
		default:
			ta_do_log (":?unbekanntes Flag %c" LINEEND, *p);
			ok = FALSE;
			break;
		}
		if (!ok)
			break;
		p += 2;
	}							/* while *p */
	if (flags == br_ind[bnr].brett->flags 
		&& rlv == br_ind[bnr].brett->lesen_level 
		&& wlv == br_ind[bnr].brett->schreib_level)
	{
		ta_do_log (":!unverndert (gleich altem Status)" LINEEND);
	}
	else if (ok)
	{
		char alt[255];
		char neu[255];

		brettflags_to_string (flags, neu, sizeof (neu));
		sprintf (neu + strlen (neu), "L%c%c", rlv + '0', wlv + '0');
		brettflags_to_string (br_ind[bnr].brett->flags, alt, sizeof (alt));
		sprintf (alt + strlen (alt), "L%c%c", 
				 br_ind[bnr].brett->lesen_level + '0', br_ind[bnr].brett->schreib_level + '0');
		ta_do_log (":! gendert auf %s, war %s" LINEEND, neu, alt);
		br_ind[bnr].brett->flags = flags;
		br_ind[bnr].brett->lesen_level = rlv;
		br_ind[bnr].brett->schreib_level = wlv;
		br_ind[bnr].brett->configaenderung=time(NULL);
	}
	bb_write_brett (bnr, TRUE, FALSE);
}


/* 
 * GIname=alias
 */
#line 1695

#line 1695
ta_command_t ta_gcmd_desc_alias = {
#line 1695
	/* Magic 1       */ "GI",
#line 1695
	/* Magic 2       */     "",
#line 1695
	/* Funktionsname */     ta_gcmd_alias,
#line 1695
	/* Min. Level    */     8,
#line 1695
	/* Textpara      */     NULL,
#line 1695
	/* Num. Parameter*/     0,
#line 1695

#line 1695
	/* ITK-ID-Nummer */     1004,
#line 1695
	/* Gruppierung   */     "G",
#line 1695
	/* ITK-Version   */     0,
#line 1695
	/* ITK-Typ       */     "B", 
#line 1695
	/* ITK-Name      */     "Aliasnamen einer Gruppe ndern", 
#line 1695
	/* ITK-Beschrei. */ 
#line 1695
	    
#line 1695
    "Hier knnen Sie den Aliasnamen einer Gruppe ndern."     
#line 1695
	, 
#line 1695

#line 1695
	/* ITK-Syntaxdefinition*/
#line 1695
	    
#line 1695
    ":CGI\r\n"
#line 1695
    ":FG\r\n"
#line 1695
    ":KGruppe\r\n"
#line 1695
    ":LWhlen Sie die Gruppe, deren Aliasnamen Sie ndern mchten.\r\n"
#line 1695
    ":C=\r\n"
#line 1695
    ":FGo\r\n"
#line 1695
    ":KAliasname\r\n"
#line 1695
    ":LGeben Sie den Aliasnamen ein. "
#line 1695
    	"Wird hier kein Name angegeben, wird der Aliasname gelscht.\r\n"     
#line 1695

#line 1695
	,
#line 1695

#line 1695
	/* Verkettungsptr*/ NULL
#line 1695
};
#line 1695


void
ta_gcmd_alias (char *cmd, 
				  const char *dummy TOM_ATTRIB_UNUSED, 
				  LONG dummy2 TOM_ATTRIB_UNUSED)
{
	BRETTNR bnr;
	char *gruppe;
	char *p;
	BRETTNR fol;

	gruppe = cmd + 2;			/* Gn berspringen */
	p=strchr(gruppe,'=');
	if (!p)
	{
		ta_do_log_error ("unvollstndiges Kommando");
		return;
	}
	*p++=0;
	strtrim(gruppe);
	strtrim(p);

	if (!*gruppe)
	{
		ta_do_log_error ("Kein Gruppenname angegeben");
		return;
	}
	if (!ta_cmd_get_group (gruppe, &bnr, TRUE))
	{
		ta_do_log (":?Kann Gruppendaten von %s nicht lesen\r\n",gruppe);
		return;
	}

	if (!*p)
	{
		if (br_ind[bnr].brett->aliasname==NULL)
		{
			bb_unlock_brett(bnr);
			ta_do_log (":!keine nderung\r\n");
		}
		else
		{
			free(br_ind[bnr].brett->aliasname);
			br_ind[bnr].brett->aliasname=NULL;
			br_ind[bnr].brett->configaenderung=time(NULL);
			ta_do_log (":!gelscht\r\n");
			bb_write_brett (bnr, TRUE, FALSE);
		}
		return;
	}

	fol=brettnr(p);
	if (fol)
	{
		bb_unlock_brett(bnr);
		ta_do_log (":?Aliasname kollidiert mit %s\r\n",
			br_ind[fol].brett->name);
		return;
	}

	free(br_ind[bnr].brett->aliasname);
	br_ind[bnr].brett->aliasname=p;
	br_ind[bnr].brett->configaenderung=time(NULL);
	bb_write_brett (bnr, TRUE, FALSE);
	ta_do_log (":!Aliasname gesetzt!\r\n");
}
