/* ta_info.c : infofiles, export */
/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/

#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <syslog.h>
#include <ctype.h>
#include <errno.h>

#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "mmread.h"
#include "modem.h"
#include "terminal.h"
#include "tomtcl.h"


#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_info.h"

#define INFOFILECNF "infofile.cnf"
#define CRCFILE "infofile.crc"

extern WORD vtmodeW;

	/* schon initialisiert? */
static BOOLEAN init_done;

	/* wenn das Flag TRUE ist, mu LOKALCRC gesichert werden */
static BOOLEAN list_changed;
	/* bzw SERVERCRC */
static BOOLEAN xlist_changed;
static infofilemenu_t *ta_infofilemenu_anker;

static infofile_t *ta_info_anker;
static xinfofile_t *ta_xinfo_anker;
static infofile_t *ta_add_infofile (char *name, char *group, char zeichen,
		char *text, char *cmd, UWORD level, INFOTYP typ, 
		BOOLEAN crc, USERNR server, ta_info_ident ident);

infofilemenu_t *
ta_get_infofilemenu_anker(void)
{
	return ta_infofilemenu_anker;
}

	/* Tabelle der fest vorhandenen Infofiles */
static infofile_t techinfotab[] =
{
	{
		"ITI",					/* Kuerzel */
		NULL,
		0,
		"technische Infofileliste",		/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		0,						/* numpara - ignoriert */
		TA_INFO_USERIDENT,
		ta_info_iti,
		NULL					/* next */
	},
	{
		"ITE",					/* Kuerzel */
		NULL,
		0,
		"technische Gruppenliste mit nderungszeiten",	/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		1,						/* numpara */
		TA_INFO_USERIDENT,
		ta_info_itg,
		NULL
	},
	{
		"ITG",					/* Kuerzel */
		NULL,
		0,
		"technische Gruppenliste",	/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		0,						/* numpara */
		TA_INFO_USERIDENT,
		ta_info_itg,
		NULL
	},
	{
		"IGK",					/* Kuerzel */
		NULL,
		0,
		"kurze Gruppenliste",	/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		0,						/* numpara - unterscheidung zwischen IGK (0) und IGL (1) */
		TA_INFO_USERIDENT,
		ta_info_igx,
		NULL
	},
	{
		"IGL",					/* Kuerzel */
		NULL,
		0,
		"lange Gruppenliste",	/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		1,						/* numpara - unterscheidung zwischen IGK (0) und IGL (1) */
		TA_INFO_USERIDENT,
		ta_info_igx,
		NULL
	},
	{
		"ITB",					/* Kuerzel */
		NULL,
		0,
		"technische Boxenliste",	/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		0,						/* numpara */
		TA_INFO_USERIDENT,
		ta_info_itb,
		NULL
	},
	{
		"ITS",					/* Kuerzel */
		NULL,
		0,
		"technische Boxenliste fr Sysops",	/* Infotext */
		ITYP_PROC,				/* Typ */
		7,						/* ab 7 */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		1,						/* numpara */
		TA_INFO_USERIDENT,
		ta_info_itb,
		NULL
	},
	{
		"ITK",					/* Kuerzel */
		NULL,
		0,
		"technische Kommandoliste",	/* Infotext */
		ITYP_PROC,				/* Typ */
		0,						/* Alle Level == alle Tauscher */
		0,						/* Sender */
		TRUE,					/* mit CRC */
		-1,						/* CRC */
		0,						/* Zeit des Uploads */
		NULL,					/* Textpara - ignoriert */
		0,						/* numpara - ignoriert */
		TA_INFO_USERIDENT,
		ta_info_itk,
		NULL
	},
	/* Endekennzeichen */
	{NULL, NULL, 0, NULL, ITYP_PROC, 0, 0, FALSE, 0, 0, NULL, 0, 0, NULL, NULL}
};

/*
   liest die Konfigurationsdateien 
 */
static void ta_info_init (void);


/* CRCFILE schreiben */
static void ta_info_write_crc (void);

/* Hngt das Infofile in inputfilename an, falls die Lnge das zult */
static void
ta_copyappend_infofile (const infofile_t * inf, const char *fname_in,
						 FILE *fout, ULONG crc, long *len);

infofile_t *
ta_get_info_anker (void)
{
	if (!init_done)
		ta_info_init ();
	return ta_info_anker;
}

static infofile_t *
ta_add_infofile (char *name, char *group, char zeichen, char *text, char *cmd, 
				 UWORD level, INFOTYP typ, BOOLEAN crc, USERNR server, 
				 ta_info_ident ident)
{
	infofile_t *new;
	static infofile_t *last=NULL;

	if (typ == ITYP_PROC)		/* geht so nicht - die sind grundsaetzlich intern */
		return NULL;
	if (level > 9)
		level = 9;

	new = XMALLOC (sizeof (infofile_t));
	new->name = XSTRDUP (name);
	if (group)
		new->group=XSTRDUP(group);
	else
		new->group=NULL;
	new->text = XSTRDUP (text);
	if (cmd)
		new->textpara = XSTRDUP (cmd);
	else
		new->textpara = NULL;

	new->level = level;
	new->mit_crc=crc;
	new->crc = (ULONG) - 1;		/* unbekannt */
	if (typ==ITYP_NET)
		new->sender=server;
	else
		new->sender = 0;			/* Absender unbekannt */
	new->zeit = (time_t) 0;
	new->typ = typ;
	new->ident = ident;
	new->zeichen = zeichen;

	if (!last)
	{
		infofile_t *akt;
		for (akt=ta_info_anker;akt;akt=akt->next)
			last=akt;
	}
	last->next=new;
	new->next=NULL;
	last=new;
	return new;
}

infofile_t *
ta_find_infofile (const char *name)
{
	infofile_t *akt;

	if (!init_done)
		ta_info_init ();
	akt = ta_info_anker;
	while (akt)
	{
		if (!stricmp (name, akt->name))
			return akt;
		akt = akt->next;
	}
	return NULL;
}

/*
 * Interne Routine zur Initialisierung der internen Daten.
 */
static void
ta_info_init (void)
{
	char fname[PATH_MAX];
	FILE *f;
	char buf[1024];

	if (init_done)
		return;
	init_done = TRUE;			/* MUSS jetzt sein */

	/*
	 * Die Tabelle der technischen Infofiles verketten
	 */
	{
		infofile_t *akt;

		akt = techinfotab;
		while (akt && akt[1].name)
		{
			akt->next = &akt[1];
			akt++;
		}
		akt->next = NULL;
		ta_info_anker = techinfotab;
	}

	sprintf(fname,"%s" INFOFILECNF,box_conf.config_dir);
	f=fopen(fname,"r");
	if (!f)
	{
		syslog(LOG_ERR,"kann Infofilekonfiguration nicht lesen");
		return;
	}
#define SEPSKIP(var) do { *((var)++)=0; while(isspace(*(var))) var++; } while(0)
#define NEXTSTRING(start,var) \
	do { if (ok) { \
		var=strpbrk(start,"\t "); \
		if (!var) ok=FALSE; \
		else SEPSKIP(var); \
	} } while (0)

	while (fgets(buf,sizeof(buf),f))
	{
		char *kuerzel;
		char *zeilentyp;
		char *p;
		BOOLEAN ok=TRUE;
		p=strpbrk(buf,"\r\n");
		if (p)
			*p=0;
		zeilentyp=strtrim(buf);
		if (!*zeilentyp || *zeilentyp=='#')
			continue;
		NEXTSTRING(zeilentyp,kuerzel);
		if (!ok)
			continue;
		if (!strhalfcasecmp(zeilentyp,"biete"))
		{
			char *crc;
			char *minlevel;
			char *identisch;
			char *infotyp;
			char *beschreibung;
			char *rest=NULL;
			char zeichen=0;
			USERNR unr=0;
			char *parameter=NULL;
			NEXTSTRING(kuerzel,crc);
			NEXTSTRING(crc,minlevel);
			NEXTSTRING(minlevel,identisch);
			NEXTSTRING(identisch,infotyp);
			NEXTSTRING(infotyp,beschreibung);
			if (!ok)
			{
				syslog(LOG_NOTICE,
					"biete-Infofileeintrag %s: unparsbare Zeile",kuerzel);
				continue;
			}
			if (*beschreibung=='"') 
			{
				beschreibung++;
				rest=strchr(beschreibung,'"');
				if (rest)
					SEPSKIP(rest);
			}
			else
			{
				NEXTSTRING(beschreibung,rest);
			}
			ok=TRUE;
			if (strtol(minlevel,NULL,10)<0 || strtol(minlevel,NULL,10) > 9)
			{
				syslog(LOG_NOTICE,
					"biete-Infofileeintrag %s: level %s ungltig",kuerzel,
					minlevel);
				continue;
			}
			if (!strhalfcasecmp(infotyp,"netz"))
			{
				char name[256];
				if (!rest || !*rest)
				{
					syslog(LOG_NOTICE,
						"biete-Infofileeintrag %s: kein Server angegeben",
						kuerzel);
					continue;
				}
				if (!ub_find_username(&unr,rest)
					&& (!do_interpret(rest, name,sizeof(name)-1)
						|| !*strtrim(name) || !ub_find_username(&unr,name)))
				{
					syslog(LOG_NOTICE,
						"hole-Infofileeintrag %s: Server %s unbekannt",kuerzel,
						rest);
					continue;
				}
			}
			else if (!strhalfcasecmp(infotyp,"lokal"))
			{
				if (rest && *rest)
				{
					syslog(LOG_NOTICE,
						"biete-Infofileeintrag %s: Mll am Ende der Zeile",
						kuerzel);
					/* kein Continue - wir haben, was wir brauchen */
				}
			}
			else if (!strhalfcasecmp(infotyp,"quarkcmd") 
				|| !strhalfcasecmp(infotyp,"extern"))
			{
				if (!rest || !*rest)
				{
					syslog(LOG_NOTICE,
						"biete-Infofileeintrag %s: Parameter/Kommando fehlt",
						kuerzel);
					continue;
				}
				parameter=rest;
			}
			else
			{
				syslog(LOG_NOTICE,
					"biete-Infofileeintrag %s: Infofiletyp %s ungltig",kuerzel,
					infotyp);
				continue;
			}
			if (strhalfcasecmp(identisch,"netz") && strhalfcasecmp(identisch,"lokal")
				&& strhalfcasecmp(identisch,"user"))
			{
				syslog(LOG_NOTICE,
					"biete-Infofileeintrag %s: Identitt %s ungltig",kuerzel,
					identisch);
				continue;
			}
			/* Alles OK */
			{
				INFOTYP typ;
				char *slash;
				ta_info_ident ident;
				if (!strhalfcasecmp(infotyp,"netz"))
					typ=ITYP_NET;
				else if (!strhalfcasecmp(infotyp,"quarkcmd"))
					typ=ITYP_INTERN;
				else if (!strhalfcasecmp(infotyp,"extern"))
					typ=ITYP_CMD;
				else
					typ=ITYP_LOKAL;
				if (!strhalfcasecmp(identisch,"netz"))
					ident=TA_INFO_NETIDENT;
				else if	(!strhalfcasecmp(identisch,"user"))
					ident=TA_INFO_USERIDENT;
				else
					ident=TA_INFO_LOKALIDENT;
				slash=strchr(kuerzel,'/');
				if (slash)
				{
					char *pz;
					*slash++=0;
					pz=strchr(slash,'=');
					if (pz)
					{
						zeichen=pz[1];
						*pz=0;
					}
				}
				ta_add_infofile (kuerzel, slash, zeichen, beschreibung, 
					parameter, 
					strtol(minlevel,NULL,10), typ,
					!strhalfcasecmp(crc,"crc"), unr, ident);
			}
		}
		else if (0==strhalfcasecmp(zeilentyp,"menue"))
		{
			/* eine Menvereinbarung */
			char *minlevel=NULL;
			char *beschreibung=NULL;
			char *rest;
			infofilemenu_t *neu;
			simplelist_t *last, *neuele;
			NEXTSTRING(kuerzel,minlevel);
			NEXTSTRING(minlevel,beschreibung);
			if (*beschreibung=='"') 
			{
				beschreibung++;
				rest=strchr(beschreibung,'"');
				if (rest)
					SEPSKIP(rest);
			}
			else
			{
				NEXTSTRING(beschreibung,rest);
			}
			if (!ok || !minlevel || !beschreibung)
			{
				syslog(LOG_NOTICE,
					"biete-Infofileeintrag %s: unparsbare Zeile",kuerzel);
				continue;
			}
			neu=XMALLOC(sizeof(infofilemenu_t));
			neu->kuerzel=XSTRDUP(kuerzel);
			neu->beschreibung=XSTRDUP(beschreibung);
			neu->minlevel=strtol(minlevel,NULL,10);
			neu->next=ta_infofilemenu_anker;
			ta_infofilemenu_anker=neu;
			last=NULL;
			neu->submenues=NULL;
			while (rest && *rest)
			{
				char *nextrest=NULL;
				infofilemenu_t *akt;
				char *pz;
				NEXTSTRING(rest,nextrest);
				pz=strchr(rest,'=');
				if (pz)
					*pz++=0;

				for (akt=ta_infofilemenu_anker;akt;akt=akt->next)
				{
					if (0==strcasecmp(rest,akt->kuerzel))
						break;
				}
				if (!akt)
				{
					syslog(LOG_NOTICE,
						   "menue-Infofileeintrag %s: submenue %s undefiniert",
						   kuerzel, rest);
					/* nicht gefunden */
				}
				else
				{
					neuele=simple_alloc(rest);
					XNULLCHECK(neuele);
					if (pz && *pz)
						neuele->xstring=XSTRDUP(pz);
					if (last)
						last->next=neuele;
					else
						neu->submenues=neuele;
					last=neuele;
				}
				rest=nextrest;
			}
		}
		else if (!strhalfcasecmp(zeilentyp,"hole"))
		{
			char *crc;
			char *server;
			NEXTSTRING(kuerzel,crc);
			NEXTSTRING(crc,server);
			if (ok)
			{
				USERNR unr;
				char name[256];
				if (!ub_find_username(&unr,server)
					&& (!do_interpret(server, name,sizeof(name)-1)
					|| !*strtrim(name) || !ub_find_username(&unr,name)))
				{
					syslog(LOG_NOTICE,
						"hole-Infofileeintrag %s: Server %s unbekannt",kuerzel,
						name);
				}
				else
				{
					xinfofile_t *sn;
					sn=XMALLOC(sizeof(xinfofile_t));
					sn->name=XSTRDUP(kuerzel);
					sn->server=unr;
					sn->mit_crc=!strhalfcasecmp(crc,"crc");
					sn->crc=(ULONG) -1;
					sn->next=ta_xinfo_anker;
					ta_xinfo_anker=sn;
				}
			}
			else
			{
				syslog(LOG_NOTICE,
					"hole-Infofileeintrag %s: unparsbare Zeile",kuerzel);
			}
		}
		else 
		{
				syslog(LOG_NOTICE,
					"infofile.cnf: unparsbare Zeile %s",zeilentyp);
		}
	}
	fclose(f);

	/*
	 * So, jetzt wissen wir, welche Infofiles konfiguriert sind. Nun
	 * die variablen Daten holen, wo sie notwendig sind.
	 * -> infofile.crc auslesen
	 */
	sprintf (fname, "%s" CRCFILE, box_conf.var_dir);
	f = fopen (fname, BINREAD);
	if (f != NULL)
	{
		char kuerzel[256];
		ULONG unr;
		ULONG crc;
		ULONG zeit;

		while (fgets(buf,sizeof(buf),f))
		{
			char *at=strchr(buf,'@');
			char *sp=strchr(buf,' ');
			if (!sp)
				continue;
			if (at && at<sp)
			{
				/* CRC bei Server */
				xinfofile_t *xi;
				*at=' ';
				if (sscanf(buf,"%255s%lu%lu",
					kuerzel,&unr,&crc)!=3)
					continue;
				xi=ta_find_xinfofile(kuerzel,unr);
				if (!xi)
					continue;
				xi->crc=crc;
				continue;
			}
			/* CRC lokal */
			if (sscanf(buf,"%255s%lu%lu%lu",
				kuerzel,&unr,&crc,&zeit)==4)
			{
				infofile_t *i;
				i=ta_find_infofile(kuerzel);
				if (i)
				{
					if (i==ITYP_LOKAL)
						i->sender=unr;
					i->crc=crc;
					i->zeit=zeit;
				}
			}
		}
		fclose (f);
	}

	/*
	 * Zustand jetzt:
	 * - alle Infofiles sind bekannt.
	 * - Uploadzeit ist bekannt.
	 * - CRC ist bekannt.
	 *
	 * Nun nachschauen, ob ein Infofile in einem der Ordner jnger ist
	 * als angegeben.
	 */
	ta_info_sync ();
}


/*
 * infofile.crc schreiben.
 */
static void
ta_info_write_crc (void)
{
	FILE *f;
	char fname[PATH_MAX];
	infofile_t *akt;
	xinfofile_t *xakt;

	/*
	 * infofile.crc schreiben
	 */
	if (!list_changed && !xlist_changed)
		return;

	sprintf (fname, "%s" CRCFILE, box_conf.var_dir );
	f = xfopen (fname, BINWRITE);

	fprintf(f,"#Keep off, if you don't know what you are doing\n");
	fprintf(f,"#Liste der CRC der lokal verfgbaren nichttechnischen Infofiles\n");
	fprintf(f,"#Format: Infofilename,Senderusernummer,crc,time_t(Uploaddatum)\n");
	akt = ta_info_anker;
	while (akt)
	{
		if (akt->typ == ITYP_NET || akt->typ == ITYP_LOKAL)
			fprintf (f, "%s %lu %lu %ld\n", akt->name, (ULONG) akt->sender,
					 (ULONG) akt->crc, (ULONG) akt->zeit);
		akt = akt->next;
	}
	xakt = ta_xinfo_anker;
	fprintf(f,"#Liste der CRC der von anderswo geholten Infofiles\n");
	fprintf(f,"#Format: Infofilename@serverusernummer crc\n");
	while (xakt)
	{
		fprintf (f, "%s@%lu %lu\n", xakt->name, (ULONG) xakt->server,
			(ULONG) xakt->crc);
		xakt = xakt->next;
	}
	fclose (f);
	list_changed = FALSE;
	xlist_changed = FALSE;
}

void
ta_info_sync (void)
{
	char crcname[PATH_MAX];
	infofile_t *akt;
	struct stat st;

	if (!init_done)
	{
		ta_info_init ();
		return;					/* weil ta_sync von ta_info_init aufgerufen wird */
	}
	akt = ta_info_anker;
	while (akt)
	{
		if (akt->typ == ITYP_NET || akt->typ == ITYP_LOKAL)
		{
			ta_info_filename (akt, crcname);
			if (stat (crcname, &st) == 0)
			{
				if (st.st_mtime > akt->zeit)
				{
					/* Das Infofile ist verndert worden */
					/* ### hier erfahren wir nichts ber den Infofilesender 
					 * Das lt sich ndern - wenn alle externen Programme
					 * <infofilename>.upl mitschicken, wenn sie etwas
					 * verndert haben, und in der Datei den Namen des
					 * Uploaders hinterlassen. TODO 
					 */
					akt->sender = 0;	/* den wissen wir leider nicht */
					/* ### CRC? Aber warum denn in aller Welt? */
					akt->crc = (st.st_mtime + st.st_size) & 0xffff;
					akt->zeit = time (NULL);
					list_changed = TRUE;
				}
			}
			else
			{
				akt->sender = 0;
				akt->zeit = 0;
				akt->crc = (ULONG) - 1;
				list_changed = TRUE;
			}
		}						/* Wenn CRC-Mechanismus */
		akt = akt->next;
	}							/* while (akt) */
	if (list_changed || xlist_changed)
		ta_info_write_crc ();
}

BOOLEAN
ta_xinfo_filename (xinfofile_t * inf, char *fname)
{
	if (!init_done)
		ta_info_init ();
	if (!inf)
		return FALSE;
	sprintf (fname, "%s%lu/%s", tom_info_net_dir, inf->server, inf->name);
	return TRUE;
}

BOOLEAN
ta_info_filename (infofile_t * inf, char *fname)
{
	if (!init_done)
		ta_info_init ();
	if (!inf)
		return FALSE;
	switch (inf->typ)
	{
	case ITYP_PROC:
	case ITYP_INTERN:
	case ITYP_CMD:
		/* Sollte nicht vorkommen, aber vielleicht erleichtert es mal
		   die Fehlersuche
		 */
		strcpy (fname, "gibt.es.nicht");
		return FALSE;
	case ITYP_LOKAL:
		sprintf (fname, "%s%s", tom_info_lokal_dir, inf->name);
		break;
	case ITYP_NET:
		sprintf (fname, "%s%lu/%s", tom_info_net_dir, inf->sender, inf->name);
		break;
	}
	return TRUE;
}

/*
 * Hngt das Infofile "inf", dessen Inhalt (ohne #inf->name) in fname_in ist, 
 * an out an. Bei allen Infofiletypen auer ITYP_PROC werden 
 * Doppelpunkte vor die Zeilen geschrieben. 
 * Als *len mu die Gesamtlaenge des Outfiles bergeben werden, sie wird 
 * hinterher korrekt modifiziert.
 * Ein Logeintrag wird geschrieben.
 * Die Funktion schreibt die "#"-Zeile selbst.
 */
#if 1
static void
ta_copyappend_infofile (const infofile_t * inf, const char *fname_in,
						FILE *out, ULONG crc, long *len)
{
	mm_file_t *in;
	char *buffer;
	size_t buffersize;
	char *p;
	size_t l=0;
	size_t got;

	in = MM_READ_OPEN(fname_in);
	if (!in)
	{
		ta_do_log (":\"%s\"\r\n", inf->name);
		ta_do_log_error ("Kann Informationstext nicht lesen");
		return;
	}

	if (len && (ULONG) (MM_READ_SIZE(in)) > ta_passinfo.maxbytes)
	{
		MM_READ_CLOSE(in)
		ta_do_log (":\"%s\"\r\n", inf->name);
		ta_do_log_error ("Maximalgre berschritten");
		return;
	}

	if (!(MM_READ_IS_MMAPPED(in)))
	{
		buffersize=1024;
		buffer=XMALLOC(buffersize);
	}

	l=fprintf (out, "#%s\r\n", inf->name);

	while ((got=MM_READ_GETLINE(in,&buffer,&buffersize))!=-1)
	{
		if (inf->typ==ITYP_PROC)
		{
			l+=got;
			fwrite(buffer,1,got,out);
			continue;
		}
	
		/* Zeilenende weglassen */
		p=buffer+got-1;
		if (*p=='\n')
		{
			if (p > buffer && p[-1]=='\r') 
			{
				fputc(':',out);
				fwrite(buffer,1,got,out);
				l+=got;
				continue;
			}
			got--;
		}
		fputc(':',out);
		fwrite(buffer,1,got,out);
		fwrite("\r\n",1,2,out);
		l+=got+3; /* +3 wegen :\r\n */
	}
	if (len)
		*len += l;
	if (!(MM_READ_IS_MMAPPED(in)))
		free(buffer);
	MM_READ_CLOSE(in);
	if (inf->mit_crc)
		ta_do_log (":$%s=%ld (CRC gendert)\r\n", inf->name, crc);
	else
		ta_do_log (":$%s (generiert)\r\n", inf->name);
}
#else
static void
ta_copyappend_infofile (const infofile_t * inf, const char *fname_in,
						FILE *out, ULONG crc, long *len)
{
	FILE *in;
	char buffer[20000];
	char *p;
	int offset=0;

	if (inf->typ!=ITYP_PROC)
	{
		offset=1;
		buffer[0]=':';
	}

	in = fopen (fname_in, BINREAD);
	if (in)
	{
		if (len && (ULONG) (filesize (in) + *len) > ta_passinfo.maxbytes)
		{
			fclose (in);
			ta_do_log (":\"%s\"\r\n", inf->name);
			ta_do_log_error ("Maximalgre berschritten");
			return;
		}
		else
		{
			size_t l=0;
			char *rpos=buffer+offset;
			l+=fprintf (out, "#%s\r\n", inf->name);
			while (fgets (rpos, sizeof (buffer) - 1 - offset, in))
			{
				size_t l_zeile; /* Lnge der Zeile inc. Doppelpunkt */
				p = strchr (rpos, '\n');
				if (p)
				{
					if (p[-1]!='\r')
					{
						*p++='\r';
						*p++='\n';
						*p=0;
						l_zeile=p-rpos+1;
					}
					else
						l_zeile=p-rpos+2;
				}
				else
				{
					l_zeile=strlen(rpos);
					p=rpos+l_zeile;
					*p++ = '\r';
					*p++ = '\n';
					*p=0;
					l_zeile+=3;
				}
				fputs(buffer,out);
				l+=l_zeile;
			}
			if (len)
				*len += l;
			fclose (in);
			if (inf->mit_crc)
				ta_do_log (":$%s=%ld (CRC gendert)\r\n", inf->name, crc);
			else
				ta_do_log (":$%s (generiert)\r\n", inf->name);
		}
	}
	else
	{
		ta_do_log (":\"%s\"\r\n", inf->name);
		ta_do_log_error ("Informationstext nicht vorhanden");
	}
}
#endif

BOOLEAN
ta_online_varinfofile (const char *fname, const char *kurz)
{
	infofile_t *inf;

	if (!init_done)
		ta_info_init ();
	inf = ta_find_infofile (kurz);
	if (!inf)
		return FALSE;
	switch(inf->typ)
	{
	case ITYP_PROC:
		{
			ULONG crc;
			crc = inf->fn (fname, inf->textpara, inf->numpara);
			if (crc==(ULONG) -1)	
			{
				unlink(fname);
				return FALSE;
			}
			return TRUE;
		}
		break;
	case ITYP_CMD:
		{
			char cmd[PATH_MAX * 2 + 1024];
			if (do_interpret (inf->textpara, cmd,
				  sizeof (cmd) - strlen (fname) - 20) > 0)
			{
				sprintf (cmd + strlen (cmd), " >%s 2>&1", fname);
				if (system (cmd) || !fileexist(fname))
				{
					unlink(fname);
					return FALSE;
				}
				return TRUE;
			}
			return FALSE;
		}
		break;
	case ITYP_INTERN:
#define IDEBUG(x)
		{
			WORD oldvtmodeW=vtmodeW;
			char cmd[PATH_MAX * 2 + 1024];
IDEBUG(fprintf(stderr,"vor do_interpret fr %s\r\n",kurz);)
			if (do_interpret (inf->textpara, cmd,
				  sizeof (cmd) - 20) <= 0)
				 return FALSE;
IDEBUG(fprintf(stderr,"vor befehls_umlenkung nach %s\r\n",fname);)
			if (!befehls_umlenkung(fname))
				return FALSE;
			vtmodeW=T_ASCII;
IDEBUG(fprintf(stderr,"vor sync_terminal 1\r\n");)
			sync_terminal();
IDEBUG(fprintf(stderr,"vor befehls_ausfhrung(%s)\r\n",cmd);)
			befehl_ausfuehrung(cmd);
IDEBUG(fprintf(stderr,"vor befehls_umlenkung, aufhebung\r\n");)
			befehls_umlenkung(NULL);
			vtmodeW=oldvtmodeW;
IDEBUG(fprintf(stderr,"vor sync_terminal 2\r\n");)
			sync_terminal();
IDEBUG(fprintf(stderr,"fertig\r\n");)
			return TRUE;
		}
		break;
	case ITYP_NET:
	case ITYP_LOKAL:
		{
			char inname[PATH_MAX];

			ta_info_filename (inf, inname);
			/* absoluter Pfad wegen symlink */
			if (*inname!='/')
			{
				char *s=concat(box_conf.quark_dir,'/',inname);
				XNULLCHECK(s);
				strcpy(inname,s);
			}
			if (
#ifdef HAVE_SYMLINK
				symlink(inname,fname) &&
#endif
#ifdef HAVE_LINK
				link(inname,fname) && 
#endif
				!file_copy(inname,fname))
				return FALSE;
			return TRUE;
		}
		break;
	}
	return FALSE; /* just in case */
}

void
ta_work_icmd (long *len, FILE *out)
{
	FILE *f;
	char buffer[1024];
	char fname[PATH_MAX];
	char *p;
	size_t l;
	infofile_t *inf;
	LONG crc=-1; /* keep gcc quiet */
	char *crcp;

	if (!init_done)
		ta_info_init ();
	sprintf (fname, "%s_later.cmd", tom_tmp_dir);
	f = fopen (fname, BINREAD);
	if (!f)
		return;
	/*
	 * Datei zeilenweise durchsuchen.
	 */
	while (fgets (buffer, sizeof (buffer) - 1, f))
	{
		p = strpbrk (buffer, "\r\n");
		if (p)
			*p = '\0';
		l = strspn (buffer, "ABCDEFGHIJKLMNOPQRSTUVWXYZ");
		if (!l || buffer[l] == '\0')
			crcp = NULL;
		else
		{
			crcp = buffer + l;
			crc = strtol (crcp, NULL, 10);
			*crcp = '\0';
		}
		inf = ta_find_infofile (buffer);
		if (!inf)
		{
			ta_do_log (":\"%s\"\r\n", buffer);
			ta_do_log_error ("unbekannter Informationstext");
			continue;
		}
		if (inf->level > level())
		{
			ta_do_log (":\"%s\"\r\n", buffer);
			ta_do_log_error ("nicht fr Sie!");
			continue;
		}
		if (inf->mit_crc)
		{
			/* Das Infofile braucht eine CRC */
			if (!crcp)
			{
				ta_do_log (":\"%s\"\r\n", buffer);
				ta_do_log (":!braucht CRC, -1 eingesetzt\r\n");
				crc = -1;
			}
		}
		/* 
		 * So, eine CRC haben wir also. Nun Fallunterscheidung. 
		 */
		switch (inf->typ)
		{
		case ITYP_NET:
		case ITYP_LOKAL:
			if (inf->mit_crc && inf->crc == crc)
				ta_do_log (":$%s=%ld (CRC unverndert)\r\n", buffer, crc);
			else
			{
				char inname[PATH_MAX];

				ta_info_filename (inf, inname);
				ta_copyappend_infofile (inf, inname, out, inf->crc,
										len);
			}
			break;
		case ITYP_PROC:
			{
				char inname[PATH_MAX];
				long new_crc;

				sprintf (inname, "%sinfofile.tmp", tom_tmp_dir);
				/* Infofile erzeugen */
				new_crc = inf->fn (inname, inf->textpara, inf->numpara);
				if (new_crc==-1)
				{
					ta_do_log (":?%s nicht verfgbar\r\n", inf->name);
				}
				else if (inf->mit_crc && new_crc == crc)
					ta_do_log (":$%s=%ld (CRC unverndert)\r\n",
							   inf->name, crc);
				else
				{
					ta_copyappend_infofile (inf, inname, out,
											new_crc, len);
				}
				unlink (inname);
			}
			break;
		case ITYP_INTERN:
			{
				char inname[PATH_MAX];
				char cmd[PATH_MAX * 2 + 1024];
				sprintf (inname, "%sinfofile.tmp", tom_tmp_dir);
				if (do_interpret (inf->textpara, cmd,
					  sizeof (cmd) - 20) <= 0
					  || !befehls_umlenkung(inname))
				{
					ta_do_log (":\"%s\"\r\n", buffer);
					ta_do_log_error ("nicht erzeugbar");
				}
				else
				{
					mm_file_t *tmpf;
					WORD oldvtmodeW=vtmodeW;
					vtmodeW=T_ASCII;
					sync_terminal();
					befehl_ausfuehrung(cmd);
					befehls_umlenkung(NULL);
					vtmodeW=oldvtmodeW;
					sync_terminal();
					tmpf=MM_READ_OPEN(inname);
					if (!tmpf)
					{
						ta_do_log (":\"%s\"\r\n", buffer);
						ta_do_log_error ("erzeugt, aber nicht vorhanden (TSNH)");
					}
					else
					{
						char *buf;
						size_t bufsize;
						LONG new_crc=0;
						size_t got;
						if (!(MM_READ_IS_MMAPPED(tmpf)))
						{
							bufsize=1024;
							buf=XMALLOC(bufsize);
						}
						while ((got=MM_READ_GETLINE(tmpf,&buf,&bufsize))!=-1)
							new_crc+=line_checksum_n(buf,got);
						new_crc&=0xffffL;
						if (!(MM_READ_IS_MMAPPED(tmpf)))
							free(buf);
						MM_READ_CLOSE(tmpf);
						if (new_crc==inf->crc)
						{
							ta_do_log (":$%s=%ld (CRC unverndert)\r\n",
									   inf->name, new_crc);
						}
						else
						{
							ta_copyappend_infofile (inf, inname, out,
												new_crc, len);
						}
					}
					unlink(inname);
				}
			}
			break;
		case ITYP_CMD:
			{
				char inname[PATH_MAX];
				char cmd[PATH_MAX * 2 + 1024];
				long new_crc;

				sprintf (inname, "%sinfofile.tmp", tom_tmp_dir);
				if (do_interpret (inf->textpara, cmd,
								  sizeof (cmd) - strlen (inname) - 20) > 0)
				{
					sprintf (cmd + strlen (cmd), " >%s 2>&1", inname);
					new_crc = system (cmd);
					if (inf->mit_crc && new_crc == crc)
						ta_do_log (":$%s=%ld (CRC unverndert)\r\n",
								   inf->name, crc);
					else
					{
						ta_copyappend_infofile (inf, inname, out,
												new_crc, len);
					}
					unlink (inname);
				}
				else
				{
					ta_do_log (":\"%s\"\r\n", buffer);
					ta_do_log_error ("nicht erzeugbar");
				}
			}
		}						/* switch */
	}							/* while */
	fclose (f);
}

void
ta_info_mark_change(void)
{
	if (!init_done)
		ta_info_init ();
	list_changed=TRUE;
}

/****************************************************************************
 * Infofilebestellungen bei anderen Boxen
 ****************************************************************************
 *
 * Leider mssen sie getrennt verwaltet werden, weil es ntzlich wre, 
 * bestimmte Infofiles bei verschiedenen Servern bestellen zu knnen.
 */
xinfofile_t *ta_find_xinfofile(const char *name,USERNR nummer)
{
	xinfofile_t *akt;

	if (!init_done)
		ta_info_init ();
	akt = ta_xinfo_anker;
	while (akt)
	{
		if (!stricmp (name, akt->name)
			&& nummer==akt->server)
			return akt;
		akt = akt->next;
	}
	return NULL;
}

void ta_xinfo_mark_change(void)
{
	if (!init_done)
		ta_info_init ();
	xlist_changed=TRUE;
}

xinfofile_t *
ta_xinfo_get_anker(void)
{
	if (!init_done)
		ta_info_init ();
	return ta_xinfo_anker;
}

