/* ta_info_igx.c : generate IGK/IGL */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
       (might be the wrong person except for offline)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "config.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <syslog.h>
#include <ctype.h>
#include <errno.h>

#include "tomsysdep.h"
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "mmread.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_info.h"

ULONG
ta_info_igx (const char *fname, const char *dummy1, long is_long)
{
	FILE *f;
	WORD i;
	LISTUSER *u;
	char buf[16384];
	char flags[20];
	char zeile[1024];
	ULONG crc = 0;
	ULONG or_status = 0;
	BRETT *b;
	BRETTNR *sortiert;

	ta_lade_installation(FALSE);

	sortiert=b_sortierung();
	if (!sortiert || !*sortiert)
		return 0;

	f = xfopen (fname, BINWRITE);

	for ( ; sortiert && *sortiert; sortiert++)
	{
		int bl;
		i=*sortiert;

		b=br_ind[i].brett;
		if (!b) /* Kann eigentlich nicht sein */
			continue;
		if (!(b->flags & (B_READ | B_WRITE)))
			continue;

		/*
		 * Zugriffsflags ermitteln.
		 */
		if (ta_blist[i]!=GRP_UNBESTELLT)	/* Wenn in installierter Bretterliste */
			buf[0] = '+';	/* hat die Gruppe bestellt */
		else if (B_MITLISTE(b->flags) && b_tst_access (i, user.nummer))
			buf[0] = '-';	/* mitglied, liest die Gruppe aber nicht */
		else
			buf[0] = ' ';
		if (b->grp_chef == user.nummer)
			buf[1] = '*';
		else
			buf[1] = ' ';
		if (b->flags & B_GEHEIM)
		{
			buf[2] = '?';
			bl=3;
		}
		else
			bl=2;

		/*
		 * Soweit der fr IGK und IGL gemeinsame Teil.
		 */
		if (is_long)
		{
			/* 
			 * IGL-Original:
			 *   [zugriff]NAME Status Flags Chef
			 *            mehrere Zeilen Infotext ...
			 * NAME kann extrem lang werden. Hmmm ...
			 * Lsung:
			 *   [zugriff]NAME 
			 *   Status Flags Chef
			 *   Infotext
			 */
			char *p;
			char *pstart;
			char *wp;
			const char *chef;
			u = ub_find_usernummer (b->grp_chef);
			memcpy(zeile,buf,bl);
			strcpy(zeile+bl,b->name);
			crc += line_checksum (zeile);
			putc(':',f);
			fputs(zeile,f);

			libquark_litoa(b->lesen_level,flags);
			libquark_litoa(b->schreib_level,flags+1);
			flags[2]=' ';
			wp=flags+3;
			or_status |= b->flags;
			if (b->flags & B_ANONYM)
				*wp++='A';
			if (b->flags & B_GEHEIM)
				*wp++='G';
			if (b->flags & B_MITGLIED)
				*wp++='M';
			if (b->flags & B_ZAHLER)
				*wp++='$';
			if (b->flags & B_PASSWORT)
				*wp++='P';
			if (b->flags & B_ZWANG)
				*wp++='Z';
			if (b->flags & B_DEFAULT)
				*wp++='D';
			if (b->flags & B_MODERIERT)
				*wp++='O';
			if (b->flags & B_ACL)
				*wp++='C';
			if (b->flags & B_LESMITGLIED)
				*wp++='L';
			if (b->flags & B_PAYWRITE)
				*wp++='Y';
			*wp=0;
			fputs(LINEEND ":  ",f);
			if (b->netzstatus)
				fputs("[vernetzt] ",f);
			else
				fputs("[lokal] ",f);
			fputs(flags,f);
			fputs(" Chef: ",f);
			chef=((u && u->name) ? u->name : "keiner eingetragen");
			fputs(chef,f);
			crc += line_checksum (flags);
			crc += line_checksum (chef);

			if (b->userinfo)
			{
				fputs(LINEEND ":  ",f);
				fputs(b->userinfo,f);
			}
			if (b->lokalinfo)
			{
				pstart = b->lokalinfo;
				while (pstart && *pstart)
				{
					p = strpbrk (pstart, "\r\n");
					if (p)
					{
						*p++ = '\0';
						if (*p=='\n')
							p++;
					}
					fputs(LINEEND ":  ",f);
					fputs(pstart,f);
					crc += line_checksum (pstart);
					pstart=p;
				}
			}
			/* Leerzeile */
			fputs(LINEEND ":" LINEEND,f);
		}
		else
			/* !is_long */
		{
			size_t sl=strlen(b->name);
			/* IGK. Wegen langer Gruppennamen auesserst primitiv */
			memcpy(zeile,buf,bl);
			zeile[bl++]=' ';
			if (sl>=10)
			{
				strcpy(zeile+bl,b->name);
				bl+=sl;
			}
			else
			{
				int j;
				strcpy (zeile+bl, b->name);
				bl+=sl;
				for (j=sl;j<=10;j++)
				{
					zeile[bl++]=' ';
				}
				zeile[bl]=0;
			}
			if (b->userinfo)
			{
				zeile[bl++]=' ';
				strcpy(zeile+bl,b->userinfo);
			}
			crc += line_checksum (zeile);
			fputc(':',f);
			fputs(zeile,f);
			fputs(LINEEND,f);
		}
	}							/* for */

	fputs (":\r\n",f);
	fputs (":     +   Diese Gruppe lesen Sie automatisch.\r\n",f);
	fputs (":     ?   Diese Gruppe ist geheim.\r\n",f);
	fputs (":     *   In dieser Gruppe sind Sie Chef.\r\n",f);
	fputs (":     -   Sie sind Mitglied, lesen diese Gruppe aber nicht.\r\n",f);
	if (is_long)
	{
		fputs (":\r\n",f);
		if (or_status & B_ANONYM)
			fputs(":     A   Anonym: Ohne Absender" LINEEND,f);
		if (or_status & B_GEHEIM)
			fputs(":     G   Geheim: Nur fr Mitglieder sicht- und lesbar" 
				LINEEND,f);
		if (or_status & B_MITGLIED)
			fputs(":     M   Mitgliedschaft: Schreiben nur fr Mitglieder"
				 LINEEND,f);
		if (or_status & B_ZAHLER)
			fputs(":     $   Nur fr Zahler lesbar" LINEEND,f);
		if (or_status & B_PASSWORT)
			fputs(":     P   Passwortzwang (nur online nutzbar)" LINEEND,f);
		if (or_status & B_ZWANG)
			fputs(":     Z   Pflichtgruppe (nicht abbestellbar)" LINEEND,f);
		if (or_status & B_DEFAULT)
			fputs(":     D   Defaultgruppe" LINEEND,f);
		if (or_status & B_MODERIERT)
			fputs(":     O   Moderierte Gruppe" LINEEND,f);
		if (or_status & B_ACL)
			fputs(":     C   Zugriffskontrolliste" LINEEND,f);
		if (or_status & B_LESMITGLIED)
			fputs(":     L   Mitgliedschaft: Lesen nur fr Mitglieder" LINEEND,f);
		if (or_status & B_PAYWRITE)
			fputs(":     Y   Schreiben nur fr Zahler" LINEEND,f);
	}
	fclose (f);
	return crc & 0x7FFF;

}								/* ta_info_igx */

