/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#ifndef TA_INTERN_H
#define TA_INTERN_H
#include "routing.h"
#include "box_conf.h"
#include "msg.h"

#define FOROFFLINE_F "4offline.txt"
#define NOEXPID_LST_F "_noexpid.lst"
#define LATER_CMD_F "_later.cmd"
#define TAUSCH_LOG_F "_tausch.log"
#define TAUSCH_BRETT_F "_tbrett.lst"
#define KOMMANDO_OUT_F "_kommandos.out"
#define INCOMING_DIR "dir"
#define PASSINFO_23 "passinfo.23"
#define PASSINFO_34 "passinfo.34"
#define SAUGCNTRL "saugcntrl"

/* Tauschinstallation des Users */
typedef struct
{
	USERNR usernr;				/* fr wen ist die geladene Installtion, 0 nobody */
	/*
	 * Aus der Tauschinstallation des Users.
	 */
	time_t last_mailtausch;		/* letzter Tausch, PM */
	time_t last_newstausch;		/* letzter Tausch, OM */
	time_t last_ren;			/* letztes Mitschicken von REN */
	time_t last_status;			/* letztes Mitschicken von PSA/PSN */
	BOOLEAN umlaut;				/* Umlautwandlung durchfhren? */
	WORD umbruch;				/* Umbruch durchfhren */
	BOOLEAN only_one_group;     /* Keine Crosspostings */
	BOOLEAN quarkoutfile;       /* Soll Quarkoutfile benutzt werden? */
	BOOLEAN async_export;       /* Soll async exportiert werden? */
	ULONG   async_max_spool;    /* 0==box_conf, sonst Bytes */
	char *pathexcludes;
		/*
		 * Maximale Nachrichtenlnge. 
		 * 0  == no limit.
		 */
	ULONG maxlen;

	/* 
	 * changed mu gesetzt werden, wenn Bretter oder obige Daten 
	 * gendert wurden, sonst wird nichts weggeschrieben 
	 */
	BOOLEAN changed;
	/*
	 * weitere variable Daten
	 */
	BOOLEAN first_cmd_block;	/* vor dem ersten CMD-Block? */
	int out_block_counter;
}
TAUSCH_OPTS;

extern TAUSCH_OPTS ta_opts;

typedef struct
{
	/*
	 * Onlineoptionen
	 */
	BOOLEAN echo_news;			/* Echo fr News? */
	BOOLEAN echo_mail;			/* Echo fr Mail */
	enum
	{
		TA_KEIN_STATUS,
		TA_STATUS_ALLE,
		TA_STATUS_NEUE
	}
	status;

	ULONG maxbytes;
	BOOLEAN send_news;			/* ON -> News senden */
	BOOLEAN send_mail;			/* PN -> Mail senden */
	BOOLEAN send_crosspostings;	/* Frontend will Crosspostings */
	BOOLEAN send_ren;			/* REN mitschicken? */
	BOOLEAN send_pe;            /* PE: Eigene PMs echoen */
	BOOLEAN send_pa;            /* PA: Alle PMs noch mal schicken */
	BOOLEAN send_star;          /* TS: Sternzeile senden */

	char *pkenn;				/* Kennung des Packers. NULL -> ungepackt. */
	char *pend;					/* Endung des Packers. */

	char protokoll;
}
TA_PASSINFO;

extern TA_PASSINFO ta_passinfo;
void ta_init_passinfo (TA_PASSINFO * i);

#define TA_OPT_HEADERLEN 256

typedef enum bestell_t
{
	GRP_UNBESTELLT,
	GRP_BESTELLT,
	GRP_IHAVE
}
bestell_t;
extern bestell_t *ta_blist;

/**************************************************************************
 *
 * Dupecheck
 *
 **************************************************************************
 */

typedef struct dupecheckstruct
{
	char *userid;
	char *realid;
	time_t zeit;
	struct dupecheckstruct *next;
}
DUSTR;

/* ta_in.c */
extern long mnk_sys;

/* ta_trans.c */
BOOLEAN ta_infile_empfangen (char *infilename);
BOOLEAN ta_infile_auspacken (char *archivname, char *infilename);

void ta_write_message(message_t *head, FILE *f,char *comment);

/* ta_artcheck.c */
BOOLEAN ta_artcheck(message_t *head, r_adr_t **, BOOLEAN *first_zuordnung, 
	const char *path_entry, BOOLEAN *need_bounce);

/****************************************************************************
 * ta_hilf.c - Hilffunktionen
 ****************************************************************************/
#ifndef __GNUC__
extern ULONG line_checksum (const char *s);
extern ULONG line_checksum_n (const char *s,size_t n);
#else
static __inline__ ULONG
line_checksum (const char *s)
{
	size_t cnt = 0;
	ULONG csum = 0;

	while (*s)
		csum += *(s++) * ++cnt;
	return csum;
}
static __inline__ ULONG
line_checksum_n (const char *s,size_t n)
{
	size_t cnt = 0;
	ULONG csum = 0;

	while (n--)
		csum += *(s++) * ++cnt;
	return csum;
}
#endif

void ta_umlautwandlung (BOOLEAN out_modeB, const char *to_change);
void ta_umbruch (int spalten, const char *infile, const char *outfile);

BOOLEAN ta_check_readperm (BRETTNR bnr, const char **err);

/* ta_dupe.c */
void ta_userdupecheck_end (void);
void ta_dupe_on (void);
void ta_dupe_off (void);
BOOLEAN ta_userdupecheck_init (void);
DUSTR *ta_userdupecheck (char *id);
void ta_userdupecheck_add_info (char *userid, char *realid, time_t zeit);

/* ta_inst.c */
void ta_do_inst (void);

/* ta_gcmd.c */
void ta_work_gcmd (void);

/* ta_log.c */
void ta_open_log (void);
void ta_do_log_error (const char *);
void ta_do_log_end (void);
void ta_msg_log (const char *,...);
void ta_msg_log_end (void);

/* ta_spec.c */
void ta_work_special (message_t *head);
void ta_saugblock(message_t *msg);

void ta_cmd_read_tcl(void);
void ta_cmd_tcl(char *cmd, const char *fname, LONG dummy2);

/* ta_stat.c */
void ta_work_status (message_t *head);

/* V wegen moderierten Gruppen optional */
/* V und S erlaubt, MAUS-kompatibilitt - extraprfung in ta_artcheck.c */
/* X ist erlaubt fr User, allerdings werden die Flags sehr genau geprft */
#define USER_KOMBI_MAIL_ZWANG "#AW:"
#define USER_KOMBI_MAIL_OPT   "VEKRT-MSVX"
#define USER_KOMBI_NEWS_ZWANG "#GW:"
#define USER_KOMBI_NEWS_OPT   "DERTF-MSVX"

#define GATE_KOMBI_MAIL_ZWANG "#VWEA:"
#define GATE_KOMBI_MAIL_OPT   "-KBIRONSTpYMX"
#define GATE_KOMBI_NEWS_ZWANG "#VEGW:"
#define GATE_KOMBI_NEWS_OPT   "-IRONDSTFpYMX"

#define BOX_KOMBI_MAIL_ZWANG "#VAWEB:"
#define BOX_KOMBI_MAIL_OPT   ">-IRONSTpYMX"
#define BOX_KOMBI_NEWS_ZWANG "#EGW:"
#define BOX_KOMBI_NEWS_OPT   "VA->IRONDTFSpYMX"

/* ta_*cmd.h: Alles was nicht ein Kommando implementiert */
/*
 * ta_cmd_work fhrt die Kommandos aus bzw. speichert sie in die passenden
 * Dateien (die offen gehalten werden, ta_cmd_end mu also aufgerufen werden,
 * wenn das letzte Kommando geschrieben wurde).
 */
void ta_cmd_work(char *s);

/*
 * Schliet die offene Datei
 */
void ta_cmd_end(void);
void ta_icmd_end(void);

/*
 * Liest die Daten des Users "username", trgt seine Nummer in *unr ein
 * und setzt auf Wunsch einen Lock.
 * Bei Fehlschlag wird eine ?-Zeile ins LOG geschrieben.
 */
BOOLEAN ta_cmd_get_user(const char *username, USERNR *unr, USER *u, BOOLEAN lock);

/*
 * Analog fr Gruppen.
 */
BOOLEAN ta_cmd_get_group(const char *grpname, BRETTNR *bnr, BOOLEAN lock);

void ta_work_gcmd(void);
void ta_work_icmd(long *aktlen,FILE *);


ULONG ta_info_itk(const char *, const char *, long);
ULONG ta_info_itb (const char *fname, const char *dummy1, long is_its);
ULONG ta_info_iti (const char *fname, const char *dummy1, long dummy2);
ULONG ta_info_igx (const char *fname, const char *dummy1, long is_long);
ULONG ta_info_itg (const char *fname, const char *dummy1, long is_long);



FILE * ta_open_out_block(const char *cmd);

#if defined(__GNUC__)
static __inline__ void
terminate_with_crlf(char *str)
{
	char *p;
	p=strchr(str,'\n');
	if (p)
	{
		if (p==str || p[-1]!='\r')
		{	
			*p++='\r';
			*p++='\n';
			*p=0;
		}
	}
	else
	{
		p=strlen(str)+p;
		*p++='\r';
		*p++='\n';
		*p=0;
	}
};
static __inline__ void
strip_crlf(char *str)
{
	char *p;
	p=strchr(str,'\n');
	if (p)
	{
		if (p==str || p[-1]!='\r')
			*p=0;
		else
			p[-1]=0;
	}
}
#else
extern void terminate_with_crlf(char *p);
extern void strip_crlf(char *p);

#endif



#endif
