/* ta_out.c: Outfileerzeugung, toplevelfunktionen 

    Copyright (C) 1994 - 1998 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@ohse.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <portab.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <limits.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
#include <errno.h>

#ifdef OFFLINE2
#include "offline/inc/version.h"
#ifndef OFFLINE_VERSION
#include "offline/inc/struktur.h"	/* wg. Distributionsflags */
#endif
#include "offline/inc/message.h"	/* wg. V_LOKAL */
#include "offline/inc/mdb_global.h"
#else
#include "offline/version.h"
#include "offline/message.h"	/* wg. V_LOKAL */
#include "offline/mdb_global.h"
#endif
#include "prototyp.h"
#include "modem.h"
#include "bin_msg.h"
#include "bin_out.h"
#include "brett.h"
#include "msg.h"
#include "user.h"
#include "str_ulong.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"
#include "ta_info.h"

const char *tau_ver = "0.5.4";

#ifndef LINEEND
#define LINEEND "\r\n"
#endif

static BOOLEAN ta_do_export (const char *fname, BOOLEAN *status_done);
void ta_exportiere_status(FILE *f, const char *fname, BOOLEAN *status_done);
extern const char *tausch_processed_file;


/*
   Syntax: 
   - TAUSCH BRETT *
   - TAUSCH BRETT nr_von
   - TAUSCH BRETT nr_von nr_bis

   05.12.92 Frank
   17.01.93 Frank, auf Datenbank vorlufig angepat
   11.05.94 Frank, auf neues Packerhandling vorlufig angepat
 */
void tausch_brett (const char *para)
{
	BOOLEAN ok = FALSE;
	ULONG nr_von = 0;
	ULONG nr_bis = (ULONG) -1;
	char outfile[PATH_MAX];
	char exp_name[PATH_MAX];
	FILE *exp_f;

	/* Parameter auswerten */
	if (!strcmp (para, "*"))
	{
		ok = TRUE;
	}
	else
	{
		nr_von=1;
		if (*para != '-' && atoi (para) > 0)
		{
			nr_von = atol (para);
			nr_bis = (ULONG) -1;
			while (isdigit (*para) || isspace (*para))
				para++;
			ok = TRUE;
		}
		if (*para == '-')
		{
			para++;
			nr_bis = atol (para);
			if (nr_bis > 1 && nr_bis > nr_von)
				ok = TRUE;
		}
	}

	if (!ok)
	{
		mboutput (" **** Syntax: T BRETT * oder von-bis" "\r\n");
		mboutput ("" "\r\n");
		mboutput ("       von  : Nummer der ersten Nachricht" "\r\n");
		mboutput ("       bis  : Nummer der letzten Nachricht" "\r\n");
		mboutput ("" "\r\n");
		return;
	}

	mboutput ("" "\r\n");

	sprintf (outfile, "%s%s", tom_utmp_dir, "outfile.txt");
	unlink (outfile);

	ta_lade_installation(FALSE);

	sprintf (exp_name, "%s%s", tom_tmp_dir, "export.lst");
	exp_f = xfopen (exp_name, "w");

	if (!brettnrW && postfach_art==TEXT_BRETT)
		fprintf (exp_f, "*%s", user.name);
	else if (!brettnrW)
		fprintf (exp_f, "Programmfach");
	else
		fputs (akt_brett.name, exp_f);
	if (nr_von || nr_bis != (ULONG) -1)
		fprintf(exp_f,",%lu-%lu",nr_von,nr_bis);
	fclose (exp_f);

	if ((brettnrW==0 && postfach_art==BINAER_BRETT) 
		|| akt_brett.art == BINAER_BRETT)
		ok = bin_export(exp_name,0,outfile);
	else
	{
#ifdef __GNUC__
		EXPORT export_conf=
		{
			TRUE,                   /* Boxmodus */
			V_LOKAL,                /* Distribution */
			(time_t) 0,             /* ab wann */
			0,                      /* maximale Outfilegrsse */
			ta_opts.only_one_group, /* Crosspostings? */
			NULL,                   /* site */
			NULL,         /* Datei mit Liste nicht zu exportierender msg. */
			FALSE,                  /* keine Sternzeilen */
			0                       /* Kein Status */
		};
#else
		EXPORT export_conf;
		export_conf.box=TRUE;
		export_conf.dist=V_LOKAL;
		export_conf.zeit=(time_t) 0;
		export_conf.maximum = 0;
		export_conf.one_group = ta_opts.only_one_group;
		export_conf.site = NULL;
		export_conf.noidfname = NULL;
		export_conf.stern = FALSE;
		export_conf.status_mode = 0;
#endif
		ok = mdb_export (
			exp_name,	/* Liste zu exportierender Bretter */
			outfile,		/* in welche Datei */
			&export_conf );
	}

	if (ok && logout_modus != 1)
	{
		FILE *f;

		f=xfopen(outfile,BINAPPEND);
		fprintf(f,"#LOG\r\n#\r\n");
		fclose(f);

		if (ta_opts.umlaut || ta_opts.umbruch)
			ta_umlautwandlung (TRUE, outfile);
#if 0
		if (!packerauswahl (PACK_TAUSCH, endung))
			return;

		mboutput ("" "\r\n");
		if (*endung)
			ok = outfile_einpacken (endung, outfile);
#endif
		if (ok)
			ta_sende_outfile (outfile,0);
	}
	mboutput ("" "\r\n");

} /* tausch_brett */



/*
   04.02.93 Frank
   29.01.94 Frank, Bugfix: Pfad/Dateiname war inkonsistent
   11.05.94 Frank, Versenden mehrer Dateien auf einmal vorbereitet
   ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
   Was soll das? Doch nicht mit *.*!
 */
BOOLEAN
ta_sende_outfile (const char *filename, char protokoll)
{
	BOOLEAN ok = TRUE;

	ok = transfer_send (&protokoll, filename) == 0;

	mboutput ("" "\r\n");

	return ok;

} /* outfile_senden */

void
ta_exportiere_status(FILE *f, const char *fname, BOOLEAN *status_done)
{
	FILE *rf=NULL;
	time_t seit=0;

	if (*status_done)
		return;
	if (ta_passinfo.status==TA_KEIN_STATUS)
		return;

	if (ta_passinfo.status!=TA_STATUS_ALLE)
		seit=ta_opts.last_status;

	if (f)
		rf=f;
	else
		rf=xfopen(fname,BINAPPEND);
	*status_done=r_st_export(rf,seit);
	if (!f)
		fclose(rf);
}


/*
   komplettes Outfile erzeugen
   inkl. Umlautwandlung usw.

   18.01.93 Frank
   30.11.93 Uwe, Anpassung an pers- und allg.-Tauschdatum
   7.12.93 Uwe, Parameter, Bugfix.
   09.08.94 Frank, Paging whrend mdb_export ausschalten
 */
static BOOLEAN
ta_do_export (const char *fname, BOOLEAN *status_done)
{
	char exp_dat[PATH_MAX];		/* Exportliste fr die DB */
	char tausch_brett_dat[PATH_MAX];		/* Ausnahmenliste fr die DB */
	char noexpid_fname[PATH_MAX];	/* Liste mit nicht zu exportierenden IDs */
	const char *path_entry=NULL;
	ULONG net_pm=0;
	ULONG dist;
	BOOLEAN ok;
	int old = user.mehr;
	FILE *tb_f;

	sprintf (exp_dat, "%s%s", tom_tmp_dir, "export.lst");
	sprintf (noexpid_fname, "%s" NOEXPID_LST_F , tom_tmp_dir);
	sprintf (tausch_brett_dat, "%s" TAUSCH_BRETT_F , tom_tmp_dir);

	if (ta_passinfo.send_mail && ta_passinfo.send_news)
	{
		if (ta_opts.last_mailtausch==ta_opts.last_newstausch)
		{
			const char *dz;
			dz=datum_zeit_string_y2k(&ta_opts.last_mailtausch);
			mbprintf (" Sammle neue Nachrichten seit %s.\r\n\n",dz);
		}
		else
		{
			const char *dz;
			dz=datum_zeit_string_y2k(&ta_opts.last_mailtausch);
			mbprintf (" Sammle neue persnliche Nachrichten seit %s," "\r\n",dz);
			dz=datum_zeit_string_y2k(&ta_opts.last_newstausch);
			mbprintf (" und neue ffentliche Nachrichten seit %s\r\n\n",dz);
		}
	}
	else if (ta_passinfo.send_mail || ta_passinfo.send_news)
	{
		const char *dz;
		dz=datum_zeit_string_y2k(
			ta_passinfo.send_mail ? &ta_opts.last_mailtausch :
				&ta_opts.last_newstausch);
		mbprintf (" Sammle neue %s Nachrichten seit %s.\r\n\n",
			ta_passinfo.send_mail ? "persnliche" : "ffentliche", dz);
	}
	ta_exportiere_status(NULL,fname,status_done);

	/* Exportliste fr Textbretter schreiben */
	ta_schreibe_exportliste (exp_dat,FALSE);
	if (!fileexist (exp_dat))
		return FALSE;

	/* "Tausch Brett" Emulation */
	tb_f = fopen (tausch_brett_dat, BINREAD);
	if (tb_f)
	{  
		FILE *exp_f;
		char buf[1024];
		exp_f = xfopen (exp_dat, BINAPPEND);
		while (fgets (buf, sizeof(buf), tb_f))
		{
			char *p;
			p=strrchr(buf,'\n');
			if (p)
				*p=0;
			p=strchr(buf,',');
			if (!p)
				continue;
			*p++=0;
			if (*p=='*')
				fprintf(exp_f,"%s,0\r\n",buf);
			else if (strchr(p,'-')) {
				fprintf(exp_f,"%s,%s\r\n",buf,p);
			} else {
				char buf2[32];
				time_t t=(time_t) strtoul(p,NULL,10);
				struct tm *tm;
				tm=localtime(&t);
				if (!tm)
					continue;
				strftime(buf2,sizeof(buf2)-1,"%d.%m.%y,%H:%M:%S",tm);
				fprintf(exp_f,"%s,%s\r\n",buf,buf2);
			}
		}
		fclose (tb_f);
		fclose (exp_f);
		unlink (tausch_brett_dat);
	}

	if (user.typ!=UTYP_USER)
	{
		path_entry=get_path_entry(user.nummer);
		net_pm=user.nummer;
	}
	else
	{
		path_entry=NULL;
		net_pm=0;
	}

	dist=V_MAUSNET;
	if (user.typ==UTYP_USER)
		dist=V_LOKAL;

	user.mehr = 0;
	
	ta_exportiere_status(NULL,fname,status_done);

	{
#ifdef __GNUC__
		EXPORT export_conf=
			{
				TRUE,         /* Boxmodus */
				dist,         /* Distribution */
				ta_opts.last_newstausch,   /* ab wann */
				0,            /* maximale Outfilegrsse */
				ta_opts.only_one_group, /* Crosspostings? */
				path_entry,   /* site */
					/* Datei mit Liste nicht zu exportierender msg. */
				(ta_passinfo.echo_news) ? NULL : noexpid_fname,
				FALSE,        /* Sternzeile */
				0 /* Status */
			};
#else
		EXPORT export_conf;
		export_conf.box=TRUE;
		export_conf.dist=dist;
		export_conf.zeit=ta_opts.last_newstausch;
		export_conf.maximum = 0;
		export_conf.one_group = ta_opts.only_one_group;
		export_conf.site = path_entry;
		export_conf.noidfname = ta_passinfo.echo_news ? NULL : noexpid_fname;
		export_conf.stern=FALSE;
		export_conf.status_mode=0;
#endif

		ok = mdb_export (
			exp_dat,	/* Liste zu exportierender Bretter */
			fname,		/* in welche Datei */
			&export_conf );
	}

	user.mehr = old;

	if (ok) 
	{
		/* Exportliste fr Binrbretter schreiben */
		ta_schreibe_exportliste (exp_dat,TRUE);
		if (!fileexist (exp_dat))
			return FALSE;

		ok = bin_export(exp_dat,ta_opts.last_newstausch,fname);
	}

	return ok;

}	/* erstelle_outfile */


static void
ta_mk_head (FILE * out_f)
{
	char tmp[128];
	time_t zeit;

	fputs ("#HEAD\r\n",out_f);
	fprintf (out_f, ":V%s\r\n", VERSION);
	fprintf (out_f, ":T%s\r\n", tau_ver);
	fputs (":BQuark\r\n",out_f);
	fprintf (out_f, ":C\r\n");
	zeit = time (NULL);
	strftime (tmp, 13, "%Y%m%d%H%M", localtime (&zeit));
	fprintf (out_f, ":D%s\r\n", tmp);
	fprintf (out_f, ":I%s\r\n", MYDOMAINPART);
	fprintf (out_f, ":W%lu\r\n", (ULONG) ta_opts.umbruch);
	if (ta_opts.only_one_group)
		fputs (":SUGR\r\n",out_f);
	else
		fputs (":SUR\r\n",out_f);
	fprintf (out_f, ":U%s\r\n", user.name);
	fprintf (out_f, ":L%lu\r\n", (ULONG) user.bs_zeilen);

}

BOOLEAN
ta_erstelle_outfile (FILE *out_f, const char *outfilename)
{
	/* 
	   Verfahren:
	   Whrend oder kurz nach dem Infilehandling wurde ein
	   Logfile (log_dat).
	   ta_do_export wird aufgerufen, um ein Outfile.txt zu
	   erzeugen (mit den Nachrichten drin).
	   Danach werden die Infofiles und das Logfile in dieser
	   Reihenfolge an das Outfile.txt angehngt, sie selbst
	   werden anschlieend gelscht.
	 */
	char inf_dat[PATH_MAX];		/* Falls vorhanden: Infofiles */
	char log_dat[PATH_MAX];		/* Logfile, sollte vorhanden sein */
	char *textS;
	char *p;
	long len;
	FILE *inf_f;
	BOOLEAN status_done=(ta_passinfo.status==TA_KEIN_STATUS);
	const char *postprozessor=NULL;

#define TEXTSLEN 32768
	textS = (char *) XMALLOC (TEXTSLEN);

	/* Falls von oben keine offene Datei bergeben wurde */
	if (!out_f)
		out_f = xfopen (outfilename, BINWRITE);


	if (user.typ!=UTYP_BOX)
		ta_mk_head (out_f);

	ta_exportiere_status(out_f,NULL,&status_done);

	/* REN-Block */
	if (ta_passinfo.send_ren)
	{
		sprintf(inf_dat,"%srename.dat",box_conf.var_dir);
		inf_f=fopen(inf_dat,BINREAD);
		if (inf_f)
		{
			fputs("#REN"LINEEND,out_f);
			while (fgets(textS,TEXTSLEN,inf_f))
			{
				time_t t;
				t=(time_t) strtoul(textS,NULL,10);
				if (t >= ta_opts.last_ren)
				{
					char *oldname;
					char *newname;
					char buf[20];
					p=strchr(textS,'@');
					if (!p)
						continue;
					oldname=p+1;
					p=strchr(oldname,'@');
					if (!p)
						continue;
					newname=p+1;
					*p=0;
					strtrim(oldname);
					strtrim(newname);
					fprintf(out_f,":O%s"LINEEND,oldname);
					fprintf(out_f,":N%s"LINEEND,newname);
					time_t_to_mtzeit(&t,buf);
					fprintf(out_f,":D%s"LINEEND,buf);
				}
			}
			fclose(inf_f);
		}
		ta_opts.last_ren=time(NULL);
		ta_opts.changed=TRUE;
	}

	ta_exportiere_status(out_f,NULL,&status_done);

	if (ta_passinfo.send_pe)
	{
		/*
		 * PE-Kommando "persnliche eigene" realisieren durch kopieren
		 * der Daten aus dem 4offline.txt in das Outfile.
		 * kommt mit "*"-Zeile klar, aber auch ohne sie aus.
		 */
/* ### diesen Code irgendwann mal auf die Sternzeile anpassen! */
		char foroffline[PATH_MAX];
		FILE *f;
		sprintf(foroffline, "%s" FOROFFLINE_F , tom_tmp_dir);
		f=fopen(foroffline, BINREAD);
		if (f)
		{
			BOOLEAN kopieren=FALSE;
			ULONG linenr=0;
			BOOLEAN nur_stern=FALSE;
			while (fgets(textS,TEXTSLEN,f))
			{
				char *at;

				if (linenr++==0 && *textS=='*')
					nur_stern=TRUE;
				switch(*textS)
				{
				case '*':
					if (textS[1]=='P')
						kopieren=TRUE;
					else
						kopieren=FALSE;
					break;
				case '#':
					if (nur_stern)
						break;
					if ((at=strchr(textS,'@'))!=NULL
						&& (p=strchr(textS,'p'))!=NULL
						&& p<at)
						kopieren=TRUE;
					else
						kopieren=FALSE;
					break;
				}
				if (kopieren)
					fputs(textS,f);
			}
			unlink(foroffline);
			fclose(f);
		}
	}

	ta_exportiere_status(out_f,NULL,&status_done);

	fclose (out_f);

	ta_open_log ();
	mbprintf("Nachrichtenexport" "\r\n");
	ta_do_export (outfilename, &status_done);

	ta_exportiere_status(NULL,outfilename,&status_done);

	mbprintf("Infofiles bereitstellen" "\r\n");
	out_f = xfopen (outfilename, BIN_R_APPEND);
	len = filesize (out_f);

	ta_work_icmd (&len, out_f);
	ta_do_log_end ();

	ta_exportiere_status(out_f,NULL,&status_done);

	/* Die Ausgaben der Kommandos anhngen */
	sprintf (inf_dat, "%s" KOMMANDO_OUT_F , tom_tmp_dir );
	inf_f = fopen (inf_dat, BINREAD);
	if (inf_f) {  
		mbprintf("Kommandoausgaben anhngen" "\r\n");
		while (fgets (textS, TEXTSLEN, inf_f))
		{
			terminate_with_crlf(textS);
			fputs (textS,out_f);
		}
		fclose (inf_f);
		unlink (inf_dat);
	}

	ta_exportiere_status(out_f,NULL,&status_done);

	/* von mauscon gespooltes Zeug anhngen */
	if (box_conf.user_dir) {
		DIR *dp;
		uostr_t dir_name={0};
		uostr_t old_name={0};
		char canon_name[UDAT_NAME_LEN*2];

		uostr_xdup_cstr(&dir_name,box_conf.user_dir);
		ub_canonical_underscore(user.name,canon_name);
		uostr_xadd_cstrmulti(&dir_name,canon_name,"/outgoing",NULL);
		uostr_x0(&dir_name);

		dp=opendir(dir_name.data);
		if (dp)
		{
			struct dirent * de;
			int pos;
			uostr_cut(&dir_name,-1);
			uostr_xadd_char(&dir_name,'/');
			pos=dir_name.len;
			while ((de=readdir(dp))!=NULL)
			{
				size_t l;
				struct stat st;
				/* aus dem Verzeichnis nur die mit "mt" beginnenden Dateien
				 * mitnehmen 
				 */
				if (de->d_name[0]!='m' || de->d_name[1]!='t')
					continue;
				l=strlen(de->d_name);
				if (l<4)
					continue;
				if (0==strcmp(de->d_name+l-4,".old"))
					continue;
				uostr_xadd_mem(&dir_name,de->d_name,l+1); /* \0 */
				if (0!=stat(dir_name.data,&st))
					continue;
				if (!S_ISREG(st.st_mode))
					continue;
				mbprintf("%s anhngen" "\r\n",de->d_name);
				inf_f = xfopen (dir_name.data, BINREAD);
				while (fgets (textS, TEXTSLEN, inf_f))
				{
					terminate_with_crlf(textS+1);
					fputs(textS,out_f);
				}
				if (0!=fclose (inf_f)) {
					mbprintf("Fehler beim Lesen der Spooldatei %s: %s\r\n",
						de->d_name, strerror(errno));
					continue;
				}
				uostr_xdup_uostr(&old_name,&dir_name);
				uostr_cut(&old_name,-1);
				uostr_xadd_cstr(&old_name,".old");
				uostr_x0(&old_name);
				rename(dir_name.data,old_name.data);
				uostr_cut(&dir_name,pos);
			}
			closedir(dp);
			uostr_freedata(&dir_name);
			uostr_freedata(&old_name);
		}
	}


	/* last chance */
	ta_exportiere_status(out_f,NULL,&status_done);
	if (!status_done)
	{
		syslog(LOG_WARNING,"Konnte Statusnachrichten nicht exportieren");
	}

	/* 
	 * nun in _tausch.log als #LOG anhngen, fr User und Gates
	 */

	sprintf (log_dat, "%s" TAUSCH_LOG_F , tom_tmp_dir);
	if (user.typ!=UTYP_BOX)
	{
		mbprintf("Logfile anhngen" "\r\n");
		inf_f = fopen (log_dat, BINREAD);
		if (inf_f)
		{
			if (!status_done)
				fputs(
					":!Nachrichtenstatus konnte nicht exportiert werden" "\r\n",
					out_f);
			while (fgets (textS, TEXTSLEN, inf_f))
			{
				terminate_with_crlf(textS);
				fputs (textS,out_f);
			}
			fclose (inf_f);
		}
		else
		{
			fprintf (out_f, "#LOG" "\r\n");
			if (!status_done)
				fprintf(out_f,
":!Nachrichtenstatus konnte nicht exportiert werden" "\r\n");
		}
	}
	/* 
	 * und fr Boxen und Gates noch einmal an den entsprechenden User
	 * mailen.
	 */
	if (user.typ!=UTYP_USER 
		&& (inf_f = fopen (log_dat, BINREAD)) != NULL)
	{
		char *empfaenger;
		char *absender=concat("Quark @ ", MYDOMAINPART, NULL);
		char *wegen=concat("Tauschfehlermeldungen an ", user.name, NULL);
		if (user.vertreter && user.vertreter[0])
			empfaenger=strdup(user.vertreter);
		else
			empfaenger=concat("Sysop @ ", MYDOMAINPART, NULL);

		if (absender && empfaenger && wegen)
		{
			char mfname[PATH_MAX];
			char what[256]="";
			BOOLEAN written=FALSE;
			time_t t=time(NULL);
			FILE *f;


			sprintf(mfname,"%sLOG.tmp",tom_tmp_dir);
			f=xfopen(mfname,"w");
			fprintf(f,"Datum/Uhrzeit: %s\n",ctime(&t));
			fprintf(f,"Folgende Probleme traten beim Tausch fr %s auf:\n",
				user.name);
			while (fgets (textS, TEXTSLEN, inf_f))
			{
				strip_crlf(textS);
				if (textS[1]=='#' || textS[1]=='\"')
				{
					strncpy(what,textS+1,sizeof(what)-1);
					what[sizeof(what)-1]='\0';
				}
				else if (textS[1]=='?')
				{
					if (*what)
					{
						fprintf (f, "%s\n", what);
						*what='\0';
					}
					fprintf (f, "%s\n", textS+1); /* ohne Doppelpunkte */
					written=TRUE;
				}
			}
			fclose(f);
			if (written)
			{
				einfachnachricht(TRUE,9,TRUE,mfname,
					'A', empfaenger,
					'V', absender,
					'W', wegen,
					0);
			}
			unlink(mfname);
		}
		fclose(inf_f);
		if (empfaenger)
			free(empfaenger);
		if (absender)
			free(absender);
		if (wegen)
			free(wegen);
	}
	unlink (log_dat);

	/*
	 * .CNF bei Vernderungen mitschicken.
	 */
	if (user.typ==UTYP_BOX)
	{
		DIR *dp;
		char dir_name[PATH_MAX];
		char *slash_p;
		sprintf(dir_name,"%s",box_conf.tausch_dir);
		slash_p=strlen(dir_name)+dir_name-1;
		while (*slash_p=='/')
			*slash_p--=0;

		dp=opendir(dir_name);
		if (dp)
		{
			struct dirent * de;
			while ((de=readdir(dp))!=NULL)
			{
				size_t l=strlen(de->d_name);
				if (l>4 && !strcmp(de->d_name+l-4,".cnf")
					&& !strncasecmp(de->d_name,box_conf.mausnet_kennung,l-4))
				{
					struct stat st;
					char fname[PATH_MAX];
					time_t vglzeit;
					sprintf(fname,"%s%s",box_conf.tausch_dir,de->d_name);
					/* last_mailtausch oder last_newstausch? Das letzte
					 * Datum von beiden ist ausreichend. Wenn eine
					 * Box das TD-Kommando verwendet, geht hier 
					 * etwas schief!
					 */
					if (ta_opts.last_mailtausch>ta_opts.last_newstausch)
						vglzeit=ta_opts.last_mailtausch;
					else
						vglzeit=ta_opts.last_newstausch;
					if (!stat(fname,&st) 
						&& st.st_mtime>vglzeit
						&& S_ISREG(st.st_mode))
					{
						mbprintf("CNF %s anhngen" "\r\n",de->d_name);
						inf_f = xfopen (fname, BINREAD);
						fputs("#CNF\r\n",out_f);
						textS[0]=':';
						while (fgets (textS+1, TEXTSLEN-1, inf_f))
						{
							terminate_with_crlf(textS+1);
							fputs(textS,out_f);
						}
						fclose (inf_f);
					}
				}
			}
			closedir(dp);
		}
	}

	/*
	 * USERNR.once - Gruppenbestellungen etc anlegen.
	 */
	if (user.typ==UTYP_BOX || user.typ==UTYP_GATE)
	{
		char fname[PATH_MAX];
		sprintf(fname,"%s%lu.once",box_conf.tausch_dir,user.nummer);
		inf_f = fopen (fname, BINREAD);
		if (inf_f)
		{
			mbprintf("Einmalkommandos anhngen" "\r\n");
			fputs("#CMD\r\n",out_f);
			textS[0]=':';
			while (fgets (textS+1, TEXTSLEN, inf_f))
			{
				terminate_with_crlf(textS+1);
				if (textS[1]=='\r')
					continue;
				fputs(textS,out_f);
			}
			fclose (inf_f);
			unlink(fname);
		}
	}

	/*
	 * USERNR.cmd anhngen.
	 */
	if (user.typ==UTYP_BOX)
	{
		char fname[PATH_MAX];
		mbprintf("CMD anhngen" "\r\n");
		sprintf(fname,"%s%lu.cmd",box_conf.tausch_dir,user.nummer);
		inf_f = fopen (fname, BINREAD);
		if (inf_f)
		{
			fputs("#CMD\r\n",out_f);
			textS[0]=':';
			while (fgets (textS+1, TEXTSLEN, inf_f))
			{
				terminate_with_crlf(textS+1);
				if (textS[1]=='\r')
					continue;
				fputs(textS,out_f);
			}
			fclose (inf_f);
		}
	}
	/* Infofilebestellungen */
	if (user.typ==UTYP_BOX || user.typ==UTYP_GATE)
	{
		xinfofile_t *x;
		x=ta_xinfo_get_anker();
		mbprintf("Infofiles bestellen" "\r\n");
		fputs("#CMD\r\n",out_f);
		while (x)
		{
			if (x->server==user.nummer)
			{
				putc(':',out_f);
				fputs(x->name,out_f);
				if (x->mit_crc)
					fprintf(out_f," %ld",x->crc);
				fputs("\r\n",out_f);
			}
			x=x->next;
		}
	}

	/* Und abschlieen */
	fputs ("#\r\n",out_f);
	fclose (out_f);

	free (textS);

	if (user.typ==UTYP_USER)
		postprozessor=box_conf.tausch_out_user_postprozessor;
	else if (user.typ==UTYP_USER)
		postprozessor=box_conf.tausch_out_gate_postprozessor;
	else
		postprozessor=box_conf.tausch_out_box_postprozessor;
	if (postprozessor && *postprozessor)
	{
		int error_occured;
		char *buf;
		
		buf=XMALLOC(PATH_MAX * 4);

		tausch_processed_file = outfilename;
		if (!do_interpret (postprozessor, buf, sizeof (buf)))
		{
			mbprintf (
				 " Interpretation des Postprozessorkommandos gescheitert" "\r\n");
			syslog (LOG_ERR,
				 " Interpretation des Postprozessorkommandos gescheitert" "\r\n");
			tausch_processed_file = NULL;
		}
		else
		{
			struct stat st1;
			struct stat st2;

			tausch_processed_file = NULL;
			stat (outfilename, &st1);
			mboutput (" Postprozessor ..." "\r\n");
			error_occured = call_extern (buf,
								 NULL,	/* spezielles Arbeitsverzeichnis */
								 akt_anruf.soft_limit + 600,	/* killzeitpunkt,just for the case */
								 FALSE,		/* ignore_exitcode */
								 FALSE);	/* nicht unter uid des Users */
			if (error_occured)
			{
				mbprintf (" Postprozessor beendete sich mit Exitcode %d" "\r\n",
						  error_occured);
				stat (outfilename, &st2);
				if (st1.st_mtime != st2.st_mtime)
				{
					ta_userdupecheck_end ();
					free(buf);
					return FALSE;
				}
			}
		}
		free(buf);
	}

	if (user.typ!=UTYP_USER && ta_opts.quarkoutfile)
	{
		int error_occured;
		uostr_t buf={0};
		uostr_t buf2={0};
		char numbuf[STR_ULONG];
		str_ulong(numbuf,(ULONG) user.nummer);
		uostr_xdup_cstrmulti(&buf2,box_conf.tausch_dir,numbuf,".qout",NULL);
		uostr_x0(&buf2);
		if (fileexist(buf2.data)) {
			uostr_xadd_uostr(&buf,&buf2); uostr_cut(&buf,-1);
			uostr_xadd_cstrmulti(&buf," ",outfilename,NULL);
			uostr_x0(&buf);
			error_occured = call_extern (buf.data,
								 NULL,	/* spezielles Arbeitsverzeichnis */
								 akt_anruf.soft_limit + 600,	/* killzeitpunkt,just for the case */
								 FALSE,		/* ignore_exitcode */
								 FALSE);	/* nicht unter uid des Users */
			uostr_freedata(&buf);
			uostr_freedata(&buf2);
			if (error_occured)
			{
				mbprintf (" Quarkoutfilescript beendete sich mit Exitcode %d" "\r\n",
						  error_occured);
				syslog (LOG_ERR,"Quarkoutfilescript: Exitcode %d\n", error_occured);
				ta_userdupecheck_end ();
				return FALSE;
			}
		} else {
			/* quarkoutfilekommando nach buf */
			uostr_xdup_cstrmulti(&buf,box_conf.bin_dir,"/quarkoutfile -0 -f U! -P -p ",
				box_conf.log_dir,datum_pro_S(),".quarkoutfile --mysite ",
				box_conf.mausnet_kennung," -l --filter-status NM -i ",
				outfilename," -o ",outfilename,NULL);

			/* gruppennamenskonvertiertungsconfig nach buf2 */
			uostr_xdup_cstrmulti(&buf2,box_conf.tausch_dir,numbuf,".grep",NULL);
			uostr_x0(&buf2);
			if (fileexist(buf2.data))
				uostr_xadd_cstrmulti(&buf," -g ",buf2,NULL);
			uostr_x0(&buf);

			mboutput("Quarkoutfile aufrufen" "\r\n");
			error_occured = call_extern (buf.data,
								 NULL,	/* spezielles Arbeitsverzeichnis */
								 akt_anruf.soft_limit + 600,	/* killzeitpunkt,just for the case */
								 FALSE,		/* ignore_exitcode */
								 FALSE);	/* nicht unter uid des Users */
			uostr_freedata(&buf);
			uostr_freedata(&buf2);
			if (error_occured)
			{
				mbprintf (" Quarkoutfile beendete sich mit Exitcode %d" "\r\n",
						  error_occured);
				syslog (LOG_ERR,"Quarkoutfile: Exitcode %d\n", error_occured);
				ta_userdupecheck_end ();
				return FALSE;
			}
		}
	}

	if (ta_opts.umlaut || ta_opts.umbruch)
		ta_umlautwandlung (TRUE, outfilename);

	return TRUE;

}								/* ta_erstelle_outfile */
