/* ta_spec.c: Spezialfilehandling (Import) */
/*
    Copyright (C) 1994 1995 1996 Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
*/
#include "tomsysdep.h"

#include <portab.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <syslog.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>			/* mkdir, S_IRWXU */
#include <dirent.h>
#include <errno.h>

#ifdef OFFLINE2
#include "offline/inc/version.h"
#include "offline/inc/mdb_global.h"  /* mdb_rename_group */
#else
#include "offline/version.h"
#include "offline/mdb_global.h"  /* mdb_rename_group */
#endif
#include "prototyp.h"
#include "user.h"
#include "brett.h"
#include "routing.h"
#include "ta_info.h"

#include "tausch.h"
#include "ta_int.h"
#include "ta_ext.h"

#ifndef LINEEND
#define LINEEND "\r\n"
#endif

static void ta_spec_work_xinfofile (message_t * head,
									xinfofile_t * server_in);
static void ta_spec_work_infofile (message_t * head,
								   infofile_t * server_in);
static void ta_spec_work_CMD (message_t * head);
static void ta_spec_work_LOG (message_t * head);
static void ta_spec_work_CNF (message_t * head);
static void ta_spec_itg_auswertung (message_t * head);
BOOLEAN ta_spec_itg_finish (char *grpname, char *netzname, 
							char *userinfo, char *sysopinfo,
							char *aliasname, 
							time_t configtime, char *followup);

/*****************************************************************************
 * Ende der Prototypen
 *****************************************************************************
 */
BOOLEAN
ta_spec_itg_finish (char *grpname, char *netzname, char *userinfo, 
					char *sysopinfo, char *aliasname, 
					time_t configtime, char *followup)
{
	BRETTNR bnr=0;
	BRETTNR flu=0;
	BRETT bdat;
	BOOLEAN x_changed=FALSE;

	if (netzname && *netzname)
		bnr=brettnr(netzname);
	if (!bnr)
		bnr=brettnr(grpname);
	if (!bnr)
		return FALSE;
	if (!br_ind[bnr].brett) /* paranoia */
		return FALSE;
	if (br_ind[bnr].brett->art==BINAER_BRETT)
		return FALSE; /* Binrbretter nicht */
	if (!br_ind[bnr].brett->netzstatus)
		return FALSE; /* lokale Bretter nicht */
	if (!bb_read_brett(bnr, TRUE))
		return FALSE;

	if (userinfo && *userinfo)
	{
		if (!br_ind[bnr].brett->userinfo 
			|| strcmp(br_ind[bnr].brett->userinfo,userinfo)!=0)
		{
			free(br_ind[bnr].brett->userinfo);
			br_ind[bnr].brett->userinfo=XSTRDUP(userinfo);
			x_changed=TRUE;
		}
	}
	if (sysopinfo && *sysopinfo)
	{
	 	if (!br_ind[bnr].brett->sysopinfo 
			|| strcmp(br_ind[bnr].brett->sysopinfo,sysopinfo)!=0)
		{
			free(br_ind[bnr].brett->sysopinfo);
			br_ind[bnr].brett->sysopinfo=XSTRDUP(sysopinfo);
			x_changed=TRUE;
		}
	}
	else if (br_ind[bnr].brett->sysopinfo)
	{
		/* wenn kein Sysopinfo mehr vorhanden, dann lschen */
		free(br_ind[bnr].brett->sysopinfo);
		br_ind[bnr].brett->sysopinfo=NULL;
		x_changed=TRUE;
	}

	if (aliasname && *aliasname)
	{
	 	if (!br_ind[bnr].brett->aliasname 
			|| strcmp(br_ind[bnr].brett->aliasname,aliasname)!=0)
		{
			free(br_ind[bnr].brett->aliasname);
			br_ind[bnr].brett->aliasname=XSTRDUP(aliasname);
			x_changed=TRUE;
		}
	}
	else if (br_ind[bnr].brett->aliasname)
	{
		/* wenn kein Sysopinfo mehr vorhanden, dann lschen */
		free(br_ind[bnr].brett->aliasname);
		br_ind[bnr].brett->aliasname=NULL;
		x_changed=TRUE;
	}

	if (followup && *followup)
		flu=brettnr(followup);
#if 0
	/* MAUS sendet keine O-Zeile mit? */
	else
		flu=0;
#endif
	if (flu!=br_ind[bnr].brett->followup_to_default)
	{
		br_ind[bnr].brett->followup_to_default=flu;
		x_changed=TRUE;
	}

	if (configtime != (time_t) -1 && configtime!=bdat.netzaenderung)
	{
		br_ind[bnr].brett->netzaenderung=configtime;
		x_changed=TRUE;
	}
	if (x_changed)
		bb_write_brett(bnr, TRUE, TRUE);
	else
		bb_unlock_brett(bnr);

	return x_changed;
}

static void
ta_spec_itg_auswertung (message_t * head)
{
	/*
	 * Kurzbeschreibung des ITG: Pro Gruppe ein Block. Jeder Block beginnt
	 * mit einer 'G' Zeile und endet bei der nchsten:
	 * 'G' lokaler Gruppenname
	 * 'N' Netzgruppenname (falls abweichend)
	 *     wir versuchen zuerst den Netznamen, dann den lokalen.
	 * 'U' Userinfo
	 *     uebernehmen wir.
	 * 'S' Sysopinfo
	 *     uebernehmen wir.
	 * 'C' Gruppenchef
	 *     fuer uns uninteressant
	 * 'F' Flags
	 *     Daraus entnehmen wir, ob die Gruppe eine Netzgruppe ist.
	 * 'E' Aenderungsdatum netzweit
	 *     uebernehmen wir.
	 * 'L' Aenderungsdatum lokal
	 *     fuer uns uninteressant.
	 * 'R' Followup-To-Default
	 *     wird uebernommen, wenn moeglich (Zielgruppe existiert)
	 */
	char *s;
	char userinfo[256]="";
	char sysopinfo[256]="";
	char aliasname[256]="";
	char netzname[256]="";
	char grpname[256]="";
	char followup[256]="";
	BOOLEAN netzgruppe=FALSE;
	time_t configtime=(time_t) -1;

	s = head->body.buf;
	while (s && *s)
	{
		char old_char = 0;		/* gcc meckerte "may be uninitialized" */
		char *p;

		p = strpbrk (s, "\r\n");
		if (p)
		{
			old_char = *p;
			*p = '\0';
		}

		switch(s[1]) /* s[0] == ':' */
		{
		case 'G':
			if (netzgruppe)
			{
				ta_spec_itg_finish (grpname,netzname,userinfo,
					sysopinfo, aliasname, configtime,followup);
			}
			*userinfo=0;
			*sysopinfo=0;
			*netzname=0;
			*aliasname=0;
			*followup=0;
			strcpy(grpname,s+2);
			netzgruppe=FALSE;
			configtime=(time_t) -1;
			break;
		case 'R':
			strcpy(followup,s+2);
			break;
		case 'N':
			strcpy(netzname,s+2);
			break;
		case 'S':
			strcpy(sysopinfo,s+2);
			break;
		case 'A':
		{
			BRETTNR j;
			j=find_brett(s+2);
			if (j) {
				syslog(LOG_WARNING,"warning: alias %s collides with group %s (alias ignored)",
					   s+2,br_ind[j].brett->name);
				break;
			}
			strcpy(aliasname,s+2);
			break;
		}
		case 'U':
			strcpy(userinfo,s+2);
			break;
		case 'E': /* netzweites nderungsdatum */
			if (!mtzeit_to_time_t(s+2, &configtime))
				configtime=(time_t) -1;
			break;
		case 'F':
			if (strstr(s+2,"V+") || strstr(s+2,"V="))
				netzgruppe=TRUE;
			break;
		}

		/* Und nun wieder zusammensetzen - wer wei, wofr wir das
		 * File spter mal brauchen (direkte ITx-Auswertung?)
		 */
		if (p)
		{
			*p++ = old_char;
			if (*p == '\n')
				p++;
		}
		s = p;
	}
	if (*grpname && netzgruppe)
		ta_spec_itg_finish (grpname,netzname,userinfo,
			sysopinfo,aliasname,configtime, followup);
}

static void
ta_spec_work_xinfofile (message_t * head, xinfofile_t * server_in)
{
	/* Infofiles von Boxen verarbeiten */
	char fname[PATH_MAX];
	infofile_t *in;
	FILE *f;
	char *s;

	/* Das Infofile in einem Verzeichnis pro User speichern */
	sprintf (fname, "%s%lu", tom_info_net_dir, (ULONG) user.nummer);
	mkdir (fname, S_IRWXU);
	sprintf (fname, "%s%lu/%s", tom_info_net_dir, (ULONG) user.nummer, server_in->name);
	f = xfopen (fname, "w");
	s = head->body.buf;
	while (s && *s)
	{
		char old_char = 0;		/* gcc meckerte "may be uninitialized" */
		char *p;

		p = strchr (s, '\n');
		if (p)
		{
			if (p>s && p[-1]=='\r')
				p--;
			old_char = *p;
			*p = '\0';
		}
		fputs(s+1,f);
		putc('\n',f);
		/* Und nun wieder zusammensetzen - wer wei, wofr wir das
		 * File spter mal brauchen (direkte ITx-Auswertung?)
		 */
		if (p)
		{
			*p++ = old_char;
			if (*p == '\n')
				p++;
		}
		s = p;
	}
	fclose (f);

	/* 
	 * So, gespeichert ist es. Jetzt nachsehen, ob das Infofile auch 
	 * in der Liste der lokalen Infofiles ist, es ITYP_NET ist und
	 * der aktuelle User als Server (Sender) eingetragen ist.
	 */

	in = ta_find_infofile (head->msg_id);
	if (in && in->typ == ITYP_NET && in->sender == user.nummer)
	{
		in->sender = user.nummer;
		in->zeit = head->importzeit;
		ta_info_mark_change ();
	}
	/* ITB/G ins info/net linken/kopieren. Vielleicht auch anders lsbar. */
	if (!strcmp (head->msg_id, "ITB") || !strcmp(head->msg_id,"ITG")
		|| !strcmp (head->msg_id, "ITS"))
	{
		char old[PATH_MAX];
		char new[PATH_MAX];

#ifdef HAVE_DOSDRIVES
		if (tom_info_net_dir[2] == ':')
#else
		if (*tom_info_net_dir == '/')
#endif
		{
			strcpy (old, tom_info_net_dir);	/* /usr/local/quark/infofile/net */
			strcpy (new,old);
			strcpy (fname, tom_info_net_dir);
		}
		else
		{
			getcwd (old, sizeof (old));		/* /usr/local/quark */
			strcat (old, "/");	/* /usr/local/quark/ */
			strcat (old, tom_info_net_dir);	/* /usr/local/quark/tom/info/net/ */
			strcpy (new, old);
			strcpy (fname, old);
		}
		strcat (old, head->msg_id);
		strcat (old, ".old");
		strcat (new, head->msg_id);
		sprintf (fname + strlen (fname), "%lu/%s" , user.nummer, head->msg_id);
		if (fileexist(new) && rename (new, old))
			syslog (LOG_NOTICE, "rename %s: %s", old, strerror (errno));
		else
		{
			if (
#ifdef HAVE_SYMLINK
				   symlink (fname, new) &&
#endif
#ifdef HAVE_LINK
				   link (fname, new) &&
#endif
				   !file_copy (fname, new))
			{
				syslog (LOG_NOTICE, "(sym)link/copy to %s: %s",
						new, strerror (errno));
				rename (old, new);
			}
			else
				unlink (old);
		}
	}

	/* ITG auswerten */

	if (!strcmp (head->msg_id, "ITG"))
		ta_spec_itg_auswertung (head);
}

static void
ta_spec_work_infofile (message_t * head, infofile_t * in)
{
	/* Infofiles von Benutzern verarbeiten */
	char fname[PATH_MAX];
	FILE *f;
	char *s;
	char *p;

	if (in->typ != ITYP_LOKAL)
	{
		ta_do_log_error ("Upload dieses Infofiles nicht erlaubt");
		return;
	}
	if (level() < 8)
	{
		ta_do_log_error ("Upload von Infofiles ist Ihnen nicht erlaubt");
		return;
	}
	sprintf (fname, "%s%s", tom_info_lokal_dir, in->name);
	f = xfopen (fname, "w");
	s = head->body.buf;
	while (s && *s)
	{
		p = strpbrk (s, "\r\n");
		if (p)
			*p++ = '\0';
		/* :-Zeile in s */
		fputs(s+1,f); /* Ohne Doppelpunkt */
		putc('\n',f);
		if (p && *p == '\n')
			p++;
		s = p;
	}
	fclose (f);
	in->sender = user.nummer;
	in->zeit = head->importzeit;
	ta_info_mark_change ();
}

static void
ta_spec_work_CMD (message_t * head)
{
	char *s;
	char *p;

	if (ta_opts.first_cmd_block)
	{
		/* 
		 * Prinzip beim Asynctausch ist, mglichst die Anweisungen aus dem
		 * letzten Infile zu nehmen - aber die Lsung hier ist alles andere
		 * als gut.
		 */
		char old_prot = ta_passinfo.protokoll;

		ta_opts.first_cmd_block = FALSE;
		ta_init_passinfo (&ta_passinfo);
		ta_passinfo.protokoll = old_prot;
	}
	s = head->body.buf;
	while (s && *s)
	{
		p = strpbrk (s, "\r\n");
		if (p)
			*p++ = '\0';
		ta_cmd_work (s + 1);
		if (*p == '\n')
			p++;
		s = p;
	}
}

static void
ta_spec_work_CNF (message_t * head)
{
	char *s;
	char *p;
	char fname_tmp[PATH_MAX];
	char fname[PATH_MAX];
	char *kuerzel=NULL;
	FILE *f;

	if (user.typ == UTYP_GATE)
	{
		ta_do_log_error ("mit Ihren CNFs kann ich nichts anfangen");
		return;
	}
	else if (user.typ == UTYP_USER && level() != ULV_MAX)
	{
		ta_do_log_error ("Sie sind nicht zum Upload von CNFs berechtigt");
		return;
	}

	sprintf (fname_tmp, "%stmp%lu.cnf", box_conf.tausch_dir, (ULONG) user.nummer);
	f = xfopen (fname_tmp, "w");
	s = head->body.buf;
	while (s && *s)
	{
		p = strpbrk (s, "\r\n");
		if (p)
			*p++ = '\0';
		if (s[1]=='*')
			kuerzel=entumlauten1(s+2);
		fputs(s+1,f);
		fputs("\r\n",f);
		if (*p == '\n')
			p++;
		s = p;
	}
	fclose (f);
	if (!kuerzel)
	{
		ta_do_log_error ("CNF enthlt kein Krzel\r\n");
		return;
	}
	for (p=kuerzel;*p;p++)
		*p=tolower(*p);
	sprintf (fname, "%s%s.cnf", box_conf.tausch_dir, kuerzel);
	unlink(fname);
	if (rename(fname_tmp,fname))
	{
		char buf[PATH_MAX+256];
		sprintf(buf,"kann CNF nicht umbenennen: %s\r\n",strerror(errno));
		ta_do_log (buf);
		unlink(fname_tmp);
		return;
	}
	ta_do_log ("!Ungeprft bernommen\r\n");
}

static void
ta_spec_work_NOCEM (message_t *head)
{
	char *s,*p;
	if (level()<ULV_MAX)
	{
		ta_do_log_error ("NOCEM-Blcke zu senden ist Ihnen nicht erlaubt");
		return;
	}
	s=head->body.buf;
	while (s && *s)
	{
		p = strpbrk (s, "\r\n");
		if (p)
			*p++ = '\0';
		if (s[0]==':' && s[1]=='V') {
			msglog("NV%s\n",s+2);
		} else if (s[0]==':' && s[1]=='#') {
			msglog("N#%s\n",s+2);
			if (mdb_kill_article(s+2)) {
				msglog("  OK\n");
			} else {
				msglog("  nicht OK/gefunden\n");
			}
		}
		if (*p == '\n')
			p++;
		s = p;
	}
}

static void
ta_spec_work_REN (message_t * head)
{
	char *s;
	char *p;
	char fname_tmp[PATH_MAX];
	char *alt=NULL;
	char *neu=NULL;
	FILE *f;

	if (user.typ == UTYP_USER && level()<ULV_MAX)
	{
		ta_do_log_error ("REN-Blcke zu senden ist Ihnen nicht erlaubt");
		return;
	}

	sprintf (fname_tmp, "%sren.tmp", tom_tmp_dir);
	f = xfopen (fname_tmp, "w");
	s = head->body.buf;
	while (s && *s)
	{
		p = strpbrk (s, "\r\n");
		if (p)
			*p++ = '\0';
		if (s[1]=='O')
		{
			alt=s+2;
			neu=NULL;
		}
		if (s[1]=='N')
			neu=s+2;
		if (s[1]=='D' && (!neu || !alt))
		{
			fprintf (f, "Umbenennung von: %s\r\n", alt ? alt : "-> [fehlt] <-");
			fprintf (f, "  nach:          %s\r\n", neu ? alt : "-> [fehlt] <-");
			fprintf (f, "  am:            %s\r\n", s+2);
			fprintf (f, "\r\n");
		}
		else if (s[1]=='D')
		{
			BRETTNR abnr;
			BRETTNR nbnr;
			abnr=brettnr(alt);
			nbnr=brettnr(neu);
			fprintf (f, "Umbenennung von: %s\r\n", alt);
			fprintf (f, "  nach:          %s\r\n", neu);
			fprintf (f, "  am:            %s\r\n", s+2);
			if (nbnr)
				fprintf (f, "  Neuer Name lokal schon vorhanden, ignoriert\r\n");
			else if (!abnr)
				fprintf (f, "  Alter Name lokal nicht vorhanden, ignoriert\r\n");
			else if (strlen(neu)>=BDAT_GRP_NAME_LEN)
				fprintf (f, "  Neuer Name zu lang, ignoriert\r\n");
			else
			{
				if (!bb_read_brett(abnr,TRUE))
					fprintf (f, "  kann Gruppe nicht locken\r\n");
				else
				{
					char buf[BDAT_GRP_NAME_LEN*2+1];
					strcpy(br_ind[abnr].brett->name,neu);

#ifdef OFFLINE_VERSION
					mdb_rename_group(alt,neu);
#else
					mdb_rename_group(alt,neu,FALSE);
#endif
					ta_add_rename_info(time(NULL),alt,neu);
					bb_write_brett (abnr, TRUE, FALSE);

					if (br_ind[abnr].canon)
						free(br_ind[abnr].canon);
					bb_canonical(neu,buf);
					br_ind[abnr].canon=XSTRDUP(buf);
					b_free_sortierung();

					fprintf (f, "  Umbenennung erfolgreich\r\n");
				}
			}
			alt=NULL;
			neu=NULL;
			fprintf (f, "\r\n");
		}

		if (*p == '\n')
			p++;
		s = p;
	}
	if (ftell(f)!=0)
	{
		char *empfaenger=concat("Sysop @ ", MYDOMAINPART, NULL);
		char *absender=concat("Quark @ ", MYDOMAINPART, NULL);
		char *wegen=concat("Gruppenumbenennung durch ", user.name, NULL);
		fclose (f);

		if (empfaenger && absender && wegen)
		{
			einfachnachricht(TRUE,9,TRUE,fname_tmp,
				'A', empfaenger,
				'V', absender,
				'W', wegen,
				0);
		}
		free(empfaenger);
		free(absender);
		free(wegen);
	}
	else
		fclose (f);
}

	/*
	 * LOGfiles von Boxen und Gates: Auf jeden Fall an den Sysop
	 * mailen (spter Adressat konfigurierbar machen?), LOGs von
	 * Boxen bezglich Infofiles auswerten.
	 */
static void
ta_spec_work_LOG (message_t * head)
{
	char *p,
	*s;
	BOOLEAN written = FALSE;
	char what[256];
	char fname[PATH_MAX];
	FILE *f = NULL;				/* keep gcc quiet */

	if (user.typ == UTYP_USER)
	{
		ta_do_log_error ("Ihre LOGs interessieren nicht");
		return;
	}

	sprintf (fname, "%sLOG.tmp", tom_tmp_dir);
	f = fopen (fname, "w");
	if (!f)
		syslog (LOG_NOTICE, "Kann LOGfile nicht temporr schreiben");

	s = head->body.buf;
	*what = '\0';
	while (s && *s)
	{
		p = strpbrk (s, "\r\n");
		if (p)
			*p = '\0';
		if (s[1] == '#' || s[1] == '\"')
		{
			strncpy (what, s + 1, sizeof (what) - 1);
			what[sizeof (what) - 1] = '\0';
		}
		if (f && s[1] == '?')
		{
			if (*what)
			{
				fputs(what,f);
				fputs(LINEEND,f);
				*what = '\0';
			}
			written = TRUE;
			fputs(s+1,f);
			fputs(LINEEND,f);
		}
		if (s[1] == '$')
		{
			char kopie[256];
			char *p2;

			strcpy (kopie, s + 2);
			p2 = strchr (kopie, '=');
			if (p2)
			{
				xinfofile_t *server_in;
				infofile_t *in;
				long l;

				*p2++ = '\0';
				strtrim (kopie);
				l = strtol (p2, NULL, 10);
				/* Erstmal die CRC in der Liste der beim Server bestellten
				 * Infofiles anpassen.
				 */
				server_in = ta_find_xinfofile (kopie, user.nummer);
				if (server_in)
				{
					server_in->crc = l;
					ta_xinfo_mark_change ();
				}
				in = ta_find_infofile (kopie);
				if (in && in->typ == ITYP_NET && user.nummer == in->sender)
				{
					in->crc = l;
					in->zeit = time (NULL);
					ta_info_mark_change ();
				}
			}
		}
		s = p + 1;
		if (*s == '\n')
			s++;
	}
	if (f)
		fclose (f);
	if (written)
	{
		char *empfaenger=concat("Sysop @ ", MYDOMAINPART, NULL);
		char *absender=concat("Quark @ ", MYDOMAINPART, NULL);
		char *wegen=concat("LOG von ", user.name, NULL);

		if (empfaenger && absender && wegen)
		{
			einfachnachricht(TRUE,9,TRUE,fname,
				'A', empfaenger,
				'V', absender,
				'W', wegen,
				0);
		}
		else
		{
			syslog(LOG_ERR,
				"kann LOG nicht an Sysop mailen, out of memory");
		}
		if (empfaenger)
			free(empfaenger);
		if (absender)
			free(absender);
		if (wegen)
			free(wegen);
	}
	unlink (fname);
}

void
ta_work_special (message_t * head)
{
	BOOLEAN just_save=FALSE;
	switch(head->msgtyp)
	{
	case gruppen_msg:
	case pers_msg:
	case status_msg:
	case false_msg:
		break; /* cannot happen */
	case quark_msg:
		ta_saugblock(head);
		break;
	case nocem_msg:
		ta_spec_work_NOCEM (head);
		break;
	case ren_msg:
		ta_spec_work_REN (head);
		break;
	case head_msg:
		/* ignored for now */
		if (user.typ != UTYP_USER)
			just_save=TRUE;
		break;
	case log_msg:
		ta_spec_work_LOG (head);
		break;
	case cmd_msg:
		ta_spec_work_CMD (head);
		break;
	case cnf_msg:
		ta_spec_work_CNF (head);
		break;
	case out_msg:
		/* ignored. for now? */
		break;
	case info_msg:
		if (user.typ == UTYP_USER)
		{
			infofile_t *in;

			in = ta_find_infofile (head->msg_id);
			if (in)
			{
				ta_spec_work_infofile (head, in);
				return;
			}
			else
				ta_do_log_error ("Unbekannte Spezialnachricht");
		}
		if (user.typ == UTYP_BOX)
		{
			xinfofile_t *in;

			in = ta_find_xinfofile (head->msg_id, user.nummer);
			if (in)
			{
				ta_spec_work_xinfofile (head, in);
				return;
			}
			else
				just_save=TRUE;
		}
		else
			ta_do_log_error ("Unbekannte Spezialnachricht");
		break;
	} /* switch */

	if (just_save) {
		char fname[PATH_MAX];
		FILE *f;
		char *s,*p;
		sprintf (fname, "%s%lu", tom_info_net_dir, (ULONG) user.nummer);
		mkdir (fname, S_IRWXU);
		sprintf (fname, "%s%lu/%s", tom_info_net_dir, (ULONG) user.nummer, head->msg_id);
		f = xfopen (fname, "w");
		s = head->body.buf;
		while (s && *s)
		{
			p = strpbrk (s, "\r\n");
			if (p)
				*p++ = '\0';
			/* :-Zeile in s */
			fputs(s+1,f); /* Ohne Doppelpunkt */
			putc('\n',f);
			if (p && *p == '\n')
				p++;
			s = p;
		}
		fclose (f);
	}

	return;
}
