/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
       Snail Mail (don't expect me to answer):
             Uwe Ohse
             Drosselstrae 2
             47055 Duisburg
             Germany
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)

    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include <portab.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <limits.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <syslog.h>
#include <signal.h>
#include <setjmp.h>
#include <errno.h>

	/* damit die in . liegen darf */
#include "conflib.h"

#include "tomsysdep.h"
#include "prototyp.h"
#include "modem.h"
#include "io.h"
#include "tompaths.h"
#include "box_conf.h"
#include "aktdata.h"

extern int logout_modus;
extern int user_in_fd;

/* =======================  Transfer - Routinen  ======================== */

typedef struct protokoll_t
{
	char key;
	const char *sendcmd;
	const char *receivecmd;
	const char *name;
	/* wo einsetzbar? */
	BOOLEAN stdinouttty;
	BOOLEAN console;
	BOOLEAN notty;
	BOOLEAN nur_tausch;
	/* Nur eine Datei jeweils bertragbar? */
	BOOLEAN one_file;
	/* Filename vor dem Empfang erfragen? */
	BOOLEAN ask_source;
	BOOLEAN ask_destination;
	BOOLEAN ask_name;
	struct protokoll_t *next;
}
protokoll_t;

static protokoll_t intern_ascii_prot_desc=
{
	'A', /* key */
	NULL, /* sendcmd */
	NULL, /* rcvcmd */
	"ASCII", /* Name */
	TRUE, /* stdinouttty */
	FALSE, /* console */
	TRUE, /* notty */
	TRUE, /* nur tausch */
	TRUE, /* Nur eine Datei jeweils bertragbar? */
	FALSE, /* ask_source */
	FALSE, /* ask_dest */
	FALSE, /* ask_name */
	NULL /* next */
};

static protokoll_t *anker=&intern_ascii_prot_desc;
static BOOLEAN init_done;

	/* globale Variablen fr die Textinterpretationsroutine */
const char *protokoll_quelle;			/* %PQ */
const char *protokoll_ziel;			/* %PZ */

static BOOLEAN read_transfer_cnf (void);
static BOOLEAN get_xmodem_filename (char *datei);
static BOOLEAN get_filename (BOOLEAN source, char *datei);

static BOOLEAN
read_transfer_cnf (void)
{
	int erg;
	protokoll_t *neu;
	cl_list_t *all_protos = NULL;
	cl_file_t *cfp;
	char fname[PATH_MAX];

	cl_var_t listtab[] =
	{
		{"protokolle", NULL, CL_LIST, NULL, 0, NULL},
		{"protocols", "protokolle", CL_ALIAS, NULL, 0,NULL},
		{"protocol", "protokolle", CL_ALIAS, NULL, 0, NULL},
		{"protokoll", "protokolle", CL_ALIAS, NULL, 0,NULL},
		{NULL, NULL, 0, NULL, 0}
	};
	listtab[0].adr=&all_protos; /* "constant initializers" */

	init_done = TRUE;
	if (tom_port_config_dir)
	{
		sprintf(fname,"%stransfer.cnf",tom_port_config_dir);
		cfp = cl_openconf (fname,"r");
	}
	else
		cfp = NULL;

	if (!cfp)
	{
		sprintf(fname,"%stransfer.cnf",box_conf.config_dir);
		cfp = cl_openconf (fname,"r");
	}
	if (!cfp)
	{
		syslog (LOG_ERR, "kann transfer.cnf nicht lesen");
		return FALSE;
	}
	erg = cl_getstanza (cfp, listtab, "liste", 0, NULL);
	if (erg)
	{
		syslog (LOG_ERR, "kann Protokolliste nicht lesen");
		cl_closeconf(cfp);
		return FALSE;
	}
	while (all_protos)
	{
		/* all_events->inhalt ist die ID des Events */
		cl_list_t *next;
		char key = '\0';
		char *sendcmd = NULL;
		char *receivecmd = NULL;
		char *ports = NULL;
		char *name = NULL;
		int one_file = 0;
		int ask_fname = 0;
		int ask_source = 0;
		int ask_destination = 0;
		int nur_tausch = 0;
		char *id;

#define CONFLIB_VAR_INIT(var,counter,p1,p2,p3,p4,p5,p6) do { \
			var[counter].varname=p1; var[counter].secdata=p2; \
			var[counter].typ=p3; var[counter].adr=p4; \
			var[counter].flags=p5; var[counter].rangeexp=p6; \
			counter++; } while(0)

		cl_var_t prot[13];
		{
			int counter=0;
			CONFLIB_VAR_INIT(prot,counter,
							 "key", NULL,   CL_CHAR,    &key,      0, "[0-9B-Zb-z]");
			CONFLIB_VAR_INIT(prot,counter,
							 "name",          NULL,   CL_STRING,  &name,     0, NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "send",          NULL,   CL_STRING,  &sendcmd,  0, NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "receive",       NULL,   CL_STRING,  &receivecmd,0,NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "ports",         NULL,   CL_STRING,  &ports,    0, NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "one-file-only", NULL,   CL_BOOLEAN, &one_file, 0, NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "ask-name",      NULL,   CL_BOOLEAN, &ask_fname,0, NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "ask-filename",  "ask-fname",CL_ALIAS,NULL,     0, NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "ask-source",    NULL,   CL_BOOLEAN, &ask_source,0,NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "only-tausch",   NULL,   CL_BOOLEAN, &nur_tausch,0,NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "ask-destination",NULL,  CL_BOOLEAN, &ask_destination,0,NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 "ask-dest", "ask-destination",CL_ALIAS,NULL,     0,NULL);
			CONFLIB_VAR_INIT(prot,counter,
							 NULL, NULL, 0, NULL, 0, NULL);
		};

		id = all_protos->inhalt;
		erg = cl_getstanza (cfp, prot, id, 0, NULL);
		if (erg)
		{
			syslog (LOG_NOTICE, "kann Protokoll %s nicht lesen", id);
		}
		else
		{
			BOOLEAN ok=TRUE;

			if (!key)
				ok = FALSE;
			if (!sendcmd)
				ok = FALSE;
			if (!receivecmd)
				ok = FALSE;
			if (!name)
				ok = FALSE;
			if (!ok)
			{
				mbprintf ("unvollstndige Angaben zu Protokoll %s\n", id);
				syslog (LOG_NOTICE, "unvollstaendige Angaben zu Protokoll %s\n", id);
			}
			else
			{

				neu = XMALLOC (sizeof (protokoll_t));
				neu->key = key;
				neu->sendcmd = XSTRDUP (sendcmd);
				neu->receivecmd = XSTRDUP (receivecmd);
				neu->name = XSTRDUP (name);
				neu->console = FALSE;
				neu->stdinouttty = FALSE;
				neu->notty = FALSE;
				if (ports)
				{
					strlwr (ports);
					if (strstr (ports, "cons"))
						neu->console = TRUE;
					if (strstr (ports, "remote") || strstr (ports, "modem"))
						neu->stdinouttty = TRUE;
					if (strstr (ports, "notty") || strstr(ports,"raw"))
						neu->notty = TRUE;
				}
				else
					neu->stdinouttty = TRUE;
				neu->one_file = one_file ? TRUE : FALSE;
				neu->ask_name = ask_fname ? TRUE : FALSE;
				neu->ask_source = ask_source ? TRUE : FALSE;
				neu->nur_tausch = nur_tausch ? TRUE : FALSE;
				neu->ask_destination = ask_destination ? TRUE : FALSE;
				neu->next = anker;
				anker = neu;
			}
		}
		cl_clearconf (prot);
		next = all_protos->next;
		free (all_protos->inhalt);
		free (all_protos);
		all_protos = next;
	}
	cl_closeconf (cfp);

	return TRUE;
}

char
transfer_protokollauswahl (BOOLEAN all, BOOLEAN space_zum_loeschen)
{
	protokoll_t *prot;
	char buf[128];
	char dec[128] = "";
	const char *fix_ende="  (ESC) Abbruch?";
	char c;
	size_t len = 0;
	int i=0;
	BOOLEAN title_shown=FALSE;

	if (!init_done)
		read_transfer_cnf ();

	c='\0';
	while (!c)
	{
		BOOLEAN can_read=io_charavail(user_in_fd);
		i++;
		if (i==1 || (i>1 && !title_shown))
		{
			title_shown=TRUE;
			for (prot = anker; prot; prot = prot->next)
			{
				BOOLEAN ja=FALSE;

				if (akt_connection.is_console && prot->console)
					ja=TRUE;
				if ((akt_connection.is_modem 
					|| (akt_connection.is_tty && !akt_connection.is_console)) 
					&& prot->stdinouttty)
					ja=TRUE;
				if (!akt_connection.is_tty && prot->notty)
					ja=TRUE;
				if (prot->nur_tausch && !akt_anruf.mitten_im_tausch)
					ja=FALSE;
				if (all)
					ja=TRUE;
				if (ja)
				{
					size_t sl;

					sl = sprintf (buf, "(%c) %s", prot->key, prot->name);
					if (sl + len >= 80)
					{
						if (!can_read)
							mboutput ("\r\n");
						len = 0;
					}
					if (!can_read)
						mbprintf ("  %s", buf);
					len += 2 + sl;
					sprintf (dec + strlen (dec), "%c", tolower (prot->key));
				}
			}
			if (space_zum_loeschen)
			{
				const char *text="  (SPACE) lschen";
				if (len + strlen(text) >= 80)
				{
					if (!can_read)
						mboutput("\r\n");
					len=0;
				}
				if (!can_read)
					mboutput(text);
				len+=strlen(text);
			}
			if (!can_read)
			{
				if (len + strlen(fix_ende) >= 80)
				{
					mboutput("\r\n");
				}
				mboutput (fix_ende);
				mboutput ("\r\n");
				/* und das fr Henry Rolofs:
				 * HR>kme. Das ist in einigen Programmen/Srcipts von mir das 
				 * HR>Schlsselwort zur Eingabe des bertragungsprotokolls.
				 */
				if (!space_zum_loeschen)
					mboutput ("Protokoll: ");
			}
		}
		c = mbchar ();
		if (logout_modus)
			return -1;
		switch (c)
		{
		case 27:				/* ESC */
		case 3:				/* ^C */
		case 24:				/* ^X */
			return '\0';
			break;
		case ' ':
			if (space_zum_loeschen)
				return ' ';
			break;
		}
		if (!strchr (dec, tolower (c)))
			c = '\0';
	}
	if (title_shown && !space_zum_loeschen)
	{
		for (prot = anker; prot; prot = prot->next)
		{
			if (prot->key==c)
			{
				mbprintf("%s\r\n",prot->name);
				break;
			}
		}
	}
	return c;
}


char
transfer_protokollcheck (char c)
{
	protokoll_t *prot;

	for (prot = anker; prot; prot = prot->next)
	{
		if (tolower(prot->key)!=tolower(c))
			continue;
		if ((akt_connection.is_console && prot->console)
			|| (akt_connection.is_modem && prot->stdinouttty)
			|| (!akt_connection.is_modem && prot->notty))
			return c;
	}
	return 0;
}

static int transfer_ascii_receive(const char *fnamepart,
								  long *laenge TOM_ATTRIB_UNUSED)
{
	FILE *f;
	BOOLEAN timed_out;
	char fname[PATH_MAX];
	BOOLEAN err=FALSE;
	int z=0;

	if (strchr(fnamepart,'/'))
	{
		mboutput("Illegale Unterverzeichnisse im Dateinamen. Abbruch\r\n");
		return 1;
	}

	sprintf(fname,"%s%s",tom_utmp_dir,fnamepart);
	f=xfopen(fname,"w");
	while (1)
	{
		char * str=mbgetline(60, &timed_out, FALSE);
		char *p;
		if (!str)
		{
			err=TRUE;
			break;
		}
		p=strpbrk(str,"\r\n");
		if (p)
			*p=0;
		fprintf(f,"%s\n",str);
		if (!strcmp(str,"#") || !strcmp(str,"."))
		{
			free(str);
			break;
		}
		mboutput(".");
		if (z++==70)
		{
			z=0;
			mboutput("\r\n");
		}
		free(str);
	}
	fclose(f);
	if (err)
	{
		mboutput("Abbruch wegen Fehler\r\n");
		return 1;
	}
	return 0;
}


/*
 * transfer_receive
 */
int 
transfer_receive (char *art, char *name, long *laenge)
{
	time_t upl_zeit;
	char datei[PATH_MAX];
	char *cmd;
	char xmodem_fname[PATH_MAX];
	char source_fname[PATH_MAX];
	BOOLEAN ok = TRUE;
	DIR *dir;
	struct dirent *eintrag;
	int status;
	protokoll_t *prot;
	size_t len;

	if (!init_done)
		read_transfer_cnf ();

	if (laenge)
		*laenge = 0;
	if (name && *name)
		(void) unlink (name);

	rm_dir_content (tom_utmp_dir);

	if (!*art)
		*art = transfer_protokollauswahl (FALSE, FALSE);
	else
		*art = transfer_protokollcheck(*art);
	if (!*art)
		return 1;
	for (prot = anker; prot; prot = prot->next)
	{
		if (tolower (*art) == tolower(prot->key))
			break;
	}
	if (!prot)
	{
		mbprintf ("Kein Protokoll %c gefunden!\n", *art);
		syslog (LOG_NOTICE, "kein Protokoll %c gefunden\n", *art);
		return 2;
	}


	if (prot->ask_name)
	{
		if (!get_xmodem_filename (xmodem_fname))
			return 1; /* zhlt als Abbruch */
	}
	else
		*xmodem_fname = '\0';

	if (prot->ask_source)
	{
		if (!get_filename (TRUE, source_fname))
			return 1; /* Abbruch */
	}
	else
		*source_fname = '\0';

	if (prot->key=='A')
	{
		mbprintf (
			"Die bertragung endet mit einer Zeile, die nur ein # enthlt\r\n");
		mbprintf ("Protokoll startet:\r\n"); 
		return transfer_ascii_receive("infile.txt",laenge);
	}

	protokoll_quelle = source_fname;
	protokoll_ziel = xmodem_fname;

	len = PATH_MAX * 2 + 4096;
	cmd = malloc (len);
	if (!cmd)
	{
		speicherproblem (FALSE, "transfer_receive");
		return 2; /* Fehler */
	}

	if (!do_interpret (prot->receivecmd, cmd, len))
	{
		len *= 4;
		free (cmd);
		cmd = malloc (len);
		if (!cmd)
		{
			speicherproblem (FALSE, "transfer_receive");
			return 2; /* Fehler */
		}

		if (!do_interpret (prot->sendcmd, cmd, len))
		{
			mbfehler ("Kommandointerpretation gescheitert");
			free (cmd);
			return 2; /* Fehler */
		}
	}

	protokoll_quelle = NULL;
	protokoll_ziel = NULL;

	upl_zeit = time (NULL);
	io_drain (user_in_fd);

	{
		time_t endzeit;

		mbprintf ("Protokoll startet:\r\n"); 
		/* Policy: 5 Minuten berziehen darf er, aber nicht ber
		 * hard_limit hinaus. Wir sind hier beim Upload.
		 */
		endzeit=akt_anruf.hard_limit;
		if (endzeit>akt_anruf.soft_limit+5*60)
			endzeit=akt_anruf.soft_limit+5*60;
		status=call_extern(cmd,
			tom_utmp_dir,              /* spezielles Arbeitsverzeichnis */
			endzeit,              /* killzeitpunkt */
			FALSE,                /* ignore_exitcode */
			FALSE);               /* Nicht unter der Kennung des Users */
	}

	free (cmd);

	if (status != 0)
	{
		mbprintf (" bertragung gescheitert\r\n\n");
		return 2; /* Fehler */
	}

	if (laenge)
		*laenge = 0;
	*datei = '\0';

	/* das ganze tmp-Verzeichnis durchgehen,
	 * unsichtbare Sachen loeschen, Zugriffsrechte korrigieren und
	 * Dateilaenge summieren.
	 */
	if ((dir = opendir (tom_utmp_dir)) != NULL)
	{
		while ((eintrag = readdir (dir)) != NULL)
		{
			sprintf (datei, "%s%s", tom_utmp_dir, eintrag->d_name);
			if (eintrag->d_name[0] == '.')
			{
				if (strcmp (eintrag->d_name, ".")
					&& strcmp (eintrag->d_name, ".."))
					unlink (datei);
				continue;
			}
			else
			{
				struct stat buf;

				/* 
				 * ggf. Schreibschutz der Datei aufheben 
				 * Nun auch unter UNIX, weil es auch Dosfilesysteme geben
				 * kann.
				 */
				stat (datei, &buf);
#ifdef HAVE_ST_ATTR
				if (!(buf.st_attr & S_IWUSR))
					chmod (datei, buf.st_attr | S_IWUSR);
#else
				if (!(buf.st_mode & S_IWUSR))
					chmod (datei, buf.st_mode | S_IWUSR);
#endif

				if (laenge)
					*laenge += len_datei (datei);
			}
		}
		closedir (dir);
	}

	if (!*datei)
		return 2; /* Fehler */

	ok = TRUE;
	if (name)
		strcpy (name, datei);

	/* Uploads wahlweise nicht auf Rest-Onlinezeit anrechnen */
	if (box_conf.upload_free)
		akt_anruf.soft_limit += (time (NULL) - upl_zeit);
	if (akt_anruf.hard_limit < akt_anruf.soft_limit)
		akt_anruf.soft_limit = akt_anruf.hard_limit;

	/* evtl. "Geistereingaben" des ZModem beseitigen */
	msleep (200);
	io_flush (user_in_fd);

	return 0;
}								/* transfer_receive */



/*
   Dateiname eingeben fr X-Modem
   FALSE: keine Eingabe bzw. Eingabe enthlt \ oder /
 */
static BOOLEAN
get_xmodem_filename (char *datei)
{
	mboutput ("\r\n");
	mbquestion (30, 2, TRUE, " Dateiname : ", "", datei);
	strtrim (datei);
	if (!*datei)
	{
		mboutput ("Abbruch!\r\n");
		return FALSE;
	}
	if (strpbrk (datei, "\\/"))
	{
		mbfehler ("fehlerhafter Dateiname");
		return FALSE;
	}

	return TRUE;
}								/* get_xmodem_filename */

/*
   Dateiname eingeben

   zurckgeliefert wird der Dateiname inkl. Pfad bzw. 0, wenn die Eingabe 
   ungltig war, Grnde:
   Konsole: keine Eingabe bzw. Datei ex. nicht
   remote : keine Eingabe bzw. Eingabe enthlt \ oder /
 */
static BOOLEAN
get_filename (BOOLEAN source, char *datei)
{
	char tmp_datei[PATH_MAX];
	char tmp_pfad[PATH_MAX];
	BOOLEAN wahl;

	const char *prompt;

	if (source)
		prompt = "Bitte Quelldatei auswhlen";
	else
		prompt = "Bitte Zieldatei/verzeichnis auswhlen";

	mbprintf("%s:\r\n",prompt);
	wahl = dateiauswahl ("", "*.*", tmp_pfad, tmp_datei);
	strtrim (tmp_datei);
	if (*tmp_pfad=='/' || (*tmp_pfad=='.' && tmp_pfad[1]=='.'))
		strcpy (datei, tmp_pfad);
	else
		sprintf(datei, "%s%s",box_conf.quark_dir, tmp_pfad);
	strcat (datei, tmp_datei);
	if (!wahl)
	{
		*datei = 0;
		mboutput ("Abbruch!\r\n");
		return FALSE;
	}
	return TRUE;
}

static int 
transfer_ascii_send(const char *fname)
{
	FILE *f;
	/* kleiner als der Puffer von mboutput! */
	char buf[512];
	BOOLEAN last=FALSE;
	const char *np;
	f=fopen(fname,"r");
	if (!f)
	{
		mbprintf ("zu versendende Datei %s verschwunden!\n", fname);
		return 1;
	}
	np=strrchr(fname,'/');
	if (!np)
		np=fname;
	else
		np++;
	while (fgets(buf,sizeof(buf),f))
	{
		if (buf[0]=='#' && (buf[1]=='\r' || buf[1]=='\n'))
			last=TRUE;
		if (0>mboutput(buf))
			break;
		if (last)
			break;
	}
	if (!last)
		mboutput("#\r\n");
	fclose(f);
	return 0;
}

/*
   Datei senden (Box -> User)

   ->           art :   Transferprotokoll, 'X', 'Y', 'Z'
   name :   Name der zu bertragenden Datei

   <-           TRUE bei Erfolg, sonst FALSE

 */
int 
transfer_send (char *art, const char *name)
{
	int errorflag=0;
	char destination[PATH_MAX];
	char *cmd;
	size_t len;
	protokoll_t *prot;

	if (!init_done)
		read_transfer_cnf ();

	if (!*art)
		*art = transfer_protokollauswahl (FALSE, FALSE);
	else
		*art = transfer_protokollcheck(*art);
	if (!*art)
		return 1; /* Abbruch */
	for (prot = anker; prot; prot = prot->next)
	{
		if (tolower (*art) == tolower(prot->key))
			break;
	}
	if (!prot)
	{
		mbprintf ("Kein Protokoll %c gefunden!\n", *art);
		syslog (LOG_NOTICE, "kein Protokoll %c gefunden\n", *art);
		return 2; /* Fehler */
	}

	if (prot->key=='A')
	{
		mbprintf (
			"Die bertragung endet mit einer Zeile, die nur ein # enthlt\r\n");
		mbprintf ("Protokoll startet:\r\n"); 
		return transfer_ascii_send(name);
	}

	*destination = '\0';

	if (prot->ask_destination)
	{
		if (!get_filename (FALSE, destination))
			return 1; /* Abbruch */
	}

	len = PATH_MAX * 2 + 4096;
	cmd = malloc (len);
	if (!cmd)
	{
		speicherproblem (FALSE, "transfer_send");
		return 2; /* Fehler */
	}

	while (name && *name)
	{
		char *p=NULL; /* keep gcc quiet */
		char c=0;
		if (prot->one_file)
		{
			p=strpbrk(name,"\t ");
			if (p)
			{
				c=*p;
				*p=0;
			}
		}
		protokoll_quelle = name;
		protokoll_ziel = destination;

		if (!do_interpret (prot->sendcmd, cmd, len))
		{
			len *= 4;
			free (cmd);
			if (!cmd)
			{
				speicherproblem (FALSE, "transfer_send");
				if (c)
					*p=c;
				return 2; /* Fehler */
			}

			if (!do_interpret (prot->sendcmd, cmd, len))
			{
				mbfehler ("Kommandointerpretation gescheitert");
				free (cmd);
				if (c)
					*p=c;
				return 2; /* Fehler */
			}
		}

		protokoll_quelle = NULL;
		protokoll_ziel = NULL;

	#if 0
		Das geht nicht - manche Scripte springen auf beide an. Und
		die erste Variante ist Standard.
		mbprintf ("Protokoll startet\r%s-Protokoll startet\r\n", prot->name);
	#else
		mbprintf ("Protokoll startet:\r\n");
	#endif
		io_drain (user_in_fd);

		{

			time_t endzeit;

			/* Policy: 5 Minuten berziehen darf er, aber nicht ber
			 * hard_limit hinaus. Wir sind hier beim Download.
			 */
			endzeit=akt_anruf.hard_limit;
			if (endzeit>akt_anruf.soft_limit+5*60)
				endzeit=akt_anruf.soft_limit+5*60;
			if (akt_anruf.max_transferzeit 
				&& akt_anruf.max_transferzeit+time(NULL) > endzeit)
				endzeit=time(NULL)+akt_anruf.max_transferzeit;
			errorflag=call_extern(cmd,
				NULL,                 /* spezielles Arbeitsverzeichnis */
				endzeit,              /* killzeitpunkt */
				FALSE,                /* ignore_exitcode */
				FALSE);               /* Nicht unter uid des Users */
		}

		if (c)
			*p=c;

		if (errorflag)
			break;
		if (prot->one_file) {
			if (p)
				name=p+1;
			else
				name=NULL;
		}
		else
			name=NULL;
	}
	/* evtl. "Geistereingaben" des ZModem beseitigen */
	msleep (200);
	io_flush (user_in_fd);

	free (cmd);

	return (errorflag == 0) ? 0 : 2;

}								/* transfer_send */
