/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/

#include "tomsysdep.h"
#include <portab.h>

#include <signal.h>
#include <limits.h>
#include <string.h>
#include <stdio.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <syslog.h>
#include <unistd.h> /* chdir, _exit */

#include "user.h"
#include "tsignal.h"
#include "modem.h"
#include "aktdata.h"
#include "prototyp.h"
#include "tsignal.h"
#include "tompaths.h"
#include "box_conf.h"
#include "io.h"


volatile int got_sigint;
volatile int got_sigusr1;
volatile int got_sigusr2;
volatile int got_sigterm;
static volatile int SIGHUP_received=0;
static volatile int carrier_lost_cleanup_needed=0;

static RETSIGTYPE SIGINT_handler(int siechtum);
static RETSIGTYPE SIGUSR2_handler(int siechtum);
static RETSIGTYPE SIGTERM_handler(int siechtum);
static RETSIGTYPE fault_sig_handler(int siechtum);
static void carrier_lost_cleanup(void);

extern BYTE modem_vorhanden;
extern int logout_modus;
extern int user_in_fd;
extern int user_out_fd;
extern int sysop_in_fd;
extern int sysop_out_fd;

extern RETSIGTYPE SIGUSR1_handler(int siechtum);

RETSIGTYPE
SIGUSR1_handler(int siechtum TOM_ATTRIB_UNUSED)
{
	got_sigusr1=1;
	signal(SIGUSR1,SIGUSR1_handler);
}


static RETSIGTYPE
SIGINT_handler(int siechtum TOM_ATTRIB_UNUSED)
{
	got_sigint=1;
	signal(SIGINT,SIGINT_handler);
}


static RETSIGTYPE
SIGUSR2_handler(int siechtum TOM_ATTRIB_UNUSED)
{
	if (!got_sigusr2)
	{
		const char *meldung;
		signal(SIGUSR2,SIG_IGN);
		meldung="Ihr Anruf muss leider beendet werden\r\n";
		got_sigusr2=1;
		logout_modus=1;
		write(user_out_fd,meldung,strlen(meldung));
		strcpy(akt_anruf.zuletzt,"URG");
	}
	signal(SIGUSR2,SIGUSR2_handler);
}

static RETSIGTYPE
SIGTERM_handler(int siechtum TOM_ATTRIB_UNUSED)
{
	if (!got_sigterm)
	{
		const char *meldung;
		signal(SIGTERM,SIG_IGN);
		meldung="Ihr Anruf muss leider beendet werden\r\n";
		got_sigterm=1;
		logout_modus=1;
		write(user_out_fd,meldung,strlen(meldung));
		strcpy(akt_anruf.zuletzt,"TER");
	}
	signal(SIGTERM,SIGTERM_handler);
}


/* Programmfehler aufgetreten */
static RETSIGTYPE 
fault_sig_handler(int siechtum)
{
	/* jedes einzelne dieser Probleme kann bedeuten, da von jetzt 
	 * an die Standardlib nicht mehr belstigt werden darf, da 
	 * smtliche Daten im Programm Schrott sind und so weiter.
	 * Die Wahrscheinlichkeit, da Strukturen der I/O-Library und/oder
	 * der Speicherverwaltung zerschossen sind, ist recht hoch.
	 * 
	 * Allerdings wre es etwas feines, wenn wir Syslog und Co.
	 * noch aufrufen drften ...
	 */
	static int leider_verloren=-1;
	char zahl[2];
	int fd;
	char path[PATH_MAX];
	const char *msg;
	int i;


	if (leider_verloren!=-1)
	{
		/* Oh weia - whrend der Abarbeitung der Routine weiter unten
		 * ist etwas ganz frchterlich schief gegangen.
		 */
		abort();
	}

	/* das geht unter obskuren Zeichenstzen schief, aber das ist nur
	 * ein kleineres Migeschick, das Groe ist schon passiert.
	 */
	zahl[0]='0'+(siechtum)/10; 
	zahl[1]='0'+(siechtum)%10;

	leider_verloren=siechtum;

	strcpy(path,box_conf.quark_dir);
	strcat(path,"/ERROR.TOM");
	fd=open(
		path,
		O_WRONLY|O_CREAT|O_APPEND
#ifdef O_FSYNC
		|O_FSYNC
#endif
		, 
		S_IRUSR|S_IWUSR);

	if (fd<0)
	{
		msg="Bad luck. Cannot open errorfile after receiving signal.\r\n";
		write(2,msg,strlen(msg));
		i=1;
	}
	else
		i=2;
	while (i--)
	{
		if (i==0)
			fd=2;
		msg="tom: Received signal ";
		write(fd,msg,strlen(msg));
		write(fd,zahl,sizeof(zahl));
		msg=". Aborting.\r\n";
		write(fd,msg,strlen(msg));
		if (i)
			close(fd);
	}
	tom_leave_after_error(128+siechtum);
	/* this is never reached. byebye. */
}

void
catch_SIGINT(BOOLEAN modus)
{
	if (modus)
		signal(SIGINT,SIGINT_handler);
	else	
		signal(SIGINT,SIG_DFL);
}

void 
catch_SIGUSR2(BOOLEAN modus)
{
	if (modus)
		signal(SIGUSR2,SIGUSR2_handler);
	else
		signal(SIGUSR2,SIG_DFL);
}

void 
catch_SIGTERM(BOOLEAN modus)
{
	if (modus)
		signal(SIGTERM,SIGTERM_handler);
	else
		signal(SIGTERM,SIG_DFL);
}

void
catch_fault_signals(BOOLEAN modus)
{
#define MODUS ((modus)? (fault_sig_handler) : SIG_DFL)
#ifdef SIGFPE
	/* floating point exception */
	signal(SIGFPE,MODUS);
#endif
#ifdef SIGILL
	/* illegal instruction */
	signal(SIGILL,MODUS);
#endif
#ifdef SIGSEGV
	/* invalid access to valid memory */
	signal(SIGSEGV,MODUS);
#endif
#ifdef SIGBUS
	/* access to invalid address */
	signal(SIGBUS,MODUS);
#endif
#ifdef SIGEMT
	/* emulator trap */
	signal(SIGEMT,MODUS);
#endif
#ifdef SIGSYS
	/* illegal system call */
	signal(SIGSYS,MODUS);
#endif
#ifdef SIGABRT
	/* abort() wurde aufgerufen. Hmm ... */
	signal(SIGABRT,MODUS);
#endif
#ifdef SIGIOT
	/* I/O-Trap ??? */
	signal(SIGIOT,MODUS);
#endif
#undef MODUS
}


/*
 * Carrier-Lost-berwachung.
 *
 * Konzept: Bei Eintreffen eines SIGHUP wird ein Flag gesetzt
 * (oben, static int SIGHUP_received), das spter an definierten 
 * Checkpoints abgefragt wird (SIGHUP_checkpoint, siehe unten).
 * SIGHUP_checkpoint prft, ob das Flag gesetzt ist, und beendet
 * bei Carrier Lost das Programm, falls der Funktion TRUE 
 * bergeben wurde.
 * carrier_still_up() fragt nur den Carrier-Lost-Status ab und
 * liefert FALSE, wenn kein Carrier mehr vorhanden ist, aber
 * einer vorhanden sein sollte (bei Modem ...)
 * SIGHUP und SIGPIPE werden gleich behandelt, da SIGPIPE erhalten
 * wird bei TCP-Verbindungen - anstelle von SIGHUP ...
 */


static RETSIGTYPE
SIGHUP_handler(int siechtum)
{
	if (SIGHUP_received==0)
	{
		carrier_lost_cleanup_needed=1;
		SIGHUP_received=1;
	}
	signal(siechtum,SIGHUP_handler);
}

void 
setup_SIGHUP(BOOLEAN modus)
{
	if (modus)
	{
		signal(SIGHUP,SIGHUP_handler);
		signal(SIGPIPE,SIGHUP_handler);
	}
	else
	{
		signal(SIGHUP,SIG_DFL);
		signal(SIGPIPE,SIG_DFL);
	}
}


#ifdef EXPLIZITE_CARRIERABFRAGE
BOOLEAN
carrier_still_up(void)
{
	if (!SIGHUP_received)
	{
		if (!akt_connection.has_modem_lines)
			return TRUE;
		if (io_test_carrier(user_in_fd))
			return TRUE;
		SIGHUP_received=1;
		carrier_lost_cleanup();
	}
	else if (carrier_lost_cleanup_needed)
		carrier_lost_cleanup();
	return FALSE;
}
#endif

/*
 * Diese Funktion rumt nach einem SIGHUP auf. Das umfat:
 * - Flags setzen, um gewisse Verhaltensweisen der Box auszuschalten
 * - Terminalfiledescriptoren ungltig machen (close() ist hier
 *   unerwnscht, da wir damit das ctty verlieren wrden, und 
 *   beim Exit eines Session Leaders bekommen alle Prozesse einer
 *   Session SIGHUP/SIGCONT - egal zu welchem User sie gehren. Von
 *   daher *mssen* wir das ctty behalten bis zum bitteren Ende)
 */
static void
carrier_lost_cleanup(void)
{
	user_out_fd=-1;
	user_in_fd=-1;
	logout_modus=1;
	carrier_lost_cleanup_needed=0;
}

void 
carrier_checkpoint(BOOLEAN leave_program)
{

	if (!SIGHUP_received)
	{
#ifdef EXPLIZITE_CARRIERABFRAGE
		if (!akt_connection.has_modem_lines)
			return;
		if (io_test_carrier(user_in_fd))
			return;
		SIGHUP_received=1;
#else
		return;
#endif
	}

	/* der Carrier ist weg */
	if (!leave_program)
	{
		if (carrier_lost_cleanup_needed)
			carrier_lost_cleanup();
		return;
	}

	/*
	 * Achtung: Keine Ausgaben mehr auf dem Terminaldevice - das
	 * knnte blockieren.
	 */
	modem_vorhanden=FALSE;
	 
	/* wir knnten den carrier verloren haben, whrend wir in einem
	 * anderen Verzeichnis waren. Also zurck.
	 */
	chdir(box_conf.quark_dir);

	syslog(LOG_INFO,"Carrier-lost Meldung (SIGHUP_checkpoint)");
#if 0
	/* vt100 Ausgabefenster zurcknehmen */
	/* ... */
	/* ta_mark_invalid(); */  /* just to be sure */

	befehls_umlenkung(NULL);

	sleep(2);

	login_protokoll_write();

	akt_anruf.anruftyp=ANRUF_NIL;

	/* User- und Brettdatenbank schlieen? */

	tom_leave(LEAVE_RESTART,NULL);
#endif
}

