/* usercl.c : still no description */
/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <portab.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>
#include <ctype.h>
#include <syslog.h>

#include "box_conf.h"
#include "prototyp.h" /* entumlauten */
#include "userbase.h"
#include "ub_int.h"
#include "modem.h"
#include "filelock.h"

/*
	User anlegen

	->			new	:	komplett initialisierte Userdatenstruktur
							(Usernummer MUSS 0 sein)

	<-			TRUE	:	User wurde angelegt
				FALSE	:	Speichermangel oder User bereits vorhanden

	Die Usernummer wird von dieser Funktion vergeben und zurckgeliefert
*/
BOOLEAN ub_create_user(USER *new)
{
	LISTUSER *newlist = NULL;
	FILE *f;
	FILE *fb;
	char fname[PATH_MAX];
	char fname_backup[PATH_MAX];
	char buf[1024];
	USERNR high_nr=0;
	int fd;
	char ufname[PATH_MAX];
	ub_fd_control_t *cont;

	if (!new || !new->name || !*new->name) {
		fprintf(stderr,"ub_create_user: no name given\n");
		syslog(LOG_ERR,"ub_create_user: no name given\n");
		return FALSE;
	}

	strcpy(stpcpy(fname,ub_basepath),USERDATNAME);
	strcpy(stpcpy(fname_backup,fname),".bak");
	
	/* Liste der User ffnen und locken */ 
	f=fopen(fname,"r+");
	if (!f) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot open %s: %s\n",fname,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot open %s: %s\n",fname,strerror(e));
		return FALSE;
	}
	if (fd_writew_lock(fileno(f),0,SEEK_SET,1)!=0) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot lock %s: %s\n",fname,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot lock %s: %s\n",fname,strerror(e));
		fclose(f);
		unlink(fname_backup);
		return FALSE;
	}

	fb=fopen(fname_backup,"w+");
	if (!fb) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot open %s: %s\n",fname_backup,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot open %s: %s\n",fname_backup,strerror(e));
		fclose(f);
		return FALSE;
	}
	while (fgets(buf,sizeof(buf),f)) {
		long n;

		n=strtol(buf,NULL,10);
		if (n > high_nr)
			high_nr=n;
		fputs(buf,fb);
	}
	if (ferror(f)) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot read %s: %s\n",fname,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot read %s: %s\n",fname,strerror(e));
		fclose(fb);
		fclose(f);
		unlink(fname_backup);
		return FALSE;
	}
	if (0!=fflush(fb) || ferror(fb)) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot write %s: %s\n",fname_backup,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot write %s: %s\n",fname_backup,strerror(e));
		fclose(f);
		fclose(fb);
		unlink(fname_backup);
		return FALSE;
	}

	/*
	 * bitte beachten: Bis jetzt ist *nichts* verndert worden.
	 */

	/*
	 * So, jetzt den eigentlichen Datensatz des Users schreiben.
	 */
	new->nummer = high_nr+1;
	new->magic=UB_MAGIC;
	sprintf(ufname,"%s/users/%lu",ub_basepath,new->nummer);

	fd=open(ufname,O_RDWR|O_CREAT|O_TRUNC,0600);
	if (fd<0) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot open %s: %s\n",ufname,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot open %s: %s\n",ufname,strerror(e));
		close(fd);
		fclose(f);
		fclose(fb);
		unlink(fname_backup);
		return FALSE;
	}
	if (fd_read_lock(fd,0,SEEK_SET,1)!=0) {
		int e=errno;
		/* this should not happen - kein anderer Quarkprozess kann das tun, da die 
		 * alle diese Routine benutzen. Anyway, besser vorsichtig sein.
		 */
		fprintf(stderr,"ub_create_user: cannot lock %s: %s\n",ufname,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot lock %s: %s\n",ufname,strerror(e));
		close(fd);
		fclose(f);
		fclose(fb);
		unlink(fname_backup);
		return FALSE;
	}
	if (-1==xwrite(fd,new,sizeof(USER))) {
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot write %s: %s\n",ufname,strerror(e));
		syslog(LOG_ERR,"ub_create_user: cannot write %s: %s\n",ufname,strerror(e));
		close(fd);
		fclose(f);
		fclose(fb);
		unlink(fname_backup);
		return FALSE;
	}

	/* jetzt in die globale Datenbank eintragen */
	fprintf(f,"%lu:%s\n",high_nr+1,new->name);
	if (0!=fflush(f) || ferror(f) || fclose(f)) {
		char fname_backup2[PATH_MAX];
		int e=errno;
		fprintf(stderr,"ub_create_user: cannot write %s: %s\n",fname,strerror(e));
		fprintf(stderr,"ub_create_user: user database possibly corrupt\n");
		syslog(LOG_CRIT,"ub_create_user: cannot write %s: %s.\n",fname,strerror(e));
		syslog(LOG_CRIT,"ub_create_user: user database possibly corrupt\n");
		fclose(fb);
		fclose(f);
		close(fd);
		/* 
		 * wir duerfen nicht das Backup umbenennen in die richtige, nun zerstrte Datei.
		 * Da stecken mehr Raceconditions drin als gut ist!
		 *
		 * wir duerfen aber auch nicht die korrupte Datei lschen - dann kommt nmlich
		 * niemand mehr in die Box, also auch nicht der Sysopknecht, der das wieder
		 * beheben muss ...
		 */
		sprintf(stpcpy(fname_backup2,fname),".%lu",(ULONG) time(NULL));
		rename(fname_backup,fname_backup2);
		return FALSE;
	}
	/* die Originaldatei ist jetzt geschlossen. */
	fclose(fb);

	/*
	 * Kontrollliste der offenen User ergaenzen.
	 */
	cont=XMALLOC(sizeof(ub_fd_control_t));
	cont->unr=new->nummer;
	cont->next=ub_fd_anker;
	cont->fd=fd;
	ub_fd_anker=cont;

	/* 
	 * Neuen User in die in-core liste eintragen.
	 */
	newlist = XCALLOC(1,sizeof(LISTUSER));
	newlist->name = XSTRDUP(new->name);
	newlist->nummer = new->nummer;
	newlist->next = ub_anker;
	ub_anker = newlist;

	ub_userzahl++;
	return TRUE;
} /* ub_create_user */

