/*
    Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
    Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
    Contact3: MausNet Newsgroups Quark & Q-Sysops.
*/
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <portab.h>
#include <errno.h>
#include <limits.h>
#include <unistd.h>
#include <ctype.h>
#include <syslog.h>

#include "box_conf.h"
#include "prototyp.h" /* entumlauten */
#include "userbase.h"
#include "ub_int.h"
#include "modem.h"
#include "filelock.h"

LISTUSER *ub_anker;
ub_fd_control_t *ub_fd_anker;

char *ub_basepath = NULL;
ULONG ub_userzahl=0;

#define USERDATNAME "user_2.dat"

static BOOLEAN
ub_load_user_dat(const char *userfile)
{
	FILE *f;
	char buf[1024];
	
	ub_anker = NULL;
	
	f = fopen(userfile,BIN_RW_OPEN);
	if (!f)
		return FALSE;

	if (fd_readw_lock(fileno(f),0,SEEK_SET,1)!=0) {
		int e=errno;
		fprintf(stderr,"ub_load_user_dat: cannot lock %s: %s\n",userfile,strerror(e));
		syslog(LOG_ERR,"ub_load_user_dat: cannot lock %s: %s\n",userfile,strerror(e));
		fclose(f);
		return FALSE;
	}
	
	while (fgets(buf,sizeof(buf),f)) {
		/* Format der Datei:
		 * #irgendwas -> kommentar
		 * Nummer:Vor Nachname:var=value...
		 */
		char *dop;
		char *p;
		LISTUSER *neu,*lauf;
		USERNR nummer;

		if (*buf=='#') 
			continue;

		dop=strchr(buf,':');
		if (!dop) {
			fprintf(stderr,"illegal line in user database: %s\n",buf);
			continue;
		}
		p=strchr(buf,'\n');
		if (p)
			*p=0;
		*dop++='\0';
		p=strchr(dop,':');
		if (p==dop) {
			fprintf(stderr,"warning: no username for user # %s\n",buf);
			continue;
		}
		if (p)
			*p++=0;
		strtrim(dop);
		if (!*dop) {
			fprintf(stderr,"warning: no username for user # %s\n",buf);
			continue;
		}
		nummer=strtol(buf,NULL,10);
		if (!nummer || nummer<0) {
			fprintf(stderr,"warning: usernumber %s illegal for %s\n",buf,dop);
			continue;
		}
		for (lauf=ub_anker;lauf;lauf=lauf->next) {
			if (lauf->nummer==nummer) {
				fprintf(stderr,"warning: usernumber %lu not uniq for %s and %sn",nummer,
						lauf->name,dop);
				break;
			}
			if (!stricmp(lauf->name,dop)) {
				fprintf(stderr,"warning: username %s not uniq for # %lu and # %lu",dop,
						lauf->nummer,nummer);
				break;
			}
		}
		if (lauf!=NULL)
			continue;
		neu=XMALLOC(sizeof(LISTUSER));
		neu->nummer=nummer;
		neu->name=XSTRDUP(dop);
		neu->next=ub_anker;
		ub_anker=neu;
		ub_userzahl++;
	}
	fclose(f);
	
	return TRUE;
}

BOOLEAN
ub_init(const char *path)
{
	char userfile[PATH_MAX];
	size_t l = strlen(path);

	if (path[l-1] == '/') {
		ub_basepath = XSTRDUP(path);
	} else {
		strcpy(stpcpy(userfile,path),"/");
		ub_basepath=XSTRDUP(userfile);
	}
	strcpy(stpcpy(userfile,ub_basepath),USERDATNAME);
	
	return ub_load_user_dat(userfile);
}

