/*
 * Copyright (C) 1994 1995 1996 Frank Baumgart, Uwe Ohse
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 * Contact: uwe@tirka.gun.de, Uwe Ohse @ DU3 (mausnet)
 * Contact2: godot@uni-paderborn.de, Frank Baumgart @ PB (mausnet)
 * Contact3: MausNet Newsgroups Quark & Q-Sysops.
 */
#include "tomsysdep.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <portab.h>
#include <errno.h>
#include <ctype.h>
#include <unistd.h>
#include <syslog.h>
#include <limits.h>
#include <sys/types.h>			/* off_t */

#include "prototyp.h"
#include "userbase.h"
#include "ub_int.h"
#include "filelock.h"

#define MAGIC	0x0369CF01L

BOOLEAN
ub_remove_user (USERNR unr)
{
	char fname[PATH_MAX];
	char fname_backup[PATH_MAX];
	char buf[1024];
	FILE *f, *f2;
	LISTUSER *li,*last;
	for (last=NULL,li=ub_anker;li && li->nummer!=unr;last=li,li=li->next) 
		/* nothing */;
	if (!li) {
		fprintf (stderr, "ub_write_user: no such user: # %lu\n",unr);
		syslog (LOG_ERR, "ub_write_user: no such user: # %lu",unr);
		return FALSE;
	}

	if (last)
		last->next=li->next;
	else
		ub_anker=li->next;
	free(li->name);
	free(li);

	/* nun die globale Datei aufrumen */
	strcpy (stpcpy (fname, ub_basepath), USERDATNAME);
	strcpy (stpcpy (fname_backup, fname), ".bak");

	/* Liste der User ffnen und locken */
	f = fopen (fname, "r+");
	if (!f) {
		int e = errno;
		fprintf (stderr, "ub_write_user: cannot open %s: %s\n",
				 fname, strerror (e));
		syslog (LOG_ERR, "ub_write_user: cannot open %s: %s\n",
				fname, strerror (e));
		return TRUE;	/* we *did* write the user */
	}
	if (fd_writew_lock (fileno (f), 0, SEEK_SET, 1) != 0) {
		int e = errno;
		fprintf (stderr, "ub_write_user: cannot lock %s: %s\n",
				 fname, strerror (e));
		syslog (LOG_ERR, "ub_write_user: cannot lock %s: %s\n",
				fname, strerror (e));
		fclose (f);
		unlink (fname_backup);
		return TRUE;	/* we *did* write the user */
	}
	/* Backup anlegen - davon mssen wir gleich lesen! */
	if (!file_copy (fname, fname_backup)) {
		int e = errno;
		fprintf (stderr, "ub_write_user: cannot backup %s: %s\n",
				 fname, strerror (e));
		syslog (LOG_ERR, "ub_write_user: cannot backup %s: %s\n",
				fname, strerror (e));
		fclose (f);
		unlink (fname_backup);
		return TRUE;	/* we *did* write the user */
	}
	f2 = fopen (fname_backup, "r");
	if (!f2) {
		int e = errno;
		fprintf (stderr, "ub_write_user: cannot open %s: %s\n",
				 fname_backup, strerror (e));
		syslog (LOG_ERR, "ub_write_user: cannot open %s: %s\n",
				fname_backup, strerror (e));
		fclose (f);
		return FALSE;
	}
	while (fgets (buf, sizeof (buf), f2)) {
		if (strtol (buf, NULL, 10) == unr) {
			if (EOF == fprintf (f,"#%s",buf))
				break;
		} else {
			fputs(buf,f);
		}
	}
	if (ferror(f2)) {
		char fname_backup2[PATH_MAX];
		int e = errno;
		fprintf (stderr, "ub_create_user: cannot write %s: %s\n",
				 fname, strerror (e));
		fprintf (stderr, 
			"ub_create_user: user database most possibly corrupt\n");
		syslog (LOG_CRIT, "ub_create_user: cannot write %s: %s",
				fname, strerror (e));
		syslog (LOG_CRIT, 
			"ub_create_user: user database most possibly corrupt\n");
		fclose (f2);
		fclose(f);
		sprintf (stpcpy (fname_backup2, fname), ".%lu", (ULONG) time (NULL));
		rename (fname_backup, fname_backup2);
		return FALSE;
	}
	if (fflush (f) || ferror (f) || fclose (f)) {
		char fname_backup2[PATH_MAX];
		int e = errno;
		fprintf (stderr, "ub_create_user: cannot write %s: %s\n",
				 fname, strerror (e));
		fprintf (stderr, 
			"ub_create_user: user database most possibly corrupt\n");
		syslog (LOG_CRIT, "ub_create_user: cannot write %s: %s",
				fname, strerror (e));
		syslog (LOG_CRIT, 
			"ub_create_user: user database most possibly corrupt\n");
		fclose (f2);
		/*
		 * wir duerfen nicht das Backup umbenennen in die richtige, 
		 * nun zerstrte Datei. Da stecken mehr Raceconditions drin 
		 * als gut ist!
		 * 
		 * wir duerfen aber auch nicht die korrupte Datei lschen, 
		 * dann kommt nmlich niemand mehr in die Box, also auch nicht 
		 * der Sysopknecht, der das wieder beheben muss ...
		 */
		sprintf (stpcpy (fname_backup2, fname), ".%lu", (ULONG) time (NULL));
		rename (fname_backup, fname_backup2);
		return FALSE;
	}					/* failed fflush, fclose or fputs */
	fclose(f2);
	sprintf(fname,"%s/users/%lu",ub_basepath,unr);
	sprintf(fname_backup,"%s/users/%lu.bak",ub_basepath,unr);
	rename (fname, fname_backup);

	ub_userzahl--;

	return TRUE;
}								/* ub_remove_user */
